# User defined gdb commands for debuging 3dvm data structures.
#
# $Id: 3dvm.gdb,v 1.1 2002/07/26 12:27:44 peteri Exp $

# prints vertex id number and vertex coordinates
define gdbVert
  printf "  id: %d   coords: (%f %f %f)\n", $arg0->id, 
          $arg0->coords[0], $arg0->coords[1],$arg0->coords[2]
end
document gdbVert
Prints vertex id number and vertex coordinates.
end


# prints id numbers of tetrahedra on the vertex adjacency stack
define gdbVertAdj
  set $t=$arg0->adjElem->head
  while ($t != 0)
    printf "  id: %d  addres: %x\n", ((Tetrah *)((struct elem *)$t->x))->id, \
            ((struct elem *)$t)->x
    set $t = (struct elem *)$t->next
  end
end
document gdbVertAdj
Prints id numbers of tetrahedra on the vertex adjacency stack.
end 

# prints tetrahedron id and id numbers is of ist vertices
define gdbTet
   printf "  id: %d   vertices: %d %d %d %d\n", $arg0->id, \
             $arg0->vertex[0]->id, $arg0->vertex[1]->id, \
             $arg0->vertex[2]->id, $arg0->vertex[3]->id
end
document gdbTet
Prints tetrahedron id and id numbers is of ist vertices.
end

# prints tetrahedron id and id numbers of its neighbours
define gdbTetAdj
   printf "id: %d\n", $arg0->id
   printf "      neighbbours id:  "
   set $i=0
   while ($i <4)
     if ($arg0->neighbour[$i] != 0)
       printf "%d  ", $arg0->neighbour[$i]->id
     else
       printf "-- "
     end
     set $i=$i+1
   end
   printf "\n      neighbour address:  "
   set $i=0
   while ($i <4)
     if ($arg0->neighbour[$i] != 0)
       printf "%x  ", $arg0->neighbour[$i]
     else
       printf "NULL "
     end
     set $i=$i+1
   end
   printf "\n"
end
document gdbTetAdj
Prints tetrahedron id and id numbers of its neighbours.
end

# prints id number and address of elements on the tetrahedron stack
define gdbTetStack
  set $t=$arg0->head
  while ($t != 0)
    printf "  id: %d  addres: %x\n", ((Tetrah *)((struct elem *)$t->x))->id, \
            ((struct elem *)$t)->x
    set $t = (struct elem *)$t->next
  end
end
document gdbTetStack
Prints id number and address of elements on the tetrahedron stack.
end

# prints id number of tetrahedron on stack and id numbers of its vertices
define gdbTetStackDetail
  set $t=$arg0->head
  while ($t != 0)
    set $b=(Tetrah*)($t->x)
    gdbTet $b
    set $t = (struct elem *)$t->next
  end
end
document gdbTetStackDetail
Prints id number of tetrahedron on stack and id numbers of its vertices
end


# prints textual represetation of flags set in tetrahedron 
define gdbTetFlags
  if (($arg0->flag & (1<<0) ) ==  (1<<0) ) 
    printf "  TET_IN_DOMAIN\n"    
  end
  if (($arg0->flag & (1<<1) ) ==  (1<<1) ) 
    printf "  TET_OUT_DOMAIN\n"
  end
  if (($arg0->flag & (1<<2) ) ==  (1<<2) ) 
    printf "  TET_PTAS\n"
  end
  if (($arg0->flag & (1<<3) ) ==  (1<<3) ) 
    printf "  TET_VISITED\n"
  end
  if (($arg0->flag & (1<<4) ) ==  (1<<4) ) 
    printf "  TET_OUTCAVITY\n"
  end
  if (($arg0->flag & (1<<5) ) ==  (1<<5) ) 
    printf "  TET_IIT\n"
  end
  if (($arg0->flag & (1<<6) ) ==  (1<<6) ) 
    printf "  TET_SIIT\n"
  end
  if (($arg0->flag & (1<<7) ) ==  (1<<7) ) 
    printf "  TET_SPLITFACESET\n"
  end
  if (($arg0->flag & (1<<8) ) ==  (1<<8) ) 
    printf "  TET_SOLIDANGLE_OK\n"
  end
  if (($arg0->flag & (1<<9) ) ==  (1<<9) ) 
    printf "  TET_MINSIGMA_OK\n"
  end
  if (($arg0->flag & (1<19) ) ==  (1<19) ) 
    printf "  TET_ESHELL_OUT\n"
  end
  if (($arg0->flag & (1<20) ) ==  (1<20) ) 
    printf "  TET_ESHELL\n"
  end
  if (($arg0->flag & (1<<21)) ==  (1<<21)) 
    printf "  TET_ROUND\n"
  end
  if (($arg0->flag & (1<<22)) ==  (1<<22)) 
    printf "  TET_NEEDLE\n"
  end
  if (($arg0->flag & (1<<23)) ==  (1<<23)) 
    printf "  TET_WEDGE\n"
  end
  if (($arg0->flag & (1<<24)) ==  (1<<24)) 
    printf "  TET_SPINDLE\n"
  end
  if (($arg0->flag & (1<<25)) ==  (1<<25)) 
    printf "  TET_SLIVER\n"
  end
  if (($arg0->flag & (1<<26)) ==  (1<<26)) 
    printf "  TET_CAP\n"
  end
  if (($arg0->flag & (1<<27)) ==  (1<<27)) 
    printf "  TET_UNKNOWN_TYPE\n"
  end
end
document gdbTetFlags
Prints textual represetation of flags set in tetrahedron. 
end

# prints textual represetation of flags set in vertex  
define gdbVertFlags
  if (($arg0->flag & (1<<0) ) ==  (1<<0) ) 
    printf "  POINT_INT_BOUNDARY\n"    
  end
  if (($arg0->flag & (1<<1) ) ==  (1<<1) ) 
    printf "  POINT_EXT_BOUNDARY\n"
  end
  if (($arg0->flag & (1<<2) ) ==  (1<<2) ) 
    printf "  POINT_BOUNDARY\n"
  end
  if (($arg0->flag & (1<<3) ) ==  (1<<3) ) 
    printf "  POINT_ON_EDGE\n"
  end
  if (($arg0->flag & (1<<4) ) ==  (1<<4) ) 
    printf "  POINT_ON_FACE\n"
  end
  if (($arg0->flag & (1<<5) ) ==  (1<<5) ) 
    printf "  POINT_CUBOID\n"
  end
  if (($arg0->flag & (1<<6) ) ==  (1<<6) ) 
    printf "  POINT_CONNECTED\n"
  end
  if (($arg0->flag & (1<<7) ) ==  (1<<7) ) 
    printf "  POINT_SHIELD\n"
  end
end
document gdbVertFlag
Prints textual represetation of flags set in vertex.
end

define gdbCheckMeshFlag
   set $counter=0
   set $tet=$arg0->tList->tail
   while ($tet != 0)
     if (($tet->flag & (1<<4)) == (1<<4))
       gdbtet $tet
       set $arg1=1
     end
     set $tet=$tet->next
     set $counter=$counter+1
   end 
   printf "%d tetrah checked\n", $counter
end
