#ifndef BASIC_OP_H
#define BASIC_OP_H

#ifdef __cplusplus
extern "C"
{
#endif

#include "mstack.h"
#include "vertex.h"
#include "edge.h"
#include "face.h"
#include "tetrah.h"


/* Set vertices of a face to correspond to vertices of given
 * tetrahdral face
 * Arguments:
 *      tet - tetrahedron (Tetrah* type)
 *      fn  - face number (int type)
 *      face - face (Face* type)
 */

#define GetFaceFromTet(tet, fn, face)                            \
  FACE_VERTEX((face), 0) = TET_VERTEX ((tet), faceDef[fn][0]);   \
  FACE_VERTEX((face), 1) = TET_VERTEX ((tet), faceDef[fn][1]);   \
  FACE_VERTEX((face), 2) = TET_VERTEX ((tet), faceDef[fn][2]);


void ExchangeVertex (Tetrah *tet, Vertex *oldVertex, Vertex *newVertex) ;
int CompareFaces (Face *f1, Face *f2);
Vertex *GetFourthVertex (Tetrah *tet, Vertex *v1, Vertex *v2, Vertex *v3);
Tetrah * GetNeighbourOpositeVertex (Tetrah *tet, Vertex *v);
void TetGenericMapping (int i1, int i2, int *o1, int *o2);
int TetHasFace (Tetrah *tet, Face *f, int *facenum);
int TetHasVertex (Tetrah *tet, Vertex *v, int *vertexnum);
int FaceHasEdge (Face *f, Edge *e, int *sidenum);
int TetHasNeighbour (Tetrah *tet, Tetrah *neighbour, int *neighbnum);
int WhichNeighbour (Tetrah *tet, Tetrah *neighbour);
int WhichVertex (Tetrah *tet, Vertex *vertex);
int IsEdgeOnStack (Tos_MStack edgeStack, Vertex *ep0, Vertex *ep1);


#ifdef __cplusplus
}
#endif

#endif /* BASIC_OP_H */





