%{
/********************************************************************
 Roman Putanowicz <romek@sect.mce.hw.ac.uk>
 SECT Research Group, Heriot-Watt University, Edinburgh    

 Changes: for log of changes see the end of the file    
 ********************************************************************/

/* $Id: ctr_lexer.l,v 1.4 2002/07/28 16:56:59 peteri Exp $ */

# include <stdio.h>
# include <string.h>
# include <unistd.h>
/* change global names which comes for the 
   parser generated by bison
*/
#include "ctr_guard.h"
#include "ctr_parser.h"

static int line;

#define YY_NO_UNPUT

%}

digit	[0-9]
exp	[eE][+-]?{digit}+
letter	[a-zA-Z_]
id	{letter}({letter}|{digit})*
eq	[ \t]*=
%x	comment
%%


%{
 /* KEYWORDS */
%}
ALPHA                        {return ALPHA;}
BETA                         {return BETA;}
INPUT_MESH_FILES             {return INPUT_MESH_FILES;}
OUTPUT_MESH_FILE             {return OUTPUT_MESH_FILE;}
REFINE_METHOD                {return REFINE_METHOD;}
NMESHES                      {return NMESHES;}
LOG_LEVEL                    {return LOG_LEVEL;}
NO_LOG                       {return NO_LOG;}
MINIMAL_LOG                  {return MINIMAL_LOG;} 
NORMAL_LOG                   {return NORMAL_LOG;}
FULL_LOG                     {return FULL_LOG;}
EDGE_LEN                     {return EDGE_LEN;}
OPTIMIZATION                 {return OPTIMIZATION;}
INSERTING                    {return INSERTING;}
SWAPPING                     {return SWAPPING;}
LAPLACE                      {return LAPLACE;}
COLLAPSING                   {return COLLAPSING;}


%{
 /* Numbers and names */
%}

[+-]?{digit}+			{yylval.i = atoi (yytext); return INTEGER;}
[+-]?{digit}+{exp}		{yylval.d = atof (yytext); return DOUBLE;}
[+-]?{digit}+\.{digit}*{exp}?	{yylval.d = atof (yytext); return DOUBLE;}
[+-]?{digit}*\.{digit}+{exp}?	{yylval.d = atof (yytext); return DOUBLE;}
\"[^"]*\"			{yylval.s = strdup (yytext + 1);
				 yylval.s [yyleng - 2] = 0; return STRING;}

%{
 /* Comments, white space, and remaining preprocessor lines (the comment
   rules are straight from the flex documentation). */
%}
#[^\n]*$			{line++;}
[ \t]+				{}
\n				{line++;}
"/*"				{BEGIN comment;}
<comment>[^*\n]*		{}
<comment>[^*\n]*\n		{line++;}
<comment>"*"+[^*/\n]*		{}
<comment>"*"+[^*/\n]*\n		{line++;}
<comment>"*"+"/"		{BEGIN INITIAL;}
<<EOF>>                         {return DATAEND;}
.				{return *yytext;}

%%

/************************************************************************
 * Function:	yyerror							*
 *									*
 * Description:	Writes a syntax error message reporting the unexpected	*
 *		token only if input is being read from a file.		*
 ************************************************************************/

void yyerror (char* msg)
{
  fprintf (stderr, "(%s) : parse error before `%s', line %d\n", msg, yytext,
                           line); 
  exit (-1);
}

/************************************************************************
 * Function:	ctr_InitLexer						*
 *									*
 * Description:	Initializes the lexer for a new file                    *
 ************************************************************************/

void ctr_InitLexer (FILE* filePtr)
{
    static int first_time = 1;

    if (first_time == 1) 
    {
    	first_time = 0;
	yyin = filePtr;
    } 
    else
    {
	yyrestart (filePtr);
    }
    line = 1;
}

int yywrap ( )
{
    return 1;
}

/* Modifications:
 * $Log: ctr_lexer.l,v $
 * Revision 1.4  2002/07/28 16:56:59  peteri
 * - the required optimization method is specified in the file
 *   OPTIMIZATION INSERTING LAPLACE SWAPPING COLLAPSING
 *
 * Revision 1.3  2002/07/26 17:53:32  peteri
 * - Internal point is not needed for flooding any more
 *
 * Revision 1.2  2002/07/26 13:06:26  peteri
 * CENTR_VERT_LEN refinement method is removed
 *
 * Revision 1.1.1.1  2002/01/30 10:20:56  peteri
 * Imported using TkCVS
 *
 */

