/********************************************************************
 ctr_read.h --
    Functions to support reading mesh generator control data file.  
 
 Roman Putanowicz <romek@sect.mce.hw.ac.uk>
 SECT Research Group, Heriot-Watt University Edinburgh    
 ********************************************************************/

/* $Id: ctr_read.h,v 1.5 2002/07/28 16:57:02 peteri Exp $ */

#ifndef CTR_READ_H
#define CTR_READ_H


#ifdef __cplusplus
extern "C"
{
#endif


#include <stdio.h> 
#include <assert.h>

#include "mesh.h"

void MeshGenCtrlRead (MeshGenCtrl *mctrl, const char *filename); 
int  MGCSet_nmeshes (MeshGenCtrl *mctrl, int nmeshes);
int  MGCSet_inputFileNames (MeshGenCtrl *mctrl, int meshNum, char* fileName);
int  MGCSet_outputFileName (MeshGenCtrl *mctrl, char* fileName);
int  MGCSet_alpha (MeshGenCtrl *mctrl, double alpha);
int  MGCSet_beta (MeshGenCtrl *mctrl, double beta);
int  MGCSet_refineMethod (MeshGenCtrl *mctrl, vm_RefineMethod refineMethod);
int  MGCSet_logLevel (MeshGenCtrl *mctrl, vm_LogLevel logLevel);
int  MGCSet_optimization (MeshGenCtrl *mctrl, int optimization);


#ifdef __cplusplus
}
#endif


#endif /* CTR_READ_H */


/* Modifications:
 * $Log: ctr_read.h,v $
 * Revision 1.5  2002/07/28 16:57:02  peteri
 * - the required optimization method is specified in the file
 *   OPTIMIZATION INSERTING LAPLACE SWAPPING COLLAPSING
 *
 * Revision 1.4  2002/07/26 17:53:35  peteri
 * - Internal point is not needed for flooding any more
 *
 * Revision 1.3  2002/01/31 22:37:40  peteri
 * Minor modifications, some renaming.
 *
 * Revision 1.2  2002/01/31 21:27:49  peteri
 * Major reorganization in the code.
 * - It is split into smaller modules.
 * - Some types, functions are renamed
 * - Proper flipping of edges is implemented
 *
 * Revision 1.1.1.1  2002/01/30 10:20:56  peteri
 * Imported using TkCVS
 *
 * Revision 1.2  2001/10/27 18:31:57  romek
 * *** empty log message ***
 *
 * Revision 1.1  2001/10/04 17:59:19  romek
 * Initial revision
 *
 */
