/* $Id: edge.c,v 1.7 2002/07/26 11:39:07 peteri Exp $ */

#include <stdio.h>
#include <stdlib.h>
#include "edge.h"


/* initialise the edge memory pool */
void EdgeMemPoolInit(void)
{
  ;
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/****f*
 * NAME
 *   EdgeCreate 
 * DESCRIPTION
 *   Allocate memory for new Edge data structure and initialize
 *   it with default values.
 * ARGUMENTS
 * RETURN VALUE
 *   Pointer to allocated structure.
 ******/
Edge *EdgeCreate (void)
{
  Edge *e;
  int i;
  
  e = (Edge *) calloc (1, sizeof (Edge));
  if (e == NULL)
  {
    return NULL;
  }

  e->splitEdges = Tos_MStackNew();
  e->mesh = -1;
  EDGE_FLAG (e) = 0;
  EDGE_NEIGHBOUR(e, 0) = NULL;
  EDGE_NEIGHBOUR(e, 1) = NULL;
  for (i=0; i<2; i++)
  {
    EDGE_VERTEX(e, i) = NULL;
  }
  return e;
} /* end of EdgeCreate */


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/****f*
 * NAME
 *   EdgeDelete
 * DESCRIPTION
 *   Free memory allocated for Tetrha data structure and sets
 *   pointer pointing to that memory to NULL.
 * ARGUMENTS
 *   edge - pointer to edge data structure
 * RETURN VALUE
 *   NONE
 ******/
void EdgeDelete (Edge *edge)
{
  edge->vertex[0] = edge->vertex[1] = NULL;
  edge->neighbour[0] = edge->neighbour[1] = NULL;
  /* FIXME: split edges maybe freed one-by-one */
  Tos_MStackFree (&edge->splitEdges);
  edge->flag = 0; 
  free (edge);
} /* end of EdgeDelete */

/* Modifications:
 * $Log: edge.c,v $
 * Revision 1.7  2002/07/26 11:39:07  peteri
 * - edge stores split edges and not splitting points
 * - edge stores the mesh number to which it belongs to
 *
 * Revision 1.6  2002/07/19 20:37:52  peteri
 * Mempool is removed for simplicity
 *
 * Revision 1.5  2002/02/24 19:01:05  romek
 * *** empty log message ***
 *
 * Revision 1.4  2002/02/16 16:55:34  romek
 * Doc fix.
 * Fixed memory leak in EdgeDelete.
 *
 */
