/* $Id: face.c,v 1.6 2002/07/26 11:39:32 peteri Exp $ */

#include <stdio.h>
#include <stdlib.h>
#include "face.h"


/* initialise the vertex memory pool */
void FaceMemPoolInit(void)
{
  ;
}



/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/****f*
 * NAME
 *  FaceCreate 
 * DESCRIPTION
 *   Allocate memory for new Face data structure and initialize
 *   it with default values.
 * ARGUMENTS
 *   NONE
 * RETURN VALUE
 *   Pointer to allocated structure.
 ******/
Face *FaceCreate (void)
{
  Face *f;
  int i;

  f = (Face *) calloc (1, sizeof (Face)); 
  if (f == NULL)
  {
    return NULL;
  } 
  
  FACE_FLAG (f) = 0;
  f->mesh = -111;
  f->splitFaces = Tos_MStackNew();
  
  for (i=0; i<3; i++)
  {
    FACE_NEIGHBOUR(f, i) = NULL;
    FACE_VERTEX(f, i) = NULL;
  }
  return f;
} /* end of FaceCreate */


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/****f*
 * NAME
 *   FaceDelete 
 * DESCRIPTION
 *   Free memory allocated for Face data structure.
 * ARGUMENTS
 *   face - pointer to face data structure.
 * RETURN VALUE
 *   NONE
 ******/
void FaceDelete (Face *face)
{
  face->vertex[0] = face->vertex[1] = face->vertex[2] = NULL;
  face->neighbour[0] = face->neighbour[1] = face->neighbour[2] = NULL;
  face->flag = 0;
  Tos_MStackFlush (face->splitFaces);
  Tos_MStackFree (&(face->splitFaces));
  free (face); 
} /* end of FaceDelete */


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/****f*
 * NAME
 *   IsFaceOnBoundary 
 * DESCRIPTION
 *   Check if face is boundary one by checking if all of its vertices
 *   have the boundary flag 'boundFlag' set.
 * ARGUMENTS
 *   face - pointer to face data structure
 *   boudFlag - flag to be checked against
 * RETURN VALUE
 *   1 if face is on boundary, 0 otherwise
 ******/
int IsFaceOnBoundary (Face *face, int boundFlag)
{
  Vertex *v0, *v1, *v2;
  v0 = FACE_VERTEX(face,0);
  v1 = FACE_VERTEX(face,1);
  v2 = FACE_VERTEX(face,2);
  
  if(   ((v0->flag & boundFlag) == boundFlag)
     && ((v1->flag & boundFlag) == boundFlag)
     && ((v2->flag & boundFlag) == boundFlag) )
  {
    return 1;
  }

  return 0;
} /* end of IsFaceOnBoundary */

