/* $Id: face.h,v 1.7 2002/07/26 11:39:33 peteri Exp $ */

#ifndef FACE_H
#define FACE_H

#ifdef __cplusplus
extern "C"
{
#endif

#include "mstack.h"
#include "vertex.h"
#include "edge.h"


/* macros to access of structure members */
#define FACE_VERTEX(fac, i)    ((fac)->vertex[i])
#define FACE_VCOORDS(fac, i)   ((fac)->vertex[i]->coords)
#define FACE_NEIGHBOUR(fac, i) ((fac)->neighbour[i])
#define FACE_FLAG(fac)         ((fac)->flag)
#define FACE_VNUMBER(fac, i)   ((fac)->vertex[i]->number)
#define FACE_NEXT(fac)         ((fac)->next)
#define FACE_PREV(fac)         ((fac)->prev)

/* Faces flags */
#define FACE_BOUNDARY  (1<<0)

typedef struct face_
{
  Vertex          *vertex[3];    /* face vertices */
  struct edge_    *neighbour[3]; /* pointers to neighbouring edges */
  Tos_MStack       splitFaces;
  int              flag;
  int              mesh;         /* which boundary mesh contains it */
  
  int              domain;
  int              number;    /* FIXME: maybe going ??? */
  int              subdomain;
  unsigned int     id;        /* unique id number (used only for debugging) */
  struct face_    *prev;      /* pointer to previous element in the list 
                                 (previously added) */
  struct face_    *next;      /* pointer to next element in the list */
}
Face;

void FaceMemPoolInit(void);
Face *FaceCreate (void);
void FaceDelete (Face *face);
int IsFaceOnBoundary (Face *face, int boundFlag);


#ifdef __cplusplus
}
#endif

#endif /* FACE_H */

/* Modifications
 * $Log: face.h,v $
 * Revision 1.7  2002/07/26 11:39:33  peteri
 * - face stores the mesh number to which it belongs to
 *
 * Revision 1.6  2002/03/18 15:54:55  peteri
 * multiple adjacency flag is removed
 *
 * Revision 1.5  2002/02/16 17:05:55  romek
 * Doc fix.
 *
 */
