#ifndef KERNEL_H
#define KERNEL_H

#ifdef __cplusplus
extern "C"
{
#endif

#include "mstack.h"
#include "tetrah.h"
#include "mesh.h"

/* bits to control the volume meshing */
#define USE_BETA          (1<<0)
#define USE_GOODPOINT     (1<<1)
#define USE_ENCROACHMENT  (1<<2)

/* number of points which are tried to find a good point */
#define NMAX_POINT    30

/* The value which is used to determine the size of the picking zone.
 * It should be larger than zero ans smaller than one. The size of
 * the picking zone is calculated as the radius of the circum-sphere
 * of the tetrahedron multiplied by this vaue.
 */
#define PICKING_SIZE  0.3


double MeshParamGetAtPos(MeshGenData *mgds, double *pos);

void UpdateCavity(TetrahMesh *tetMesh,
                  Tos_MStack cavity,
                  Tos_MStack newCavity);
void ReconnectCavity (Tos_MStack cavity,
                      Tos_MStack newCavity,
                      Vertex *point);

int DelaunayKernel (TetrahMesh *tetMesh, Tos_MStack waitingPoints,
                    int force);

int EmptyTriang (MeshGenData *mgds);

int VolumeTriang (int method, MeshGenData *data, TetrahMesh *tetMesh,
                  double alpha, double beta, int mode);

Vertex *GoodPointSelection(MeshGenData *mgds, TetrahMesh *tetMesh,
                           double *coords, double radius,
                           int encroachment, int optimize,
                           double ms_min, double ms_max,
                           double md_min, double md_max);


int CheckEdgeEncroachment(MeshGenData *data,
                          double *coords, double *newcoords,
                          Edge **encedge);
int CheckFaceEncroachment(MeshGenData *data,
                          double *coords, double *newcoords,
                          Face **encface, Edge **encedge);


#ifdef __cplusplus
}
#endif

#endif /* KERNEL_H */



