
#ifndef LIST_H
#define LIST_H

#ifdef __cplusplus
extern "C"
{
#endif

#include <stdio.h>
#include "vertex.h"
#include "edge.h"
#include "face.h"
#include "tetrah.h"


/* Direction of list traversal */
#define HEAD_TAIL -1
#define TAIL_HEAD  1


#define LIST_HEAD(list) ((list)->head)
#define LIST_TAIL(list) ((list)->tail)


typedef struct tetlist_
{
  Tetrah *head;     /* head of the list (the most recent added element) */
  Tetrah *tail;     /* tail of the list (the oldest element on list) */
  Tetrah *nextItem; /* pointer to the element returned by list 
                     * traversal routine */
}
TetList;


typedef struct edgelist_
{
  Edge *head;     /* head of the list (the most recent added element) */
  Edge *tail;     /* tail of the list (the oldest element on list) */
  Edge *nextItem; /* pointer to the element returned by list 
                   * traversal routine */
}
EdgeList;

typedef struct facelist_
{
  Face *head;     /* head of the list (the most recent added element) */
  Face *tail;     /* tail of the list (the oldest element on list) */
  Face *nextItem; /* pointer to the element returned by list 
                   * traversal routine */
}
FaceList;

typedef struct vertexlist_
{
  Vertex *head;     /* head of the list (the most recent added element) */
  Vertex *tail;     /* tail of the list (the oldest element on list) */
  Vertex *nextItem; /* pointer to the element returned by list
                     * traversal routine */  
}
VertexList;



TetList    *CreateTetList();
FaceList   *CreateFaceList();
EdgeList   *CreateEdgeList();
VertexList *CreateVertexList();

void PutTetOnListHead(TetList *list, Tetrah *item);
void PutFaceOnListHead(FaceList *list, Face *item);
void PutEdgeOnListHead(EdgeList *list, Edge *item);
void PutVertexOnListHead(VertexList *list, Vertex *item);

void PutTetOnListTail(TetList *list, Tetrah *item);
void PutFaceOnListTail(FaceList *list, Face *item);
void PutEdgeOnListTail(EdgeList *list, Edge *item);
void PutVertexOnListTail(VertexList *list, Vertex *item);

void RemoveTetFromList(TetList *list, Tetrah *item);
void RemoveFaceFromList(FaceList *list, Face *item);
void RemoveEdgeFromList(EdgeList *list, Edge *item);
void RemoveVertexFromList(VertexList *list, Vertex *item);

void TetListTraversalInit(TetList *list, int direction);
void EdgeListTraversalInit(EdgeList *list, int direction);
void FaceListTraversalInit(FaceList *list, int direction);
void VertexListTraversalInit(VertexList *list, int direction);

Tetrah *TetListTraverse(TetList *list, int direction);
Edge *EdgeListTraverse(EdgeList *list, int direction);
Face *FaceListTraverse(FaceList *list, int direction);
Vertex *VertexListTraverse(VertexList *list, int direction);


#ifdef __cplusplus
}
#endif

#endif /* LIST_H */
