/********************************************************************
 tet_measures.h --     
        Various shape measures of tetrahedron. 
 
 Roman Putanowicz <romek@sect.mce.hw.ac.uk>
 SECT Research Group, Heriot-Watt University Edinburgh    
 ********************************************************************/

/* $Id: measures.h,v 1.4 2002/07/03 21:26:54 peteri Exp $ */

#ifndef MEASURES_H
#define MEASURES_H

#include "tetrah.h"
#include "mesh.h"


#define NTYPES_TET 7 /* number of different tetrah types */

/* Types of tetrahedron related to number and kind of bad angles */
enum TetrahType {ROUND_TET_TYPE  =0, 
                 NEEDLE_TET_TYPE =1, 
                 WEDGE_TET_TYPE  =2,
                 SPINDLE_TET_TYPE=3, 
                 SLIVER_TET_TYPE =4, 
                 CAP_TET_TYPE    =5, 
                 UNKNOWN_TET_TYPE=6}; 

/* Types of tetrahdron  angles*/
enum AngleType {DIHEDRAL_ANGLE    =100,
                SOLID_ANGLE       =101};


/* Types of tetrahdron shape measures */
enum MeasureType {SIGMA_MEASURE       =200,
                  RADIUS_RATIO_MEASURE=201,
                  MEAN_RATIO_MEASURE  =202};

extern const char* tetTypeNames[]; 

void GetTetEdgesLengthTable (double *vcoords[4], double *ltable[4]);
double SinHalfSolidAngle (int vertexIndex, double volume, double *ltable[4]);
double RadiusRatio (double *vcoords[4], double *ltable[4]);
double MeanRatio(double volume, double *ltable[4]);
double Tetrah_MinSigmaMeasure (Tetrah *tet, int *index);
double Tetrah_RadiusRatio (Tetrah *tet);
double Tetrah_MeanRatio (Tetrah *tet);
double Tetrah_SolidAngle (Tetrah *tet, int index);
double Tetrah_DihedralAngle (Tetrah *tet, int i);
double Tetrah_Measure (Tetrah *tet, enum MeasureType measure);   

void Tetrah_MinAndMaxAngle (Tetrah *tet, enum AngleType angle,
                            double *minAngle, int *minind, 
                            double *maxAngle, int *maxind);

void Tetrah_MinAndMaxDihedralAngle (Tetrah *tet, 
                                    double *mindangle, int *eminind, 
                                    double* maxdangle, int *emaxind);
void Tetrah_MinAndMaxSolidAngle (Tetrah *tet, 
                                 double *minsangle, int *vminind, 
                                 double* maxsangle, int *vmaxind);
void Mesh_MinAndMaxSolidAngle (TetrahMesh *tetMesh, 
                              Tetrah **tet_min, double *sa_min, int *index_min,
                              Tetrah **tet_max, double *sa_max, int *index_max);
void Mesh_MinAndMaxDihedralAngle (TetrahMesh *tetMesh, double *mindangle, 
                                 double *maxdangle);

void  Mesh_MinAndMaxMeasure (TetrahMesh *tetMesh, enum MeasureType measure, 
                             double *minMeasure, double *maxMeasure);

enum TetrahType TetrahSetTypeFlag (Tetrah *tet, double ms_min, double ms_max, 
                                            double md_min, double md_max);
void ClassifyTetrah (TetrahMesh *tetMesh);
void CountTetrahType(TetrahMesh *tetMesh, int* tetTypeCount);

void FindMeasureLimits (Tos_MStack tetAssembly, enum MeasureType measure, 
                        double *minMeasure, double *maxMeasure);


void GoodAnglesLimits (TetrahMesh *tetMesh, 
                  double *ms_min, double *ms_max,
                  double *md_min, double *md_max);
#endif /* MEASURES_H */

