#ifndef MESH_H
#define MESH_H


#ifdef __cplusplus
extern "C"
{
#endif

/* CAUTION value=1<<(VM_MAX_NMESHES-1) might by assigned  to 
 * variable of type int so if you are going to increase VM_MAX_NMESHES
 * check for overflow (and eventualy change type of 'domain' member in
 * Tetrah data structure ans 'domainFlag' in RecoverFace and 
 * DeleteOuterElements
 */
#define VM_MAX_NMESHES 10  /* max. number of input meshes */


#include "e_lib.h"
#include "list.h"
#include "vertex.h"
#include "edge.h"
#include "face.h"
#include "tetrah.h"

/* Structure of generated tetrahedral mesh */
typedef struct 
{
  TetList    *tList;    /* list of tetrahedra */
  VertexList *pList;    /* list of meshpoints */

  int NMeshPoints;      /* number of mehs points in final mesh */
  int NTetrah;          /* number of tetrahedra in final mesh */
}
TetrahMesh; 

typedef enum
{
    VM_NO_LOG,
    VM_MINIMAL_LOG,
    VM_NORMAL_LOG,
    VM_FULL_LOG
}
vm_LogLevel;

typedef enum
{
    VM_REFINE_EDGE_LEN
}
vm_RefineMethod;

#define OPT_INSERTING          (1<<0)
#define OPT_SWAPPING           (1<<1)
#define OPT_LAPLACE            (1<<2)
#define OPT_COLLAPSING         (1<<3)

/* mesh generator control data structure */
typedef struct
{
  double            alpha;
  double            beta;
  int               includeEmbededMeshes;
  vm_RefineMethod   refineMethod;
  int               optimizeMethod;
  int               nmeshes;
  char             *inputFileNames[VM_MAX_NMESHES];
  MESH             *inputMeshes[VM_MAX_NMESHES];
  char             *outputFileName;
  char             *ctrlFileName;
  char             *logFileName;
  FILE             *logFile;
  double            subdomainPoint[VM_MAX_NMESHES][3];
  int               subdomainMeshes[VM_MAX_NMESHES];
  int               nsubdomains;
  vm_LogLevel       logLevel;
}
MeshGenCtrl;


/* The main data structure for mesh generator */
/* The array 'inputMeshes' contains input meshes to the generator
 * inputMeshes[0] contains the mesh of the most outer boundary if the meshed
 * domain.  
 */
typedef struct
{
  int      NMeshes;                     /* number of input meshes */
  
  /* tables for faces, edges and meshpoints of input meshes */
  Face   **fTable[VM_MAX_NMESHES];
  Edge   **eTable[VM_MAX_NMESHES];
  Vertex **pTable[VM_MAX_NMESHES];

  /* lengths of faces, edges and meshpoints tables */
  int fTableLength[VM_MAX_NMESHES];
  int eTableLength[VM_MAX_NMESHES];
  int pTableLength[VM_MAX_NMESHES];

  double low[3];
  double high[3];
  
  /* Tetrahedral mesh */
  TetrahMesh tetMesh;

  /* mesh generator control data structure */
  MeshGenCtrl *mgCtrl;
}
MeshGenData;


#ifdef __cplusplus
}
#endif

#endif /* MESH_H */

