#include <stdio.h>
#include <stdlib.h>

#include "tetrah.h"


/* generic tetrahedron face numbering */
const int faceDef[4][3] = {{1,2,3}, {3,2,0}, {0,1,3}, {2,1,0}};

/* generic tiangular side numbering */
const int sideDef[3][2] = {{1,2}, {2,0}, {0,1}}; 

/* generics tetrahedron edges numbering */
const int edgeDef[6][2] = {{0, 1}, {0, 2}, {0, 3}, {1, 2}, {1, 3}, {2, 3}};



/* initialise the tetrahedron memory pool */
void TetrahMemPoolInit(void)
{
  ;
}


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 TetrahCreate --
        Allocate memory for new Tetrah data structure and initialize
        it with default values.
 Return value:
        Pointer to allocated structure.
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
Tetrah *TetrahCreate (void)
{
  static unsigned int idCounter=1; /* couter for creating uniue Id
				      numbers for tetrahedra */
  Tetrah *tet;

  tet = (Tetrah* ) calloc (1, sizeof (Tetrah ));
  if (tet == NULL)
  {
    return NULL;
  }  
  TET_FLAG (tet) = 0;
  TET_NUMBER (tet) = 0;
  TET_DOMAIN (tet) = 0;
  TET_SUBDOMAIN(tet) = MAX_SUBDOMAIN_NUM;

  TET_NEIGHBOUR(tet, 0) = NULL;
  TET_VERTEX(tet, 0) = NULL;

  TET_NEIGHBOUR(tet, 1) = NULL;
  TET_VERTEX(tet, 1) = NULL;

  TET_NEIGHBOUR(tet, 2) = NULL;
  TET_VERTEX(tet, 2) = NULL;

  TET_NEIGHBOUR(tet, 3) = NULL;
  TET_VERTEX(tet, 3) = NULL;
  tet->id = idCounter;
  idCounter++;
  return tet;
} /* end of TetrahCreate */



/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 TetrahDelete --
        Free memory allocated for Tetrha data structure and sets
        pointer pointing to that memory to NULL.
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void TetrahDelete (Tetrah *tet)
{
  tet->id = 0;
  tet->neighbour[0] = NULL;
  tet->neighbour[1] = NULL;
  tet->neighbour[2] = NULL;
  tet->neighbour[3] = NULL;
  tet->vertex[0] = NULL;
  tet->vertex[1] = NULL;
  tet->vertex[2] = NULL;
  tet->vertex[3] = NULL;
  tet->flag=0;
  free (tet);
} /* end of DeleteTetrah */



/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 TetrahClearFlag
        Pop all elements from the stack and clear their specified 
        flag.
        
 Arguments:       
        stk - stack of Tetrah, Faces  or Edges elements
        flag - flag to be cleared
 Note:
         This function is intendet to clear flags in Tetrah, Edges
 or Faces elements. As all mentioned types are implemented
 with the same data structure this function can bee safely used
 to stack holding any of Tetrah, Edges or Faces items.
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void TetrahClearFlag (Tos_MStack stk, int flag)
{
  Tetrah *tet; 
  while (Tos_MStackCount (stk) > 0)
  {
    tet = (Tetrah *) Tos_MStackPop (stk);
    TET_FLAG(tet) &= ~flag;  /* clear the flag 'flag' */    
  }
  return;
} /* end of TetrahClearFlag */


