#include <stdio.h>
#include <stdlib.h>
#include "vertex.h"


/* initialise the vertex memory pool */
void VertexMemPoolInit(void)
{
  ;
}


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 VertexCreate --
        Allocate memory for new Vertex data structure and initialize
        it with default values.
 Return value:
        Pointer to allocated structure.
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
Vertex *VertexCreate (void)
{
  Vertex *point;
  static unsigned int idCounter=0; /* counter to create unique
				      id numbers for Points */

  point = (Vertex *) calloc (1, sizeof (Vertex)); 
  if (point == NULL)
  {
    return NULL;
  }
  point->flag = 0;
  point->number = 0;
  point->coords[0] = 0.0;
  point->coords[1] = 0.0;
  point->coords[2] = 0.0;
  point->meshParam = 0.0;
  /* create adjacency stack */
  point->adjElem = Tos_MStackNew ();
  point->id = idCounter;
  idCounter++;
  return point;
} /* end of CreateVertex */


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 VertexDelete --
        Free memory allocated for Vertex data structure and sets
        pointer pointing to that memory to NULL.
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void VertexDelete (Vertex *point)
{
  /* free stack of adjacent elements */
  Tos_MStackFree (&(point->adjElem));
  /* free point itself */
  free (point); 
} /* end of DeleteVertex */


