/* ////////////////////////////////////////////////////////////////////////// */
/*                                                                            */
/*                      e_GetMeshData(), e_PutMeshData()                      */
/*                                                                            */
/* ////////////////////////////////////////////////////////////////////////// */

#include "e_pltfrm.h"
#include <stdio.h>
#include <string.h>
#include <e_lib.h>

#include "comsrc.h"
#include "meshdata.h"


/* ////////////////////////////////////////////////////////////////////////// */
/* NOTE: It is currently assumed that the remeshing data are the last data to
         be read from the file.                                               */

/* ////////////////////////////////////////////////////////////////////////// */
/* e_GetMeshData: To read in mesh data. */
/* The return value is 1 if no input errors occur, else 0. */


int e_GetMeshData(MESH **mesh, char *InputFile)
{
   *mesh = NULL;
   FullPathName = NULL;

   if (            (*mesh = (MESH *)e_calloc(1,sizeof(MESH))) == NULL
       || (FullPathName = OpenInputFile(InputFile,E_MDF,&fp)) == NULL)
   {
      e_free(*mesh);
      e_free(FullPathName);
      return 0;
   }

   Initialise();

   if (   GetData()
       && AllCompulsoryKeywordsSet()
       && OptionalDataOkay2()
       && SetMeshData(*mesh)        )
   {
      e_free(FullPathName);
      fclose(fp);
      return 1;
   }
   else
   {
      DeallocateAllMemory(*mesh);
      fclose(fp);
      return 0;
   }
}


/* ////////////////////////////////////////////////////////////////////////// */
/* initialise: To initialise variables. */


static void Initialise()
{
   int i;


   strcpy(Keyword[E_KW_NUM_TITLE],E_KW_TITLE);
   strcpy(Keyword[E_KW_NUM_NMESHPOINTS],E_KW_NMESHPOINTS);
   strcpy(Keyword[E_KW_NUM_NNODES],E_KW_NNODES);

   strcpy(Keyword[E_KW_NUM_NELEMS_LINK1],E_KW_NELEMS_LINK1);
   strcpy(Keyword[E_KW_NUM_NELEMS_LINK2],E_KW_NELEMS_LINK2);
   strcpy(Keyword[E_KW_NUM_NELEMS_LINK3],E_KW_NELEMS_LINK3);
   strcpy(Keyword[E_KW_NUM_NELEMS_LINK4],E_KW_NELEMS_LINK4);
   strcpy(Keyword[E_KW_NUM_NELEMS_LINK5],E_KW_NELEMS_LINK5);
   strcpy(Keyword[E_KW_NUM_NELEMS_TRIANG1],E_KW_NELEMS_TRIANG1);
   strcpy(Keyword[E_KW_NUM_NELEMS_TRIANG2],E_KW_NELEMS_TRIANG2);
   strcpy(Keyword[E_KW_NUM_NELEMS_TRIANG3],E_KW_NELEMS_TRIANG3);
   strcpy(Keyword[E_KW_NUM_NELEMS_TRIANG4],E_KW_NELEMS_TRIANG4);
   strcpy(Keyword[E_KW_NUM_NELEMS_TRIANG5],E_KW_NELEMS_TRIANG5);
   strcpy(Keyword[E_KW_NUM_NELEMS_QUAD1],E_KW_NELEMS_QUAD1);
   strcpy(Keyword[E_KW_NUM_NELEMS_QUAD3],E_KW_NELEMS_QUAD3);
   strcpy(Keyword[E_KW_NUM_NELEMS_TETRAH1],E_KW_NELEMS_TETRAH1);
   strcpy(Keyword[E_KW_NUM_NELEMS_BLOCK1],E_KW_NELEMS_BLOCK1);

   strcpy(Keyword[E_KW_NUM_NBC_NODES],E_KW_NBC_NODES);
   strcpy(Keyword[E_KW_NUM_NLOADED_NODES],E_KW_NLOADED_NODES);
   strcpy(Keyword[E_KW_NUM_NMATS],E_KW_NMATS);
   strcpy(Keyword[E_KW_NUM_NCOMP_MATS_TYPE1],E_KW_NCOMP_MATS_TYPE1);
   strcpy(Keyword[E_KW_NUM_NNURBS_CURVES],E_KW_NNURBS_CURVES);
   strcpy(Keyword[E_KW_NUM_TIMESTEP],E_KW_TIMESTEP);
   strcpy(Keyword[E_KW_NUM_NTIMESTEPS],E_KW_NTIMESTEPS);
   strcpy(Keyword[E_KW_NUM_NINTERNAL_TIMESTEPS],E_KW_NINTERNAL_TIMESTEPS);
   strcpy(Keyword[E_KW_NUM_NEXTERNAL_TIMESTEPS],E_KW_NEXTERNAL_TIMESTEPS);
   strcpy(Keyword[E_KW_NUM_DAMPING_FACTOR],E_KW_DAMPING_FACTOR);
   strcpy(Keyword[E_KW_NUM_BETA],E_KW_BETA);
   strcpy(Keyword[E_KW_NUM_GAMMA],E_KW_GAMMA);

   strcpy(Keyword[E_KW_NUM_NSTRESS_POINTS_LINK1],E_KW_NSTRESS_POINTS_LINK1);
   strcpy(Keyword[E_KW_NUM_NSTRESS_POINTS_LINK2],E_KW_NSTRESS_POINTS_LINK2);
   strcpy(Keyword[E_KW_NUM_NSTRESS_POINTS_LINK3],E_KW_NSTRESS_POINTS_LINK3);
   strcpy(Keyword[E_KW_NUM_NSTRESS_POINTS_LINK4],E_KW_NSTRESS_POINTS_LINK4);
   strcpy(Keyword[E_KW_NUM_NSTRESS_POINTS_LINK5],E_KW_NSTRESS_POINTS_LINK5);
   strcpy(Keyword[E_KW_NUM_NSTRESS_POINTS_TRIANG1],E_KW_NSTRESS_POINTS_TRIANG1);
   strcpy(Keyword[E_KW_NUM_NSTRESS_POINTS_TRIANG2],E_KW_NSTRESS_POINTS_TRIANG2);
   strcpy(Keyword[E_KW_NUM_NSTRESS_POINTS_TRIANG3],E_KW_NSTRESS_POINTS_TRIANG3);
   strcpy(Keyword[E_KW_NUM_NSTRESS_POINTS_TRIANG4],E_KW_NSTRESS_POINTS_TRIANG4);
   strcpy(Keyword[E_KW_NUM_NSTRESS_POINTS_TRIANG5],E_KW_NSTRESS_POINTS_TRIANG5);
   strcpy(Keyword[E_KW_NUM_NSTRESS_POINTS_QUAD1],E_KW_NSTRESS_POINTS_QUAD1);
   strcpy(Keyword[E_KW_NUM_NSTRESS_POINTS_QUAD3],E_KW_NSTRESS_POINTS_QUAD3);
   strcpy(Keyword[E_KW_NUM_NSTRESS_POINTS_TETRAH1],E_KW_NSTRESS_POINTS_TETRAH1);
   strcpy(Keyword[E_KW_NUM_NSTRESS_POINTS_BLOCK1],E_KW_NSTRESS_POINTS_BLOCK1);

   strcpy(Keyword[E_KW_NUM_MESHPOINT_COORDS],E_KW_MESHPOINT_COORDS);

   strcpy(Keyword[E_KW_NUM_NODES_LINK1],E_KW_NODES_LINK1);
   strcpy(Keyword[E_KW_NUM_NODES_LINK2],E_KW_NODES_LINK2);
   strcpy(Keyword[E_KW_NUM_NODES_LINK3],E_KW_NODES_LINK3);
   strcpy(Keyword[E_KW_NUM_NODES_LINK4],E_KW_NODES_LINK4);
   strcpy(Keyword[E_KW_NUM_NODES_LINK5],E_KW_NODES_LINK5);
   strcpy(Keyword[E_KW_NUM_NODES_TRIANG1],E_KW_NODES_TRIANG1);
   strcpy(Keyword[E_KW_NUM_NODES_TRIANG2],E_KW_NODES_TRIANG2);
   strcpy(Keyword[E_KW_NUM_NODES_TRIANG3],E_KW_NODES_TRIANG3);
   strcpy(Keyword[E_KW_NUM_NODES_TRIANG4],E_KW_NODES_TRIANG4);
   strcpy(Keyword[E_KW_NUM_NODES_TRIANG5],E_KW_NODES_TRIANG5);
   strcpy(Keyword[E_KW_NUM_NODES_QUAD1],E_KW_NODES_QUAD1); 
   strcpy(Keyword[E_KW_NUM_NODES_QUAD3],E_KW_NODES_QUAD3); 
   strcpy(Keyword[E_KW_NUM_NODES_TETRAH1],E_KW_NODES_TETRAH1);
   strcpy(Keyword[E_KW_NUM_NODES_BLOCK1],E_KW_NODES_BLOCK1);

   strcpy(Keyword[E_KW_NUM_BCS],E_KW_BCS);

   strcpy(Keyword[E_KW_NUM_LOADS],E_KW_LOADS);

   strcpy(Keyword[E_KW_NUM_MATS_LINK1],E_KW_MATS_LINK1);
   strcpy(Keyword[E_KW_NUM_MATS_LINK2],E_KW_MATS_LINK2);
   strcpy(Keyword[E_KW_NUM_MATS_LINK3],E_KW_MATS_LINK3);
   strcpy(Keyword[E_KW_NUM_MATS_LINK4],E_KW_MATS_LINK4);
   strcpy(Keyword[E_KW_NUM_MATS_LINK5],E_KW_MATS_LINK5);
   strcpy(Keyword[E_KW_NUM_MATS_TRIANG1],E_KW_MATS_TRIANG1);
   strcpy(Keyword[E_KW_NUM_COMP_MATS_TRIANG2],E_KW_COMP_MATS_TRIANG2);
   strcpy(Keyword[E_KW_NUM_MATS_TRIANG3],E_KW_MATS_TRIANG3);
   strcpy(Keyword[E_KW_NUM_MATS_TRIANG4],E_KW_MATS_TRIANG4);
   strcpy(Keyword[E_KW_NUM_MATS_TRIANG5],E_KW_MATS_TRIANG5);
   strcpy(Keyword[E_KW_NUM_MATS_QUAD1],E_KW_MATS_QUAD1);
   strcpy(Keyword[E_KW_NUM_COMP_MATS_QUAD3],E_KW_COMP_MATS_QUAD3);
   strcpy(Keyword[E_KW_NUM_MATS_TETRAH1],E_KW_MATS_TETRAH1);
   strcpy(Keyword[E_KW_NUM_MATS_BLOCK1],E_KW_MATS_BLOCK1);

   strcpy(Keyword[E_KW_NUM_REMESH_DATA],E_KW_REMESH_DATA);

   for (i = 0; i < E_NKEYWORDS; i++)
   {
      KeywordIsOptional[i] = TRUE;
      KeywordSet[i] = FALSE;
   }

   KeywordIsOptional[E_KW_NUM_TITLE] = FALSE;
   KeywordIsOptional[E_KW_NUM_NMESHPOINTS] = FALSE;
   KeywordIsOptional[E_KW_NUM_NNODES] = FALSE;
   KeywordIsOptional[E_KW_NUM_MESHPOINT_COORDS] = FALSE;
   
   for (i = 0; i < E_NELEM_TYPES; i++)
      FirstNetIndex[i] = 0;

   NElemsLink1 = 0;
   NElemsLink2 = 0;
   NElemsLink3 = 0;
   NElemsLink4 = 0;
   NElemsLink5 = 0;
   NElemsTriang1 = 0;
   NElemsTriang2 = 0;
   NElemsTriang3 = 0;
   NElemsTriang4 = 0;
   NElemsTriang5 = 0;
   NElemsQuad1 = 0;
   NElemsQuad3 = 0;
   NElemsTetrah1 = 0;
   NElemsBlock1 = 0;
   TotalNElems = 0;
   TotalNodalDOF = 0;
   NodalDOF = NULL;
   NBCNodes = 0;
   NLoadedNodes = 0;
   NMats = 0;
   NCompMatsType1 = 0;
   CurrentNMats = 0;
   CurrentNCompMatsType1 = 0;
   Geom.NNurbsCurves = 0;
   CurrentNNurbsCurves = 0;
   LineNumber = 0;
   ReadNextLine = TRUE;
   DefinedStresses = 0;
}


/* ////////////////////////////////////////////////////////////////////////// */
/* GetData: To read in the input data. */

/* The return value is 1 if the data are read successfully or the end of file
   is encountered, else 0.                                                    */

/* NOTE: Under no circumstances whatsoever must this function be altered! */


static int GetData()
{
   int i = 0;


   if (GetKeyword(line,word,&LineNumber,FullPathName,fp) == EOF)
      return 1;

   while (TRUE)
   {
      while (i < E_NKEYWORDS && KeywordIsOptional[i])
      {
         if (WordMatchesKeyword(Keyword[i],word,TRUE))
         {
            if (GetKeywordData(i++) == 1)
            {
               if (   ReadNextLine
                   && GetKeyword(line,word,&LineNumber,FullPathName,fp) == EOF)
                  return 1;
               else
                  ReadNextLine = TRUE;
            }
            else
               return 0;
         }
         else
            i++;
      }

      if (i == E_NKEYWORDS)
      {
         WordMatchesKeyword(" ",word,FALSE);
         return 0;
      }

      if (   !WordMatchesKeyword(Keyword[i],word,FALSE)
          || !GetKeywordData(i++)                      )
      {
         return 0;
      }

      if (   ReadNextLine
          && GetKeyword(line,word,&LineNumber,FullPathName,fp) == EOF)
         return 1;
      else
         ReadNextLine = TRUE;
   }
}


/* ////////////////////////////////////////////////////////////////////////// */
/* WordMatchesKeyword: To return whether a word is a specified keyword. */

/* The return value is 1 if the word matches the keyword, else 0. */


static int WordMatchesKeyword(char *Keyword, char *word, char optional)
{
   if (strcmp(Keyword,word) == 0)
   {
      return 1;
   }
   else if (optional)
   {
      return 0;
   }
   else
   {
      if (IsAKeyword(word))
         e_FileLineError(LineNumber,FullPathName,"keyword out of sequence");
      else
         e_FileLineError(LineNumber,FullPathName,"undefined keyword");

      return 0;
   }
}


/* ////////////////////////////////////////////////////////////////////////// */
/* GetKeywordData: To read in keyword data. */

/* The return value is 1 if the data are read successfully, else 0. */


static int GetKeywordData(int i)
{
   switch (i)
   {
      case E_KW_NUM_TITLE:
      {
         return GetTitle();
      }
      case E_KW_NUM_NMESHPOINTS:
      {
         return GetNMeshPoints();
      }
      case E_KW_NUM_NNODES:
      {
         if (   GetNNodes()
             && (NodalDOF = e_alloc_i_vect(NNodes)) != NULL)
            return 1;
         else
            return 0;
      }
      case E_KW_NUM_NELEMS_LINK1:
      {
         return GetNElems(&NElemsLink1,E_ELEM_TYPE_LINK1,
                          E_KW_NUM_NELEMS_LINK1,E_KW_NUM_NODES_LINK1);
      }
      case E_KW_NUM_NELEMS_LINK2:
      {
         return GetNElems(&NElemsLink2,E_ELEM_TYPE_LINK2,
                          E_KW_NUM_NELEMS_LINK2,E_KW_NUM_NODES_LINK2);
      }
      case E_KW_NUM_NELEMS_LINK3:
      {
         return GetNElems(&NElemsLink3,E_ELEM_TYPE_LINK3,
                          E_KW_NUM_NELEMS_LINK3,E_KW_NUM_NODES_LINK3);
      }
      case E_KW_NUM_NELEMS_LINK4:
      {
         return GetNElems(&NElemsLink4,E_ELEM_TYPE_LINK4,
                          E_KW_NUM_NELEMS_LINK4,E_KW_NUM_NODES_LINK4);
      }
      case E_KW_NUM_NELEMS_LINK5:
      {
         return GetNElems(&NElemsLink5,E_ELEM_TYPE_LINK5,
                          E_KW_NUM_NELEMS_LINK5,E_KW_NUM_NODES_LINK5);
      }
      case E_KW_NUM_NELEMS_TRIANG1:
      {
         return GetNElems(&NElemsTriang1,E_ELEM_TYPE_TRIANG1,
                          E_KW_NUM_NELEMS_TRIANG1,E_KW_NUM_NODES_TRIANG1);
      }
      case E_KW_NUM_NELEMS_TRIANG2:
      {
         return GetNElems(&NElemsTriang2,E_ELEM_TYPE_TRIANG2,
                          E_KW_NUM_NELEMS_TRIANG2,E_KW_NUM_NODES_TRIANG2);
      }
      case E_KW_NUM_NELEMS_TRIANG3:
      {
         return GetNElems(&NElemsTriang3,E_ELEM_TYPE_TRIANG3,
                          E_KW_NUM_NELEMS_TRIANG3,E_KW_NUM_NODES_TRIANG3);
      }
      case E_KW_NUM_NELEMS_TRIANG4:
      {
         return GetNElems(&NElemsTriang4,E_ELEM_TYPE_TRIANG4,
                          E_KW_NUM_NELEMS_TRIANG4,E_KW_NUM_NODES_TRIANG4);
      }
      case E_KW_NUM_NELEMS_TRIANG5:
      {
         return GetNElems(&NElemsTriang5,E_ELEM_TYPE_TRIANG5,
                          E_KW_NUM_NELEMS_TRIANG5,E_KW_NUM_NODES_TRIANG5);
      }
      case E_KW_NUM_NELEMS_QUAD1:
      {
         return GetNElems(&NElemsQuad1,E_ELEM_TYPE_QUAD1,
                          E_KW_NUM_NELEMS_QUAD1,E_KW_NUM_NODES_QUAD1);
      }
      case E_KW_NUM_NELEMS_QUAD3:
      {
         return GetNElems(&NElemsQuad3,E_ELEM_TYPE_QUAD3,
                          E_KW_NUM_NELEMS_QUAD3,E_KW_NUM_NODES_QUAD3);
      }
      case E_KW_NUM_NELEMS_TETRAH1:
      {
         return GetNElems(&NElemsTetrah1,E_ELEM_TYPE_TETRAH1,
                          E_KW_NUM_NELEMS_TETRAH1,E_KW_NUM_NODES_TETRAH1);
      }
      case E_KW_NUM_NELEMS_BLOCK1:
      {
         return GetNElems(&NElemsBlock1,E_ELEM_TYPE_BLOCK1,
                          E_KW_NUM_NELEMS_BLOCK1,E_KW_NUM_NODES_BLOCK1);
      }
      case E_KW_NUM_NBC_NODES:
      {
         return GetNBCNodes();
      }
      case E_KW_NUM_NLOADED_NODES:
      {
         return GetNLoadedNodes();
      }
      case E_KW_NUM_NMATS:
      {
         return GetNMats();
      }
      case E_KW_NUM_NCOMP_MATS_TYPE1:
      {
         return GetNCompMatsType1();
      }
      case E_KW_NUM_NNURBS_CURVES:
      {
         return GetNNurbsCurves();
      }
      case E_KW_NUM_TIMESTEP:
      {
         return GetTimeStep();
      }
      case E_KW_NUM_NTIMESTEPS:
      {
         return GetNTimeSteps();
      }
      case E_KW_NUM_NINTERNAL_TIMESTEPS:
      {
         return GetNInternalTimeSteps();
      }
      case E_KW_NUM_NEXTERNAL_TIMESTEPS:
      {
         return GetNExternalTimeSteps();
      }
      case E_KW_NUM_DAMPING_FACTOR:
      {
         return GetDampingFactor();
      }
      case E_KW_NUM_BETA:
      {
         return GetBeta();
      }
      case E_KW_NUM_GAMMA:
      {
         return GetGamma();
      }

      case E_KW_NUM_NSTRESS_POINTS_LINK1:
      {
         return GetNStress(&NStressPointsLink1, 
                           E_KW_NUM_NSTRESS_POINTS_LINK1);
      }
      case E_KW_NUM_NSTRESS_POINTS_LINK2:
      {
         return GetNStress(&NStressPointsLink2, 
                           E_KW_NUM_NSTRESS_POINTS_LINK2);
      }
      case E_KW_NUM_NSTRESS_POINTS_LINK3:
      {
         return GetNStress(&NStressPointsLink3, 
                           E_KW_NUM_NSTRESS_POINTS_LINK3);
      }
      case E_KW_NUM_NSTRESS_POINTS_LINK4:
      {
         return GetNStress(&NStressPointsLink4, 
                           E_KW_NUM_NSTRESS_POINTS_LINK4);
      }
      case E_KW_NUM_NSTRESS_POINTS_LINK5:
      {
         return GetNStress(&NStressPointsLink5, 
                           E_KW_NUM_NSTRESS_POINTS_LINK5);
      }
      case E_KW_NUM_NSTRESS_POINTS_TRIANG1:
      {
         return GetNStress(&NStressPointsTriang1,
                           E_KW_NUM_NSTRESS_POINTS_TRIANG1);
      }
      case E_KW_NUM_NSTRESS_POINTS_TRIANG2:
      {
         return GetNStress(&NStressPointsTriang2,
                           E_KW_NUM_NSTRESS_POINTS_TRIANG2);
      }
      case E_KW_NUM_NSTRESS_POINTS_TRIANG3:
      {
         return GetNStress(&NStressPointsTriang3,
                           E_KW_NUM_NSTRESS_POINTS_TRIANG3);
      }
      case E_KW_NUM_NSTRESS_POINTS_TRIANG4:
      {
         return GetNStress(&NStressPointsTriang4,
                           E_KW_NUM_NSTRESS_POINTS_TRIANG4);
      }
      case E_KW_NUM_NSTRESS_POINTS_TRIANG5:
      {
         return GetNStress(&NStressPointsTriang5,
                           E_KW_NUM_NSTRESS_POINTS_TRIANG5);
      }
      case E_KW_NUM_NSTRESS_POINTS_QUAD1:
      {
         return GetNStress(&NStressPointsQuad1,
                           E_KW_NUM_NSTRESS_POINTS_QUAD1);
      }
      case E_KW_NUM_NSTRESS_POINTS_QUAD3:
      {
         return GetNStress(&NStressPointsQuad3,
                           E_KW_NUM_NSTRESS_POINTS_QUAD3);
      }
      case E_KW_NUM_NSTRESS_POINTS_TETRAH1:
      {
         return GetNStress(&NStressPointsTetrah1,
                           E_KW_NUM_NSTRESS_POINTS_TETRAH1);
      }
      case E_KW_NUM_NSTRESS_POINTS_BLOCK1:
      {
         return GetNStress(&NStressPointsBlock1,
                           E_KW_NUM_NSTRESS_POINTS_BLOCK1);
      }
      case E_KW_NUM_MESHPOINT_COORDS:
      {
         if (   OptionalDataOkay1()
             && GetMeshPointCoords())
            return 1;
         else
            return 0;
      }
      case E_KW_NUM_NODES_LINK1:
      {
         return GetLinkNodes(NElemsLink1,&NodesLink1,
                             E_NNPE_LINK1,E_VERTEX_NODE_DOF_LINK1,
                             E_KW_NUM_NELEMS_LINK1,E_KW_NUM_NODES_LINK1);
      }
      case E_KW_NUM_NODES_LINK2:
      {
         return GetLinkNodes(NElemsLink2,&NodesLink2,
                             E_NNPE_LINK2,E_VERTEX_NODE_DOF_LINK2,
                             E_KW_NUM_NELEMS_LINK2,E_KW_NUM_NODES_LINK2);
      }
      case E_KW_NUM_NODES_LINK3:
      {
         return GetLinkNodes(NElemsLink3,&NodesLink3,
                             E_NNPE_LINK3,E_VERTEX_NODE_DOF_LINK3,
                             E_KW_NUM_NELEMS_LINK3,E_KW_NUM_NODES_LINK3);
      }
      case E_KW_NUM_NODES_LINK4:
      {
         return GetLinkNodes(NElemsLink4,&NodesLink4,
                             E_NNPE_LINK4,E_VERTEX_NODE_DOF_LINK4,
                             E_KW_NUM_NELEMS_LINK4,E_KW_NUM_NODES_LINK4);
      }
      case E_KW_NUM_NODES_LINK5:
      {
         return GetLinkNodes(NElemsLink5,&NodesLink5,
                             E_NNPE_LINK5,E_VERTEX_NODE_DOF_LINK5,
                             E_KW_NUM_NELEMS_LINK5,E_KW_NUM_NODES_LINK5);
      }
      case E_KW_NUM_NODES_TRIANG1:
      {
         return GetTriangNodes(NElemsTriang1,&NodesTriang1,
                               E_NNPE_TRIANG1,E_VERTEX_NODE_DOF_TRIANG1,
                               E_KW_NUM_NELEMS_TRIANG1,E_KW_NUM_NODES_TRIANG1);
      }
      case E_KW_NUM_NODES_TRIANG2:
      {
         return GetNodesTriang2();
      }
      case E_KW_NUM_NODES_TRIANG3:
      {
         return GetTriangNodes(NElemsTriang3,&NodesTriang3,
                               E_NNPE_TRIANG3,E_VERTEX_NODE_DOF_TRIANG3,
                               E_KW_NUM_NELEMS_TRIANG3,E_KW_NUM_NODES_TRIANG3);
      }
      case E_KW_NUM_NODES_TRIANG4:
      {
         return GetTriangNodes(NElemsTriang4,&NodesTriang4,
                               E_NNPE_TRIANG4,E_VERTEX_NODE_DOF_TRIANG4,
                               E_KW_NUM_NELEMS_TRIANG4,E_KW_NUM_NODES_TRIANG4);
      }
      case E_KW_NUM_NODES_TRIANG5:
      {
         return GetTriangNodes(NElemsTriang5,&NodesTriang5,
                               E_NNPE_TRIANG5,E_VERTEX_NODE_DOF_TRIANG5,
                               E_KW_NUM_NELEMS_TRIANG5,E_KW_NUM_NODES_TRIANG5);
      }
      case E_KW_NUM_NODES_QUAD1:
      {
         return GetNodesQuad1();
      }
      case E_KW_NUM_NODES_QUAD3:
      {
         return GetNodesQuad3();
      }
      case E_KW_NUM_NODES_TETRAH1:
      {
         return GetNodesTetrah1();
      }
      case E_KW_NUM_NODES_BLOCK1:
      {
         return GetNodesBlock1();
      }
      case E_KW_NUM_BCS:
      {
         return GetBoundaryConditions();
      }
      case E_KW_NUM_LOADS:
      {
         return GetLoads();
      }
      case E_KW_NUM_MATS_LINK1:
      {
         return GetMaterials(NElemsLink1,E_ELEM_TYPE_LINK1,
                                        E_KW_NUM_MATS_LINK1);
      }
      case E_KW_NUM_MATS_LINK2:
      {
         return GetMaterials(NElemsLink2,E_ELEM_TYPE_LINK2,
                                        E_KW_NUM_MATS_LINK2);
      }
      case E_KW_NUM_MATS_LINK3:
      {
         return GetMaterials(NElemsLink3,E_ELEM_TYPE_LINK3,
                                        E_KW_NUM_MATS_LINK3);
      }
      case E_KW_NUM_MATS_LINK4:
      {
         return GetMaterials(NElemsLink4,E_ELEM_TYPE_LINK4,
                                        E_KW_NUM_MATS_LINK4);
      }
      case E_KW_NUM_MATS_LINK5:
      {
         return GetMaterials(NElemsLink5,E_ELEM_TYPE_LINK5,
                                        E_KW_NUM_MATS_LINK5);
      }
      case E_KW_NUM_MATS_TRIANG1:
      {
         return GetMaterials(NElemsTriang1,E_ELEM_TYPE_TRIANG1,
                                          E_KW_NUM_MATS_TRIANG1);
      }
      case E_KW_NUM_COMP_MATS_TRIANG2:
      {
         return GetCompMatsType1(NElemsTriang2,E_ELEM_TYPE_TRIANG2,
                                         E_KW_NUM_COMP_MATS_TRIANG2);
      }
      case E_KW_NUM_MATS_TRIANG3:
      {
         return GetMaterials(NElemsTriang3,E_ELEM_TYPE_TRIANG3,
                                          E_KW_NUM_MATS_TRIANG3);
      }
      case E_KW_NUM_MATS_TRIANG4:
      {
         return GetMaterials(NElemsTriang4,E_ELEM_TYPE_TRIANG4,
                                          E_KW_NUM_MATS_TRIANG4);
      }
      case E_KW_NUM_MATS_TRIANG5:
      {
         return GetMaterials(NElemsTriang5,E_ELEM_TYPE_TRIANG5,
                                          E_KW_NUM_MATS_TRIANG5);
      }
      case E_KW_NUM_MATS_QUAD1:
      {
         return GetMaterials(NElemsQuad1,E_ELEM_TYPE_QUAD1,
                                        E_KW_NUM_MATS_QUAD1);
      }
      case E_KW_NUM_COMP_MATS_QUAD3:
      {
         return GetCompMatsType1(NElemsQuad3,E_ELEM_TYPE_QUAD3,
                                         E_KW_NUM_COMP_MATS_QUAD3);
      }
      case E_KW_NUM_MATS_TETRAH1:
      {
         return GetMaterials(NElemsTetrah1,E_ELEM_TYPE_TETRAH1,
                                          E_KW_NUM_MATS_TETRAH1);
      }
      case E_KW_NUM_MATS_BLOCK1:
      {
         return GetMaterials(NElemsBlock1,E_ELEM_TYPE_BLOCK1,
                                         E_KW_NUM_MATS_BLOCK1);
      }
      case E_KW_NUM_REMESH_DATA:
      {
         return GetRemeshData();
      }
      default:
      {
         return 0;
      }
   }
}


/* ////////////////////////////////////////////////////////////////////////// */
/* Get... input functions: To read in data corresponding to a keyword. */

/* The return value is 1 if the data are read successfully, else 0. */


static int GetTitle()
{
   int TitleLength;             /* title length */

   char *pFirstQuote;           /* pointer to first '"' in 'line' */
   char *pLastQuote;            /* point to last '"' in 'line' */


   if (   (pFirstQuote = strchr(line,'"')) != NULL
       && (pLastQuote = strrchr(line,'"')) != pFirstQuote)
   {
      TitleLength = (int)(pLastQuote - pFirstQuote) - 1;

      if ((Title = e_alloc_c_vect(TitleLength + 1)) == NULL)
         return 0;

      strncpy(Title,pFirstQuote + 1,TitleLength);
      Title[TitleLength] = '\0';
   }
   else
   {
      e_FileLineError(LineNumber,FullPathName,"cannot read title");
      return 0;
   }

   KeywordSet[E_KW_NUM_TITLE] = TRUE;
   return 1;
}


/* ////////////////////////////////// */


static int GetNMeshPoints()
{
   if (sscanf(line,"%*s %d",&NMeshPoints) == 1)
   {
      if (NMeshPoints < 1)
      {
         e_FileLineError(LineNumber,FullPathName,
                         "ill-defined number of mesh-points");
         return 0;
      }
   }
   else
   {
      e_FileLineError(LineNumber,FullPathName,
                      "cannot read number of mesh-points");
      return 0;
   }

   KeywordSet[E_KW_NUM_NMESHPOINTS] = TRUE;
   return 1;
}


/* ////////////////////////////////// */


static int GetNNodes()
{
   if (sscanf(line,"%*s %d",&NNodes) == 1)
   {
      if (NNodes < NMeshPoints)
      {
         e_FileLineError(LineNumber,FullPathName,
                         "ill-defined number of nodes");
         return 0;
      }
   }
   else
   {
      e_FileLineError(LineNumber,FullPathName,
                      "cannot read number of nodes");
      return 0;
   }

   KeywordSet[E_KW_NUM_NNODES] = TRUE;
   return 1;
}


/* ////////////////////////////////// */


static int GetNElems(int *NElems, int ElemType, int NElemsKeywordNumber,
                                                  int NodesKeywordNumber)
{
   if (sscanf(line,"%*s %d",NElems) == 1)
   {
      if (*NElems < 1)
      {
         e_FileLineError(LineNumber,FullPathName,
                         "ill-defined number of elements");
         return 0;
      }
   }
   else
   {
      e_FileLineError(LineNumber,FullPathName,
                      "cannot read number of elements");
      return 0;
   }

   FirstNetIndex[ElemType] = TotalNElems + 1;
   TotalNElems += *NElems;
   KeywordSet[NElemsKeywordNumber] = TRUE;
   KeywordIsOptional[NodesKeywordNumber] = FALSE;
   return 1;
}


/* ////////////////////////////////// */


static int GetNBCNodes()
{
   if (sscanf(line,"%*s %d",&NBCNodes) == 1)
   {
      if (NBCNodes < 1 || NBCNodes > NNodes)
      {
         e_FileLineError(LineNumber,FullPathName,
                         "ill-defined number of boundary conditions");
         return 0;
      }
   }
   else
   {
      e_FileLineError(LineNumber,FullPathName,
                      "cannot read number of boundary condition nodes");
      return 0;
   }

   KeywordSet[E_KW_NUM_NBC_NODES] = TRUE;
   KeywordIsOptional[E_KW_NUM_BCS] = FALSE;
   return 1;
}


/* ////////////////////////////////// */


static int GetNLoadedNodes()
{
   if (sscanf(line,"%*s %d",&NLoadedNodes) == 1)
   {
      if (NLoadedNodes < 1 || NLoadedNodes > NNodes)
      {
         e_FileLineError(LineNumber,FullPathName,
                         "ill-defined number of loaded nodes");
         return 0;
      }
   }
   else
   {
      e_FileLineError(LineNumber,FullPathName,
                      "cannot read number of loaded nodes");
      return 0;
   }

   KeywordSet[E_KW_NUM_NLOADED_NODES] = TRUE;
   KeywordIsOptional[E_KW_NUM_LOADS] = FALSE;
   return 1;
}


/* ////////////////////////////////// */


static int GetNMats()
{
   int k;


   if (sscanf(line,"%*s %d",&NMats) == 1)
   {
      if (NMats < 1)
      {
         e_FileLineError(LineNumber,FullPathName,
                         "ill-defined number of materials");
         return 0;
      }
   }
   else
   {
      e_FileLineError(LineNumber,FullPathName,
                      "cannot read number of materials");
      return 0;
   }

   Mats = NULL;
   Props = NULL;

   if (          (Mats = e_alloc_vp_vect(TotalNElems)) == NULL
       || (Props = (MAT *)e_calloc(NMats,sizeof(MAT))) == NULL)
   {
      e_free(Mats);
      e_free(Props);
      return 0;
   }

   for (k = 0; k < NMats; k++)
      Props[k].Name = NULL;

   if (KeywordSet[E_KW_NUM_NELEMS_LINK1])
      MatsLink1 = (MAT **)(Mats + FirstNetIndex[E_ELEM_TYPE_LINK1] - 1);

   if (KeywordSet[E_KW_NUM_NELEMS_LINK2])
      MatsLink2 = (MAT **)(Mats + FirstNetIndex[E_ELEM_TYPE_LINK2] - 1);

   if (KeywordSet[E_KW_NUM_NELEMS_LINK3])
      MatsLink3 = (MAT **)(Mats + FirstNetIndex[E_ELEM_TYPE_LINK3] - 1);

   if (KeywordSet[E_KW_NUM_NELEMS_LINK4])
      MatsLink4 = (MAT **)(Mats + FirstNetIndex[E_ELEM_TYPE_LINK4] - 1);

   if (KeywordSet[E_KW_NUM_NELEMS_LINK5])
      MatsLink5 = (MAT **)(Mats + FirstNetIndex[E_ELEM_TYPE_LINK5] - 1);

   if (KeywordSet[E_KW_NUM_NELEMS_TRIANG1])
      MatsTriang1 = (MAT **)(Mats + FirstNetIndex[E_ELEM_TYPE_TRIANG1] - 1);

   if (KeywordSet[E_KW_NUM_NELEMS_TRIANG2]) 
      CompMatsTriang2
         = (COMPMAT1 **)(Mats + FirstNetIndex[E_ELEM_TYPE_TRIANG2] - 1);

   if (KeywordSet[E_KW_NUM_NELEMS_TRIANG3])
      MatsTriang3 = (MAT **)(Mats + FirstNetIndex[E_ELEM_TYPE_TRIANG3] - 1);

   if (KeywordSet[E_KW_NUM_NELEMS_TRIANG4])
      MatsTriang4 = (MAT **)(Mats + FirstNetIndex[E_ELEM_TYPE_TRIANG4] - 1);

   if (KeywordSet[E_KW_NUM_NELEMS_TRIANG5])
      MatsTriang5 = (MAT **)(Mats + FirstNetIndex[E_ELEM_TYPE_TRIANG5] - 1);

   if (KeywordSet[E_KW_NUM_NELEMS_QUAD1]) 
      MatsQuad1 = (MAT **)(Mats + FirstNetIndex[E_ELEM_TYPE_QUAD1] - 1);

   if (KeywordSet[E_KW_NUM_NELEMS_QUAD3]) 
      CompMatsQuad3
         = (COMPMAT1 **)(Mats + FirstNetIndex[E_ELEM_TYPE_QUAD3] - 1);

   if (KeywordSet[E_KW_NUM_NELEMS_TETRAH1]) 
      MatsTetrah1 = (MAT **)(Mats + FirstNetIndex[E_ELEM_TYPE_TETRAH1] - 1);

   if (KeywordSet[E_KW_NUM_NELEMS_BLOCK1]) 
      MatsBlock1 = (MAT **)(Mats + FirstNetIndex[E_ELEM_TYPE_BLOCK1] - 1);

   KeywordSet[E_KW_NUM_NMATS] = TRUE;
   return 1;
}


/* ////////////////////////////////// */


static int GetNCompMatsType1()
{
   int k;


   if (!KeywordSet[E_KW_NUM_NMATS])
   {
      e_FileLineError(LineNumber,FullPathName,"keyword out of sequence");
      return 0;
   }
   else if (sscanf(line,"%*s %d",&NCompMatsType1) == 1)
   {
      if (NCompMatsType1 < 1)
      {
         e_FileLineError(LineNumber,FullPathName,
                         "ill-defined number of composite materials");
         return 0;
      }
   }
   else
   {
      e_FileLineError(LineNumber,FullPathName,
                      "cannot read number of composite materials");
      return 0;
   }

   if ((CompPropsType1
           = (COMPMAT1 *)e_calloc(NCompMatsType1,sizeof(COMPMAT1))) == NULL)
   {
      return 0;
   }

   for (k = 0; k < NCompMatsType1; k++)
      CompPropsType1[k].Name = NULL;

   KeywordSet[E_KW_NUM_NCOMP_MATS_TYPE1] = TRUE;
   return 1;
}


/* ////////////////////////////////// */


static int GetNNurbsCurves()
{
   if (sscanf(line,"%*s %d",&Geom.NNurbsCurves) == 1)
   {
      if (Geom.NNurbsCurves < 1)
      {
         e_FileLineError(LineNumber,FullPathName,
                         "ill-defined number of NURBS curves");
         return 0;
      }
   }
   else
   {
      e_FileLineError(LineNumber,FullPathName,
                      "cannot read number of NURBS curves");
      return 0;
   }

   KeywordSet[E_KW_NUM_NNURBS_CURVES] = TRUE;
   KeywordIsOptional[E_KW_NUM_REMESH_DATA] = FALSE;
   return 1;
}


/* ////////////////////////////////// */


static int GetTimeStep()
{
   if (sscanf(line,"%*s %lf",&TimeStep) == 1)
   {
      if (!(TimeStep > 0.0))
      {
         e_FileLineError(LineNumber,FullPathName,"ill-defined time-step");
         return 0;
      }
   }
   else
   {
      e_FileLineError(LineNumber,FullPathName,"cannot read time-step");
      return 0;
   }

   KeywordSet[E_KW_NUM_TIMESTEP] = TRUE;
   return 1;
}


/* ////////////////////////////////// */


static int GetNTimeSteps()
{
   if (!KeywordSet[E_KW_NUM_TIMESTEP])
   {
      e_FileLineError(LineNumber,FullPathName,"keyword out of sequence");
      return 0;
   }
   else if (sscanf(line,"%*s %d",&NTimeSteps) == 1)
   {
      if (NTimeSteps < 1)
      {
         e_FileLineError(LineNumber,FullPathName,
                         "ill-defined number of time-steps");
         return 0;
      }
   }
   else
   {
      e_FileLineError(LineNumber,FullPathName,
                      "cannot read number of time-steps");
      return 0;
   }

   KeywordSet[E_KW_NUM_NTIMESTEPS] = TRUE;
   return 1;
}


/* ////////////////////////////////// */


static int GetNInternalTimeSteps()
{
   if (KeywordSet[E_KW_NUM_NTIMESTEPS])
   {
      e_FileLineError(LineNumber,FullPathName,
                      "number of time-steps multiply defined");
      return 0;
   }
   else if (!KeywordSet[E_KW_NUM_TIMESTEP])
   {
      e_FileLineError(LineNumber,FullPathName,"keyword out of sequence");
      return 0;
   }
   else if (sscanf(line,"%*s %d",&NInternalTimeSteps) == 1)
   {
      if (NInternalTimeSteps < 1)
      {
         e_FileLineError(LineNumber,FullPathName,
                         "ill-defined number of internal time-steps");
         return 0;
      }
   }
   else
   {
      e_FileLineError(LineNumber,FullPathName,
                      "cannot read number of internal time-steps");
      return 0;
   }

   KeywordSet[E_KW_NUM_NINTERNAL_TIMESTEPS] = TRUE;
   KeywordIsOptional[E_KW_NUM_NEXTERNAL_TIMESTEPS] = FALSE;
   return 1;
}


/* ////////////////////////////////// */


static int GetNExternalTimeSteps()
{
   if (!KeywordSet[E_KW_NUM_NINTERNAL_TIMESTEPS])
   {
      e_FileLineError(LineNumber,FullPathName,"keyword out of sequence");
      return 0;
   }
   else if (sscanf(line,"%*s %d",&NExternalTimeSteps) == 1)
   {
      if (NExternalTimeSteps < 1)
      {
         e_FileLineError(LineNumber,FullPathName,
                         "ill-defined number of external time-steps");
         return 0;
      }
   }
   else
   {
      e_FileLineError(LineNumber,FullPathName,
                      "cannot read number of external time-steps");
      return 0;
   }

   KeywordSet[E_KW_NUM_NEXTERNAL_TIMESTEPS] = TRUE;
   return 1;
}


/* ////////////////////////////////// */


static int GetDampingFactor()
{
   if (sscanf(line,"%*s %lf",&DampingFactor) == 1)
   {
      if (DampingFactor < 0.0)
      {
         e_FileLineError(LineNumber,FullPathName,
                         "ill-defined damping factor");
         return 0;
      }
   }
   else
   {
      e_FileLineError(LineNumber,FullPathName,
                      "cannot read damping factor");
      return 0;
   }

   KeywordSet[E_KW_NUM_DAMPING_FACTOR] = TRUE;
   return 1;
}


/* ////////////////////////////////// */


static int GetBeta()
{
   if (sscanf(line,"%*s %lf",&Beta) == 1)
   {
      if (!(Beta > 0.0))
      {
         e_FileLineError(LineNumber,FullPathName,"ill-defined beta value");
         return 0;
      }
   }
   else
   {
      e_FileLineError(LineNumber,FullPathName,"cannot read beta value");
      return 0;
   }

   KeywordSet[E_KW_NUM_BETA] = TRUE;
   return 1;
}


/* ////////////////////////////////// */


static int GetGamma()
{
   if (sscanf(line,"%*s %lf",&Gamma) == 1)
   {
      if (!(Gamma > 0.0))
      {
         e_FileLineError(LineNumber,FullPathName,"ill-defined gamma value");
         return 0;
      }
   }
   else
   {
      e_FileLineError(LineNumber,FullPathName,"cannot read gamma value");
      return 0;
   }

   KeywordSet[E_KW_NUM_GAMMA] = TRUE;
   return 1;
}


/* ////////////////////////////////// */


static int GetNStress(int *NStressPoint, int KeywordNumber)
{
   if (sscanf(line,"%*s %d", NStressPoint) == 1)
   {
      if (*NStressPoint < 1)
      {
         e_FileLineError(LineNumber,FullPathName,
                         "ill-defined number of stress points");
         return 0;
      }
   }
   else
   {
      e_FileLineError(LineNumber,FullPathName,
                      "cannot read number of stress points");
      return 0;
   }

   KeywordSet[KeywordNumber] = TRUE;
   DefinedStresses++;
   return 1;
}


/* ////////////////////////////////// */


static int GetMeshPointCoords()
{
   int i;
   int CoordsIndex;


   if ((MeshPointCoords = e_alloc_cont_d_matr(NMeshPoints,E_NDIMS)) == NULL)
      return 0;

   for (i = 0; i < NMeshPoints; i++)
   {
      if (e_GetNextLine(line,&LineNumber,FullPathName,fp) == 0)
      {
         e_free_cont_d_matr(NMeshPoints,MeshPointCoords);
         return 0;
      }

      if (sscanf(line,"%d %lf %lf %lf",&CoordsIndex,
                             &MeshPointCoords[i][0],
                             &MeshPointCoords[i][1],
                             &MeshPointCoords[i][2] ) == 4)
      {
         if (CoordsIndex != i + 1)
         {
            e_FileLineError(LineNumber,FullPathName,
                            "coordinates index out of sequence");
            e_free_cont_d_matr(NMeshPoints,MeshPointCoords);
            return 0;
         }
      }
      else
      {
         e_FileLineError(LineNumber,FullPathName,
                         "cannot read mesh-point coordinates");
         e_free_cont_d_matr(NMeshPoints,MeshPointCoords);
         return 0;
      }
   }

   KeywordSet[E_KW_NUM_MESHPOINT_COORDS] = TRUE;
   return 1;
}


/* ////////////////////////////////// */


static int GetLinkNodes(int NElems, int ***Nodes,
                        int NNodesPerElem, int VertexNodeDOF,
                        int NElemsKeywordNumber, int NodesKeywordNumber)
{
   int i,j,k;
   int NodesIndex;


   if (!KeywordSet[NElemsKeywordNumber])
   {
      e_FileLineError(LineNumber,FullPathName,"keyword out of sequence");
      return 0;
   }
   else if ((*Nodes = e_alloc_cont_i_matr(NElems,NNodesPerElem)) == NULL)
   {
      return 0;
   }

   for (i = 0; i < NElems; i++)
   {
      if (e_GetNextLine(line,&LineNumber,FullPathName,fp) == 0)
      {
         e_free_cont_i_matr(NElems,*Nodes);
         return 0;
      }

      if (sscanf(line,"%d %d %d",&NodesIndex,
                             &(*Nodes)[i][0],
                             &(*Nodes)[i][1] ) == 3)
      {
         if (NodesIndex != i + 1)
         {
            e_FileLineError(LineNumber,FullPathName,
                            "nodes index out of sequence");
            e_free_cont_i_matr(NElems,*Nodes);
            return 0;
         }

         if (   (*Nodes)[i][0] < 1
             || (*Nodes)[i][0] > NNodes
             || (*Nodes)[i][1] < 1
             || (*Nodes)[i][1] > NNodes)
         {
            e_FileLineError(LineNumber,FullPathName,"ill-defined node index");
            e_free_cont_i_matr(NElems,*Nodes);
            return 0;
         }

         for (j = 0; j < NNodesPerElem; j++)
         {
            k = (*Nodes)[i][j] - 1;

            if (NodalDOF[k] == 0)
            {
               TotalNodalDOF += VertexNodeDOF;
               NodalDOF[k] = VertexNodeDOF;
            }
         }
      }
      else
      {
         e_FileLineError(LineNumber,FullPathName,"cannot read node indices");
         e_free_cont_i_matr(NElems,*Nodes);
         return 0;
      }
   }

   KeywordSet[NodesKeywordNumber] = TRUE;
   return 1;
}


/* ////////////////////////////////// */


static int GetTriangNodes(int NElems, int ***Nodes,
                          int NNodesPerElem, int VertexNodeDOF,
                          int NElemsKeywordNumber, int NodesKeywordNumber)
{
   int i,j,k;
   int NodesIndex;


   if (!KeywordSet[NElemsKeywordNumber])
   {
      e_FileLineError(LineNumber,FullPathName,"keyword out of sequence");
      return 0;
   }
   else if ((*Nodes = e_alloc_cont_i_matr(NElems,NNodesPerElem)) == NULL)
   {
      return 0;
   }

   for (i = 0; i < NElems; i++)
   {
      if (e_GetNextLine(line,&LineNumber,FullPathName,fp) == 0)
      {
         e_free_cont_i_matr(NElems,*Nodes);
         return 0;
      }

      if (sscanf(line,"%d %d %d %d",&NodesIndex,
                                &(*Nodes)[i][0],
                                &(*Nodes)[i][1],
                                &(*Nodes)[i][2] ) == 4)
      {
         if (NodesIndex != i + 1)
         {
            e_FileLineError(LineNumber,FullPathName,
                            "nodes index out of sequence");
            e_free_cont_i_matr(NElems,*Nodes);
            return 0;
         }

         if (   (*Nodes)[i][0] < 1
             || (*Nodes)[i][0] > NNodes
             || (*Nodes)[i][1] < 1
             || (*Nodes)[i][1] > NNodes
             || (*Nodes)[i][2] < 1
             || (*Nodes)[i][2] > NNodes)
         {
            e_FileLineError(LineNumber,FullPathName,"ill-defined node index");
            e_free_cont_i_matr(NElems,*Nodes);
            return 0;
         }

         for (j = 0; j < NNodesPerElem; j++)
         {
            k = (*Nodes)[i][j] - 1;

            if (NodalDOF[k] == 0)
            {
               TotalNodalDOF += VertexNodeDOF;
               NodalDOF[k] = VertexNodeDOF;
            }
         }
      }
      else
      {
         e_FileLineError(LineNumber,FullPathName,"cannot read node indices");
         e_free_cont_i_matr(NElems,*Nodes);
         return 0;
      }
   }

   KeywordSet[NodesKeywordNumber] = TRUE;
   return 1;
}


/* ////////////////////////////////// */


static int GetNodesTriang2()
{
   int i,j,k;
   int NodesIndex;


   if (!KeywordSet[E_KW_NUM_NELEMS_TRIANG2])
   {
      e_FileLineError(LineNumber,FullPathName,"keyword out of sequence");
      return 0;
   }
   else if ((NodesTriang2
                = e_alloc_cont_i_matr(NElemsTriang2,E_NNPE_TRIANG2)) == NULL)
   {
      return 0;
   }

   for (i = 0; i < NElemsTriang2; i++)
   {
      if (e_GetNextLine(line,&LineNumber,FullPathName,fp) == 0)
      {
         e_free_cont_i_matr(NElemsTriang2,NodesTriang2);
         return 0;
      }

      if (sscanf(line,"%d %d %d %d %d %d %d",&NodesIndex,
                                     &NodesTriang2[i][0],
                                     &NodesTriang2[i][1],
                                     &NodesTriang2[i][2],
                                     &NodesTriang2[i][3],
                                     &NodesTriang2[i][4],
                                     &NodesTriang2[i][5] ) == 7)
      {
         if (NodesIndex != i + 1)
         {
            e_FileLineError(LineNumber,FullPathName,
                            "nodes index out of sequence");
            e_free_cont_i_matr(NElemsTriang2,NodesTriang2);
            return 0;
         }

         if (   NodesTriang2[i][0] <  1
             || NodesTriang2[i][0] >  NMeshPoints
             || NodesTriang2[i][1] <  1
             || NodesTriang2[i][1] >  NMeshPoints
             || NodesTriang2[i][2] <  1
             || NodesTriang2[i][2] >  NMeshPoints
             || NodesTriang2[i][3] <= NMeshPoints
             || NodesTriang2[i][3] >  NNodes
             || NodesTriang2[i][4] <= NMeshPoints
             || NodesTriang2[i][4] >  NNodes
             || NodesTriang2[i][5] <= NMeshPoints
             || NodesTriang2[i][5] >  NNodes     )
         {
            e_FileLineError(LineNumber,FullPathName,"ill-defined node index");
            e_free_cont_i_matr(NElemsTriang2,NodesTriang2);
            return 0;
         }

         for (j = 0; j < E_NVPE_TRIANG2; j++)
         {
            k = NodesTriang2[i][j] - 1;

            if (NodalDOF[k] == 0)
            {
               TotalNodalDOF += E_VERTEX_NODE_DOF_TRIANG2;
               NodalDOF[k] = E_VERTEX_NODE_DOF_TRIANG2;
            }
         }

         for ( ; j < E_NNPE_TRIANG2; j++)
         {
            k = NodesTriang2[i][j] - 1;

            if (NodalDOF[k] == 0)
            {
               TotalNodalDOF += E_MIDSIDE_NODE_DOF_TRIANG2;
               NodalDOF[k] = E_MIDSIDE_NODE_DOF_TRIANG2;
            }
         }
      }
      else
      {
         e_FileLineError(LineNumber,FullPathName,"cannot read node indices");
         e_free_cont_i_matr(NElemsTriang2,NodesTriang2);
         return 0;
      }
   }

   KeywordSet[E_KW_NUM_NODES_TRIANG2] = TRUE;
   return 1;
}


/* ////////////////////////////////// */


static int GetNodesQuad1()
{
   int i,j,k;
   int NodesIndex;


   if (!KeywordSet[E_KW_NUM_NELEMS_QUAD1])
   {
      e_FileLineError(LineNumber,FullPathName,"keyword out of sequence");
      return 0;
   }
   else if ((NodesQuad1
                = e_alloc_cont_i_matr(NElemsQuad1,E_NNPE_QUAD1)) == NULL)
   {
      return 0;
   }

   for (i = 0; i < NElemsQuad1; i++)
   {
      if (e_GetNextLine(line,&LineNumber,FullPathName,fp) == 0)
      {
         e_free_cont_i_matr(NElemsQuad1,NodesQuad1);
         return 0;
      }

      if (sscanf(line,"%d %d %d %d %d",&NodesIndex,
                                 &NodesQuad1[i][0],
                                 &NodesQuad1[i][1],
                                 &NodesQuad1[i][2],
                                 &NodesQuad1[i][3] ) == 5)
      {
         if (NodesIndex != i + 1)
         {
            e_FileLineError(LineNumber,FullPathName,
                            "nodes index out of sequence");
            e_free_cont_i_matr(NElemsQuad1,NodesQuad1);
            return 0;
         }

         if (   NodesQuad1[i][0] < 1
             || NodesQuad1[i][0] > NNodes
             || NodesQuad1[i][1] < 1
             || NodesQuad1[i][1] > NNodes
             || NodesQuad1[i][2] < 1
             || NodesQuad1[i][2] > NNodes
             || NodesQuad1[i][3] < 1
             || NodesQuad1[i][3] > NNodes)
         {
            e_FileLineError(LineNumber,FullPathName,"ill-defined node index");
            e_free_cont_i_matr(NElemsQuad1,NodesQuad1);
            return 0;
         }

         for (j = 0 ; j < E_NNPE_QUAD1; j++)
         {
            k = NodesQuad1[i][j] - 1;

            if (NodalDOF[k] == 0)
            {
               TotalNodalDOF += E_VERTEX_NODE_DOF_QUAD1;
               NodalDOF[k] = E_VERTEX_NODE_DOF_QUAD1;
            }
         }
      }
      else
      {
         e_FileLineError(LineNumber,FullPathName,"cannot read node indices");
         e_free_cont_i_matr(NElemsQuad1,NodesQuad1);
         return 0;
      }
   }

   KeywordSet[E_KW_NUM_NODES_QUAD1] = TRUE;
   return 1;
}

/* ////////////////////////////////// */


static int GetNodesQuad3()
{
   int i,j,k;
   int NodesIndex;


   if (!KeywordSet[E_KW_NUM_NELEMS_QUAD3])
   {
      e_FileLineError(LineNumber,FullPathName,"keyword out of sequence");
      return 0;
   }
   else if ((NodesQuad3
                = e_alloc_cont_i_matr(NElemsQuad3,E_NNPE_QUAD3)) == NULL)
   {
      return 0;
   }

   for (i = 0; i < NElemsQuad3; i++)
   {
      if (e_GetNextLine(line,&LineNumber,FullPathName,fp) == 0)
      {
         e_free_cont_i_matr(NElemsQuad3,NodesQuad3);
         return 0;
      }

      if (sscanf(line,"%d %d %d %d %d %d %d %d %d %d",&NodesIndex,
                                 &NodesQuad3[i][0],
                                 &NodesQuad3[i][1],
                                 &NodesQuad3[i][2],
                                 &NodesQuad3[i][3],
                                 &NodesQuad3[i][4],
                                 &NodesQuad3[i][5],
                                 &NodesQuad3[i][6],
                                 &NodesQuad3[i][7],
                                 &NodesQuad3[i][8] ) == 10)
      {
         if (NodesIndex != i + 1)
         {
            e_FileLineError(LineNumber,FullPathName,
                            "nodes index out of sequence");
            e_free_cont_i_matr(NElemsQuad3,NodesQuad3);
            return 0;
         }

         if (   NodesQuad3[i][0] <  1
             || NodesQuad3[i][0] >  NMeshPoints
             || NodesQuad3[i][1] <  1
             || NodesQuad3[i][1] >  NMeshPoints
             || NodesQuad3[i][2] <  1
             || NodesQuad3[i][2] >  NMeshPoints
             || NodesQuad3[i][3] <  1
             || NodesQuad3[i][3] >  NMeshPoints
             || NodesQuad3[i][4] <= NMeshPoints
             || NodesQuad3[i][4] >  NNodes
             || NodesQuad3[i][5] <= NMeshPoints
             || NodesQuad3[i][5] >  NNodes
             || NodesQuad3[i][6] <= NMeshPoints
             || NodesQuad3[i][6] >  NNodes
             || NodesQuad3[i][7] <= NMeshPoints
             || NodesQuad3[i][7] >  NNodes
             || NodesQuad3[i][8] <= NMeshPoints
             || NodesQuad3[i][8] >  NNodes)
         {
            e_FileLineError(LineNumber,FullPathName,"ill-defined node index");
            e_free_cont_i_matr(NElemsQuad3,NodesQuad3);
            return 0;
         }

         for (j = 0 ; j < E_NVPE_QUAD3; j++)
         {
            k = NodesQuad3[i][j] - 1;

            if (NodalDOF[k] == 0)
            {
               TotalNodalDOF += E_VERTEX_NODE_DOF_QUAD3;
               NodalDOF[k] = E_VERTEX_NODE_DOF_QUAD3;
            }
         }
         for ( ; j < E_NNPE_QUAD3; j++)
         {
            k = NodesQuad3[i][j] - 1;

            if (NodalDOF[k] == 0)
            {
               TotalNodalDOF += E_MIDSIDE_NODE_DOF_QUAD3;
               NodalDOF[k] = E_MIDSIDE_NODE_DOF_QUAD3;
            }
         }
      }
      else
      {
         e_FileLineError(LineNumber,FullPathName,"cannot read node indices");
         e_free_cont_i_matr(NElemsQuad3,NodesQuad3);
         return 0;
      }
   }

   KeywordSet[E_KW_NUM_NODES_QUAD3] = TRUE;
   return 1;
}


/* ////////////////////////////////// */


static int GetNodesTetrah1()
{
   int i,j,k;
   int NodesIndex;


   if (!KeywordSet[E_KW_NUM_NELEMS_TETRAH1])
   {
      e_FileLineError(LineNumber,FullPathName,"keyword out of sequence");
      return 0;
   }
   else if ((NodesTetrah1
                = e_alloc_cont_i_matr(NElemsTetrah1,E_NNPE_TETRAH1)) == NULL)
   {
      return 0;
   }

   for (i = 0; i < NElemsTetrah1; i++)
   {
      if (e_GetNextLine(line,&LineNumber,FullPathName,fp) == 0)
      {
         e_free_cont_i_matr(NElemsTetrah1,NodesTetrah1);
         return 0;
      }

      if (sscanf(line,"%d %d %d %d %d",&NodesIndex,
                               &NodesTetrah1[i][0],
                               &NodesTetrah1[i][1],
                               &NodesTetrah1[i][2],
                               &NodesTetrah1[i][3] ) == 5)
      {
         if (NodesIndex != i + 1)
         {
            e_FileLineError(LineNumber,FullPathName,
                            "nodes index out of sequence");
            e_free_cont_i_matr(NElemsTetrah1,NodesTetrah1);
            return 0;
         }

         if (   NodesTetrah1[i][0] < 1
             || NodesTetrah1[i][0] > NNodes
             || NodesTetrah1[i][1] < 1
             || NodesTetrah1[i][1] > NNodes
             || NodesTetrah1[i][2] < 1
             || NodesTetrah1[i][2] > NNodes
             || NodesTetrah1[i][3] < 1
             || NodesTetrah1[i][3] > NNodes)
         {
            e_FileLineError(LineNumber,FullPathName,"ill-defined node index");
            e_free_cont_i_matr(NElemsTetrah1,NodesTetrah1);
            return 0;
         }

         for (j = 0 ; j < E_NNPE_TETRAH1; j++)
         {
            k = NodesTetrah1[i][j] - 1;

            if (NodalDOF[k] == 0)
            {
               TotalNodalDOF += E_VERTEX_NODE_DOF_TETRAH1;
               NodalDOF[k] = E_VERTEX_NODE_DOF_TETRAH1;
            }
         }
      }
      else
      {
         e_FileLineError(LineNumber,FullPathName,"cannot read node indices");
         e_free_cont_i_matr(NElemsTetrah1,NodesTetrah1);
         return 0;
      }
   }

   KeywordSet[E_KW_NUM_NODES_TETRAH1] = TRUE;
   return 1;
}


/* ////////////////////////////////// */


static int GetNodesBlock1()
{
   int i,j,k;
   int NodesIndex;


   if (!KeywordSet[E_KW_NUM_NELEMS_BLOCK1])
   {
      e_FileLineError(LineNumber,FullPathName,"keyword out of sequence");
      return 0;
   }
   else if ((NodesBlock1
                = e_alloc_cont_i_matr(NElemsBlock1,E_NNPE_BLOCK1)) == NULL)
   {
      return 0;
   }

   KeywordIsOptional[E_KW_NUM_NODES_BLOCK1] = FALSE;

   for (i = 0; i < NElemsBlock1; i++)
   {
      if (e_GetNextLine(line,&LineNumber,FullPathName,fp) == 0)
      {
         e_free_cont_i_matr(NElemsBlock1,NodesBlock1);
         return 0;
      }

      if (sscanf(line,"%d %d %d %d %d %d %d %d %d",
                                &NodesIndex,
                                &NodesBlock1[i][0],
                                &NodesBlock1[i][1],
                                &NodesBlock1[i][2],
                                &NodesBlock1[i][3],
                                &NodesBlock1[i][4],
                                &NodesBlock1[i][5],
                                &NodesBlock1[i][6],
                                &NodesBlock1[i][7] ) == 9)
      {
         if (NodesIndex != i + 1)
         {
            e_FileLineError(LineNumber,FullPathName,
                            "nodes index out of sequence");
            e_free_cont_i_matr(NElemsBlock1,NodesBlock1);
            return 0;
         }

         if (   NodesBlock1[i][0] < 1
             || NodesBlock1[i][0] > NNodes
             || NodesBlock1[i][1] < 1
             || NodesBlock1[i][1] > NNodes
             || NodesBlock1[i][2] < 1
             || NodesBlock1[i][2] > NNodes
             || NodesBlock1[i][3] < 1
             || NodesBlock1[i][3] > NNodes
             || NodesBlock1[i][4] < 1
             || NodesBlock1[i][4] > NNodes
             || NodesBlock1[i][5] < 1
             || NodesBlock1[i][5] > NNodes
             || NodesBlock1[i][6] < 1
             || NodesBlock1[i][6] > NNodes
             || NodesBlock1[i][7] < 1
             || NodesBlock1[i][7] > NNodes)
         {
            e_FileLineError(LineNumber,FullPathName,"ill-defined node index");
            e_free_cont_i_matr(NElemsBlock1,NodesBlock1);
            return 0;
         }

         for (j = 0 ; j < E_NNPE_BLOCK1; j++)
         {
            k = NodesBlock1[i][j] - 1;

            if (NodalDOF[k] == 0)
            {
               TotalNodalDOF += E_VERTEX_NODE_DOF_BLOCK1;
               NodalDOF[k] = E_VERTEX_NODE_DOF_BLOCK1;
            }
         }
      }
      else
      {
         e_FileLineError(LineNumber,FullPathName,"cannot read node indices");
         e_free_cont_i_matr(NElemsBlock1,NodesBlock1);
         return 0;
      }
   }

   KeywordSet[E_KW_NUM_NODES_BLOCK1] = TRUE;
   return 1;
}


/* ////////////////////////////////// */


static int GetBoundaryConditions()
{
   int i,j;
   int ReturnValue;

   char *BoundaryConditionSet;
   char *BoundaryConditionType;
   char message[E_LINE_MAX + 1];
   char *pLine;


   if (!KeywordSet[E_KW_NUM_NBC_NODES])
   {
      e_FileLineError(LineNumber,FullPathName,"keyword out of sequence");
      return 0;
   }
   
   BCNodes = NULL;
   BCTypes = NULL;
   BCDispls = NULL;
   BCSpringConsts = NULL;
   BoundaryConditionSet = NULL;
   BoundaryConditionType = NULL;

   if (            (BoundaryConditionSet = e_alloc_c_vect(NNodes)) == NULL
       || (BoundaryConditionType = e_alloc_c_vect(E_LINE_MAX + 1)) == NULL)
   {
      e_free(BoundaryConditionSet);
      e_free(BoundaryConditionType);
      return 0;
   }

   i = 0;

   while (TRUE)
   {
      ReturnValue = GetKeyword(line,word,&LineNumber,FullPathName,fp);

      if (ReturnValue == EOF)
      {
         if (NBCNodes != i)
         {
            sprintf(message,"incorrect number of boundary condition nodes: actual number is %d",i);
            e_FileWarning(FullPathName,message);
         }

         NBCNodes = i;

         if (NBCNodes > 0)
            KeywordSet[E_KW_NUM_BCS] = TRUE;
         else
            FreeBoundaryConditions(i);

         e_free(BoundaryConditionSet);
         e_free(BoundaryConditionType);
         return 1;
      }
      else if (IsAKeyword(word))
      {
         ReadNextLine = FALSE;

         if (NBCNodes != i)
         {
            sprintf(message,"incorrect number of boundary condition nodes: actual number is %d",i);
            e_FileWarning(FullPathName,message);
         }

         NBCNodes = i;

         if (NBCNodes > 0)
            KeywordSet[E_KW_NUM_BCS] = TRUE;
         else
            FreeBoundaryConditions(i);

         e_free(BoundaryConditionSet);
         e_free(BoundaryConditionType);
         return 1;
      }
      else if (ReturnValue == 0)
      {
         FreeBoundaryConditions(i);
         e_free(BoundaryConditionSet);
         e_free(BoundaryConditionType);
         return 0;
      }

      if (          (BCNodes = e_incr_i_vect(i,BCNodes,NVECT_MAX0)) == NULL
          ||   (BCTypes = e_incr_frag_i_matr(i,BCTypes,NVECT_MAX0)) == NULL
          || (BCDispls = e_incr_frag_d_matr(i,BCDispls,NVECT_MAX0)) == NULL
          || (BCSpringConsts
                 = e_incr_frag_d_matr(i,BCSpringConsts,NVECT_MAX0)) == NULL)
      {
         FreeBoundaryConditions(i);
         e_free(BoundaryConditionSet);
         e_free(BoundaryConditionType);
         return 0;
      }

      if (          (BCTypes[i] = e_alloc_i_vect(E_DOF_MAX)) == NULL
          ||       (BCDispls[i] = e_alloc_d_vect(E_DOF_MAX)) == NULL
          || (BCSpringConsts[i] = e_alloc_d_vect(E_DOF_MAX)) == NULL)
      {
         FreeBoundaryConditions(i + 1);
         e_free(BoundaryConditionSet);
         e_free(BoundaryConditionType);
         return 0;
      }

      pLine = GetStrArg(line,"%d",&BCNodes[i]);

      if (pLine == NULL)
      {
         e_FileLineError(LineNumber,FullPathName,
                         "cannot read boundary conditions");
         FreeBoundaryConditions(i + 1);
         e_free(BoundaryConditionSet);
         e_free(BoundaryConditionType);
         return 0;
      }
      else if (BCNodes[i] < 1 || BCNodes[i] > NNodes)
      {
         e_FileLineError(LineNumber,FullPathName,"ill-defined node index");
         FreeBoundaryConditions(i + 1);
         e_free(BoundaryConditionSet);
         e_free(BoundaryConditionType);
         return 0;
      }
      else if (BoundaryConditionSet[BCNodes[i] - 1])
      {
         e_FileLineError(LineNumber,FullPathName,
                         "boundary conditions multiply defined");
         FreeBoundaryConditions(i + 1);
         e_free(BoundaryConditionSet);
         e_free(BoundaryConditionType);
         return 0;
      }

      for (j = 0; j < NodalDOF[BCNodes[i] - 1]; j++)
      {
         pLine = GetStrArg(pLine,"%s",BoundaryConditionType);

         if (pLine == NULL)
         {
            e_FileLineError(LineNumber,FullPathName,
                            "cannot read boundary conditions");
            FreeBoundaryConditions(i + 1);
            e_free(BoundaryConditionSet);
            e_free(BoundaryConditionType);
            return 0;
         }
         else if (strcmp(E_BC_FREE,BoundaryConditionType) == 0)
         {
            BCTypes[i][j] = E_BC_NUM_FREE;
         }
         else if (strcmp(E_BC_FIXED,BoundaryConditionType) == 0)
         {
            BCTypes[i][j] = E_BC_NUM_FIXED;
            pLine = GetStrArg(pLine,"%lf",&BCDispls[i][j]);

            if (pLine == NULL)
            {
               e_FileLineError(LineNumber,FullPathName,
                               "cannot read boundary conditions");
               FreeBoundaryConditions(i + 1);
               e_free(BoundaryConditionSet);
               e_free(BoundaryConditionType);
               return 0;
            }
         }
         else if (strcmp(E_BC_SPRING,BoundaryConditionType) == 0)
         {
            BCTypes[i][j] = E_BC_NUM_SPRING;
            pLine = GetStrArg(pLine,"%lf",&BCSpringConsts[i][j]);

            if (pLine == NULL)
            {
               e_FileLineError(LineNumber,FullPathName,
                               "cannot read boundary conditions");
               FreeBoundaryConditions(i + 1);
               e_free(BoundaryConditionSet);
               e_free(BoundaryConditionType);
               return 0;
            }
            else if (BCSpringConsts[i][j] < 0.0)
            {
               e_FileLineError(LineNumber,FullPathName,
                               "ill-defined spring constant");
               FreeBoundaryConditions(i + 1);
               e_free(BoundaryConditionSet);
               e_free(BoundaryConditionType);
               return 0;
            }
         }
         else
         {
            e_FileLineError(LineNumber,FullPathName,
                            "undefined boundary condition type");
            FreeBoundaryConditions(i + 1);
            e_free(BoundaryConditionSet);
            e_free(BoundaryConditionType);
            return 0;
         }
      }

      BoundaryConditionSet[BCNodes[i] - 1] = TRUE;
      i++;
   }
}


/* ////////////////////////////////// */


static void FreeBoundaryConditions(int NBCNodes)
{
   e_free(BCNodes);
   e_free_frag_i_matr(NBCNodes,BCTypes);
   e_free_frag_d_matr(NBCNodes,BCDispls);
   e_free_frag_d_matr(NBCNodes,BCSpringConsts);
}


/* ////////////////////////////////// */


static int GetLoads()
{
   int i,j;
   int ReturnValue;

   char *LoadsSet;
   char message[E_LINE_MAX + 1];
   char *pLine;


   if (!KeywordSet[E_KW_NUM_NLOADED_NODES])
   {
      e_FileLineError(LineNumber,FullPathName,"keyword out of sequence");
      return 0;
   }

   LoadedNodes = NULL;
   Loads = NULL;
   LoadsSet = NULL;

   if ((LoadsSet = e_alloc_c_vect(NNodes)) == NULL)
      return 0;

   i = 0;

   while (TRUE)
   {
      ReturnValue = GetKeyword(line,word,&LineNumber,FullPathName,fp);

      if (ReturnValue == EOF)
      {
         if (NLoadedNodes != i)
         {
            sprintf(message,"incorrect number of loaded nodes: actual number is %d",i);
            e_FileWarning(FullPathName,message);
         }

         NLoadedNodes = i;

         if (NLoadedNodes > 0)
            KeywordSet[E_KW_NUM_LOADS] = TRUE;
         else
            FreeLoads(i);

         e_free(LoadsSet);
         return 1;
      }
      else if (IsAKeyword(word))
      {
         ReadNextLine = FALSE;

         if (NLoadedNodes != i)
         {
            sprintf(message,"incorrect number of loaded nodes: actual number is %d",i);
            e_FileWarning(FullPathName,message);
         }

         NLoadedNodes = i;

         if (NLoadedNodes > 0)
            KeywordSet[E_KW_NUM_LOADS] = TRUE;
         else
            FreeLoads(i);

         e_free(LoadsSet);
         return 1;
      }
      else if (ReturnValue == 0)
      {
         FreeLoads(i);
         e_free(LoadsSet);
         return 0;
      }

      if (   (LoadedNodes = e_incr_i_vect(i,LoadedNodes,NVECT_MAX0)) == NULL
          ||        (Loads = e_incr_frag_d_matr(i,Loads,NVECT_MAX0)) == NULL)
      {
         FreeLoads(i);
         e_free(LoadsSet);
         return 0;
      }

      if ((Loads[i] = e_alloc_d_vect(E_DOF_MAX)) == NULL)
      {
         FreeLoads(i + 1);
         e_free(LoadsSet);
         return 0;
      }

      pLine = GetStrArg(line,"%d",&LoadedNodes[i]);

      if (pLine == NULL)
      {
         e_FileLineError(LineNumber,FullPathName,"cannot read loads");
         FreeLoads(i + 1);
         e_free(LoadsSet);
         return 0;
      }
      else if (LoadedNodes[i] < 1 || LoadedNodes[i] > NNodes)
      {
         e_FileLineError(LineNumber,FullPathName,"ill-defined node index");
         FreeLoads(i + 1);
         e_free(LoadsSet);
         return 0;
      }
      else if (LoadsSet[LoadedNodes[i] - 1])
      {
         e_FileLineError(LineNumber,FullPathName,"loads multiply defined");
         FreeLoads(i + 1);
         e_free(LoadsSet);
         return 0;
      }

      for (j = 0; j < NodalDOF[LoadedNodes[i] - 1]; j++)
      {
         pLine = GetStrArg(pLine,"%lf",&Loads[i][j]);

         if (pLine == NULL)
         {
            e_FileLineError(LineNumber,FullPathName,"cannot read loads");
            FreeLoads(i + 1);
            e_free(LoadsSet);
            return 0;
         }
      }

      LoadsSet[LoadedNodes[i] - 1] = TRUE;
      i++;
   }
}


/* ////////////////////////////////// */


static void FreeLoads(int NLoadedNodes)
{
   e_free(LoadedNodes);
   e_free_frag_d_matr(NLoadedNodes,Loads);
}


/* ////////////////////////////////// */


static int GetMaterials(int NElements, int ElementType, int KeywordNumber)
{
   int i,k;
   int MaterialIndex;
   int NameLength;
   int PropsIndex;

   char *pFirstQuote;
   char *pLastQuote;
   char *string;


   k = FirstNetIndex[ElementType] - 1;

   if (!KeywordSet[E_KW_NUM_NMATS])
   {
      e_FileLineError(LineNumber,FullPathName,"keyword out of sequence");
      return 0;
   }
   else if ((string = e_alloc_c_vect(E_LINE_MAX + 1)) == NULL)
   {
      return 0;
   }

   for (i = 0; i < NElements; i++)
   {
      if (e_GetNextLine(line,&LineNumber,FullPathName,fp) == 0)
      {
         e_free(string);
         return 0;
      }
      else if (   sscanf(line,"%d %s",&MaterialIndex,string) != 2
               ||         (pFirstQuote = strchr(string,'"')) == NULL
               ||         (pLastQuote = strrchr(string,'"')) == pFirstQuote
               ||         (int)(pLastQuote - pFirstQuote)    == 1          )
      {
         e_FileLineError(LineNumber,FullPathName,
                         "cannot read material name");
         e_free(string);
         return 0;
      }
      else if (MaterialIndex != i + 1)
      {
         e_FileLineError(LineNumber,FullPathName,
                         "material index out of sequence");
         e_free(string);
         return 0;
      }

      PropsIndex = GetPropsIndex(string);

      if (PropsIndex > 0)
      {
         Mats[k] = &Props[PropsIndex - 1];
      }
      else
      {
         if (CurrentNMats == NMats)
         {
            e_GeneralError("incorrect number of materials");
            e_free(string);
            return 0;
         }

         NameLength = (int)(pLastQuote - pFirstQuote) - 1;
         Props[CurrentNMats].Name = e_alloc_c_vect(NameLength + 1);

         if (Props[CurrentNMats].Name == NULL)
         {
            e_free(string);
            return 0;
         }

         Mats[k] = &Props[CurrentNMats];
         strncpy(Props[CurrentNMats].Name,pFirstQuote + 1,NameLength);
         Props[CurrentNMats].Name[NameLength] = '\0';
         CurrentNMats++;
      }

      k++;
   }

   e_free(string);
   KeywordSet[KeywordNumber] = TRUE;
   return 1;
}


/* ////////////////////////////////// */


static int GetCompMatsType1(int NElements, int ElementType,
                                   int CompMatKeywordNumber)
{
   int i,k;
   int CompMaterialIndex;
   int CompNameLength;
   int CompPropsIndex;

   char *pFirstQuote;
   char *pLastQuote;
   char *string;


   k = FirstNetIndex[ElementType] - 1;

   if (!KeywordSet[E_KW_NUM_NCOMP_MATS_TYPE1])
   {
      e_FileLineError(LineNumber,FullPathName,"keyword out of sequence");
      return 0;
   }
   else if ((string = e_alloc_c_vect(E_LINE_MAX + 1)) == NULL)
   {
      return 0;
   }

   for (i = 0; i < NElements; i++)
   {
      if (e_GetNextLine(line,&LineNumber,FullPathName,fp) == 0)
      {
         e_free(string);
         return 0;
      }
      else if (   sscanf(line,"%d %s",&CompMaterialIndex,string) != 2
               ||             (pFirstQuote = strchr(string,'"')) == NULL
               ||             (pLastQuote = strrchr(string,'"')) == pFirstQuote
               ||             (int)(pLastQuote - pFirstQuote)    == 1          )
      {
         e_FileLineError(LineNumber,FullPathName,
                         "cannot read composite material name");
         e_free(string);
         return 0;
      }
      else if (CompMaterialIndex != i + 1)
      {
         e_FileLineError(LineNumber,FullPathName,
                         "composite material index out of sequence");
         e_free(string);
         return 0;
      }

      CompPropsIndex = GetCompPropsType1Index(string);

      if (CompPropsIndex > 0)
      {
         Mats[k] = &CompPropsType1[CompPropsIndex - 1];
      }
      else
      {
         if (CurrentNCompMatsType1 == NCompMatsType1)
         {
            e_GeneralError("incorrect number of TYPE1 composite materials");
            e_free(string);
            return 0;
         }

         CompNameLength = (int)(pLastQuote - pFirstQuote) - 1;
         CompPropsType1[CurrentNCompMatsType1].Name
            = e_alloc_c_vect(CompNameLength + 1);

         if (CompPropsType1[CurrentNCompMatsType1].Name == NULL)
         {
            e_free(string);
            return 0;
         }

         Mats[k] = &CompPropsType1[CurrentNCompMatsType1];
         strncpy(CompPropsType1[CurrentNCompMatsType1].Name,
                              pFirstQuote + 1,CompNameLength);
         CompPropsType1[CurrentNCompMatsType1].Name[CompNameLength] = '\0';
         CurrentNCompMatsType1++;
      }

      k++;
   }

   e_free(string);
   KeywordSet[CompMatKeywordNumber] = TRUE;
   return 1;
}


/* ////////////////////////////////// */


static int GetRemeshData()
{
   int k;
   int NodeIndex;


   Geom.NurbsCurves = NULL;
   NodeTypes = NULL;
   NNurbsCurves = NULL;
   NurbsCurveParams = NULL;
   NurbsCurves = NULL;

   if (   (Geom.NurbsCurves
               = (NURBSCURVE *)
                 e_calloc(Geom.NNurbsCurves,sizeof(NURBSCURVE))) == NULL
       ||              (NodeTypes = e_alloc_i_vect(NMeshPoints)) == NULL
       ||           (NNurbsCurves = e_alloc_i_vect(NMeshPoints)) == NULL
       ||   (NurbsCurveParams = e_init_frag_d_matr(NMeshPoints)) == NULL
       || (NurbsCurves
                  = (NURBSCURVE ***)
                    e_calloc(NMeshPoints,sizeof(NURBSCURVE **))) == NULL)
   {
      e_free(Geom.NurbsCurves);
      e_free(NodeTypes);
      e_free(NNurbsCurves);
      e_free(NurbsCurveParams);
      e_free(NurbsCurves);
      return 0;
   }

   for (k = 0; k < Geom.NNurbsCurves; k++)
      Geom.NurbsCurves[k].Name = NULL;

   for (k = 0; k < NMeshPoints; k++)
      NurbsCurves[k] = NULL;

   for (k = 0; k < NMeshPoints; k++)
   {
      if (   ReadNextLine
          && e_GetNextLine(line,&LineNumber,FullPathName,fp) == 0)
      {
         FreeRemeshData();
         return 0;
      }
      else if (sscanf(line,"%d %s",&NodeIndex,word) != 2)
      {
         e_FileLineError(LineNumber,FullPathName,
                         "cannot read remeshing data");
         FreeRemeshData();
         return 0;
      }
      else if (NodeIndex != k + 1)
      {
         e_FileLineError(LineNumber,FullPathName,
                         "node index out of sequence");
         FreeRemeshData();
         return 0;
      }
      else if (strcmp(E_RM_EXTERIOR,word) == 0)
      {
         if (!GetRemeshExterior(k))
         {
            FreeRemeshData();
            return 0;
         }

         NodeTypes[k] = E_RM_NUM_EXTERIOR;
      }
      else if (strcmp(E_RM_INTERIOR,word) == 0)
      {
         NodeTypes[k] = E_RM_NUM_INTERIOR;
         ReadNextLine = TRUE;
      }
      else if (strcmp(E_RM_ISD_BOUNDARY,word) == 0)
      {
         NodeTypes[k] = E_RM_NUM_ISD_BOUNDARY;
         ReadNextLine = TRUE;
      }
      else
      {
         e_FileLineError(LineNumber,FullPathName,
                         "undefined remeshing node type");
         FreeRemeshData();
         return 0;
      }
   }

   if (CurrentNNurbsCurves != Geom.NNurbsCurves)
   {
      e_GeneralError("incorrect number of NURBS curves");
      FreeRemeshData();
      return 0;
   }

   KeywordSet[E_KW_NUM_REMESH_DATA] = TRUE;
   return 1;
}


/* ////////////////////////////////// */


static int GetRemeshExterior(int k)
{
   int j;
   int index;
   int NameIndex;
   char *pFirstQuote;
   char *pLastQuote;


   if (GetKeyword(line,word,&LineNumber,FullPathName,fp) == EOF)
   {
      if (k + 1 < NMeshPoints)
      {
         e_FileError(FullPathName,"unexpected end of file");
         return 0;
      }
      else
      {
         ReadNextLine = TRUE;
         return 1;
      }
   }
   else if (strcmp(E_RMKW_NNURBS_CURVES,word) != 0)
   {
      ReadNextLine = FALSE;
      return 1;
   }
   else if (sscanf(line,"%*s %d",&NNurbsCurves[k]) != 1)
   {
      e_FileLineError(LineNumber,FullPathName,
                      "cannot read curve/surface data");
      return 0;
   }
   else if (NNurbsCurves[k] < 1)
   {
      e_FileLineError(LineNumber,FullPathName,
                      "ill-defined number of NURBS curves");
      return 0;
   }
   else if (   (NurbsCurveParams[k] = e_alloc_d_vect(NNurbsCurves[k])) == NULL
            || (NurbsCurves[k] = (NURBSCURVE **)
                                 e_calloc(NNurbsCurves[k],
                                                sizeof(NURBSCURVE *))) == NULL)
   {
      return 0;
   }

   for (j = 0; j < NNurbsCurves[k]; j++)
   {
      if (e_GetNextLine(line,&LineNumber,FullPathName,fp) == 0)
      {
         return 0;
      }
      else if (sscanf(line,"%d %s",&index,word) != 2)
      {
         e_FileLineError(LineNumber,FullPathName,
                         "cannot read NURBS curve data");
         return 0;
      }
      else if (index != j + 1)
      {
         e_FileLineError(LineNumber,FullPathName,
                         "NURBS curve index out of sequence");
         return 0;
      }
      else if (strcmp(E_RMKW_NC_PARAM,word) == 0)
      {
         e_FileLineError(LineNumber,FullPathName,
                         "keyword out of sequence");
         return 0;
      }
      else if (strcmp(E_RMKW_NURBS_CURVE,word) != 0)
      {
         e_FileLineError(LineNumber,FullPathName,"undefined keyword");
         return 0;
      }
      else if (     sscanf(line,"%*s %*s %s",word) != 1
               || (pFirstQuote = strchr(word,'"')) == NULL
               || (pLastQuote = strrchr(word,'"')) == pFirstQuote
               || ((int)(pLastQuote - pFirstQuote) - 1) == 0)
      {
         e_FileLineError(LineNumber,FullPathName,
                         "cannot read NURBS curve data");
         return 0;
      }
      else if ((NameIndex = GetNurbsCurveName(word)) == 0)
      {
         return 0;
      }
      else if (e_GetNextLine(line,&LineNumber,FullPathName,fp) == 0)
      {
         return 0;
      }
      else if (sscanf(line,"%s %lf",word,&NurbsCurveParams[k][j]) != 2)
      {
         e_FileLineError(LineNumber,FullPathName,
                         "cannot read NURBS curve data");
         return 0;
      }
      else if (strcmp(E_RMKW_NURBS_CURVE,word) == 0)
      {
         e_FileLineError(LineNumber,FullPathName,
                         "keyword out of sequence");
         return 0;
      }
      else if (strcmp(E_RMKW_NC_PARAM,word) != 0)
      {
         e_FileLineError(LineNumber,FullPathName,"undefined keyword");
         return 0;
      }
      else if (   NurbsCurveParams[k][j] < 0.0
               || NurbsCurveParams[k][j] > 1.0)
      {
         e_FileLineError(LineNumber,FullPathName,
                         "ill-defined NURBS curve parameter");
         return 0;
      }

      NurbsCurves[k][j] = &Geom.NurbsCurves[NameIndex - 1];
   }

   ReadNextLine = TRUE;

   return 1;
}


/* ////////////////////////////////////////////////////////////////////////// */
/* GetNurbsCurveName: To set the name of a NURBS curve. */

/* The return value is the name index if the operation is successful, else 0. */


static int GetNurbsCurveName(char *Name)
{
   int k;
   int  NewNameLength;
   char NewName[E_LINE_MAX + 1];
   
   /* assemble quote-less string */
   strcpy(NewName, Name+1);
   NewName[strlen(Name) - 2] = '\0';
   NewNameLength = strlen(NewName);
   
   for (k = 0; k < CurrentNNurbsCurves; k++)
   {
      if (strcmp(Geom.NurbsCurves[k].Name, NewName) == 0)
         break;
   }

   if (k == CurrentNNurbsCurves)
   {
      if (CurrentNNurbsCurves == Geom.NNurbsCurves)
      {
         e_GeneralError("incorrect number of NURBS curves");
         return 0;
      }
      else if ((Geom.NurbsCurves[CurrentNNurbsCurves].Name
                          = e_alloc_c_vect(NewNameLength + 1)) == NULL)
      {
         return 0;
      }

      strcpy(Geom.NurbsCurves[CurrentNNurbsCurves].Name, NewName);
      CurrentNNurbsCurves++;
   }

   return k + 1;
}


/* ////////////////////////////////////////////////////////////////////////// */
/* FreeRemeshData: To free memory allocated by "GetRemeshData". */


static void FreeRemeshData()
{
   int k;


   for (k = 0; k < Geom.NNurbsCurves; k++)
      e_free(Geom.NurbsCurves[k].Name);

   for (k = 0; k < NMeshPoints; k++)
      e_free(NurbsCurves[k]);

   e_free(Geom.NurbsCurves);
   e_free(NodeTypes);
   e_free(NNurbsCurves);
   e_free_frag_d_matr(NMeshPoints,NurbsCurveParams);
   e_free(NurbsCurves);
}


/* ////////////////////////////////////////////////////////////////////////// */
/* OptionalDataOkay1: To return whether first groups of optional data is
   okay.                                                                      */

/* The return value is 1 if the optional data are okay, else 0. */


static int OptionalDataOkay1()
{
   if (   !KeywordSet[E_KW_NUM_NELEMS_LINK1]
       && !KeywordSet[E_KW_NUM_NELEMS_LINK2]
       && !KeywordSet[E_KW_NUM_NELEMS_LINK3]
       && !KeywordSet[E_KW_NUM_NELEMS_LINK4]
       && !KeywordSet[E_KW_NUM_NELEMS_LINK5]
       && !KeywordSet[E_KW_NUM_NELEMS_TRIANG1]
       && !KeywordSet[E_KW_NUM_NELEMS_TRIANG2]
       && !KeywordSet[E_KW_NUM_NELEMS_TRIANG3]
       && !KeywordSet[E_KW_NUM_NELEMS_TRIANG4]
       && !KeywordSet[E_KW_NUM_NELEMS_TRIANG5]
       && !KeywordSet[E_KW_NUM_NELEMS_QUAD1]
       && !KeywordSet[E_KW_NUM_NELEMS_QUAD3]
       && !KeywordSet[E_KW_NUM_NELEMS_TETRAH1]
       && !KeywordSet[E_KW_NUM_NELEMS_BLOCK1]  )
   {
      e_GeneralError("elements of no element type declared");
      return 0;
   }
   else if (    KeywordSet[E_KW_NUM_TIMESTEP]
            && !KeywordSet[E_KW_NUM_NTIMESTEPS]
            && !KeywordSet[E_KW_NUM_NINTERNAL_TIMESTEPS]
            && !KeywordSet[E_KW_NUM_NEXTERNAL_TIMESTEPS])
   {
      e_GeneralError("time-step data not defined");
      return 0;
   }

   if(DefinedStresses > 0)
   {
     if(KeywordSet[E_KW_NUM_NELEMS_LINK1])
       DefinedStresses--;
     if(KeywordSet[E_KW_NUM_NELEMS_LINK2])
       DefinedStresses--;
     if(KeywordSet[E_KW_NUM_NELEMS_LINK3])
       DefinedStresses--;
     if(KeywordSet[E_KW_NUM_NELEMS_LINK4])
       DefinedStresses--;
     if(KeywordSet[E_KW_NUM_NELEMS_LINK5])
       DefinedStresses--;
     if(KeywordSet[E_KW_NUM_NELEMS_TRIANG1])
       DefinedStresses--;
     if(KeywordSet[E_KW_NUM_NELEMS_TRIANG2])
       DefinedStresses--;
     if(KeywordSet[E_KW_NUM_NELEMS_TRIANG3])
       DefinedStresses--;
     if(KeywordSet[E_KW_NUM_NELEMS_TRIANG4])
       DefinedStresses--;
     if(KeywordSet[E_KW_NUM_NELEMS_TRIANG5])
       DefinedStresses--;
     if(KeywordSet[E_KW_NUM_NELEMS_QUAD1])
       DefinedStresses--;
     if(KeywordSet[E_KW_NUM_NELEMS_QUAD3])
       DefinedStresses--;
     if(KeywordSet[E_KW_NUM_NELEMS_TETRAH1])
       DefinedStresses--;
     if(KeywordSet[E_KW_NUM_NELEMS_BLOCK1])
       DefinedStresses--;

     if(DefinedStresses != 0)
     {
       e_GeneralError("The number of element types and the number of defined, stress point types are not equal !");
       return 0;
     }
   }

   return 1;
}


/* ////////////////////////////////////////////////////////////////////////// */
/* OptionalDataOkay2: To return whether second group of optional data is
   okay.                                                                      */

/* The return value is 1 if the optional data are okay, else 0. */


static int OptionalDataOkay2()
{
   if (KeywordSet[E_KW_NUM_NCOMP_MATS_TYPE1])
   {
      if (    KeywordSet[E_KW_NUM_NELEMS_TRIANG2]
          && !KeywordSet[E_KW_NUM_COMP_MATS_TRIANG2])
      {
         e_GeneralError("no composite material data declared for element type TRIANG2");
         return 0;
      }
      if (    KeywordSet[E_KW_NUM_NELEMS_QUAD3]
          && !KeywordSet[E_KW_NUM_COMP_MATS_QUAD3])
      {
         e_GeneralError("no composite material data declared for element type QUAD3");
         return 0;
      }
      else if (CurrentNCompMatsType1 != NCompMatsType1)
      {
         e_GeneralError("incorrect number of TYPE1 composite materials");
         return 0;
      }
   }

   if (KeywordSet[E_KW_NUM_NMATS])
   {
      if (    KeywordSet[E_KW_NUM_NELEMS_LINK1]
          && !KeywordSet[E_KW_NUM_MATS_LINK1]  )
      {
         e_GeneralError("no material data declared for element type LINK1");
         return 0;
      }
      else if (    KeywordSet[E_KW_NUM_NELEMS_LINK2]
               && !KeywordSet[E_KW_NUM_MATS_LINK2]  )
      {
         e_GeneralError("no material data declared for element type LINK2");
         return 0;
      }
      else if (    KeywordSet[E_KW_NUM_NELEMS_LINK3]
               && !KeywordSet[E_KW_NUM_MATS_LINK3]  )
      {
         e_GeneralError("no material data declared for element type LINK3");
         return 0;
      }
      else if (    KeywordSet[E_KW_NUM_NELEMS_LINK4]
               && !KeywordSet[E_KW_NUM_MATS_LINK4]  )
      {
         e_GeneralError("no material data declared for element type LINK4");
         return 0;
      }
      else if (    KeywordSet[E_KW_NUM_NELEMS_LINK5]
               && !KeywordSet[E_KW_NUM_MATS_LINK5]  )
      {
         e_GeneralError("no material data declared for element type LINK5");
         return 0;
      }
      else if (    KeywordSet[E_KW_NUM_NELEMS_TRIANG1]
               && !KeywordSet[E_KW_NUM_MATS_TRIANG1]  )
      {
         e_GeneralError("no material data declared for element type TRIANG1");
         return 0;
      }
      else if (    KeywordSet[E_KW_NUM_NELEMS_TRIANG3]
               && !KeywordSet[E_KW_NUM_MATS_TRIANG3]  )
      {
         e_GeneralError("no material data declared for element type TRIANG3");
         return 0;
      }
      else if (    KeywordSet[E_KW_NUM_NELEMS_TRIANG4]
               && !KeywordSet[E_KW_NUM_MATS_TRIANG4]  )
      {
         e_GeneralError("no material data declared for element type TRIANG4");
         return 0;
      }
      else if (    KeywordSet[E_KW_NUM_NELEMS_TRIANG5]
               && !KeywordSet[E_KW_NUM_MATS_TRIANG5]  )
      {
         e_GeneralError("no material data declared for element type TRIANG5");
         return 0;
      }
      else if (    KeywordSet[E_KW_NUM_NELEMS_QUAD1]
               && !KeywordSet[E_KW_NUM_MATS_QUAD1]  )
      {
         e_GeneralError("no material data declared for element type QUAD1");
         return 0;
      }
      else if (    KeywordSet[E_KW_NUM_NELEMS_TETRAH1]
               && !KeywordSet[E_KW_NUM_MATS_TETRAH1]  )
      {
         e_GeneralError("no material data declared for element type TETRAH1");
         return 0;
      }
      else if (    KeywordSet[E_KW_NUM_NELEMS_BLOCK1]
               && !KeywordSet[E_KW_NUM_MATS_BLOCK1]  )
      {
         e_GeneralError("no material data declared for element type BLOCK1");
         return 0;
      }
      else if (!KeywordSet[E_KW_NUM_NCOMP_MATS_TYPE1] && CurrentNMats != NMats)
      {
         e_GeneralError("incorrect number of materials");
         return 0;
      }
   }

   return 1;
}


/* ////////////////////////////////////////////////////////////////////////// */
/* IsAKeyword: To return whether a word is a keyword. */

/* The return value is 1 if "word" is a keyword, else 0. */


static int IsAKeyword(char *word)
{
   int i;


   for (i = 0; i < E_NKEYWORDS; i++)
   {
      if (strcmp(Keyword[i],word) == 0)
         return TRUE;
   }

   return FALSE;
}


/* ////////////////////////////////////////////////////////////////////////// */
/* GetPropsIndex: To return the properties index corresponding to a material
   name if the material name has already been read, else 0.                   */


static int GetPropsIndex(char *string)
{
   int i;
   char newname[E_LINE_MAX + 1];
   
   /* assemble quote-less string */
   strcpy(newname, string+1);
   newname[strlen(string) - 2] = '\0';
   
   for (i = 0; i < CurrentNMats; i++)
   {
      if (strcmp(Props[i].Name, newname) == 0)
        return i + 1;
   }

   return 0;
}


/* ////////////////////////////////////////////////////////////////////////// */
/* GetCompPropsType1Index: To return the properties index corresponding to a
   TYPE1 composite material name if the composite material name has already
   been read, else 0.                                                         */


static int GetCompPropsType1Index(char *string)
{
   int i;
   char newname[E_LINE_MAX + 1];
   
   /* assemble quote-less string */
   strcpy(newname, string+1);
   newname[strlen(string) - 2] = '\0';

   for (i = 0; i < CurrentNCompMatsType1; i++)
   {
      if (strcmp(CompPropsType1[i].Name, newname) == 0)
        return i + 1;
   }

   return 0;
}


/* ////////////////////////////////////////////////////////////////////////// */
/* AllCompulsorySet: To return whether all compulsory keywords have been set. */


static int AllCompulsoryKeywordsSet()
{
   int i;


   for (i = 0; i < E_NKEYWORDS; i++)
   {
      if (!KeywordIsOptional[i] && !KeywordSet[i])
      {
         e_FileError(FullPathName,"unexpected end of file");
         return 0;
      }
   }

   return 1;
}


/* ////////////////////////////////////////////////////////////////////////// */
/* SetMeshData: To set the MESH structure data. */

/* The return value is 1 if the data are set successfully, else 0. */


static int SetMeshData(MESH *mesh)
{
   int i;


   e_NullifyMesh(mesh);

   mesh->GNIs = NULL;
   mesh->GEIs = NULL;

   if (        (mesh->GNIs = e_alloc_i_vect(NNodes)) == NULL
       || (mesh->GEIs = e_alloc_i_vect(TotalNElems)) == NULL)
   {
      e_free(mesh->GNIs);
      e_free(mesh->GEIs);
      return 0;
   }

   mesh->Title = Title;

   mesh->NMeshPoints = NMeshPoints;
   mesh->NNodes = NNodes;

   mesh->NElemsLink1 = NElemsLink1;
   mesh->NElemsLink2 = NElemsLink2;
   mesh->NElemsLink3 = NElemsLink3;
   mesh->NElemsLink4 = NElemsLink4;
   mesh->NElemsLink5 = NElemsLink5;
   mesh->NElemsTriang1 = NElemsTriang1;
   mesh->NElemsTriang2 = NElemsTriang2;
   mesh->NElemsTriang3 = NElemsTriang3;
   mesh->NElemsTriang4 = NElemsTriang4;
   mesh->NElemsTriang5 = NElemsTriang5;
   mesh->NElemsQuad1 = NElemsQuad1;
   mesh->NElemsQuad3 = NElemsQuad3;
   mesh->NElemsTetrah1 = NElemsTetrah1;
   mesh->NElemsBlock1 = NElemsBlock1;
   mesh->TotalNElems = TotalNElems;

   mesh->NBCNodes = NBCNodes;
   mesh->NLoadedNodes = NLoadedNodes;
   mesh->NMats = NMats;
   mesh->NMdfMats = CurrentNMats;
   mesh->NCompMatsType1 = NCompMatsType1;
   mesh->TimeStep = TimeStep;
   mesh->NTimeSteps = NTimeSteps;
   mesh->NInternalTimeSteps = NInternalTimeSteps;
   mesh->NExternalTimeSteps = NExternalTimeSteps;
   mesh->DampingFactor = DampingFactor;
   mesh->Beta = Beta;
   mesh->Gamma = Gamma;

   mesh->NStressPointsLink1 = NStressPointsLink1;
   mesh->NStressPointsLink2 = NStressPointsLink2;
   mesh->NStressPointsLink3 = NStressPointsLink3;
   mesh->NStressPointsLink4 = NStressPointsLink4;
   mesh->NStressPointsLink5 = NStressPointsLink5;
   mesh->NStressPointsTriang1 = NStressPointsTriang1;
   mesh->NStressPointsTriang2 = NStressPointsTriang2;
   mesh->NStressPointsTriang3 = NStressPointsTriang3;
   mesh->NStressPointsTriang4 = NStressPointsTriang4;
   mesh->NStressPointsTriang5 = NStressPointsTriang5;
   mesh->NStressPointsQuad1 = NStressPointsQuad1;
   mesh->NStressPointsQuad3 = NStressPointsQuad3;
   mesh->NStressPointsTetrah1 = NStressPointsTetrah1;
   mesh->NStressPointsBlock1 = NStressPointsBlock1;

   mesh->MeshPointCoords = MeshPointCoords;

   mesh->NodesLink1 = NodesLink1;
   mesh->NodesLink2 = NodesLink2;
   mesh->NodesLink3 = NodesLink3;
   mesh->NodesLink4 = NodesLink4;
   mesh->NodesLink5 = NodesLink5;
   mesh->NodesTriang1 = NodesTriang1;
   mesh->NodesTriang2 = NodesTriang2;
   mesh->NodesTriang3 = NodesTriang3;
   mesh->NodesTriang4 = NodesTriang4;
   mesh->NodesTriang5 = NodesTriang5;
   mesh->NodesQuad1 = NodesQuad1;
   mesh->NodesQuad3 = NodesQuad3;
   mesh->NodesTetrah1 = NodesTetrah1;
   mesh->NodesBlock1 = NodesBlock1;

   mesh->TotalNodalDOF = TotalNodalDOF;
   mesh->NodalDOF = NodalDOF;

   mesh->BCNodes = BCNodes;
   mesh->BCTypes = BCTypes;
   mesh->BCDispls = BCDispls;
   mesh->BCSpringConsts = BCSpringConsts;

   mesh->LoadedNodes = LoadedNodes;
   mesh->Loads = Loads;

   mesh->Geom.NNurbsCurves = Geom.NNurbsCurves;
   mesh->Geom.NurbsCurves = Geom.NurbsCurves;

   mesh->Mats = Mats;
   mesh->MatsLink1 = MatsLink1;
   mesh->MatsLink2 = MatsLink2;
   mesh->MatsLink3 = MatsLink3;
   mesh->MatsLink4 = MatsLink4;
   mesh->MatsLink5 = MatsLink5;
   mesh->MatsTriang1 = MatsTriang1;
   mesh->CompMatsTriang2 = CompMatsTriang2;
   mesh->MatsTriang3 = MatsTriang3;
   mesh->MatsTriang4 = MatsTriang4;
   mesh->MatsTriang5 = MatsTriang5;
   mesh->MatsQuad1 = MatsQuad1;
   mesh->CompMatsQuad3 = CompMatsQuad3;
   mesh->MatsTetrah1 = MatsTetrah1;
   mesh->MatsBlock1 = MatsBlock1;
   mesh->Props = Props;
   mesh->CompPropsType1 = CompPropsType1;

   mesh->NodeTypes = NodeTypes;
   mesh->NNurbsCurves = NNurbsCurves;
   mesh->NurbsCurveParams = NurbsCurveParams;
   mesh->NurbsCurves = NurbsCurves;

   for (i = 0; i < E_NELEM_TYPES; i++)
      mesh->FirstNetIndex[i] = FirstNetIndex[i];

   for (i = 0; i < NNodes; i++)
      mesh->GNIs[i] = i + 1;

   for (i = 0; i < TotalNElems; i++)
      mesh->GEIs[i] = i + 1;

   mesh->GEIsLink1 = mesh->GEIs
                     + mesh->FirstNetIndex[E_ELEM_TYPE_LINK1] - 1;
   mesh->GEIsLink2 = mesh->GEIs
                     + mesh->FirstNetIndex[E_ELEM_TYPE_LINK2] - 1;
   mesh->GEIsLink3 = mesh->GEIs
                     + mesh->FirstNetIndex[E_ELEM_TYPE_LINK3] - 1;
   mesh->GEIsLink4 = mesh->GEIs
                     + mesh->FirstNetIndex[E_ELEM_TYPE_LINK4] - 1;
   mesh->GEIsLink5 = mesh->GEIs
                     + mesh->FirstNetIndex[E_ELEM_TYPE_LINK5] - 1;
   mesh->GEIsTriang1 = mesh->GEIs
                       + mesh->FirstNetIndex[E_ELEM_TYPE_TRIANG1] - 1;
   mesh->GEIsTriang2 = mesh->GEIs
                       + mesh->FirstNetIndex[E_ELEM_TYPE_TRIANG2] - 1;
   mesh->GEIsTriang3 = mesh->GEIs
                       + mesh->FirstNetIndex[E_ELEM_TYPE_TRIANG3] - 1;
   mesh->GEIsTriang4 = mesh->GEIs
                       + mesh->FirstNetIndex[E_ELEM_TYPE_TRIANG4] - 1;
   mesh->GEIsTriang5 = mesh->GEIs
                       + mesh->FirstNetIndex[E_ELEM_TYPE_TRIANG5] - 1;
   mesh->GEIsQuad1 = mesh->GEIs
                     + mesh->FirstNetIndex[E_ELEM_TYPE_QUAD1] - 1;
   mesh->GEIsQuad3 = mesh->GEIs
                     + mesh->FirstNetIndex[E_ELEM_TYPE_QUAD3] - 1;
   mesh->GEIsTetrah1 = mesh->GEIs
                       + mesh->FirstNetIndex[E_ELEM_TYPE_TETRAH1] - 1;
   mesh->GEIsBlock1 = mesh->GEIs
                      + mesh->FirstNetIndex[E_ELEM_TYPE_BLOCK1] - 1;

   for (i = 0; i < E_NKEYWORDS; i++)
      mesh->KeywordSet[i] = KeywordSet[i];

   for (i = 0; i < E_NSUBSTRUCTS; i++)
      mesh->SubStructSet[i] = FALSE;

   return 1;
}


/* ////////////////////////////////////////////////////////////////////////// */
/* DeallocateAllMemory: To deallocate all memory. */


static void DeallocateAllMemory(MESH *mesh)
{
   int k;


   if (KeywordSet[E_KW_NUM_TITLE]) 
      e_free(Title);

   if (KeywordSet[E_KW_NUM_MESHPOINT_COORDS]) 
      e_free_cont_d_matr(NMeshPoints,MeshPointCoords);

   if (KeywordSet[E_KW_NUM_NODES_LINK1])
      e_free_cont_i_matr(NElemsLink1,NodesLink1);

   if (KeywordSet[E_KW_NUM_NODES_LINK2])
      e_free_cont_i_matr(NElemsLink2,NodesLink2);

   if (KeywordSet[E_KW_NUM_NODES_LINK3])
      e_free_cont_i_matr(NElemsLink3,NodesLink3);

   if (KeywordSet[E_KW_NUM_NODES_LINK4])
      e_free_cont_i_matr(NElemsLink4,NodesLink4);

   if (KeywordSet[E_KW_NUM_NODES_LINK5])
      e_free_cont_i_matr(NElemsLink5,NodesLink5);

   if (KeywordSet[E_KW_NUM_NODES_TRIANG1]) 
      e_free_cont_i_matr(NElemsTriang1,NodesTriang1);

   if (KeywordSet[E_KW_NUM_NODES_TRIANG2])
      e_free_cont_i_matr(NElemsTriang2,NodesTriang2);

   if (KeywordSet[E_KW_NUM_NODES_TRIANG3])
      e_free_cont_i_matr(NElemsTriang3,NodesTriang3);

   if (KeywordSet[E_KW_NUM_NODES_TRIANG4])
      e_free_cont_i_matr(NElemsTriang4,NodesTriang4);

   if (KeywordSet[E_KW_NUM_NODES_TRIANG5])
      e_free_cont_i_matr(NElemsTriang5,NodesTriang5);

   if (KeywordSet[E_KW_NUM_NODES_QUAD1])
      e_free_cont_i_matr(NElemsQuad1,NodesQuad1);

   if (KeywordSet[E_KW_NUM_NODES_QUAD3])
      e_free_cont_i_matr(NElemsQuad3,NodesQuad3);

   if (KeywordSet[E_KW_NUM_NODES_TETRAH1])
      e_free_cont_i_matr(NElemsTetrah1,NodesTetrah1);

   if (KeywordSet[E_KW_NUM_NODES_BLOCK1])
      e_free_cont_i_matr(NElemsBlock1,NodesBlock1);

   e_free(NodalDOF);

   if (KeywordSet[E_KW_NUM_BCS])
      FreeBoundaryConditions(NBCNodes);

   if (KeywordSet[E_KW_NUM_LOADS])
      FreeLoads(NLoadedNodes);

   if (KeywordSet[E_KW_NUM_NMATS])
   {
      for (k = 0; k < NMats; k++)
         e_free(Props[k].Name);

      e_free(Mats);
      e_free(Props);
   }

   if (KeywordSet[E_KW_NUM_NCOMP_MATS_TYPE1])
   {
      for (k = 0; k < NCompMatsType1; k++)
         e_free(CompPropsType1[k].Name);

      e_free(CompPropsType1);
   }

   if (KeywordSet[E_KW_NUM_REMESH_DATA])
      FreeRemeshData();

   /* NOTE: No need to free global indices */

   e_free(mesh);
   e_free(FullPathName);
}


/* ////////////////////////////////////////////////////////////////////////// */










/* ////////////////////////////////////////////////////////////////////////// */
/* e_PutMeshData: To output mesh data. */
/* The return value is 1 if the data are written successfully, else 0. */


int e_PutMeshData(MESH *mesh, char *OutputFile)
{
   int i,j,k,l;

   char Gap = FALSE;

   FILE *fp;            /* output file pointer */


   if (OpenOutputFile(OutputFile,E_MDF,&fp) == NULL)
      return 0;

   if (mesh->KeywordSet[E_KW_NUM_TITLE])
      fprintf(fp,"%s  \"%s\"\n\n",E_KW_TITLE,mesh->Title);

   if (mesh->KeywordSet[E_KW_NUM_NMESHPOINTS])
      fprintf(fp,"%s  %d\n",E_KW_NMESHPOINTS,mesh->NMeshPoints);

   if (mesh->KeywordSet[E_KW_NUM_NNODES])
      fprintf(fp,"%s  %d\n",E_KW_NNODES,mesh->NNodes);

   if (mesh->KeywordSet[E_KW_NUM_NELEMS_LINK1])
      fprintf(fp,"%s  %d\n",E_KW_NELEMS_LINK1,mesh->NElemsLink1);

   if (mesh->KeywordSet[E_KW_NUM_NELEMS_LINK2])
      fprintf(fp,"%s  %d\n",E_KW_NELEMS_LINK2,mesh->NElemsLink2);

   if (mesh->KeywordSet[E_KW_NUM_NELEMS_LINK3])
      fprintf(fp,"%s  %d\n",E_KW_NELEMS_LINK3,mesh->NElemsLink3);

   if (mesh->KeywordSet[E_KW_NUM_NELEMS_LINK4])
      fprintf(fp,"%s  %d\n",E_KW_NELEMS_LINK4,mesh->NElemsLink4);

   if (mesh->KeywordSet[E_KW_NUM_NELEMS_LINK5])
      fprintf(fp,"%s  %d\n",E_KW_NELEMS_LINK5,mesh->NElemsLink5);

   if (mesh->KeywordSet[E_KW_NUM_NELEMS_TRIANG1])
      fprintf(fp,"%s  %d\n",E_KW_NELEMS_TRIANG1,mesh->NElemsTriang1);

   if (mesh->KeywordSet[E_KW_NUM_NELEMS_TRIANG2])
      fprintf(fp,"%s  %d\n",E_KW_NELEMS_TRIANG2,mesh->NElemsTriang2);

   if (mesh->KeywordSet[E_KW_NUM_NELEMS_TRIANG3])
      fprintf(fp,"%s  %d\n",E_KW_NELEMS_TRIANG3,mesh->NElemsTriang3);

   if (mesh->KeywordSet[E_KW_NUM_NELEMS_TRIANG4])
      fprintf(fp,"%s  %d\n",E_KW_NELEMS_TRIANG4,mesh->NElemsTriang4);

   if (mesh->KeywordSet[E_KW_NUM_NELEMS_TRIANG5])
      fprintf(fp,"%s  %d\n",E_KW_NELEMS_TRIANG5,mesh->NElemsTriang5);

   if (mesh->KeywordSet[E_KW_NUM_NELEMS_QUAD1])
      fprintf(fp,"%s  %d\n",E_KW_NELEMS_QUAD1,mesh->NElemsQuad1);

   if (mesh->KeywordSet[E_KW_NUM_NELEMS_QUAD3])
      fprintf(fp,"%s  %d\n",E_KW_NELEMS_QUAD3,mesh->NElemsQuad3);

   if (mesh->KeywordSet[E_KW_NUM_NELEMS_TETRAH1])
      fprintf(fp,"%s  %d\n",E_KW_NELEMS_TETRAH1,mesh->NElemsTetrah1);

   if (mesh->KeywordSet[E_KW_NUM_NELEMS_BLOCK1])
      fprintf(fp,"%s  %d\n",E_KW_NELEMS_BLOCK1,mesh->NElemsBlock1);

   if (mesh->KeywordSet[E_KW_NUM_NBC_NODES])
      fprintf(fp,"%s  %d\n",E_KW_NBC_NODES,mesh->NBCNodes);

   if (mesh->KeywordSet[E_KW_NUM_NLOADED_NODES])
      fprintf(fp,"%s  %d\n",E_KW_NLOADED_NODES,mesh->NLoadedNodes);

   if (mesh->KeywordSet[E_KW_NUM_NMATS])
      fprintf(fp,"%s  %d\n",E_KW_NMATS,mesh->NMats);

   if (mesh->KeywordSet[E_KW_NUM_NCOMP_MATS_TYPE1])
      fprintf(fp,"%s  %d\n",E_KW_NCOMP_MATS_TYPE1,mesh->NCompMatsType1);

   if (mesh->KeywordSet[E_KW_NUM_NNURBS_CURVES])
      fprintf(fp,"%s  %d\n",E_KW_NNURBS_CURVES,mesh->Geom.NNurbsCurves);

   if (mesh->KeywordSet[E_KW_NUM_TIMESTEP])
   {
      if (!Gap) { fprintf(fp,"\n"); Gap = TRUE; }

      fprintf(fp,"%s  ",E_KW_TIMESTEP);

      if (PrintEng(fp,"%e",mesh->TimeStep) < 0)
      {
         fclose(fp);
         return 0;
      }

      fprintf(fp,"\n");
   }

   if (mesh->KeywordSet[E_KW_NUM_NTIMESTEPS])
   {
      if (!Gap) { fprintf(fp,"\n"); Gap = TRUE; }

      fprintf(fp,"%s  %d\n",E_KW_NTIMESTEPS,mesh->NTimeSteps);
   }

   if (mesh->KeywordSet[E_KW_NUM_NINTERNAL_TIMESTEPS])
   {
      if (!Gap) { fprintf(fp,"\n"); Gap = TRUE; }

      fprintf(fp,"%s  %d\n",E_KW_NINTERNAL_TIMESTEPS,
                             mesh->NInternalTimeSteps);
   }

   if (mesh->KeywordSet[E_KW_NUM_NEXTERNAL_TIMESTEPS])
   {
      if (!Gap) { fprintf(fp,"\n"); Gap = TRUE; }

      fprintf(fp,"%s  %d\n",E_KW_NEXTERNAL_TIMESTEPS,
                             mesh->NExternalTimeSteps);
   }

   if (mesh->KeywordSet[E_KW_NUM_DAMPING_FACTOR])
   {
      if (!Gap) { fprintf(fp,"\n"); Gap = TRUE; }

      fprintf(fp,"%s  ",E_KW_DAMPING_FACTOR);

      if (PrintEng(fp,"%e",mesh->DampingFactor) < 0)
      {
         fclose(fp);
         return 0;
      }

      fprintf(fp,"\n");
   }

   if (mesh->KeywordSet[E_KW_NUM_BETA])
   {
      if (!Gap) { fprintf(fp,"\n"); Gap = TRUE; }

      fprintf(fp,"%s  %8.6f\n",E_KW_BETA,mesh->Beta);
   }

   if (mesh->KeywordSet[E_KW_NUM_GAMMA])
   {
      if (!Gap) { fprintf(fp,"\n"); Gap = TRUE; }

      fprintf(fp,"%s  %8.6f\n",E_KW_GAMMA,mesh->Gamma);
   }

   if (mesh->KeywordSet[E_KW_NUM_NSTRESS_POINTS_LINK1])
   {
      if (!Gap) { fprintf(fp,"\n"); Gap = TRUE; }

      fprintf(fp,"%s  %d\n",
              E_KW_NSTRESS_POINTS_LINK1, mesh->NStressPointsLink1);
   }
   if (mesh->KeywordSet[E_KW_NUM_NSTRESS_POINTS_LINK2])
   {
      if (!Gap) { fprintf(fp,"\n"); Gap = TRUE; }

      fprintf(fp,"%s  %d\n",
              E_KW_NSTRESS_POINTS_LINK2, mesh->NStressPointsLink2);
   }
   if (mesh->KeywordSet[E_KW_NUM_NSTRESS_POINTS_LINK3])
   {
      if (!Gap) { fprintf(fp,"\n"); Gap = TRUE; }

      fprintf(fp,"%s  %d\n",
              E_KW_NSTRESS_POINTS_LINK3, mesh->NStressPointsLink3);
   }
   if (mesh->KeywordSet[E_KW_NUM_NSTRESS_POINTS_LINK4])
   {
      if (!Gap) { fprintf(fp,"\n"); Gap = TRUE; }

      fprintf(fp,"%s  %d\n",
              E_KW_NSTRESS_POINTS_LINK4, mesh->NStressPointsLink4);
   }
   if (mesh->KeywordSet[E_KW_NUM_NSTRESS_POINTS_LINK5])
   {
      if (!Gap) { fprintf(fp,"\n"); Gap = TRUE; }

      fprintf(fp,"%s  %d\n",
              E_KW_NSTRESS_POINTS_LINK5, mesh->NStressPointsLink5);
   }
   if (mesh->KeywordSet[E_KW_NUM_NSTRESS_POINTS_TRIANG1])
   {
      if (!Gap) { fprintf(fp,"\n"); Gap = TRUE; }

      fprintf(fp,"%s  %d\n",
              E_KW_NSTRESS_POINTS_TRIANG1, mesh->NStressPointsTriang1);
   }
   if (mesh->KeywordSet[E_KW_NUM_NSTRESS_POINTS_TRIANG2])
   {
      if (!Gap) { fprintf(fp,"\n"); Gap = TRUE; }

      fprintf(fp,"%s  %d\n",
              E_KW_NSTRESS_POINTS_TRIANG2, mesh->NStressPointsTriang2);
   }
   if (mesh->KeywordSet[E_KW_NUM_NSTRESS_POINTS_TRIANG3])
   {
      if (!Gap) { fprintf(fp,"\n"); Gap = TRUE; }

      fprintf(fp,"%s  %d\n",
              E_KW_NSTRESS_POINTS_TRIANG3, mesh->NStressPointsTriang3);
   }
   if (mesh->KeywordSet[E_KW_NUM_NSTRESS_POINTS_TRIANG4])
   {
      if (!Gap) { fprintf(fp,"\n"); Gap = TRUE; }

      fprintf(fp,"%s  %d\n",
              E_KW_NSTRESS_POINTS_TRIANG4, mesh->NStressPointsTriang4);
   }
   if (mesh->KeywordSet[E_KW_NUM_NSTRESS_POINTS_TRIANG5])
   {
      if (!Gap) { fprintf(fp,"\n"); Gap = TRUE; }

      fprintf(fp,"%s  %d\n",
              E_KW_NSTRESS_POINTS_TRIANG5, mesh->NStressPointsTriang5);
   }
   if (mesh->KeywordSet[E_KW_NUM_NSTRESS_POINTS_QUAD1])
   {
      if (!Gap) { fprintf(fp,"\n"); Gap = TRUE; }

      fprintf(fp,"%s  %d\n",
              E_KW_NSTRESS_POINTS_QUAD1, mesh->NStressPointsQuad1);
   }
   if (mesh->KeywordSet[E_KW_NUM_NSTRESS_POINTS_QUAD3])
   {
      if (!Gap) { fprintf(fp,"\n"); Gap = TRUE; }

      fprintf(fp,"%s  %d\n",
              E_KW_NSTRESS_POINTS_QUAD3, mesh->NStressPointsQuad3);
   }
   if (mesh->KeywordSet[E_KW_NUM_NSTRESS_POINTS_TETRAH1])
   {
      if (!Gap) { fprintf(fp,"\n"); Gap = TRUE; }

      fprintf(fp,"%s  %d\n",
              E_KW_NSTRESS_POINTS_TETRAH1, mesh->NStressPointsTetrah1);
   }
   if (mesh->KeywordSet[E_KW_NUM_NSTRESS_POINTS_BLOCK1])
   {
      if (!Gap) { fprintf(fp,"\n"); Gap = TRUE; }

      fprintf(fp,"%s  %d\n",
              E_KW_NSTRESS_POINTS_BLOCK1, mesh->NStressPointsBlock1);
   }


   if (mesh->KeywordSet[E_KW_NUM_MESHPOINT_COORDS])
   {
      fprintf(fp,"\n\n# Mesh-point coordinates\n");
      fprintf(fp,"# node index");
      fprintf(fp,"           x-coord");
      fprintf(fp,"               y-coord");
      fprintf(fp,"               z-coord\n");

      fprintf(fp,"\n%s\n",E_KW_MESHPOINT_COORDS);

      for (i = 0; i < mesh->NMeshPoints; i++)
      {
         fprintf(fp,"%7d   ",i + 1);
         fprintf(fp,"%20.8f  ",mesh->MeshPointCoords[i][0]);
         fprintf(fp,"%20.8f  ",mesh->MeshPointCoords[i][1]);
         fprintf(fp,"%20.8f\n",mesh->MeshPointCoords[i][2]);
      }
   }

   if (mesh->KeywordSet[E_KW_NUM_NODES_LINK1])
   {
      fprintf(fp,"\n\n# LINK1 element node indices\n");
      fprintf(fp,"# element index\n");
      fprintf(fp,"#         _ node indices _\n");
      fprintf(fp,"#               1        2\n");

      fprintf(fp,"\n%s\n",E_KW_NODES_LINK1);

      for (i = 0; i < mesh->NElemsLink1; i++)
      {
         fprintf(fp,"%7d   ",i + 1);
         fprintf(fp,"%7d  ",mesh->NodesLink1[i][0]);
         fprintf(fp,"%7d\n",mesh->NodesLink1[i][1]);
      }
   }

   if (mesh->KeywordSet[E_KW_NUM_NODES_LINK2])
   {
      fprintf(fp,"\n\n# LINK2 element node indices\n");
      fprintf(fp,"# element index\n");
      fprintf(fp,"#         _ node indices _\n");
      fprintf(fp,"#               1        2\n");

      fprintf(fp,"\n%s\n",E_KW_NODES_LINK2);

      for (i = 0; i < mesh->NElemsLink2; i++)
      {
         fprintf(fp,"%7d   ",i + 1);
         fprintf(fp,"%7d  ",mesh->NodesLink2[i][0]);
         fprintf(fp,"%7d\n",mesh->NodesLink2[i][1]);
      }
   }

   if (mesh->KeywordSet[E_KW_NUM_NODES_LINK3])
   {
      fprintf(fp,"\n\n# LINK3 element node indices\n");
      fprintf(fp,"# element index\n");
      fprintf(fp,"#         _ node indices _\n");
      fprintf(fp,"#               1        2\n");

      fprintf(fp,"\n%s\n",E_KW_NODES_LINK3);

      for (i = 0; i < mesh->NElemsLink3; i++)
      {
         fprintf(fp,"%7d   ",i + 1);
         fprintf(fp,"%7d  ",mesh->NodesLink3[i][0]);
         fprintf(fp,"%7d\n",mesh->NodesLink3[i][1]);
      }
   }

   if (mesh->KeywordSet[E_KW_NUM_NODES_LINK4])
   {
      fprintf(fp,"\n\n# LINK4 element node indices\n");
      fprintf(fp,"# element index\n");
      fprintf(fp,"#         _ node indices _\n");
      fprintf(fp,"#               1        2\n");

      fprintf(fp,"\n%s\n",E_KW_NODES_LINK4);

      for (i = 0; i < mesh->NElemsLink4; i++)
      {
         fprintf(fp,"%7d   ",i + 1);
         fprintf(fp,"%7d  ",mesh->NodesLink4[i][0]);
         fprintf(fp,"%7d\n",mesh->NodesLink4[i][1]);
      }
   }

   if (mesh->KeywordSet[E_KW_NUM_NODES_LINK5])
   {
      fprintf(fp,"\n\n# LINK5 element node indices\n");
      fprintf(fp,"# element index\n");
      fprintf(fp,"#         _ node indices _\n");
      fprintf(fp,"#               1        2\n");

      fprintf(fp,"\n%s\n",E_KW_NODES_LINK5);

      for (i = 0; i < mesh->NElemsLink5; i++)
      {
         fprintf(fp,"%7d   ",i + 1);
         fprintf(fp,"%7d  ",mesh->NodesLink5[i][0]);
         fprintf(fp,"%7d\n",mesh->NodesLink5[i][1]);
      }
   }

   if (mesh->KeywordSet[E_KW_NUM_NODES_TRIANG1])
   {
      fprintf(fp,"\n\n# TRIANG1 element node indices\n");
      fprintf(fp,"# element index\n");
      fprintf(fp,"#         ______ node indices _____\n");
      fprintf(fp,"#               1        2        3\n");

      fprintf(fp,"\n%s\n",E_KW_NODES_TRIANG1);

      for (i = 0; i < mesh->NElemsTriang1; i++)
      {
         fprintf(fp,"%7d   ",i + 1);
         fprintf(fp,"%7d  ",mesh->NodesTriang1[i][0]);
         fprintf(fp,"%7d  ",mesh->NodesTriang1[i][1]);
         fprintf(fp,"%7d\n",mesh->NodesTriang1[i][2]);
      }
   }

   if (mesh->KeywordSet[E_KW_NUM_NODES_TRIANG2])
   {
      fprintf(fp,"\n\n# TRIANG2 element node indices\n");
      fprintf(fp,"# element index\n");
      fprintf(fp,"#           __________________ ");
      fprintf(fp,"node indices __________________\n");
      fprintf(fp,"#               ");
      fprintf(fp,"1        2        3        4        5        6\n");

      fprintf(fp,"\n%s\n",E_KW_NODES_TRIANG2);

      for (i = 0; i < mesh->NElemsTriang2; i++)
      {
         fprintf(fp,"%7d   ",i + 1);
         fprintf(fp,"%7d  ",mesh->NodesTriang2[i][0]);
         fprintf(fp,"%7d  ",mesh->NodesTriang2[i][1]);
         fprintf(fp,"%7d  ",mesh->NodesTriang2[i][2]);
         fprintf(fp,"%7d  ",mesh->NodesTriang2[i][3]);
         fprintf(fp,"%7d  ",mesh->NodesTriang2[i][4]);
         fprintf(fp,"%7d\n",mesh->NodesTriang2[i][5]);
      }
   }

   if (mesh->KeywordSet[E_KW_NUM_NODES_TRIANG3])
   {
      fprintf(fp,"\n\n# TRIANG3 element node indices\n");
      fprintf(fp,"# element index\n");
      fprintf(fp,"#         ______ node indices _____\n");
      fprintf(fp,"#               1        2        3\n");

      fprintf(fp,"\n%s\n",E_KW_NODES_TRIANG3);

      for (i = 0; i < mesh->NElemsTriang3; i++)
      {
         fprintf(fp,"%7d   ",i + 1);
         fprintf(fp,"%7d  ",mesh->NodesTriang3[i][0]);
         fprintf(fp,"%7d  ",mesh->NodesTriang3[i][1]);
         fprintf(fp,"%7d\n",mesh->NodesTriang3[i][2]);
      }
   }

   if (mesh->KeywordSet[E_KW_NUM_NODES_TRIANG4])
   {
      fprintf(fp,"\n\n# TRIANG4 element node indices\n");
      fprintf(fp,"# element index\n");
      fprintf(fp,"#         ______ node indices _____\n");
      fprintf(fp,"#               1        2        3\n");

      fprintf(fp,"\n%s\n",E_KW_NODES_TRIANG4);

      for (i = 0; i < mesh->NElemsTriang4; i++)
      {
         fprintf(fp,"%7d   ",i + 1);
         fprintf(fp,"%7d  ",mesh->NodesTriang4[i][0]);
         fprintf(fp,"%7d  ",mesh->NodesTriang4[i][1]);
         fprintf(fp,"%7d\n",mesh->NodesTriang4[i][2]);
      }
   }

   if (mesh->KeywordSet[E_KW_NUM_NODES_TRIANG5])
   {
      fprintf(fp,"\n\n# TRIANG5 element node indices\n");
      fprintf(fp,"# element index\n");
      fprintf(fp,"#         ______ node indices _____\n");
      fprintf(fp,"#               1        2        3\n");

      fprintf(fp,"\n%s\n",E_KW_NODES_TRIANG5);

      for (i = 0; i < mesh->NElemsTriang5; i++)
      {
         fprintf(fp,"%7d   ",i + 1);
         fprintf(fp,"%7d  ",mesh->NodesTriang5[i][0]);
         fprintf(fp,"%7d  ",mesh->NodesTriang5[i][1]);
         fprintf(fp,"%7d\n",mesh->NodesTriang5[i][2]);
      }
   }

   if (mesh->KeywordSet[E_KW_NUM_NODES_QUAD1])
   {
      fprintf(fp,"\n\n# QUAD1 element node indices\n");
      fprintf(fp,"# element index\n");
      fprintf(fp,"#         __________ node indices __________\n");
      fprintf(fp,"#               1        2        3        4\n");

      fprintf(fp,"\n%s\n",E_KW_NODES_QUAD1);

      for (i = 0; i < mesh->NElemsQuad1; i++)
      {
         fprintf(fp,"%7d   ",i + 1);
         fprintf(fp,"%7d  ",mesh->NodesQuad1[i][0]);
         fprintf(fp,"%7d  ",mesh->NodesQuad1[i][1]);
         fprintf(fp,"%7d  ",mesh->NodesQuad1[i][2]);
         fprintf(fp,"%7d\n",mesh->NodesQuad1[i][3]);
      }
   }

   if (mesh->KeywordSet[E_KW_NUM_NODES_QUAD3])
   {
      fprintf(fp,"\n\n# QUAD3 element node indices\n");
      fprintf(fp,"# element index\n");
      fprintf(fp,"#         ____________________________________ ");
      fprintf(fp,"node indices ____________________________\n");
      fprintf(fp,"#               1        2        3        4   ");
      fprintf(fp,"     5        6        7       8        9\n");

      fprintf(fp,"\n%s\n",E_KW_NODES_QUAD3);

      for (i = 0; i < mesh->NElemsQuad3; i++)
      {
         fprintf(fp,"%7d   ",i + 1);
         fprintf(fp,"%7d  ",mesh->NodesQuad3[i][0]);
         fprintf(fp,"%7d  ",mesh->NodesQuad3[i][1]);
         fprintf(fp,"%7d  ",mesh->NodesQuad3[i][2]);
         fprintf(fp,"%7d  ",mesh->NodesQuad3[i][3]);
         fprintf(fp,"%7d  ",mesh->NodesQuad3[i][4]);
         fprintf(fp,"%7d  ",mesh->NodesQuad3[i][5]);
         fprintf(fp,"%7d  ",mesh->NodesQuad3[i][6]);
         fprintf(fp,"%7d  ",mesh->NodesQuad3[i][7]);
         fprintf(fp,"%7d\n",mesh->NodesQuad3[i][8]);
      }
   }

   if (mesh->KeywordSet[E_KW_NUM_NODES_TETRAH1])
   {
      fprintf(fp,"\n\n# TETRAH1 element node indices\n");
      fprintf(fp,"# element index\n");
      fprintf(fp,"#         __________ node indices __________\n");
      fprintf(fp,"#               1        2        3        4\n");

      fprintf(fp,"\n%s\n",E_KW_NODES_TETRAH1);

      for (i = 0; i < mesh->NElemsTetrah1; i++)
      {
         fprintf(fp,"%7d   ",i + 1);
         fprintf(fp,"%7d  ",mesh->NodesTetrah1[i][0]);
         fprintf(fp,"%7d  ",mesh->NodesTetrah1[i][1]);
         fprintf(fp,"%7d  ",mesh->NodesTetrah1[i][2]);
         fprintf(fp,"%7d\n",mesh->NodesTetrah1[i][3]);
      }
   }

   if (mesh->KeywordSet[E_KW_NUM_NODES_BLOCK1])
   {
      fprintf(fp,"\n\n# BLOCK1 element node indices\n");
      fprintf(fp,"# element index\n");
      fprintf(fp,"#         ____________________________ ");
      fprintf(fp,"node indices ____________________________\n");
      fprintf(fp,"#               1        2        3");
      fprintf(fp,"        4        5        6        7        8\n");

      fprintf(fp,"\n%s\n",E_KW_NODES_BLOCK1);

      for (i = 0; i < mesh->NElemsBlock1; i++)
      {
         fprintf(fp,"%7d   ",i + 1);
         fprintf(fp,"%7d  ",mesh->NodesBlock1[i][0]);
         fprintf(fp,"%7d  ",mesh->NodesBlock1[i][1]);
         fprintf(fp,"%7d  ",mesh->NodesBlock1[i][2]);
         fprintf(fp,"%7d  ",mesh->NodesBlock1[i][3]);
         fprintf(fp,"%7d  ",mesh->NodesBlock1[i][4]);
         fprintf(fp,"%7d  ",mesh->NodesBlock1[i][5]);
         fprintf(fp,"%7d  ",mesh->NodesBlock1[i][6]);
         fprintf(fp,"%7d\n",mesh->NodesBlock1[i][7]);
      }
   }

   if (mesh->KeywordSet[E_KW_NUM_BCS])
   {
      fprintf(fp,"\n\n# Boundary conditions\n");
      fprintf(fp,"# node index\n");
      fprintf(fp,"#         ");
      fprintf(fp,"____________________________________ ");
      fprintf(fp,"nodal degrees of freedom ");
      fprintf(fp,"_____________________________________\n");
      fprintf(fp,"#         ");
      fprintf(fp,"______________ 1 ______________   ");
      fprintf(fp,"______________ 2 ______________   ");
      fprintf(fp,"______________ 3 ______________\n");

      fprintf(fp,"\n%s\n",E_KW_BCS);

      for (i = 0; i < mesh->NBCNodes; i++)
      {
         fprintf(fp,"%7d   ",mesh->BCNodes[i]);

         for (j = 0; j < mesh->NodalDOF[mesh->BCNodes[i] - 1]; j++)
         {
            if (mesh->BCTypes[i][j] == E_BC_NUM_FREE)
            {
               fprintf(fp,"%8s   ",E_BC_FREE);
               fprintf(fp,"                    ");
            }
            else if (mesh->BCTypes[i][j] == E_BC_NUM_FIXED)
            {
               fprintf(fp,"%8s   ",E_BC_FIXED);
               fprintf(fp,"%20.8f",mesh->BCDispls[i][j]);
            }
            else
            {
               fprintf(fp,"%8s   ",E_BC_SPRING);

               if (PrintEng(fp,"%20.6e",mesh->BCSpringConsts[i][j]) < 0)
               {
                  fclose(fp);
                  return 0;
               }
            }

            (j < mesh->NodalDOF[mesh->BCNodes[i] - 1] - 1)
                ? fprintf(fp,"   ") : fprintf(fp,"\n");
         }
      }
   }

   if (mesh->KeywordSet[E_KW_NUM_LOADS])
   {
      fprintf(fp,"\n\n# Loads\n");
      fprintf(fp,"# node index\n");
      fprintf(fp,"#         ");
      fprintf(fp,"______________ ");
      fprintf(fp,"nodal degrees of freedom ");
      fprintf(fp,"______________\n");
      fprintf(fp,"#                  1                  2");
      fprintf(fp,"                  3\n");

      fprintf(fp,"\n%s\n",E_KW_LOADS);

      for (i = 0; i < mesh->NLoadedNodes; i++)
      {
         fprintf(fp,"%7d   ",mesh->LoadedNodes[i]);

         for (j = 0; j < mesh->NodalDOF[mesh->LoadedNodes[i] - 1]; j++)
         {
            if (PrintEng(fp,"%16.6e",mesh->Loads[i][j]) < 0)
            {
               fclose(fp);
               return 0;
            }

            (j < mesh->NodalDOF[mesh->LoadedNodes[i] - 1] - 1)
               ? fprintf(fp,"   ") : fprintf(fp,"\n");
         }
      }
   }

   if (mesh->KeywordSet[E_KW_NUM_MATS_LINK1])
   {
      fprintf(fp,"\n\n# LINK1 element materials\n");
      fprintf(fp,"# element index\n");
      fprintf(fp,"#         material name\n");

      fprintf(fp,"\n%s\n",E_KW_MATS_LINK1);

      for (i = 0; i < mesh->NElemsLink1; i++)
      {
         k = (int)(mesh->MatsLink1[i] - mesh->Props);
         fprintf(fp,"%7d   \"%s\"\n",i + 1,mesh->Props[k].Name);
      }
   }

   if (mesh->KeywordSet[E_KW_NUM_MATS_LINK2])
   {
      fprintf(fp,"\n\n# LINK2 element materials\n");
      fprintf(fp,"# element index\n");
      fprintf(fp,"#         material name\n");

      fprintf(fp,"\n%s\n",E_KW_MATS_LINK2);

      for (i = 0; i < mesh->NElemsLink2; i++)
      {
         k = (int)(mesh->MatsLink2[i] - mesh->Props);
         fprintf(fp,"%7d   \"%s\"\n",i + 1,mesh->Props[k].Name);
      }
   }

   if (mesh->KeywordSet[E_KW_NUM_MATS_LINK3])
   {
      fprintf(fp,"\n\n# LINK3 element materials\n");
      fprintf(fp,"# element index\n");
      fprintf(fp,"#         material name\n");

      fprintf(fp,"\n%s\n",E_KW_MATS_LINK3);

      for (i = 0; i < mesh->NElemsLink3; i++)
      {
         k = (int)(mesh->MatsLink3[i] - mesh->Props);
         fprintf(fp,"%7d   \"%s\"\n",i + 1,mesh->Props[k].Name);
      }
   }

   if (mesh->KeywordSet[E_KW_NUM_MATS_LINK4])
   {
      fprintf(fp,"\n\n# LINK4 element materials\n");
      fprintf(fp,"# element index\n");
      fprintf(fp,"#         material name\n");

      fprintf(fp,"\n%s\n",E_KW_MATS_LINK4);

      for (i = 0; i < mesh->NElemsLink4; i++)
      {
         k = (int)(mesh->MatsLink4[i] - mesh->Props);
         fprintf(fp,"%7d   \"%s\"\n",i + 1,mesh->Props[k].Name);
      }
   }

   if (mesh->KeywordSet[E_KW_NUM_MATS_LINK5])
   {
      fprintf(fp,"\n\n# LINK5 element materials\n");
      fprintf(fp,"# element index\n");
      fprintf(fp,"#         material name\n");

      fprintf(fp,"\n%s\n",E_KW_MATS_LINK5);

      for (i = 0; i < mesh->NElemsLink5; i++)
      {
         k = (int)(mesh->MatsLink5[i] - mesh->Props);
         fprintf(fp,"%7d   \"%s\"\n",i + 1,mesh->Props[k].Name);
      }
   }

   if (mesh->KeywordSet[E_KW_NUM_MATS_TRIANG1])
   {
      fprintf(fp,"\n\n# TRIANG1 element materials\n");
      fprintf(fp,"# element index\n");
      fprintf(fp,"#         material name\n");

      fprintf(fp,"\n%s\n",E_KW_MATS_TRIANG1);

      for (i = 0; i < mesh->NElemsTriang1; i++)
      {
         k = (int)(mesh->MatsTriang1[i] - mesh->Props);
         fprintf(fp,"%7d   \"%s\"\n",i + 1,mesh->Props[k].Name);
      }
   }

   if (mesh->KeywordSet[E_KW_NUM_COMP_MATS_TRIANG2])
   {
      fprintf(fp,"\n\n# TRIANG2 element composite materials\n");
      fprintf(fp,"# element index\n");
      fprintf(fp,"#         composite material name\n");

      fprintf(fp,"\n%s\n",E_KW_COMP_MATS_TRIANG2);

      for (i = 0; i < mesh->NElemsTriang2; i++)
      {
         k = (int)(mesh->CompMatsTriang2[i] - mesh->CompPropsType1);
         fprintf(fp,"%7d   \"%s\"\n",i + 1,mesh->CompPropsType1[k].Name);
      }
   }

   if (mesh->KeywordSet[E_KW_NUM_MATS_TRIANG3])
   {
      fprintf(fp,"\n\n# TRIANG3 element materials\n");
      fprintf(fp,"# element index\n");
      fprintf(fp,"#         material name\n");

      fprintf(fp,"\n%s\n",E_KW_MATS_TRIANG3);

      for (i = 0; i < mesh->NElemsTriang3; i++)
      {
         k = (int)(mesh->MatsTriang3[i] - mesh->Props);
         fprintf(fp,"%7d   \"%s\"\n",i + 1,mesh->Props[k].Name);
      }
   }

   if (mesh->KeywordSet[E_KW_NUM_MATS_TRIANG4])
   {
      fprintf(fp,"\n\n# TRIANG4 element materials\n");
      fprintf(fp,"# element index\n");
      fprintf(fp,"#         material name\n");

      fprintf(fp,"\n%s\n",E_KW_MATS_TRIANG4);

      for (i = 0; i < mesh->NElemsTriang4; i++)
      {
         k = (int)(mesh->MatsTriang4[i] - mesh->Props);
         fprintf(fp,"%7d   \"%s\"\n",i + 1,mesh->Props[k].Name);
      }
   }

   if (mesh->KeywordSet[E_KW_NUM_MATS_TRIANG5])
   {
      fprintf(fp,"\n\n# TRIANG5 element materials\n");
      fprintf(fp,"# element index\n");
      fprintf(fp,"#         material name\n");

      fprintf(fp,"\n%s\n",E_KW_MATS_TRIANG5);

      for (i = 0; i < mesh->NElemsTriang5; i++)
      {
         k = (int)(mesh->MatsTriang5[i] - mesh->Props);
         fprintf(fp,"%7d   \"%s\"\n",i + 1,mesh->Props[k].Name);
      }
   }

   if (mesh->KeywordSet[E_KW_NUM_MATS_QUAD1])
   {
      fprintf(fp,"\n\n# QUAD1 element materials\n");
      fprintf(fp,"# element index\n");
      fprintf(fp,"#         material name\n");

      fprintf(fp,"\n%s\n",E_KW_MATS_QUAD1);

      for (i = 0; i < mesh->NElemsQuad1; i++)
      {
         k = (int)(mesh->MatsQuad1[i] - mesh->Props);
         fprintf(fp,"%7d   \"%s\"\n",i + 1,mesh->Props[k].Name);
      }
   }

   if (mesh->KeywordSet[E_KW_NUM_COMP_MATS_QUAD3])
   {
      fprintf(fp,"\n\n# QUAD3 element composite materials\n");
      fprintf(fp,"# element index\n");
      fprintf(fp,"#         composite material name\n");

      fprintf(fp,"\n%s\n",E_KW_COMP_MATS_QUAD3);

      for (i = 0; i < mesh->NElemsQuad3; i++)
      {
         k = (int)(mesh->CompMatsQuad3[i] - mesh->CompPropsType1);
         fprintf(fp,"%7d   \"%s\"\n",i + 1,mesh->CompPropsType1[k].Name);
      }
   }

   if (mesh->KeywordSet[E_KW_NUM_MATS_TETRAH1])
   {
      fprintf(fp,"\n\n# TETRAH1 element materials\n");
      fprintf(fp,"# element index\n");
      fprintf(fp,"#         material name\n");

      fprintf(fp,"\n%s\n",E_KW_MATS_TETRAH1);

      for (i = 0; i < mesh->NElemsTetrah1; i++)
      {
         k = (int)(mesh->MatsTetrah1[i] - mesh->Props);
         fprintf(fp,"%7d   \"%s\"\n",i + 1,mesh->Props[k].Name);
      }
   }

   if (mesh->KeywordSet[E_KW_NUM_MATS_BLOCK1])
   {
      fprintf(fp,"\n\n# BLOCK1 element materials\n");
      fprintf(fp,"# element index\n");
      fprintf(fp,"#         material name\n");

      fprintf(fp,"\n%s\n",E_KW_MATS_BLOCK1);

      for (i = 0; i < mesh->NElemsBlock1; i++)
      {
         k = (int)(mesh->MatsBlock1[i] - mesh->Props);
         fprintf(fp,"%7d   \"%s\"\n",i + 1,mesh->Props[k].Name);
      }
   }

   if (mesh->KeywordSet[E_KW_NUM_REMESH_DATA])
   {
      fprintf(fp,"\n\n# Remeshing node data\n");
      fprintf(fp,"# vertex-node index\n");
      fprintf(fp,"#         node type and/or data\n");

      fprintf(fp,"\n%s\n",E_KW_REMESH_DATA);

      for (k = 0; k < mesh->NMeshPoints; k++)
      {
         if (mesh->NodeTypes[k] == E_RM_NUM_EXTERIOR)
         {
            fprintf(fp,"%7d  %s\n",k + 1,E_RM_EXTERIOR);

            if (mesh->NNurbsCurves[k] > 0)
               fprintf(fp,"            %s  %d\n",
                       E_RMKW_NNURBS_CURVES,mesh->NNurbsCurves[k]);

            for (j = 0; j < mesh->NNurbsCurves[k]; j++)
            {
               l = (int)(mesh->NurbsCurves[k][j] - mesh->Geom.NurbsCurves);
               fprintf(fp,"           %4d  ",j + 1);
               fprintf(fp,"%10s  ",E_RMKW_NURBS_CURVE);
               fprintf(fp,"\"%s\"  \n",mesh->Geom.NurbsCurves[l].Name);
               fprintf(fp,"                ");
               fprintf(fp,"%10s   ",E_RMKW_NC_PARAM);
               fprintf(fp,"%8.6f\n",mesh->NurbsCurveParams[k][j]);
            }
         }
         else if (mesh->NodeTypes[k] == E_RM_NUM_INTERIOR)
         {
            fprintf(fp,"%7d  %s\n",k + 1,E_RM_INTERIOR);
         }
         else
         {
            fprintf(fp,"%7d  %s\n",k + 1,E_RM_ISD_BOUNDARY);
         }
      }
   }

   fclose(fp);
   return 1;
}
