/********************************************************************
  
 ********************************************************************/

 /* $Id: etliCmdComposite.c,v 1.1.1.1 2002/08/04 13:19:11 peteri Exp $ */


#include "etliInt.h"


/*
 * composite object --
 */  

typedef struct
{
  COMPMAT1 *composite; 
  /*Tcl_HashTable *compositeInfo; */
} CompositeObj; 


/* Prototypes for procedures referenced in this file only */
int EtliCompositeCmd (ClientData clientData, Tcl_Interp *interp,
             int objc, Tcl_Obj *CONST objv[]);
int EtliCompositeObjCmd (ClientData clientData, Tcl_Interp *interp, 
                int objc, Tcl_Obj *CONST objv[]);
void EtliCompositeObjDelete (ClientData compositeObjPtr);


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 Etli_InitCompositeCmd -- 

 Results:
         TCL_OK

 Side effects:
         Create etli_composite command. 
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        int        
Etli_InitCompositeCmd (Tcl_Interp *interp)
{

    Tcl_CreateObjCommand (interp, "etli_composite", EtliCompositeCmd, 
                      (ClientData) NULL,  (Tcl_CmdDeleteProc *) NULL);

    return (TCL_OK); 
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 EtliCompositeCmd -- 
             Create composite object and accociated Tcl command
 Results:           
         The results in normally TCL_OK.
         TCL_ERROR will be returned if
           1) wrong numbers of argument is supplied
           2) allocation of new composite object failed
         In any case apropriate error message will be left
         in result object.
         
 Side effects:
         Allocate memory of new composite object.
         Create new Tcl command for composite object.
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        int
EtliCompositeCmd (ClientData clientData, Tcl_Interp *interp,
             int objc, Tcl_Obj *CONST objv[])
{
  char name[20];  
  static int id = 0; 

  Tcl_Obj *resultObjPtr;  
  CompositeObj *compositeObjPtr;  

  if (objc != 1)
    {
      Tcl_WrongNumArgs (interp, 1, objv, "no options");
      return (TCL_ERROR); 
    }    

  /*
  RP_TRY
     RP_NEW(compositeObjPtr);        
  RP_EXCEPT(RP_MemFailed)
      resultObjPtr = Tcl_GetObjResult(interp);
      Tcl_SetStringObj (resultObjPtr, name, -1); 
      RP_RETURN (TCL_ERROR);
  RP_END_TRY 
  */

  compositeObjPtr = (CompositeObj *) Tcl_Alloc (sizeof (CompositeObj)); 
  
  compositeObjPtr->composite = NULL;
  
  sprintf (name, "etliComposite%d", id); 
  id++;
  
  Tcl_CreateObjCommand (interp, name, EtliCompositeObjCmd, (ClientData) compositeObjPtr, 
                                      EtliCompositeObjDelete);
  
  resultObjPtr = Tcl_GetObjResult(interp);
  Tcl_SetStringObj (resultObjPtr, name, -1); 

  return (TCL_OK); 
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 EtliCompositeObjCmd -- 
         Parse and process requests to composite object
         
 Results:        
         The return value is normally TCL_OK.
         TCL_ERROR can be returned if command parse error ocures
         or request return error condition.

 Side effecs:
         Depending on the request.
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        int
EtliCompositeObjCmd (ClientData clientData, Tcl_Interp *interp, 
                int objc, Tcl_Obj *CONST objv[])
{
  int index;   
  Tcl_Obj *resultObjPtr;  
  char *subCmds[] = {"get", "put", "info", "delete", NULL};
  enum subCmdsIx {GetIx, PutIx, InfoIx, DeleteIx};  

  if (objc != 2)
    {
       Tcl_WrongNumArgs (interp, 1, objv, "option ?arg ...?"); 
       return (TCL_ERROR); 
    }

  if (Tcl_GetIndexFromObj (interp, objv[1], subCmds, "option", TCL_EXACT, 
                          &index) != TCL_OK)
    {
       return (TCL_ERROR);
    }
  switch (index)
    {
      case GetIx:
                {
                 resultObjPtr = Tcl_GetObjResult(interp);
                 Tcl_SetStringObj (resultObjPtr, "doing_get_command", -1); 
                 return (TCL_OK); 
                }
      default :          
                {
                 resultObjPtr = Tcl_GetObjResult(interp);
                 Tcl_SetStringObj (resultObjPtr, "doing_other_command", -1); 
                 return (TCL_OK); 
                }
            
    }
  return (TCL_OK); /* to avoid commpiler warning */
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 EtliCompositeObjDelete --
         Delete composite object and associated Tcl command

 Results:        
         NONE
 
 Side effects:
         Free memory allocated for composite object.
         Delete Tcl command associated with composite object. 
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        void
EtliCompositeObjDelete (ClientData compositeObjPtr)
{
}

/* Modifications:
 * $Id: etliCmdComposite.c,v 1.1.1.1 2002/08/04 13:19:11 peteri Exp $
 */ 
