/********************************************************************
  
 ********************************************************************/

 /* $Id: etliCmdGeom.c,v 1.1.1.1 2002/08/04 13:19:11 peteri Exp $ */


#include "etliInt.h"


/*
 * geom object --
 */  

typedef struct
{
  GEOM *geom; 
  /*Tcl_HashTable *geomInfo; */
} GeomObj; 

/*
 * nurbs object --
 */ 


/* Prototypes for procedures referenced in this file only */
int EtliGeomCmd (ClientData clientData, Tcl_Interp *interp,
             int objc, Tcl_Obj *CONST objv[]);
int EtliGeomObjCmd (ClientData clientData, Tcl_Interp *interp, 
                int objc, Tcl_Obj *CONST objv[]);
void EtliGeomObjDelete (ClientData geomObjPtr);


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 Etli_InitGeomCmd -- 

 Results:
         TCL_OK

 Side effects:
         Create etli_geom command. 
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        int        
Etli_InitGeomCmd (Tcl_Interp *interp)
{

    Tcl_CreateObjCommand (interp, "etli_geom", EtliGeomCmd, 
                      (ClientData) NULL,  (Tcl_CmdDeleteProc *) NULL);

    return (TCL_OK); 
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 EtliGeomCmd -- 
             Create geom object and accociated Tcl command
 Results:           
         The results in normally TCL_OK.
         TCL_ERROR will be returned if
           1) wrong numbers of argument is supplied
           2) allocation of new geom object failed
         In any case apropriate error message will be left
         in result object.
         
 Side effects:
         Allocate memory of new geom object.
         Create new Tcl command for geom object.
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        int
EtliGeomCmd (ClientData clientData, Tcl_Interp *interp,
             int objc, Tcl_Obj *CONST objv[])
{
  char name[20];  
  static int id = 0; 

  Tcl_Obj *resultObjPtr;  
  GeomObj *geomObjPtr;  

  if (objc != 1)
    {
      Tcl_WrongNumArgs (interp, 1, objv, "no options");
      return (TCL_ERROR); 
    }    

  /*
  RP_TRY
     RP_NEW(geomObjPtr);        
  RP_EXCEPT(RP_MemFailed)
      resultObjPtr = Tcl_GetObjResult(interp);
      Tcl_SetStringObj (resultObjPtr, name, -1); 
      RP_RETURN (TCL_ERROR);
  RP_END_TRY 
  */

  geomObjPtr = (GeomObj*) Tcl_Alloc (sizeof (GeomObj)); 
  
  geomObjPtr->geom = NULL;
  
  sprintf (name, "etliGeom%d", id); 
  id++;
  
  Tcl_CreateObjCommand (interp, name, EtliGeomObjCmd, (ClientData) geomObjPtr, 
                                      EtliGeomObjDelete);
  
  resultObjPtr = Tcl_GetObjResult(interp);
  Tcl_SetStringObj (resultObjPtr, name, -1); 

  return (TCL_OK); 
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 EtliGeomObjCmd -- 
         Parse and process requests to geom object
         
 Results:        
         The return value is normally TCL_OK.
         TCL_ERROR can be returned if command parse error ocures
         or request return error condition.

 Side effecs:
         Depending on the request.
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        int
EtliGeomObjCmd (ClientData clientData, Tcl_Interp *interp, 
                int objc, Tcl_Obj *CONST objv[])
{
  int index;   
  Tcl_Obj *resultObjPtr;  
  char *subCmds[] = {"get", "put", "info", "delete", NULL};
  enum subCmdsIx {GetIx, PutIx, InfoIx, DeleteIx};  

  if (objc != 2)
    {
       Tcl_WrongNumArgs (interp, 1, objv, "option ?arg ...?"); 
       return (TCL_ERROR); 
    }

  if (Tcl_GetIndexFromObj (interp, objv[1], subCmds, "option", TCL_EXACT, 
                          &index) != TCL_OK)
    {
       return (TCL_ERROR);
    }
  switch (index)
    {
      case GetIx:
                {
                 resultObjPtr = Tcl_GetObjResult(interp);
                 Tcl_SetStringObj (resultObjPtr, "doing_get_command", -1); 
                 return (TCL_OK); 
                }
      default :          
                {
                 resultObjPtr = Tcl_GetObjResult(interp);
                 Tcl_SetStringObj (resultObjPtr, "doing_other_command", -1); 
                 return (TCL_OK); 
                }
            
    }
  return (TCL_OK); /* to avoid commpiler warning */
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 EtliGeomObjDelete --
         Delete geom object and associated Tcl command

 Results:        
         NONE
 
 Side effects:
         Free memory allocated for geom object.
         Delete Tcl command associated with geom object. 
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        void
EtliGeomObjDelete (ClientData geomObjPtr)
{
}

/* Modifications:
 * $Id: etliCmdGeom.c,v 1.1.1.1 2002/08/04 13:19:11 peteri Exp $
 */ 
