/********************************************************************
  
 ********************************************************************/

 /* $Id: etliCmdMaterial.c,v 1.1.1.1 2002/08/04 13:19:11 peteri Exp $ */


#include "etliInt.h"


/*
 * material object --
 */  

typedef struct
{
  MAT *material; 
  /*Tcl_HashTable *materialInfo; */
} MaterialObj; 


/* Prototypes for procedures referenced in this file only */
int EtliMaterialCmd (ClientData clientData, Tcl_Interp *interp,
             int objc, Tcl_Obj *CONST objv[]);
int EtliMaterialObjCmd (ClientData clientData, Tcl_Interp *interp, 
                int objc, Tcl_Obj *CONST objv[]);
void EtliMaterialObjDelete (ClientData materialObjPtr);


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 Etli_InitMaterialCmd -- 

 Results:
         TCL_OK

 Side effects:
         Create etli_material command. 
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        int        
Etli_InitMaterialCmd (Tcl_Interp *interp)
{

    Tcl_CreateObjCommand (interp, "etli_material", EtliMaterialCmd, 
                      (ClientData) NULL,  (Tcl_CmdDeleteProc *) NULL);

    return (TCL_OK); 
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 EtliMaterialCmd -- 
             Create material object and accociated Tcl command
 Results:           
         The results in normally TCL_OK.
         TCL_ERROR will be returned if
           1) wrong numbers of argument is supplied
           2) allocation of new material object failed
         In any case apropriate error message will be left
         in result object.
         
 Side effects:
         Allocate memory of new material object.
         Create new Tcl command for material object.
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        int
EtliMaterialCmd (ClientData clientData, Tcl_Interp *interp,
             int objc, Tcl_Obj *CONST objv[])
{
  char name[20];  
  static int id = 0; 

  Tcl_Obj *resultObjPtr;  
  MaterialObj *materialObjPtr;  

  if (objc != 1)
    {
      Tcl_WrongNumArgs (interp, 1, objv, "no options");
      return (TCL_ERROR); 
    }    

  /*
  RP_TRY
     RP_NEW(materialObjPtr);        
  RP_EXCEPT(RP_MemFailed)
      resultObjPtr = Tcl_GetObjResult(interp);
      Tcl_SetStringObj (resultObjPtr, name, -1); 
      RP_RETURN (TCL_ERROR);
  RP_END_TRY 
  */

  materialObjPtr = (MaterialObj *) Tcl_Alloc (sizeof (MaterialObj)); 
  
  materialObjPtr->material = NULL;
  
  sprintf (name, "etliMaterial%d", id); 
  id++;
  
  Tcl_CreateObjCommand (interp, name, EtliMaterialObjCmd, (ClientData) materialObjPtr, 
                                      EtliMaterialObjDelete);
  
  resultObjPtr = Tcl_GetObjResult(interp);
  Tcl_SetStringObj (resultObjPtr, name, -1); 

  return (TCL_OK); 
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 EtliMaterialObjCmd -- 
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        int
EtliMaterialObjCmd (ClientData clientData, Tcl_Interp *interp, 
                int objc, Tcl_Obj *CONST objv[])
{
  int index;   
  Tcl_Obj *resultObjPtr;  
  char *subCmds[] = {"get", "put", "info", "delete", NULL};
  enum subCmdsIx {GetIx, PutIx, InfoIx, DeleteIx};  

  if (objc != 2)
    {
       Tcl_WrongNumArgs (interp, 1, objv, "option ?arg ...?"); 
       return (TCL_ERROR); 
    }

  if (Tcl_GetIndexFromObj (interp, objv[1], subCmds, "option", TCL_EXACT, 
                          &index) != TCL_OK)
    {
       return (TCL_ERROR);
    }
  switch (index)
    {
      case GetIx:
                {
                 resultObjPtr = Tcl_GetObjResult(interp);
                 Tcl_SetStringObj (resultObjPtr, "doing_get_command", -1); 
                 return (TCL_OK); 
                }
      default :          
                {
                 resultObjPtr = Tcl_GetObjResult(interp);
                 Tcl_SetStringObj (resultObjPtr, "doing_other_command", -1); 
                 return (TCL_OK); 
                }
            
    }
  return (TCL_OK); /* to avoid commpiler warning */
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 EtliMaterialObjDelete --
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        void
EtliMaterialObjDelete (ClientData materialObjPtr)
{
}

/* Modifications:
 * $Id: etliCmdMaterial.c,v 1.1.1.1 2002/08/04 13:19:11 peteri Exp $
 */ 
