/********************************************************************
 etliCmdMeshGet.c --
      This file implements get subcommand of mesh objects.
      
 ******************************************************************** 
** Copyright (C) 1999 Roman Putanowicz <romek@sect.mce.hw.ac.uk>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, 
** Boston, MA 02111-1307, USA.
 ***********************************************************************
*/


 /* $Id: etliCmdMeshGet.c,v 1.1.1.1 2002/08/04 13:19:11 peteri Exp $ */

#include "etliInt.h"
#include "etliCmdMesh.h"
#include "mesh_supp.h"

/* Prototypes for procedures refernced in this file only */
static void InitMeshObj (MeshObj *meshObj);


static void GetMesh (ClientData  clientData, Tcl_Interp *interp, 
                 int objc, Tcl_Obj *CONST objv[], char *filename, 
                 int *result, Tcl_Obj **resultObjPtr);

static void GetMaterials (ClientData  clientData, Tcl_Interp *interp, 
                 int objc, Tcl_Obj *CONST objv[], char *filename, 
                 int *result, Tcl_Obj **resultObjPtr);

static void GetDecomp (ClientData  clientData, Tcl_Interp *interp, 
                 int objc, Tcl_Obj *CONST objv[], char *filename, 
                 int *result, Tcl_Obj **resultObjPtr);

static void GetStresses (ClientData  clientData, Tcl_Interp *interp, 
                 int objc, Tcl_Obj *CONST objv[], char *filename, 
                 int *result, Tcl_Obj **resultObjPtr);

static void GetMeshParams (ClientData  clientData, Tcl_Interp *interp, 
                 int objc, Tcl_Obj *CONST objv[], char *filename, 
                 int *result, Tcl_Obj **resultObjPtr);

static void GetFEErrors (ClientData  clientData, Tcl_Interp *interp, 
                 int objc, Tcl_Obj *CONST objv[], char *filename, 
                 int *result, Tcl_Obj **resultObjPtr);

static void GetGeom (ClientData  clientData, Tcl_Interp *interp, 
                 int objc, Tcl_Obj *CONST objv[], char *filename, 
                 int *result, Tcl_Obj **resultObjPtr);

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 EtliMeshObjGet 
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        CmdReturn*
EtliMeshObjGet (ClientData clientData, Tcl_Interp *interp, 
              int objc, Tcl_Obj *CONST objv[])
{
  int index;
  static char *subOpts[] = {"-mesh", "-materials", "-decomp", 
             "-meshparams", "-stresses", "-feerrors","-geom", NULL};
  enum subCmdIx {MeshIx, MaterialsIx, DecompIx, MeshparamsIx, 
                 StressesIx, FeerrorsIx, GeomIx};         

  int result = TCL_OK; 
  CmdReturn *returnStructPtr = NULL;
  Tcl_Obj *resultObjPtr = NULL;

  char *filename; /* name of input file (without extension!!!) */
  
  /* Allocate the space for the return structure */
  /* Use Tcl memory allocators to make possible monitor memory usage */ 
  returnStructPtr = (CmdReturn *) Tcl_Alloc (sizeof (CmdReturn));

  /* find option in the list of valid options for this subcommand */
  if (Tcl_GetIndexFromObj (interp, objv[2], subOpts, "option",
                          TCL_EXACT, &index) != TCL_OK)
    {
      result = TCL_ERROR;
      goto done;
    }
  /* get filename from command argument 
   * There is actualy no checking if the string is valid file name 
   * Because E-Lib function e_GetMeshData does exit on invalid file name
   * or invalid file contest a program will terminate here on invalid
   * file. It can be worked round but I'll beter wait for changing
   * E-lib library behaviour :-)
   */

  /* the string is mamaged by object manager and should not be modified or
   *  deleted 
   */  
   filename =  Tcl_GetString (objv[3]); 

   switch (index)
     {
       case MeshIx:
         {
           GetMesh (clientData, interp, objc, objv, filename,
                           &result, &resultObjPtr);
           break;
         } 
       case MaterialsIx:
         {
           GetMaterials (clientData, interp, objc, objv, filename,
                           &result, &resultObjPtr);
           break;
         }
       case DecompIx:
         {
           GetDecomp (clientData, interp, objc, objv, filename,
                           &result, &resultObjPtr);
           break;
         } 
       case StressesIx:
         {
           GetStresses (clientData, interp, objc, objv, filename,
                           &result, &resultObjPtr);
           break;
         }
       case MeshparamsIx:  
         {
           GetMeshParams (clientData, interp, objc, objv, filename,
                           &result, &resultObjPtr);
           break;
         } 
       case FeerrorsIx:
         {
           GetFEErrors (clientData, interp, objc, objv, filename,
                           &result, &resultObjPtr);
           break;
         } 
       case GeomIx:  
         {
           GetGeom (clientData, interp, objc, objv, filename,
                           &result, &resultObjPtr);
           break;
         } 
     }
  done:

  /* initialize return structure */
      returnStructPtr->object = resultObjPtr;
      returnStructPtr->status = result;

  return returnStructPtr;
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 InitMeshObj -- 
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        static void
InitMeshObj (MeshObj *meshObj)
{
  /* initialize mesh bounding box */
  RP_MeshPointsBBox (meshObj->mesh, meshObj->bBox, meshObj->bBox+3);   
  /* check if mesh is of one element type */
  if (e_IsOfOneElementType (meshObj->mesh) == 1)
    {
      /* take type of the first element */
      meshObj->elemType = RP_MeshElementType (meshObj->mesh, 1); 
    } 
  else
    {
      meshObj->elemType = ETLI_MIXED;
    }
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 GetMesh --
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        static void
GetMesh (ClientData  clientData, Tcl_Interp *interp, int objc, 
          Tcl_Obj *CONST objv[], char *filename, 
          int *result, Tcl_Obj **resultObjPtr)
{
  MeshObj *meshObjPtr = (MeshObj *) clientData;
  MESH *mesh = meshObjPtr->mesh; /* convinience variable */

  if (mesh == NULL)
    {     
     e_GetMeshData (&(meshObjPtr->mesh), filename); 
     /* Initialize other members of mesh object (bounding box, 
      * hash table, etc.  
      */
     InitMeshObj (meshObjPtr); 
     *resultObjPtr = Tcl_NewStringObj ("mesh data read in", -1); 
    }
  else
    {
     *resultObjPtr = Tcl_NewStringObj ("mesh already read in; do nothing", -1); 
     *result = TCL_ERROR;                
    }  
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 GetMaterials --
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        static void
GetMaterials (ClientData  clientData, Tcl_Interp *interp, int objc, 
          Tcl_Obj *CONST objv[], char *filename, 
          int *result, Tcl_Obj **resultObjPtr)
{
  MeshObj *meshObjPtr = (MeshObj *) clientData;
  MESH *mesh = meshObjPtr->mesh; /* convinience variable */

  if (mesh != NULL && mesh->SubStructSet[E_SS_NUM_MAT_PROPS] == FALSE)
    {     
     e_GetMaterials (mesh, filename); 
     *resultObjPtr = Tcl_NewStringObj ("materials data read in", -1); 
    }
  else if (mesh != NULL && mesh->SubStructSet[E_SS_NUM_MAT_PROPS] == TRUE)
    {
     *resultObjPtr = Tcl_NewStringObj 
                 ("materials data already read in; do nothing", -1); 
     *result = TCL_ERROR;                
    }  
  else if ( mesh == NULL )
    {
      *resultObjPtr = Tcl_NewStringObj 
         ("mesh not initialized; use \"get -mesh\" to initialize", -1);
      *result = TCL_ERROR;   
    }
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 GetDecomp --
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        static void
GetDecomp (ClientData  clientData, Tcl_Interp *interp, int objc, 
          Tcl_Obj *CONST objv[], char *filename, 
          int *result, Tcl_Obj **resultObjPtr)
{
  MeshObj *meshObjPtr = (MeshObj *) clientData;
  MESH *mesh = meshObjPtr->mesh; /* convinience variable */

  if (mesh != NULL && mesh->SubStructSet[E_SS_NUM_DECOMP] == FALSE)
    {     
     e_GetDecompData (mesh, filename); 
     *resultObjPtr = Tcl_NewStringObj ("decomposition data read in", -1); 
    }
  else if (mesh != NULL && mesh->SubStructSet[E_SS_NUM_MAT_PROPS] == TRUE)
    {
     *resultObjPtr = Tcl_NewStringObj 
                 ("decomposition data already read in; do nothing", -1); 
     *result = TCL_ERROR;                
    }  
  else if ( mesh == NULL )
    {
      *resultObjPtr = Tcl_NewStringObj 
         ("mesh not initialized; use \"get -mesh\" to initialize", -1);
      *result = TCL_ERROR;   
    }
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 GetStresses --
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        static void
GetStresses (ClientData  clientData, Tcl_Interp *interp, int objc, 
          Tcl_Obj *CONST objv[], char *filename, 
          int *result, Tcl_Obj **resultObjPtr)
{
  MeshObj *meshObjPtr = (MeshObj *) clientData;
  MESH *mesh = meshObjPtr->mesh; /* convinience variable */

  if (mesh != NULL && mesh->SubStructSet[E_SS_NUM_STRESS] == FALSE)
    {     
     e_GetStresses (mesh, filename); 
     *resultObjPtr = Tcl_NewStringObj ("stresses data read in", -1); 
    }
  else if (mesh != NULL && mesh->SubStructSet[E_SS_NUM_STRESS] == TRUE)
    {
     *resultObjPtr = Tcl_NewStringObj 
                 ("stresses data already read in; do nothing", -1); 
     *result = TCL_ERROR;                
    }  
  else if ( mesh == NULL )
    {
      *resultObjPtr = Tcl_NewStringObj 
         ("mesh not initialized; use \"get -mesh\" to initialize", -1);
      *result = TCL_ERROR;   
    }
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 GetMeshParams --
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        static void
GetMeshParams (ClientData  clientData, Tcl_Interp *interp, int objc, 
          Tcl_Obj *CONST objv[], char *filename, 
          int *result, Tcl_Obj **resultObjPtr)
{
  MeshObj *meshObjPtr = (MeshObj *) clientData;
  MESH *mesh = meshObjPtr->mesh; /* convinience variable */

  if (mesh != NULL && mesh->SubStructSet[E_SS_NUM_MESHPARAM] == FALSE)
    {     
     e_GetMeshParams (mesh, filename); 
     *resultObjPtr = Tcl_NewStringObj ("mesh parameters data read in", -1); 
    }
  else if (mesh != NULL && mesh->SubStructSet[E_SS_NUM_MESHPARAM] == TRUE)
    {
     *resultObjPtr = Tcl_NewStringObj 
                 ("mesh parameters data already read in; do nothing", -1); 
     *result = TCL_ERROR;                
    }  
  else if ( mesh == NULL )
    {
      *resultObjPtr = Tcl_NewStringObj 
         ("mesh not initialized; use \"get -mesh\" to initialize", -1);
      *result = TCL_ERROR;   
    }
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 GetFEErrors --
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        static void
GetFEErrors (ClientData  clientData, Tcl_Interp *interp, int objc, 
          Tcl_Obj *CONST objv[], char *filename, 
          int *result, Tcl_Obj **resultObjPtr)
{
  MeshObj *meshObjPtr = (MeshObj *) clientData;
  MESH *mesh = meshObjPtr->mesh; /* convinience variable */

  if (mesh != NULL && mesh->SubStructSet[E_SS_NUM_FEERROR] == FALSE)
    {     
     e_GetFEErrors (mesh, filename); 
     *resultObjPtr = Tcl_NewStringObj ("FE error data read in", -1); 
    }
  else if (mesh != NULL && mesh->SubStructSet[E_SS_NUM_FEERROR] == TRUE)
    {
     *resultObjPtr = Tcl_NewStringObj 
                 ("FE error data already read in; do nothing", -1); 
     *result = TCL_ERROR;                
    }  
  else if ( mesh == NULL )
    {
      *resultObjPtr = Tcl_NewStringObj 
         ("mesh not initialized; use \"get -mesh\" to initialize", -1);
      *result = TCL_ERROR;   
    }
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 GetGeom --
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        static void
GetGeom (ClientData  clientData, Tcl_Interp *interp, int objc, 
          Tcl_Obj *CONST objv[], char *filename, 
          int *result, Tcl_Obj **resultObjPtr)
{
  MeshObj *meshObjPtr = (MeshObj *) clientData;
  MESH *mesh = meshObjPtr->mesh; /* convinience variable */

  if (mesh != NULL && mesh->SubStructSet[E_SS_NUM_GEOM] == FALSE)
    {     
     e_GetGeomModel (mesh, filename); 
     *resultObjPtr = Tcl_NewStringObj ("geometry data read in", -1); 
    }
  else if (mesh != NULL && mesh->SubStructSet[E_SS_NUM_GEOM] == TRUE)
    {
     *resultObjPtr = Tcl_NewStringObj 
                 ("geometry data already read in; do nothing", -1); 
     *result = TCL_ERROR;                
    }  
  else if ( mesh == NULL )
    {
      *resultObjPtr = Tcl_NewStringObj 
         ("mesh not initialized; use \"get -mesh\" to initialize", -1);
      *result = TCL_ERROR;   
    }
}
/* Modifications:
 * $Id: etliCmdMeshGet.c,v 1.1.1.1 2002/08/04 13:19:11 peteri Exp $
 */ 
