/********************************************************************
 etliCmdMeshInfo.c --
      This file implements info subcommand for mesh objects. 
      
 ******************************************************************** 
** Copyright (C) 1999 Roman Putanowicz <romek@sect.mce.hw.ac.uk>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, 
** Boston, MA 02111-1307, USA.
 ***********************************************************************
*/


 /* $Id: etliCmdMeshInfo.c,v 1.1.1.1 2002/08/04 13:19:11 peteri Exp $ */

#include <assert.h>
#include "etliInt.h"
#include "etliCmdMesh.h" /* for declaration of mesh internal procedures */
#include "mesh_supp.h" /* for additional mesh support (eg RP_MeshElementType)*/

/* string array for error messages */
static char errorMessage[200];

/* Mesh info subcommand options */
static char *meshInfoSubOpts[] = {
                 /* 1 */      "-Title",
                 /* 2 */      "-NNodes", 
                 /* 3 */      "-NMeshPoints",
                 /* 4 */      "-NElemsLink1",
                 /* 5 */      "-NElemsLink2",
                 /* 6 */      "-NElemsLink3",
                 /* 7 */      "-NElemsLink4",
                 /* 8 */      "-NElemsLink5",
                 /* 9 */      "-NElemsTriang1",
                 /* 10 */     "-NElemsTriang2",
                 /* 12 */     "-NElemsTriang3",
                 /* 13 */     "-NElemsTriang4",
                 /* 14 */     "-NElemsTriang5",
                 /* 15 */     "-NElemsQuad1",
                 /* 16 */     "-NElemsQuad3",
                 /* 17 */     "-NElemsTetrah1",
                 /* 18 */     "-NElemsBlock1",
                 /* 19 */     "-TotalNElems",
                 /* 20 */     "-NBCNodes",
                 /* 21 */     "-NLoadedNodes",
                 /* 22 */     "-NMats",
                 /* 23 */     "-NMdfMats",
                 /* 24 */     "-NCompMatsType1",
                 /* 25 */     "-NTimeSteps",
                 /* 26 */     "-NInternalTimeSteps",
                 /* 27 */     "-NExternalTimeSteps",
                 /* 28 */     "-FirstNetIndex",
                 /* 29 */     "-TimeStep",
                 /* 30 */     "-DampingFactor",
                 /* 31 */     "-Beta",
                 /* 32 */     "-Gamma",
                 /* 33 */     "-NStressPointsLink1",         
                 /* 34 */     "-NStressPointsLink2",         
                 /* 35 */     "-NStressPointsLink3",         
                 /* 36 */     "-NStressPointsLink4",         
                 /* 37 */     "-NStressPointsLink5",         
                 /* 38 */     "-NStressPointsTriang1",       
                 /* 39 */     "-NStressPointsTriang2",       
                 /* 40 */     "-NStressPointsTriang3",       
                 /* 41 */     "-NStressPointsTriang4",       
                 /* 42 */     "-NStressPointsTriang5",       
                 /* 43 */     "-NStressPointsQuad1",         
                 /* 44 */     "-NStressPointsQuad3",         
                 /* 45 */     "-NStressPointsTetrah1",       
                 /* 46 */     "-NStressPointsBlock1",        
                 /* 47 */     "-MeshPointCoords",       
                 /* 48 */     "-NodesLink1",               
                 /* 49 */     "-NodesLink2",               
                 /* 50 */     "-NodesLink3",               
                 /* 52 */     "-NodesLink4",               
                 /* 53 */     "-NodesLink5",               
                 /* 54 */     "-NodesTriang1",             
                 /* 55 */     "-NodesTriang2",             
                 /* 56 */     "-NodesTriang3",             
                 /* 57 */     "-NodesTriang4",             
                 /* 58 */     "-NodesTriang5",             
                 /* 59 */     "-NodesQuad1",               
                 /* 60 */     "-NodesQuad3",               
                 /* 61 */     "-NodesTetrah1",             
                 /* 62 */     "-NodesBlock1",              
                 /* 63 */     "-TotalNodalDOF",              
                 /* 64 */     "-NodalDOF",                  
                 /* 65 */     "-BCNodes",                  
                 /* 66 */     "-BCTypes",                  
                 /* 67 */     "-BCDispls",              
                 /* 68 */     "-BCSpringConsts",        
                 /* 69 */     "-LoadedNodes",               
                 /* 70 */     "-Loads",                 
//               /* 71 */     "-Mats",                    
//               /* 72 */     "-MatsLink1",               
//               /* 73 */     "-MatsLink2",               
//               /* 74 */     "-MatsLink3",               
//               /* 75 */     "-MatsLink4",               
//               /* 76 */     "-MatsLink5",               
//               /* 77 */     "-MatsTriang1",             
//               /* 78 */     "-CompMatsTriang2",     
//               /* 79 */     "-MatsTriang3",             
//               /* 80 */     "-MatsTriang4",             
//               /* 81 */     "-MatsTriang5",             
//               /* 82 */     "-MatsQuad1",               
//               /* 83 */     "-CompMatsQuad3",       
//               /* 84 */     "-MatsTetrah1",             
//               /* 85 */     "-MatsBlock1",              
//               /* 86 */     "-NodeTypes",                 
                 /* 87 */     "-NNurbsCurves",               
//               /* 88 */     "-NurbsCurveParams",      
                 /* 89 */     "-NurbsCurves",      
//               /* 90 */     "-GNIs",                      
//               /* 92 */     "-GEIs",                      
//               /* 93 */     "-GEIsLink1",                 
//               /* 94 */     "-GEIsLink2",                 
//               /* 95 */     "-GEIsLink3",                 
//               /* 96 */     "-GEIsLink4",                 
//               /* 97 */     "-GEIsLink5",                 
//               /* 98 */     "-GEIsTriang1",               
//               /* 99 */     "-GEIsTriang2",               
//               /* 100 */    "-GEIsTriang3",               
//               /* 101 */    "-GEIsTriang4",               
//               /* 102 */    "-GEIsTriang5",               
//               /* 103 */    "-GEIsQuad1",                 
//               /* 104 */    "-GEIsQuad3",                 
//               /* 105 */    "-GEIsTetrah1",               
//               /* 106 */    "-GEIsBlock1",                
//               /* 107 */    "-Geom",                      
//               /* 108 */    "-Props",                     
//               /* 119 */    "-CompPropsType1",       
//               /* 110 */    "-Decomp",                  
//               /* 111 */    "-MeshParam",            
//               /* 112 */    "-FEError",                
//               /* 113 */    "-Stresses",              
//               /* 114 */    "-KeywordSet",    
//               /* 115 */    "-SubStructSet",
   /* this are additional mesh poroperties which does not correspond 
    *  directly to members of MESH structure.
    */            
                /* 116 */      "-BBox",
                /* 117 */      "-element", 
                /* 118 */      "-IsOfOneElementType",
                /* 119 */      "-LinkElementsOnly",
                /* 120 */      "-PlanarElementsOnly",        
                /* 121 */      "-NSubDoms",
                /* 122 */      "-SubDoms",        
                /* 123 */      "-SubDomsLink1",               
                /* 124 */      "-SubDomsLink2",               
                /* 125 */      "-SubDomsLink3",               
                /* 126 */      "-SubDomsLink4",               
                /* 127 */      "-SubDomsLink5",               
                /* 128 */      "-SubDomsTriang1",             
                /* 129 */      "-SubDomsTriang2",             
                /* 130 */      "-SubDomsTriang3",             
                /* 131 */      "-SubDomsTriang4",             
                /* 132 */      "-SubDomsTriang5",             
                /* 133 */      "-SubDomsQuad1",               
                /* 134 */      "-SubDomsQuad3",               
                /* 135 */      "-SubDomsTetrah1",
                /* 136 */      "-SubDomsBlock1",
                           NULL};
                 

/* nurbs utility from Jelle Muylle */
extern void crv_getcoords   (NURBSCURVE *curve,double t,double *crds);

static void NurbsInfoAll (ClientData clientData, Tcl_Interp *interp, int objc, 
              Tcl_Obj *CONST objv[], int *result, Tcl_Obj **resultObjPtr);

static void NurbsInfoControlPoint (ClientData clientData, Tcl_Interp *interp, 
              int objc, Tcl_Obj *CONST objv[], int *result,
              Tcl_Obj **resultObjPtr);

static void NurbsInfoCoords (ClientData clientData, Tcl_Interp *interp, int objc, 
              Tcl_Obj *CONST objv[], int *result, Tcl_Obj **resultObjPtr);


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 InfoTitle --     
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        static void 
InfoTitle (ClientData clientData, Tcl_Interp *interp, 
               int objc, Tcl_Obj *CONST objv[], int *result,
               Tcl_Obj **resultObjPtr)
{
  MeshObj *meshObjPtr = (MeshObj *) clientData; 

  *resultObjPtr = Tcl_NewStringObj (meshObjPtr->mesh->Title, -1); 
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 InfoFirstNetIndex --
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        static void
InfoFirstNetIndex (ClientData clientData, Tcl_Interp *interp, 
               int objc, Tcl_Obj *CONST objv[], int *result,
               Tcl_Obj **resultObjPtr)
{
  char error[80];
  int elemType;

  MeshObj *meshObjPtr = (MeshObj *) clientData; 
  MESH *mesh = meshObjPtr->mesh; /* convinience variable */

  /* check if valid number of arguments is supplied for -FirstNetIndex */
  if (objc != 4)
    {
      Tcl_WrongNumArgs (interp, 1, objv, "info -FirstNetIndex elementtype");
      *result = TCL_ERROR;
      return; 
    }
 /* get element type form argument list */
  if (Tcl_GetIntFromObj (interp, objv[3], &elemType) != TCL_OK)
    {
      *result = TCL_ERROR;
      return;
    }
 /* check if element type is a valid type
  * (as specified in e_lib.h)   
  */
 if (elemType < 0 || elemType >= E_NELEM_TYPES)
   {
     sprintf (error, "wrong element type %d : expected integer 0-%d", 
                   elemType,  E_NELEM_TYPES-1); 
     *resultObjPtr = Tcl_NewStringObj (error, -1);
     *result = TCL_ERROR; 
     return;
   } 

 /* everytking is OK, return FirstNetIndex */
 *resultObjPtr = Tcl_NewIntObj (mesh->FirstNetIndex[elemType]);
 return; 
}    

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 InfoNodalDOF --
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        static void
InfoNodalDOF (ClientData clientData, Tcl_Interp *interp, 
               int objc, Tcl_Obj *CONST objv[], int *result,
               Tcl_Obj **resultObjPtr)
{
  int nodeNumber;
  char error[80];  

  MeshObj *meshObjPtr = (MeshObj *) clientData; 
  MESH *mesh = meshObjPtr->mesh; /* convinience variable */
  
 /* check if valid number of arguments is supplied for -NodalDOF */ 
  if (objc != 4)
    {
      Tcl_WrongNumArgs (interp, 1, objv, "info -NodalDOF nodenumber"); 
      *result = TCL_ERROR;
      return;       
    }

  /* get the node number from arguments list */
  if (Tcl_GetIntFromObj (interp, objv[3], &nodeNumber) != TCL_OK)
    {
      *result = TCL_ERROR;
      return;
    }

  /* check if node number is valid */
  if (nodeNumber<1 || nodeNumber>mesh->NNodes)
    {
      sprintf (error, "wrong node number %d : expected integer 1-%d", 
                    nodeNumber,  mesh->NNodes); 
      *resultObjPtr = Tcl_NewStringObj (error, -1);
      *result = TCL_ERROR; 
      return;
    }

  /* everything is OK, can retutn number of dofs */

  nodeNumber--; /* nodes are numbered from 1 but this is C */
 
  *resultObjPtr = Tcl_NewIntObj (mesh->NodalDOF[nodeNumber]);
  return; 
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 InfoBCNodes --
          Return node index for boundary condition node nuber.
 Syntax:      
        etliNurbs  info -BCNodes number
        
 Description:       
       'number' = 1,...,NBCNodes
 Result:
        Return node index where boundary condition of 'number' is set
        or error message if 'number' is invalid.
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        static void
InfoBCNodes (ClientData clientData, Tcl_Interp *interp, 
               int objc, Tcl_Obj *CONST objv[], int *result,
               Tcl_Obj **resultObjPtr)
{
  int bcNodeNumber;
  char error[80];  

  MeshObj *meshObjPtr = (MeshObj *) clientData; 
  MESH *mesh = meshObjPtr->mesh; /* convinience variable */
  
 /* check if valid number of arguments is supplied for -BCNodes */ 
  if (objc != 4)
    {
      Tcl_WrongNumArgs (interp, 1, objv, "info -BCNodes number"); 
      *result = TCL_ERROR;
      return;       
    }

  /* get the boundary condition node number from arguments list */
  if (Tcl_GetIntFromObj (interp, objv[3], &bcNodeNumber) != TCL_OK)
    {
      *result = TCL_ERROR;
      return;
    }

  /* check if boundary condition node number is valid */
  if (bcNodeNumber<1 || bcNodeNumber>mesh->NBCNodes)
    {
      sprintf (error, "wrong number %d : expected integer 1-%d", 
                    bcNodeNumber,  mesh->NBCNodes); 
      *resultObjPtr = Tcl_NewStringObj (error, -1);
      *result = TCL_ERROR; 
      return;
    }

  /* everything is OK, can return index of node */

  bcNodeNumber--; /* BC nodes are numbered from 1 but this is C */
 
  *resultObjPtr = Tcl_NewIntObj (mesh->BCNodes[bcNodeNumber]);
  return;    
}


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 InfoBCTypes --
       Return boundary type for boundary condition of 'number'.
 Syntax:      
     etliNurbs info -BCTypes number ?-dof dofnumber?

 Description:       
      Return boundary condition type for all degrees of freedom
      or if number of degree of freedom is suplied only for that degree
      of freedom.      
       'number' = 1,...,NBCNodes  
       'dofnumber' = 1,...,L  where L is number of degrees of freedom 
                              at the node.
 Result:
        Return boundary condition type for boundary of 'number' 
        or error message if 'number' is invalid.
 Notes:
       Maximal number of dofs (which is 5 for QUAD3) is hardcoded
       in this function. When adding new element type this must be checked
       and updated when necessary. 
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        static void
InfoBCTypes (ClientData clientData, Tcl_Interp *interp, 
               int objc, Tcl_Obj *CONST objv[], int *result,
               Tcl_Obj **resultObjPtr)
{
  int bcNodeNumber;
  int node, numberOfDofs; /* node number and number of DOFs at this node */
  char error[80];  

  static char *subOpts[] = {"-dof", NULL};
  enum subOptsIx   {AllIx, DofIx}; 
  int index;

  MeshObj *meshObjPtr = (MeshObj *) clientData; 
  MESH *mesh = meshObjPtr->mesh; /* convinience variable */
  
 /* check if valid number of arguments is supplied for -BCTypes */ 
  if (objc != 4 && objc != 6)
    {
      Tcl_WrongNumArgs (interp, 1, objv,
                      "info -BCTypes number ?-dof dofnumber?"); 
      *result = TCL_ERROR;
      return;       
    }

  /* get the boundary condition node number from arguments list */
  if (Tcl_GetIntFromObj (interp, objv[3], &bcNodeNumber) != TCL_OK)
    {
      *result = TCL_ERROR;
      return;
    }

  /* check if boundary condition node number is valid */
  if (bcNodeNumber<1 || bcNodeNumber>mesh->NBCNodes)
    {
      sprintf (error, "wrong number %d : expected integer 1-%d", 
                    bcNodeNumber,  mesh->NBCNodes); 
      *resultObjPtr = Tcl_NewStringObj (error, -1);
      *result = TCL_ERROR; 
      return;
    }

   bcNodeNumber--; /* BC nodes are numbered from 1 but this is C */

   node = mesh->BCNodes[bcNodeNumber]; /* get node number of BC node */
   numberOfDofs = mesh->NodalDOF[node-1]; /* get number of dofs for node*/

  /* switch either to case where we have to return list of dof types
   *  or only doftype for particular dof
   */ 

  if (objc == 4) /* returning list of dof types */
    {
      /* max number of dofs is 5 for nodes of QUAD3 */
      /* This must be changed if new element type is added */
      static int maxDofs = 5;
      Tcl_Obj *listPtrPtr[maxDofs]; /* for holding dofs type */
      int i;
      for (i=0; i<numberOfDofs; i++)
        {
          listPtrPtr[i] = Tcl_NewIntObj (*(mesh->BCTypes[bcNodeNumber]+i));
        }
      *resultObjPtr = Tcl_NewListObj (numberOfDofs, listPtrPtr);
      return;
    }
  else /* returning only type for given dof */
    {
      int dofNumber;
      /* check if proper suboption is suplied */
       if (Tcl_GetIndexFromObj (interp, objv[4], subOpts, "suboption", 
                              TCL_EXACT, &index) != TCL_OK)
        {
          *result = TCL_ERROR;
          return;
        }
      /* get dof number */ 
       if (Tcl_GetIntFromObj (interp, objv[5], &dofNumber) != TCL_OK)
         {
           *result = TCL_ERROR;
         }
      /* check if dof number is a valid one */
       if (dofNumber<1 || dofNumber>numberOfDofs)
         {
           sprintf (error, "wrong DOF number %d : expected integer 1-%d", 
                    dofNumber,  numberOfDofs); 
           *resultObjPtr = Tcl_NewStringObj (error, -1);
           *result = TCL_ERROR; 
           return;
         }
       dofNumber--; /* Dof are numbered from 1 but this is C */
      /* everything is ok, can return dof type */ 
       *resultObjPtr = Tcl_NewIntObj (*(mesh->BCTypes[bcNodeNumber]+dofNumber));
       return;
    }
}


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 InfoBCDispls --
         Return initial nodal displacement for displacement boundary
         condition type
 Syntax:
      meshObj info -BCDispls bc_number -dof dof_number

 Description:
      Return value of initial displacement for displacement boundary
      condition of given dof at particular boundary condition.
      If specified degree of freedom is not subject to displacement type
      boundary condion  return an error message.
 Side effects:
      None.
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        static void
InfoBCDispls (ClientData clientData, Tcl_Interp *interp, 
               int objc, Tcl_Obj *CONST objv[], int *result,
               Tcl_Obj **resultObjPtr)
{
  int bcNumber;
  int dofNumber;
  int node, numberOfDofs; /* node number and number of DOFs at this node */
  char error[80];  

  static char *subOpts[] = {"-dof", NULL};
  enum subOptsIx   {DofIx}; 
  int index;

  MeshObj *meshObjPtr = (MeshObj *) clientData; 
  MESH *mesh = meshObjPtr->mesh; /* convinience variable */
  
 /* check if valid number of arguments is supplied for -BCTypes */ 
  if (objc != 6)
    {
      Tcl_WrongNumArgs (interp, 1, objv,
                      "info -BCDispls number -dof dofnumber"); 
      *result = TCL_ERROR;
      return;       
    }

  /* get the boundary condition number from arguments list */
  if (Tcl_GetIntFromObj (interp, objv[3], &bcNumber) != TCL_OK)
    {
      *result = TCL_ERROR;
      return;
    }

  /* check if boundary condition number is valid */
  if (bcNumber<1 || bcNumber>mesh->NBCNodes)
    {
      sprintf (error, "wrong number %d : expected integer 1-%d", 
                    bcNumber,  mesh->NBCNodes); 
      *resultObjPtr = Tcl_NewStringObj (error, -1);
      *result = TCL_ERROR; 
      return;
    }

   /* check if proper suboption is suplied */
   if (Tcl_GetIndexFromObj (interp, objv[4], subOpts, "suboption", 
                         TCL_EXACT, &index) != TCL_OK)
     {
       *result = TCL_ERROR;
       return;
     }
   /* get dof number */ 
   if (Tcl_GetIntFromObj (interp, objv[5], &dofNumber) != TCL_OK)
     {
       *result = TCL_ERROR;
     }

   bcNumber--; /* boundary conditions are numbered from 1 but this is C */
   node = mesh->BCNodes[bcNumber]; /* get node number of BC node */
   numberOfDofs = mesh->NodalDOF[node-1]; /* get number of dofs for node*/

   /* check if dof number is a valid one */
   if (dofNumber<1 || dofNumber>numberOfDofs)
     {
       sprintf (error, "wrong DOF number %d : expected integer 1-%d", 
                 dofNumber,  numberOfDofs); 
       *resultObjPtr = Tcl_NewStringObj (error, -1);
       *result = TCL_ERROR; 
       return;
     }
   dofNumber--; /* Dof are numbered from 1 but this is C */

   /* check if given node and dof is subject to displacement condition */
   if ((*(mesh->BCTypes[bcNumber]+dofNumber)) != E_BC_NUM_FIXED)
     {
       sprintf (error, "Boundary condition is not of displacement type");
       *resultObjPtr = Tcl_NewStringObj (error, -1);
       *result = TCL_ERROR; 
       return;
     }

   /* everything is ok, can return initial displacement */ 
   *resultObjPtr = Tcl_NewIntObj (*(mesh->BCDispls[bcNumber]+dofNumber));
   return;
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 InfoBCSpringConsts --
         Return spring constant for spring oundary condition type.
 Syntax:
      meshObj info -BCSpringConsts bc_node_number -dof dof_number

 Description:
      Return value of spring constant for spring boundary
      condition of given dof at particular boundary condition node.
      If specified degree of freedom is not subject to spring 
      boundary condion return an error message.
 Side effects:
      None.
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        static void
InfoBCSpringConsts (ClientData clientData, Tcl_Interp *interp, 
               int objc, Tcl_Obj *CONST objv[], int *result,
               Tcl_Obj **resultObjPtr)
{
  int bcNodeNumber;
  int dofNumber;
  int node, numberOfDofs; /* node number and number of DOFs at this node */
  char error[80];  

  static char *subOpts[] = {"-dof", NULL};
  enum subOptsIx   {DofIx}; 
  int index;

  MeshObj *meshObjPtr = (MeshObj *) clientData; 
  MESH *mesh = meshObjPtr->mesh; /* convinience variable */
  
 /* check if valid number of arguments is supplied for -BCSpringConstants */ 
  if (objc != 6)
    {
      Tcl_WrongNumArgs (interp, 1, objv,
                      "info -BCSpringConstants number -dof dofnumber"); 
      *result = TCL_ERROR;
      return;       
    }

  /* get the boundary condition node number from arguments list */
  if (Tcl_GetIntFromObj (interp, objv[3], &bcNodeNumber) != TCL_OK)
    {
      *result = TCL_ERROR;
      return;
    }

  /* check if boundary condition node number is valid */
  if (bcNodeNumber<1 || bcNodeNumber>mesh->NBCNodes)
    {
      sprintf (error, "wrong number %d : expected integer 1-%d", 
                    bcNodeNumber,  mesh->NBCNodes); 
      *resultObjPtr = Tcl_NewStringObj (error, -1);
      *result = TCL_ERROR; 
      return;
    }

   /* check if proper suboption is suplied */
   if (Tcl_GetIndexFromObj (interp, objv[4], subOpts, "suboption", 
                         TCL_EXACT, &index) != TCL_OK)
     {
       *result = TCL_ERROR;
       return;
     }
   /* get dof number */ 
   if (Tcl_GetIntFromObj (interp, objv[5], &dofNumber) != TCL_OK)
     {
       *result = TCL_ERROR;
     }

   bcNodeNumber--; /* BC nodes are numbered from 1 but this is C */
   node = mesh->BCNodes[bcNodeNumber]; /* get node number of BC node */
   numberOfDofs = mesh->NodalDOF[node-1]; /* get number of dofs for node*/

   /* check if dof number is a valid one */
   if (dofNumber<1 || dofNumber>numberOfDofs)
     {
       sprintf (error, "wrong DOF number %d : expected integer 1-%d", 
                 dofNumber,  numberOfDofs); 
       *resultObjPtr = Tcl_NewStringObj (error, -1);
       *result = TCL_ERROR; 
       return;
     }
   dofNumber--; /* Dof are numbered from 1 but this is C */

   /* check if given node and dof is subject to spring condition */
   if ((*(mesh->BCTypes[bcNodeNumber]+dofNumber)) != E_BC_NUM_SPRING)
     {
       sprintf (error, "Boundary condition is not of spring type");
       *resultObjPtr = Tcl_NewStringObj (error, -1);
       *result = TCL_ERROR; 
       return;
     }
    
   /* everything is ok, can return spring constant */ 
   *resultObjPtr = Tcl_NewIntObj
           (*(mesh->BCSpringConsts[bcNodeNumber]+dofNumber));
   return;
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 InfoLoadedNodes --
          Return node index for loaded node of given number. 
 Syntax:      
        etliNurbs  info -LoadedNodes number
        
 Description:       
       'number' = 1,...,NLoadedNodes
 Result:
        Return node index where load of 'number' is applied 
        or error message if 'number' is invalid.
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        static void
InfoLoadedNodes (ClientData clientData, Tcl_Interp *interp, 
               int objc, Tcl_Obj *CONST objv[], int *result,
               Tcl_Obj **resultObjPtr)
{
  int loadNumber;
  char error[80];  

  MeshObj *meshObjPtr = (MeshObj *) clientData; 
  MESH *mesh = meshObjPtr->mesh; /* convinience variable */
  
 /* check if valid number of arguments is supplied for -LoadedNodes */ 
  if (objc != 4)
    {
      Tcl_WrongNumArgs (interp, 1, objv, "info -LoadedNodes number"); 
      *result = TCL_ERROR;
      return;       
    }

  /* get the load number from arguments list */
  if (Tcl_GetIntFromObj (interp, objv[3], &loadNumber) != TCL_OK)
    {
      *result = TCL_ERROR;
      return;
    }

  /* check if load number is valid */
  if (loadNumber<1 || loadNumber>mesh->NLoadedNodes)
    {
      sprintf (error, "wrong number %d : expected integer 1-%d", 
                    loadNumber,  mesh->NLoadedNodes); 
      *resultObjPtr = Tcl_NewStringObj (error, -1);
      *result = TCL_ERROR; 
      return;
    }

  /* everything is OK, can return index of node */

  loadNumber--; /* loads are numbered from 1 but this is C */
 
  *resultObjPtr = Tcl_NewIntObj (mesh->LoadedNodes[loadNumber]);
  return;    
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 InfoLoads --
          Return list of node loads for given load number. 
 Syntax:      
        etliNurbs  info -Loads number
        
 Description:       
       'number' = 1,...,NLoadedNodes
 Result:
        Return list of nodal loads for  load of 'number'  
        or error message if 'number' is invalid.
        
 Notes:
       Maximal number of dofs (which is 5 for QUAD3) is hardcoded
       in this function. When adding new element type this must be checked
       and updated when necessary. 
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        static void
InfoLoads (ClientData clientData, Tcl_Interp *interp, 
               int objc, Tcl_Obj *CONST objv[], int *result,
               Tcl_Obj **resultObjPtr)
{
  int loadNumber;
  char error[80];  

  /* max number of dofs is 5 for nodes of QUAD3 */
  /* This must be changed if new element type is added */
  static int maxDofs = 5;
  Tcl_Obj *listPtrPtr[maxDofs]; /* for holding dofs type */
  int i, node, numberOfDofs;

  MeshObj *meshObjPtr = (MeshObj *) clientData; 
  MESH *mesh = meshObjPtr->mesh; /* convinience variable */
  
 /* check if valid number of arguments is supplied for -Loads */ 
  if (objc != 4)
    {
      Tcl_WrongNumArgs (interp, 1, objv, "info -Loads number"); 
      *result = TCL_ERROR;
      return;       
    }

  /* get the load number from arguments list */
  if (Tcl_GetIntFromObj (interp, objv[3], &loadNumber) != TCL_OK)
    {
      *result = TCL_ERROR;
      return;
    }

  /* check if load number is valid */
  if (loadNumber<1 || loadNumber>mesh->NLoadedNodes)
    {
      sprintf (error, "wrong number %d : expected integer 1-%d", 
                    loadNumber,  mesh->NLoadedNodes); 
      *resultObjPtr = Tcl_NewStringObj (error, -1);
      *result = TCL_ERROR; 
      return;
    }

  /* everything is OK; can fo further */

  /*get the node index for load and number of degrees of freedom in that node*/
  loadNumber--; /* loads are numbered from 1 but this is C */
 
  node = mesh->LoadedNodes[loadNumber]; /* get node number of BC node */
  numberOfDofs = mesh->NodalDOF[node-1]; /* get number of dofs for node*/
   
  /* make list of nodal loads */
  for (i=0; i<numberOfDofs; i++)
    {
      listPtrPtr[i] = Tcl_NewDoubleObj (*(mesh->Loads[loadNumber]+i));
    }

  *resultObjPtr = Tcl_NewListObj (numberOfDofs, listPtrPtr);
  return;
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 InfoMeshPointCoords --
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        static void
InfoMeshPointCoords (ClientData clientData, Tcl_Interp *interp, 
               int objc, Tcl_Obj *CONST objv[], int *result,
               Tcl_Obj **resultObjPtr)
{
  int pointNumber;
  char error[80];
  Tcl_Obj *listPtrPtr[3]; /* for holding coords */
  int index=0; 
  static char *subOpts[] = {"-xyz", "-xy", "-x", "-y", "-z", NULL};
  enum subOptsIx {XYZIx, XYIx, XIx, YIx, ZIx};  

  MeshObj *meshObjPtr = (MeshObj *) clientData; 
  MESH *mesh = meshObjPtr->mesh; /* convinience variable */

  /* check if valid number of arguments is supplied for -MeshPointCoords */ 
  if (objc <4 || objc >5)
    {
      Tcl_WrongNumArgs (interp, 1, objv, 
                      "info -MeshPointCoords number ?option?"); 
      *result = TCL_ERROR;
      return;       
    }

  /* get the meshpoint number from arguments list */
  if (Tcl_GetIntFromObj (interp, objv[3], &pointNumber) != TCL_OK)
    {
      *result = TCL_ERROR;
      return;
    }
  /* check if mesh point number is valid */
  if (pointNumber<1 || pointNumber>mesh->NMeshPoints)
    {
      sprintf (error, "wrong point number %d : expected integer 1-%d", 
                    pointNumber,  mesh->NMeshPoints); 
      *resultObjPtr = Tcl_NewStringObj (error, -1);
      *result = TCL_ERROR; 
      return;
    }

  pointNumber--; /* mesh points are numbered from 1 but this is C */
  
  /* check if coord suboption is suplied and find option in the
   * list of valid suboptions 
   */
  index = 0; /* this is when no suboption is supplied */
  if (objc == 5) 
    {
      if (Tcl_GetIndexFromObj (interp, objv[4], subOpts, "suboption", 
                              TCL_EXACT, &index) != TCL_OK)
        {
          *result = TCL_ERROR;
          return;
        }
    }

  switch (index)
    {
      case XYZIx:
        {
          listPtrPtr[0] = 
             Tcl_NewDoubleObj (*(mesh->MeshPointCoords[pointNumber]+0)); 
          listPtrPtr[1] = 
             Tcl_NewDoubleObj (*(mesh->MeshPointCoords[pointNumber]+1)); 
          listPtrPtr[2] = 
             Tcl_NewDoubleObj (*(mesh->MeshPointCoords[pointNumber]+2)); 
          *resultObjPtr = Tcl_NewListObj(3, listPtrPtr); 
          return;
        }
      case XYIx:
        {
          listPtrPtr[0] = 
             Tcl_NewDoubleObj (*(mesh->MeshPointCoords[pointNumber]+0)); 
          listPtrPtr[1] = 
             Tcl_NewDoubleObj (*(mesh->MeshPointCoords[pointNumber]+1)); 
          *resultObjPtr = Tcl_NewListObj(2, listPtrPtr); 
          return;
        }
      case XIx: /* return only X coordinate */
        {
          listPtrPtr[0] = 
             Tcl_NewDoubleObj (*(mesh->MeshPointCoords[pointNumber]+0)); 
          *resultObjPtr = Tcl_NewListObj(1, listPtrPtr); 
          return;
        }  
      case YIx: /* return only Y coordinate */
        {
          listPtrPtr[0] = 
             Tcl_NewDoubleObj (*(mesh->MeshPointCoords[pointNumber]+1)); 
          *resultObjPtr = Tcl_NewListObj(1, listPtrPtr); 
          return;
        }
      case ZIx: /* retutn only Z coordinate */
        {
          listPtrPtr[0] = 
             Tcl_NewDoubleObj (*(mesh->MeshPointCoords[pointNumber]+2)); 
          *resultObjPtr = Tcl_NewListObj(1, listPtrPtr); 
          return;
        }
    } /* end of switch(index) */ 
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 InfoIsOfOneElementType --
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        static void
InfoIsOfOneElementType (ClientData clientData, Tcl_Interp * intetp,
        int objc, Tcl_Obj *CONST objv[], int *result,        
        Tcl_Obj **resultObjPtr)
{
  MeshObj *meshObjPtr = (MeshObj *) clientData; 
  *resultObjPtr = Tcl_NewIntObj (e_IsOfOneElementType (meshObjPtr->mesh));
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 InfoLinkElementsOnly --
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        static void
InfoLinkElementsOnly (ClientData clientData, Tcl_Interp * intetp,
        int objc, Tcl_Obj *CONST objv[], int *result,        
        Tcl_Obj **resultObjPtr)
{
  MeshObj *meshObjPtr = (MeshObj *) clientData; 
  *resultObjPtr = Tcl_NewIntObj (e_LinkElementsOnly (meshObjPtr->mesh));
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 InfoPlanarElementsOnly --
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        static void
InfoPlanarElementsOnly (ClientData clientData, Tcl_Interp * intetp,
        int objc, Tcl_Obj *CONST objv[], int *result,        
        Tcl_Obj **resultObjPtr)
{
  MeshObj *meshObjPtr = (MeshObj *) clientData; 
  *resultObjPtr = Tcl_NewIntObj (e_PlanarElementsOnly (meshObjPtr->mesh));
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 Macro for generation Info* procedures which acces int or 
 double mesh members 
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define INFO_PROC_SIMPLE(MEMBER, TYPE) \
         static void \
Info##MEMBER (ClientData clientData, Tcl_Interp *interp, \
               int objc, Tcl_Obj *CONST objv[], int *result, \
               Tcl_Obj **resultObjPtr) \
{\
  MeshObj *meshObjPtr = (MeshObj *) clientData; \
  *resultObjPtr = Tcl_New##TYPE##Obj (meshObjPtr->mesh->##MEMBER); \
}
 
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 Generation of Info procedures 
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
 INFO_PROC_SIMPLE(NNodes, Int);
 INFO_PROC_SIMPLE(NMeshPoints, Int);
 INFO_PROC_SIMPLE(NElemsLink1, Int);
 INFO_PROC_SIMPLE(NElemsLink2, Int);
 INFO_PROC_SIMPLE(NElemsLink3, Int);
 INFO_PROC_SIMPLE(NElemsLink4, Int);
 INFO_PROC_SIMPLE(NElemsLink5, Int);
 INFO_PROC_SIMPLE(NElemsTriang1, Int);
 INFO_PROC_SIMPLE(NElemsTriang2, Int);
 INFO_PROC_SIMPLE(NElemsTriang3, Int);
 INFO_PROC_SIMPLE(NElemsTriang4, Int);
 INFO_PROC_SIMPLE(NElemsTriang5, Int);
 INFO_PROC_SIMPLE(NElemsQuad1, Int);
 INFO_PROC_SIMPLE(NElemsQuad3, Int);
 INFO_PROC_SIMPLE(NElemsTetrah1, Int);
 INFO_PROC_SIMPLE(NElemsBlock1, Int);
 INFO_PROC_SIMPLE(TotalNElems, Int);
 INFO_PROC_SIMPLE(NBCNodes, Int);
 INFO_PROC_SIMPLE(NLoadedNodes, Int);
 INFO_PROC_SIMPLE(NMats, Int);
 INFO_PROC_SIMPLE(NMdfMats, Int);
 INFO_PROC_SIMPLE(NCompMatsType1, Int);
 INFO_PROC_SIMPLE(NTimeSteps, Int);
 INFO_PROC_SIMPLE(NInternalTimeSteps, Int);
 INFO_PROC_SIMPLE(NExternalTimeSteps, Int);
 
 INFO_PROC_SIMPLE(TimeStep, Double);
 INFO_PROC_SIMPLE(DampingFactor, Double);
 INFO_PROC_SIMPLE(Beta, Double);
 INFO_PROC_SIMPLE(Gamma, Double);
         
 INFO_PROC_SIMPLE(NStressPointsLink1, Int);
 INFO_PROC_SIMPLE(NStressPointsLink2, Int);
 INFO_PROC_SIMPLE(NStressPointsLink3, Int);
 INFO_PROC_SIMPLE(NStressPointsLink4, Int);
 INFO_PROC_SIMPLE(NStressPointsLink5, Int);
 INFO_PROC_SIMPLE(NStressPointsTriang1, Int);
 INFO_PROC_SIMPLE(NStressPointsTriang2, Int);
 INFO_PROC_SIMPLE(NStressPointsTriang3, Int);
 INFO_PROC_SIMPLE(NStressPointsTriang4, Int);
 INFO_PROC_SIMPLE(NStressPointsTriang5, Int);
 INFO_PROC_SIMPLE(NStressPointsQuad1, Int);
 INFO_PROC_SIMPLE(NStressPointsQuad3, Int);
 INFO_PROC_SIMPLE(NStressPointsTetrah1, Int);
 INFO_PROC_SIMPLE(NStressPointsBlock1, Int);
 INFO_PROC_SIMPLE(TotalNodalDOF, Int);
#undef INFO_PROC_SIMPLE 
        
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 Macro for generation Info* procedures which return element nodes list
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define INFO_PROC_NODES(TYPE1, TYPE2, TYPENAME) \
        static void \
InfoNodes##TYPE2 (ClientData clientData, Tcl_Interp *interp, \
               int objc, Tcl_Obj *CONST objv[], int *result, \
               Tcl_Obj **resultObjPtr) \
{ \
  int elemNumber; \
  char error[80]; \
  Tcl_Obj *listPtrPtr[E_NNPE_##TYPE1]; /* for holding nodes numbers */ \
  int i; \
  \
  MeshObj *meshObjPtr = (MeshObj *) clientData; \
  MESH *mesh = meshObjPtr->mesh; /* convinience variable */ \
  \
  /* check if valid number of arguments is supplied */ \
  if (objc != 4) \
    { \
      sprintf (error, "info -Nodes%s element", TYPENAME);\
      Tcl_WrongNumArgs (interp, 1, objv, error); \
      *result = TCL_ERROR; \
      return;  \
    } \
 /* check if there are Link1 elements in the mesh */ \
 if (mesh->NElems##TYPE2 < 1) \
   { \
    sprintf (error, "mesh does not contain %s elements", TYPENAME); \
      *resultObjPtr = Tcl_NewStringObj (error, -1); \
      *result = TCL_ERROR;  \
      return; \
   }   \
 /* get element number form argument list */ \
  if (Tcl_GetIntFromObj (interp, objv[3], &elemNumber) != TCL_OK) \
    { \
      *result = TCL_ERROR; \
      return; \
    } \
 /* check if element number is a valid number  
    By convention we assume that elements are numbered from 1.
  */ \
 if (elemNumber < 1 || elemNumber > mesh->NElems##TYPE2) \
   {\
    sprintf (error, "wrong element number %d : expected integer 1-%d", \
                    elemNumber,  mesh->NElems##TYPE2); \
      *resultObjPtr = Tcl_NewStringObj (error, -1); \
      *result = TCL_ERROR;  \
      return; \
   }  \
 elemNumber--; /* as we accesing C array */ \
 for (i=0; i<E_NNPE_##TYPE1; i++) \
    { \
      listPtrPtr[i] = Tcl_NewIntObj (*(mesh->Nodes##TYPE2[elemNumber]+i)); \
    } \
  *resultObjPtr = Tcl_NewListObj (E_NNPE_##TYPE1, listPtrPtr); \
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 Generation of Info procedures 
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
INFO_PROC_NODES(LINK1, Link1, "Link1");
INFO_PROC_NODES(LINK2, Link2, "Link2");
INFO_PROC_NODES(LINK3, Link3, "Link3");
INFO_PROC_NODES(LINK4, Link4, "Link4");
INFO_PROC_NODES(LINK5, Link5, "Link5");
INFO_PROC_NODES(TRIANG1, Triang1, "Triang1");
INFO_PROC_NODES(TRIANG2, Triang2, "Triang2");
INFO_PROC_NODES(TRIANG3, Triang3, "Triang3");
INFO_PROC_NODES(TRIANG4, Triang4, "Triang4");
INFO_PROC_NODES(TRIANG5, Triang5, "Triang5");
INFO_PROC_NODES(QUAD1, Quad1, "Quad1");
INFO_PROC_NODES(QUAD3, Quad3, "Quad3");
INFO_PROC_NODES(TETRAH1, Tetrah1, "Tetrah1"); 
INFO_PROC_NODES(BLOCK1, Block1, "Block1"); 


#undef INFO_PROC_NODES        

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 InfoBBox -- 
         Return mesh points bounding box (x1 y1 z1 x2 y2 z2).
 Syntax:
       meshObj info -BBox
       meshObj info -BBox -xyz
       meshObj info -BBox -xy
 Results:
      Depemding on the suboption returns list of four or six coordinates
      of the bounding box points.
 Side effects:
      None. 
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        static void
InfoBBox (ClientData clientData, Tcl_Interp *interp, 
               int objc, Tcl_Obj *CONST objv[], int *result,
               Tcl_Obj **resultObjPtr)
{
  int j;
  Tcl_Obj *listPtrPtr[6]; /* for holding coords */
  int index=0; 
  static char *subOpts[] = {"-xyz", "-xy", NULL};
  enum subOptsIx {XYZIx, XYIx};  

  MeshObj *meshObjPtr = (MeshObj *) clientData; 

  /* check if valid number of arguments is supplied for -BBox */ 
  if (objc <3 || objc >4)
    {
      Tcl_WrongNumArgs (interp, 1, objv, "info -BBox ?option?"); 
      *result = TCL_ERROR;
      return;       
    }
  /* check if coord suboption is suplied and find option in the
   * list of valid suboptions 
   */
  index = 0; /* this is when no suboption is supplied */
  if (objc == 4) 
    {
      if (Tcl_GetIndexFromObj (interp, objv[3], subOpts, "suboption", 
                              TCL_EXACT, &index) != TCL_OK)
        {
          *result = TCL_ERROR;
          return;
        }
    }

  switch (index)
    {
      case XYZIx: /* all coordinates */
        {
          for (j=0; j<6; j++)
            {
              listPtrPtr[j] = Tcl_NewDoubleObj (meshObjPtr->bBox[j]); 
            }
          *resultObjPtr = Tcl_NewListObj(6, listPtrPtr); 
          return;
        }
      case XYIx: /* ommit Z coordinate */
        {
          listPtrPtr[0] = Tcl_NewDoubleObj (meshObjPtr->bBox[0]); 
          listPtrPtr[1] = Tcl_NewDoubleObj (meshObjPtr->bBox[1]); 
          listPtrPtr[2] = Tcl_NewDoubleObj (meshObjPtr->bBox[3]); 
          listPtrPtr[3] = Tcl_NewDoubleObj (meshObjPtr->bBox[4]); 
          *resultObjPtr = Tcl_NewListObj(4, listPtrPtr); 
          return;
        }
    } /* end of switch(index) */ 
}

/* declaration of element info low level functions */
static void InfoElementNodes (MESH *mesh, int element, Tcl_Obj **resultObjPtr);
static void InfoElementVertices (MESH *mesh, int element, Tcl_Obj **resultObjPtr);
static void InfoElementNodesCoord (MESH *mesh, int element, Tcl_Obj **resultObjPtr);
static void InfoElementVerticesCoord (MESH *mesh, int element, Tcl_Obj **resultObjPtr);

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 InfoElement --
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        static void
InfoElement (ClientData clientData, Tcl_Interp *interp, 
               int objc, Tcl_Obj *CONST objv[], int *result,
               Tcl_Obj **resultObjPtr)
{
  int elemNumber;
  char error[80];
  int index=0; 
  static char *subOpts[] = {"-type", "-nodes", "-vertices",
                            "-nodescoord", "-verticescoord", NULL};
  enum subOptsIx {TypeIx, NodesIx, VerticesIx, NodescoordIx, 
                  VerticescoordIx};  

                   
  MeshObj *meshObjPtr = (MeshObj *) clientData; 
  MESH *mesh = meshObjPtr->mesh; /* convinience variable */

  /* check if at least element number and suboption is supplied */
  if (objc < 5)
    {
      Tcl_WrongNumArgs (interp, 1, objv,
                 "info -element number -suboption ?args...?");
      *result = TCL_ERROR;
      return; 
    } 

  /* get element number and check if it is valid */ 
  if (Tcl_GetIntFromObj (interp, objv[3], &elemNumber) != TCL_OK)
    {
      *result = TCL_ERROR;
      return;
    }
  if (elemNumber<1 || elemNumber>mesh->TotalNElems)
    {
      sprintf (error, "wrong element number %d : expected integer 1-%d", 
                    elemNumber,  mesh->TotalNElems); 
      *resultObjPtr = Tcl_NewStringObj (error, -1);
      *result = TCL_ERROR; 
      return;
    }

  /* check if valid options is suplied */
  if (Tcl_GetIndexFromObj (interp, objv[4], subOpts, "suboption", 
                           TCL_EXACT, &index) != TCL_OK)
    {
      *result = TCL_ERROR; 
      return; 
    }

  switch (index)
    {
      case TypeIx:
        {        
          if (meshObjPtr->elemType == ETLI_MIXED ) /* mixed element type in
                                                      mesh */
            {
              *resultObjPtr = Tcl_NewIntObj(RP_MeshElementType(mesh,
                                      elemNumber));
            }
          else /* mesh of one element type */
            {
              *resultObjPtr = Tcl_NewIntObj(meshObjPtr->elemType); 
            }     
          break;
        }
      case NodesIx: 
        {
          InfoElementNodes (mesh, elemNumber, resultObjPtr); 
          break;  
        }
      case VerticesIx: 
        {
          InfoElementVertices (mesh, elemNumber, resultObjPtr); 
          break;  
        }
      case NodescoordIx:
        {
          InfoElementNodesCoord (mesh, elemNumber, resultObjPtr); 
          break;  
        }
      case VerticescoordIx:          
        {
          InfoElementVerticesCoord (mesh, elemNumber, resultObjPtr); 
          break;  
        }
      default:  /* program never should go here; it is only for safety check */
        {
          char error [100]; 
          snprintf (error, 99, 
                    "Bad suboption %s. Has no switch entry",
                     Tcl_GetStringFromObj (objv[index], NULL)); 
          *resultObjPtr = Tcl_NewStringObj (error, -1); 
          *result = TCL_ERROR; 
        }
    } /* end of switch(index) */ 
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 InfoElementNodes --
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/* Maximal number of nodes per element is 9 for QUAD3 element */
#define _ETLI_MAX_NODES 9
        static void
InfoElementNodes (MESH *mesh, int element, Tcl_Obj **resultObjPtr)
{
  Tcl_Obj *listPtrPtr[_ETLI_MAX_NODES]; /* for holding nodes numbers */
  int *nodesList; 
  int llength, i;

  RP_MeshElementNodes (mesh, element, &nodesList, &llength);  
  assert (llength <= _ETLI_MAX_NODES); 
  
  for (i=0; i<llength; i++)
    {
      listPtrPtr[i] = Tcl_NewIntObj (nodesList[i]);       
    }
  *resultObjPtr = Tcl_NewListObj (llength, listPtrPtr); 
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 InfoElementNodesCoord --
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        static void
InfoElementNodesCoord (MESH *mesh, int element, Tcl_Obj **resultObjPtr)
{
  Tcl_Obj *listPtrPtr[3*_ETLI_MAX_NODES]; /* for holding nodes coords,
                                           * 3*_ETLI_MAX_NODES because
                                           * of three coordinates
                                           */
  int *nodesList;
  int llength, i, node;

  RP_MeshElementNodes (mesh, element, &nodesList, &llength);  
  assert (llength <= 3*_ETLI_MAX_NODES); 
  
  for (i=0; i<llength; i++)
    {
      node = nodesList[i]-1; 
      listPtrPtr[3*i] = Tcl_NewDoubleObj (*mesh->MeshPointCoords[node]);       
      listPtrPtr[3*i+1] = Tcl_NewDoubleObj (*(mesh->MeshPointCoords[node]+1));
      listPtrPtr[3*i+2] = Tcl_NewDoubleObj (*(mesh->MeshPointCoords[node]+2));
    }
  *resultObjPtr = Tcl_NewListObj (2*llength, listPtrPtr); 
}
#undef _ETLI_MAX_NODES

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 InfoElementVertices --
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/* Maximal number of vertices per element is 8 for BLOCK1 element */
#define _ETLI_MAX_VERTICES 8
        static void
InfoElementVertices (MESH *mesh, int element, Tcl_Obj **resultObjPtr)
{
  Tcl_Obj *listPtrPtr[_ETLI_MAX_VERTICES]; /* for holding vertices numbers */
  int *verticesList; 
  int llength, i;

  RP_MeshElementVertices (mesh, element, &verticesList, &llength);  
  assert (llength <= _ETLI_MAX_VERTICES); 
  
  for (i=0; i<llength; i++)
    {
      listPtrPtr[i] = Tcl_NewIntObj (verticesList[i]);       
    }
  *resultObjPtr = Tcl_NewListObj (llength, listPtrPtr); 
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 InfoElementVerticesCoord --
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        static void
InfoElementVerticesCoord (MESH *mesh, int element, Tcl_Obj **resultObjPtr)
{
  Tcl_Obj *listPtrPtr[3*_ETLI_MAX_VERTICES]; /* for holding nodes coords,
                                           * 3*_ETLI_MAX_NODES because
                                           * of three coordinates
                                           */
  int *verticesList;
  int llength, i, vertex;

  RP_MeshElementVertices (mesh, element, &verticesList, &llength);  
  assert (llength <= 3*_ETLI_MAX_VERTICES); 
  
  for (i=0; i<llength; i++)
    {
      vertex = verticesList[i]; 
      listPtrPtr[3*i] = Tcl_NewDoubleObj ((*mesh->MeshPointCoords[vertex]));       
      listPtrPtr[3*i+1] = Tcl_NewDoubleObj (*(mesh->MeshPointCoords[vertex]+1)); 
      listPtrPtr[3*i+2] = Tcl_NewDoubleObj (*(mesh->MeshPointCoords[vertex]+2)); 
    }
  *resultObjPtr = Tcl_NewListObj (3*llength, listPtrPtr); 
}
#undef _ETLI_MAX_VERTICES

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 InfoNSubDoms --
           Return number of subdomains.
 Syntax:
       meshObj info -NSubDoms
 Results:
       Normally return number of subdomains.
       If decomposition data has not been read return error code
       and print error message.
 Side effects:
       None. 
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        static void
InfoNSubDoms (ClientData clientData, Tcl_Interp * intetp,
        int objc, Tcl_Obj *CONST objv[], int *result,        
        Tcl_Obj **resultObjPtr)
{
  MeshObj *meshObjPtr = (MeshObj *) clientData; 
  MESH *mesh = meshObjPtr->mesh; /* convinience variable */ 

  /* check if mesh decomposition structure has been initialized */  
  if (mesh->SubStructSet[E_SS_NUM_DECOMP] == FALSE)
    {
      *resultObjPtr = Tcl_NewStringObj
          ("decomp not initialized; use \"get -decomp\" to initialize", -1);
      *result = TCL_ERROR; 
      return; 
    }
  *resultObjPtr = Tcl_NewIntObj (mesh->Decomp.NSubDoms);
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 InfoSubDoms --
            Return subdomain index for given element.
 Syntax:
        meshObj info -SubDoms elemNumber

 Results:       
        Normally return subdomain index.
        If element number is invalid or decomposition data has not 
        been read return error code and print an error message
 Side effects:
        None.1 
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        static void
InfoSubDoms (ClientData clientData, Tcl_Interp *interp, 
               int objc, Tcl_Obj *CONST objv[], int *result,
               Tcl_Obj **resultObjPtr)
{
  int elemNumber;
  char error[80];
                   
  MeshObj *meshObjPtr = (MeshObj *) clientData; 
  MESH *mesh = meshObjPtr->mesh; /* convinience variable */

  /* check proper number of arguments supplied */
  if (objc != 4)
    {
      Tcl_WrongNumArgs (interp, 1, objv,
                 "info -SubDoms elemnumber");
      *result = TCL_ERROR;
      return; 
    } 

  /* get element number and check if it is valid */ 
  if (Tcl_GetIntFromObj (interp, objv[3], &elemNumber) != TCL_OK)
    {
      *result = TCL_ERROR;
      return;
    }
  if (elemNumber<1 || elemNumber>mesh->TotalNElems)
    {
      sprintf (error, "wrong element number %d : expected integer 1-%d", 
                    elemNumber,  mesh->TotalNElems); 
      *resultObjPtr = Tcl_NewStringObj (error, -1);
      *result = TCL_ERROR; 
      return;
    }
  /* check if mesh decomposition structure has been initialized */  
  if (mesh->SubStructSet[E_SS_NUM_DECOMP] == FALSE)
    {
      *resultObjPtr = Tcl_NewStringObj
          ("decomp not initialized; use \"get -decomp\" to initialize", -1);
      *result = TCL_ERROR; 
      return; 
    }
  
  /* everything OK can returnsubdomain index */
  *resultObjPtr = Tcl_NewIntObj (mesh->Decomp.SubDoms[elemNumber-1]);  
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 InfoSubDoms#ELEMTYPE# --
            Return subdomain index for given element of particular type.
 Syntax:
        meshObj info -SubDomsLink1 elemNumber
        ....................................
        meshObj info -SubDomsLink5 elemNumber
        meshObj info -SubDomsTriang1 elemNumber
        ......................................
        meshObj info -SubDomsTriang5 elemNumber
        meshObj info -SubDomsQuad1 elemNumber
        meshObj info -SubDomsQuad3 elemNumber
        meshObj info -SubDomsTetrah1 elemNumber
        meshObj info -SubDomsBlock1 elemNumber

 Results:       
        Normally return subdomain index.
        If element number is invalid, decomposition data has not 
        been read, or mesh does not contain elements of partivular type
        return error code and print an error message

 Side effects:
        None. 
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/* Those functions described above are generated from the macro */

#define INFO_PROC_SUBDOM(TYPE1, TYPE2, TYPENAME) /*TYPE1 is not used */ \
        static void \
InfoSubDoms##TYPE2 (ClientData clientData, Tcl_Interp *interp, \
               int objc, Tcl_Obj *CONST objv[], int *result, \
               Tcl_Obj **resultObjPtr) \
{ \
  int elemNumber; \
  char error[80]; \
  \
  MeshObj *meshObjPtr = (MeshObj *) clientData; \
  MESH *mesh = meshObjPtr->mesh; /* convinience variable */ \
   \
  /* check if mesh contains particular type of elements */ \
  if ( mesh->NElems##TYPE2 < 1) \
    { \
      sprintf (error, "mesh does not contain elements of type %s", ##TYPENAME); \
      *resultObjPtr = Tcl_NewStringObj (error, -1); \
      *result = TCL_ERROR; \
      return; \
    } \
  /* check proper number of arguments supplied */ \
  if (objc != 4) \
    { \
      sprintf (error, "info -SubDoms%s elemnumber", ##TYPENAME); \
      Tcl_WrongNumArgs (interp, 1, objv, error); \
      *result = TCL_ERROR; \
      return; \
    } \
   \
  /* get element number and check if it is valid */ \
  if (Tcl_GetIntFromObj (interp, objv[3], &elemNumber) != TCL_OK) \
    { \
      *result = TCL_ERROR; \
      return; \
    } \
  if (elemNumber<1 || elemNumber>mesh->NElems##TYPE2) \
    { \
      sprintf (error, "wrong element number %d : expected integer 1-%d", \
                    elemNumber,  mesh->NElems##TYPE2); \
      *resultObjPtr = Tcl_NewStringObj (error, -1); \
      *result = TCL_ERROR;  \
      return; \
    } \
  /* check if mesh decomposition structure has been initialized */   \
  if (mesh->SubStructSet[E_SS_NUM_DECOMP] == FALSE) \
    { \
      *resultObjPtr = Tcl_NewStringObj \
          ("decomp not initialized; use \"get -decomp\" to initialize", -1); \
      *result = TCL_ERROR; \
      return; \
    } \
   \
  /* everything OK can returnsubdomain index */ \
  *resultObjPtr = Tcl_NewIntObj (mesh->Decomp.SubDoms##TYPE2[elemNumber-1]);  \
}

INFO_PROC_SUBDOM(LINK1, Link1, "Link1");
INFO_PROC_SUBDOM(LINK2, Link2, "Link2");
INFO_PROC_SUBDOM(LINK3, Link3, "Link3");
INFO_PROC_SUBDOM(LINK4, Link4, "Link4");
INFO_PROC_SUBDOM(LINK5, Link5, "Link5");
INFO_PROC_SUBDOM(TRIANG1, Triang1, "Triang1");
INFO_PROC_SUBDOM(TRIANG2, Triang2, "Triang2");
INFO_PROC_SUBDOM(TRIANG3, Triang3, "Triang3");
INFO_PROC_SUBDOM(TRIANG4, Triang4, "Triang4");
INFO_PROC_SUBDOM(TRIANG5, Triang5, "Triang5");
INFO_PROC_SUBDOM(QUAD1, Quad1, "Quad1");
INFO_PROC_SUBDOM(QUAD3, Quad3, "Quad3");
INFO_PROC_SUBDOM(TETRAH1, Tetrah1, "Tetrah1"); 
INFO_PROC_SUBDOM(BLOCK1, Block1, "Block1"); 

#undef INFO_PROC_SUBDOM 

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 InfoNNurbsCurves --
           Return number of nurbs curves associated with the mesh.
 Syntax:
       meshObj info -NNurbsCurves
 Results:
       Normally return number of nurbs curves associated with the mesh.
       If geometry data has not been read return error code
       and print error message.
 Side effects:
       None. 
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        static void
InfoNNurbsCurves (ClientData clientData, Tcl_Interp * intetp,
        int objc, Tcl_Obj *CONST objv[], int *result,        
        Tcl_Obj **resultObjPtr)
{
  MeshObj *meshObjPtr = (MeshObj *) clientData; 
  MESH *mesh = meshObjPtr->mesh; /* convinience variable */ 

  /* check if mesh geom substructure has been initialized */  
  if (mesh->SubStructSet[E_SS_NUM_GEOM] == FALSE)
    {
      *resultObjPtr = Tcl_NewStringObj
          ("geom not initialized; use \"get -geom\" to initialize", -1);
      *result = TCL_ERROR; 
      return; 
    }
  *resultObjPtr = Tcl_NewIntObj (mesh->Geom.NNurbsCurves);
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 InfoNurbsCurves --
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        static void
InfoNurbsCurves (ClientData clientData, Tcl_Interp *interp, 
               int objc, Tcl_Obj *CONST objv[], int *result,
               Tcl_Obj **resultObjPtr)
{
  int nurbsNumber;
  char error[80];
  NurbsObj nuObj;
  int index=0; 
  static char *subOpts[] = {"-all", "-degree", "-ep1", "-ep2", "-ctrlpoint", 
                             "-coords", "-name", NULL};
  enum subOptsIx {AllIx, DegreeIx, Ep1Ix, Ep2Ix, CtrlIx, CoordsIx, NameIx};

  MeshObj *meshObjPtr = (MeshObj *) clientData; 
  MESH *mesh = meshObjPtr->mesh; /* convinience variable */

  /* check if mesh geom substructure has been initialized */  
  if (mesh->SubStructSet[E_SS_NUM_GEOM] == FALSE)
    {
      *resultObjPtr = Tcl_NewStringObj
          ("geom not initialized; use \"get -geom\" to initialize", -1);
      *result = TCL_ERROR; 
      return; 
    }

  /* check if valid number of arguments is supplied for -NurbsCurves */ 
  if (objc <4 || objc >6)
    {
      Tcl_WrongNumArgs (interp, 1, objv, 
                      "info -NurbsCurves number ?option?"); 
      *result = TCL_ERROR;
      return;       
    }

  /* get the nurbs number from arguments list */
  if (Tcl_GetIntFromObj (interp, objv[3], &nurbsNumber) != TCL_OK)
    {
      *result = TCL_ERROR;
      return;
    }
  /* check if nurbs number is valid */
  if (nurbsNumber<1 || nurbsNumber>mesh->Geom.NNurbsCurves)
    {
      sprintf (error, "wrong NURBS number %d : expected integer 1-%d", 
                    nurbsNumber,  mesh->Geom.NNurbsCurves); 
      *resultObjPtr = Tcl_NewStringObj (error, -1);
      *result = TCL_ERROR; 
      return;
    }

  nurbsNumber--; /* nurbs are numbered from 1 but this is C */
  
  /* check if suboption is suplied and find it in the
   * list of valid suboptions 
   */
  index = 0; /* this is when no suboption is supplied  -all will be assumed */
  if (objc > 4) 
    {
      if (Tcl_GetIndexFromObj (interp, objv[4], subOpts, "suboption", 
                              TCL_EXACT, &index) != TCL_OK)
        {
          *result = TCL_ERROR;
          return;
        }
    }
 nuObj.nurbs = &(mesh->Geom.NurbsCurves[nurbsNumber]);
 switch (index)
    {
      case AllIx:
        {
           NurbsInfoAll (&nuObj, interp, objc, objv, 
                             result, resultObjPtr); 
           break;
        }
      case DegreeIx:
        {
          *result = TCL_OK;
          *resultObjPtr = Tcl_NewIntObj (nuObj.nurbs->Degree); 
          break;
        }
      case Ep1Ix:
        {
          *result = TCL_OK;
          *resultObjPtr = Tcl_NewIntObj (nuObj.nurbs->EndPoint1); 
          break;
        }
      case Ep2Ix:
        {
          *result = TCL_OK;
          *resultObjPtr = Tcl_NewIntObj (nuObj.nurbs->EndPoint2);
          break; 
        }
      case CtrlIx: 
        {
           NurbsInfoControlPoint (&nuObj, interp, objc, objv, 
                             result, resultObjPtr); 
           break;
        }         
      case CoordsIx:
        {
          NurbsInfoCoords (&nuObj, interp, objc, objv, 
                          result, resultObjPtr); 
          break;
        }  
      case NameIx:
        {
          *resultObjPtr = Tcl_NewStringObj (nuObj.nurbs->Name, 
                                           strlen (nuObj.nurbs->Name));
          break;
        }
    } /* end of switch */                
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 InfoAll --
       Return list of all nurbs parametrs (in the form which is
       the same as when creating new nurbs object)
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        static void
NurbsInfoAll (ClientData clientData, 
         Tcl_Interp *interp, 
         int objc, 
         Tcl_Obj *CONST objv[],
         int *result,
         Tcl_Obj **resultObjPtr)
{
  /* for holding return list */
  Tcl_Obj *listPtrPtr[3+4*(1+ETLI_MAX_NURBS_DEGREE)];
  int llength;  /* return list list length */
  int pointNumber; /*control point number */
  int j, i;
  NurbsObj *nurbsObjPtr = (NurbsObj *) clientData; 
  NURBSCURVE *nurbs = nurbsObjPtr->nurbs; /* convinience variable */

  /* check number of arguments */
  if (objc != 5)
    {
      sprintf(errorMessage, 
          "wrong # args: should be \"info -NurbsCurves number -all\"");
      *result = TCL_ERROR;

       *resultObjPtr = Tcl_NewStringObj (errorMessage, -1); 
       *result = TCL_ERROR;
       return;         
     }

  /* calculate length of the return list:
   *       3 items for   degree, ep1, ep2, 
   *  and  4* (degree+1) items for coords and weigth of each control
   *  point
   */
  llength = 3 + 4*(nurbs->Degree+1);
  
  /* add degree and end points number to return list */
  listPtrPtr[0] = Tcl_NewIntObj (nurbs->Degree);
  listPtrPtr[1] = Tcl_NewIntObj (nurbs->EndPoint1);
  listPtrPtr[2]= Tcl_NewIntObj (nurbs->EndPoint2);
  
  /* construc list of control point coords and weight */
  j=3; /* we already filled thre items */
  for (pointNumber=0; pointNumber < nurbs->Degree+1; pointNumber++)
    {
      for (i=0; i<4; i++)
        {
          listPtrPtr[j] = 
                 Tcl_NewDoubleObj (*(nurbs->CntlPoints[pointNumber]+i));
          j++;
        } 
    }

   *resultObjPtr = Tcl_NewListObj(llength, listPtrPtr); 

   return;
}


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 NurbsInfoControlPoint --
           Returns control point coords and weight.
 Syntax:          
      nurbsObj info -ctrlpoint number
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        static void
NurbsInfoControlPoint (ClientData clientData, 
                  Tcl_Interp *interp, 
                  int objc, 
                  Tcl_Obj *CONST objv[],
                  int *result,
                  Tcl_Obj **resultObjPtr)
{
  int pointNumber,j;
  Tcl_Obj *listPtrPtr[4]; /* for holding coords and weight */

  NurbsObj *nurbsObjPtr = (NurbsObj *) clientData; 
  NURBSCURVE *nurbs = nurbsObjPtr->nurbs; /* convinience variable */

  /* check if control point number is supplied for -ctrlpoint option */
  if (objc != 6)
    {
      sprintf(errorMessage, 
      "wrong # args: should be \"info -NurbsCurves number -ctrlpoint number\"");
       *resultObjPtr = Tcl_NewStringObj (errorMessage, -1); 
       *result = TCL_ERROR;
       return;         
     }
  if (Tcl_GetIntFromObj (interp, objv[5], &pointNumber) != TCL_OK)
    {
      *result = TCL_ERROR; 
      return;
    }  

  /* check if control point number is valid */
  if ( pointNumber<1 || pointNumber>nurbs->Degree+1 )
    {
      sprintf(errorMessage, "wrong point number: expected integer 1-%d", 
                     nurbs->Degree+1); 
      *resultObjPtr = Tcl_NewStringObj (errorMessage, -1); 
      *result = TCL_ERROR; 
      return; 
    }

  pointNumber--; /* control point are numbered from 1 but this is C */

  /* construc list of control point coords and weight */
  for (j=0; j<4; j++)
    {
      listPtrPtr[j] = Tcl_NewDoubleObj (*(nurbs->CntlPoints[pointNumber]+j));
    } 
   *resultObjPtr = Tcl_NewListObj(4, listPtrPtr); 

   return;
}    

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 NurbsInfoCoords --
        Return list of x,y,z coords of point on nurbs curve for
        given curve parameter t.
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        static void
NurbsInfoCoords (ClientData clientData, 
            Tcl_Interp *interp, 
            int objc, 
            Tcl_Obj *CONST objv[],
            int *result,
            Tcl_Obj **resultObjPtr)
{
  int j;
  double t; /* curve parameter */
  double coords[3]; /* curve point coords */
  Tcl_Obj *listPtrPtr[3]; /* for holding x,y,z  coords */

  NurbsObj *nurbsObjPtr = (NurbsObj *) clientData; 
  NURBSCURVE *nurbs = nurbsObjPtr->nurbs; /* convinience variable */

  /* check if curve parameter is supplied for -coords option */
  if (objc != 6)
    {
      sprintf(errorMessage, 
      "wrong # args: should be \"info -NurbsCurves number -coords t_param\"");
       *resultObjPtr = Tcl_NewStringObj (errorMessage, -1); 
       *result = TCL_ERROR;
       return;         
     }

  /* retrieve curve parameter from arguments list */ 
  if (Tcl_GetDoubleFromObj (interp, objv[5], &t) != TCL_OK)
    {
      *result = TCL_ERROR; 
      return;
    }  

  /* check if curve parameter is valid  <0,1>*/
  if ( t<0.0 || t>1.0 )
    {
      sprintf(errorMessage, "wrong curve parameter: expected double <0,1>"); 
      *resultObjPtr = Tcl_NewStringObj (errorMessage, -1); 
      *result = TCL_ERROR; 
      return; 
    }

  /* get curve point coords for given parameter t (Jelle function) */
      crv_getcoords (nurbs, t, coords); 
  /* new function supplied by Jelle, this was only for testing purposes */
  /*    SetCurvePoint (nurbs, t, coords); */
 
  
  /* construc list of x,y,z coords for giver t */
  for (j=0; j<3; j++)
    {
      listPtrPtr[j] = Tcl_NewDoubleObj (coords[j]);
    } 
   *resultObjPtr = Tcl_NewListObj(3, listPtrPtr); 

   return;
} 



/* meshInfoProc is an array of pointer to Info* functions */
static void (*meshInfoProc[]) (ClientData clientData, 
               Tcl_Interp *interp, 
               int objc, 
               Tcl_Obj *CONST objv[],
               int *result,
               Tcl_Obj **resultObjPtr)={
                 /* 1 */      InfoTitle,
                 /* 2 */      InfoNNodes, 
                 /* 3 */      InfoNMeshPoints,
                 /* 4 */      InfoNElemsLink1,
                 /* 5 */      InfoNElemsLink2,
                 /* 6 */      InfoNElemsLink3,
                 /* 7 */      InfoNElemsLink4,
                 /* 8 */      InfoNElemsLink5,
                 /* 9 */      InfoNElemsTriang1,
                 /* 10 */     InfoNElemsTriang2,
                 /* 12 */     InfoNElemsTriang3,
                 /* 13 */     InfoNElemsTriang4,
                 /* 14 */     InfoNElemsTriang5,
                 /* 15 */     InfoNElemsQuad1,
                 /* 16 */     InfoNElemsQuad3,
                 /* 17 */     InfoNElemsTetrah1,
                 /* 18 */     InfoNElemsBlock1,
                 /* 19 */     InfoTotalNElems,
                 /* 20 */     InfoNBCNodes,
                 /* 21 */     InfoNLoadedNodes,
                 /* 22 */     InfoNMats,
                 /* 23 */     InfoNMdfMats,
                 /* 24 */     InfoNCompMatsType1,
                 /* 25 */     InfoNTimeSteps,
                 /* 26 */     InfoNInternalTimeSteps,
                 /* 27 */     InfoNExternalTimeSteps,
                 /* 28 */     InfoFirstNetIndex,
                 /* 29 */     InfoTimeStep,
                 /* 30 */     InfoDampingFactor,
                 /* 31 */     InfoBeta,
                 /* 32 */     InfoGamma,
                 /* 33 */     InfoNStressPointsLink1,         
                 /* 34 */     InfoNStressPointsLink2,         
                 /* 35 */     InfoNStressPointsLink3,         
                 /* 36 */     InfoNStressPointsLink4,         
                 /* 37 */     InfoNStressPointsLink5,         
                 /* 38 */     InfoNStressPointsTriang1,       
                 /* 39 */     InfoNStressPointsTriang2,       
                 /* 40 */     InfoNStressPointsTriang3,       
                 /* 41 */     InfoNStressPointsTriang4,       
                 /* 42 */     InfoNStressPointsTriang5,       
                 /* 43 */     InfoNStressPointsQuad1,         
                 /* 44 */     InfoNStressPointsQuad3,         
                 /* 45 */     InfoNStressPointsTetrah1,       
                 /* 46 */     InfoNStressPointsBlock1,        
                 /* 47 */     InfoMeshPointCoords,       
                 /* 48 */     InfoNodesLink1,               
                 /* 49 */     InfoNodesLink2,               
                 /* 50 */     InfoNodesLink3,               
                 /* 52 */     InfoNodesLink4,               
                 /* 53 */     InfoNodesLink5,               
                 /* 54 */     InfoNodesTriang1,             
                 /* 55 */     InfoNodesTriang2,             
                 /* 56 */     InfoNodesTriang3,             
                 /* 57 */     InfoNodesTriang4,             
                 /* 58 */     InfoNodesTriang5,             
                 /* 59 */     InfoNodesQuad1,               
                 /* 60 */     InfoNodesQuad3,               
                 /* 61 */     InfoNodesTetrah1,             
                 /* 62 */     InfoNodesBlock1,              
                 /* 63 */     InfoTotalNodalDOF,              
                 /* 64 */     InfoNodalDOF,                  
                 /* 65 */     InfoBCNodes,                  
                 /* 66 */     InfoBCTypes,                  
                 /* 67 */     InfoBCDispls,              
                 /* 68 */     InfoBCSpringConsts,        
                 /* 69 */     InfoLoadedNodes,               
                 /* 70 */     InfoLoads,                 
//               /* 71 */     InfoMats,                    
//               /* 72 */     InfoMatsLink1,               
//               /* 73 */     InfoMatsLink2,               
//               /* 74 */     InfoMatsLink3,               
//               /* 75 */     InfoMatsLink4,               
//               /* 76 */     InfoMatsLink5,               
//               /* 77 */     InfoMatsTriang1,             
//               /* 78 */     InfoCompMatsTriang2,     
 //              /* 79 */     InfoMatsTriang3,             
//               /* 80 */     InfoMatsTriang4,             
//               /* 81 */     InfoMatsTriang5,             
//               /* 82 */     InfoMatsQuad1,               
//               /* 83 */     InfoCompMatsQuad3,       
//               /* 84 */     InfoMatsTetrah1,             
//               /* 85 */     InfoMatsBlock1,              
//               /* 86 */     InfoNodeTypes,                 
                 /* 87 */     InfoNNurbsCurves,               
//               /* 88 */     InfoNurbsCurveParams,      
                 /* 89 */     InfoNurbsCurves,      
//               /* 90 */     InfoGNIs,                      
//               /* 92 */     InfoGEIs,                      
//               /* 93 */     InfoGEIsLink1,                 
//               /* 94 */     InfoGEIsLink2,                 
//               /* 95 */     InfoGEIsLink3,                 
//               /* 96 */     InfoGEIsLink4,                 
//               /* 97 */     InfoGEIsLink5,                 
//               /* 98 */     InfoGEIsTriang1,               
//               /* 99 */     InfoGEIsTriang2,               
//               /* 100 */    InfoGEIsTriang3,               
//               /* 101 */    InfoGEIsTriang4,               
//               /* 102 */    InfoGEIsTriang5,               
//               /* 103 */    InfoGEIsQuad1,                 
//               /* 104 */    InfoGEIsQuad3,                 
//               /* 105 */    InfoGEIsTetrah1,               
//               /* 106 */    InfoGEIsBlock1,                
//               /* 107 */    InfoGeom,                      
//               /* 108 */    InfoProps,                     
//               /* 119 */    InfoCompPropsType1,       
//               /* 110 */    InfoDecomp,                  
//               /* 111 */    InfoMeshParam,            
//               /* 112 */    InfoFEError,                
//               /* 113 */    InfoStresses,              
//               /* 114 */    InfoKeywordSet,    
//               /* 115 */    InfoSubStructSet,
                 /* 116 */    InfoBBox,
                 /* 117 */    InfoElement, 
                 /* 118 */    InfoIsOfOneElementType,
                 /* 119 */    InfoLinkElementsOnly,
                 /* 120 */    InfoPlanarElementsOnly,
                 /* 121 */    InfoNSubDoms,
                 /* 122 */    InfoSubDoms,        
                 /* 123 */    InfoSubDomsLink1,               
                 /* 124 */    InfoSubDomsLink2,               
                 /* 125 */    InfoSubDomsLink3,               
                 /* 126 */    InfoSubDomsLink4,               
                 /* 127 */    InfoSubDomsLink5,               
                 /* 128 */    InfoSubDomsTriang1,             
                 /* 129 */    InfoSubDomsTriang2,             
                 /* 130 */    InfoSubDomsTriang3,             
                 /* 131 */    InfoSubDomsTriang4,             
                 /* 132 */    InfoSubDomsTriang5,             
                 /* 133 */    InfoSubDomsQuad1,               
                 /* 134 */    InfoSubDomsQuad3,               
                 /* 135 */    InfoSubDomsTetrah1,             
                 /* 135 */    InfoSubDomsBlock1             
                           }; 

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 EtliMeshObjInfo -- 
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        CmdReturn*
EtliMeshObjInfo (ClientData clientData, Tcl_Interp *interp, 
              int objc, Tcl_Obj *CONST objv[])
{
  int index;
  int result = TCL_OK; 
  CmdReturn *returnStructPtr = NULL;
  Tcl_Obj *resultObjPtr = NULL;

  MeshObj *meshObjPtr = (MeshObj *) clientData; 
  MESH *mesh = meshObjPtr->mesh; /* convinience variable */

  /* allocate the space for the return structure */
  /* Use Tcl memory allocators to make possible monitor memory usage */ 
  returnStructPtr = (CmdReturn *)  Tcl_Alloc (sizeof (CmdReturn));


  /* check if mesh has been initialized */
  if (mesh == (MESH*) NULL)   
    {
      resultObjPtr = Tcl_NewStringObj 
          ("mesh not initialized; use get subcommand to initialize", -1); 
      result = TCL_ERROR; 
      goto done;
    }
  /* find option in the list of valid options for this subcommad */
  if (Tcl_GetIndexFromObj (interp, objv[2], meshInfoSubOpts, 
                          "options", TCL_EXACT, &index) != TCL_OK)
    {
      result = TCL_ERROR;
      goto done;
    }

  /* calling appropriate info function */
  meshInfoProc[index](clientData, interp, objc, objv, 
                      &result, &resultObjPtr); 
  done:
  /* initialize return structure */
     returnStructPtr->object = resultObjPtr;
     returnStructPtr->status = result;

  return returnStructPtr;
}

/* Modifications:
 * $Id: etliCmdMeshInfo.c,v 1.1.1.1 2002/08/04 13:19:11 peteri Exp $
 */ 
