/********************************************************************
 etliCmdMeshPut.c --
      This file implements put subcommand of mesh objects.
      
 ******************************************************************** 
** Copyright (C) 1999 Roman Putanowicz <romek@sect.mce.hw.ac.uk>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, 
** Boston, MA 02111-1307, USA.
 ***********************************************************************
*/


 /* $Id: etliCmdMeshPut.c,v 1.1.1.1 2002/08/04 13:19:11 peteri Exp $ */

#include "etliInt.h"
#include "etliCmdMesh.h"


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 EtliMeshObjPut 
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        CmdReturn*
EtliMeshObjPut (ClientData clientData, Tcl_Interp *interp, 
              int objc, Tcl_Obj *CONST objv[])
{
  int index;
  static char *subOpts[] = {"-mesh", "-materials", "-decomp", 
             "-meshparams", "-stresses", "-feerrors","-geom", NULL};
  enum subCmdIx {MeshIx, MaterialsIx, DecompIx, MeshparamsIx, 
                 StressesIx, FeerrorsIx, GeomIx};         

  int result = TCL_OK; 
  CmdReturn *returnStructPtr = NULL;
  Tcl_Obj *resultObjPtr = NULL;

  MeshObj *meshObjPtr = (MeshObj *) clientData; 
  MESH *mesh = meshObjPtr->mesh; /* convinience variable */

  char *filename;
  
  /* Allocate the space for the return structure */
  /* Use Tcl memory allocators to make possible monitor memory usage */ 
  returnStructPtr = (CmdReturn *) Tcl_Alloc (sizeof (CmdReturn));

  /* find option in the list of valid options for this subcommand */
  if (Tcl_GetIndexFromObj (interp, objv[2], subOpts, "option",
                          TCL_EXACT, &index) != TCL_OK)
    {
      result = TCL_ERROR;
      goto done;
    }

  /* check if mesh object has been initialized */
  if (mesh == NULL)
    {
      resultObjPtr = Tcl_NewStringObj
         ("mesh not initialized; use \"get -mesh\" to initialize", -1);
      result = TCL_ERROR; 
      goto done; 
    }
  /* get filename from command argument 
   * the string is mamaged by object manager and should not be modified or
   *  deleted 
   */  
   filename =  Tcl_GetString (objv[2]); 

   /* there should be chec if writing finishes with succes 
    * but as E-lib *Put* functions does not return it cannot
    * be done easily 
    */  
   switch (index)
     {
       case MeshIx:
         {
           e_PutMeshData (mesh, filename);
           break;
         } 
       case MaterialsIx:
         {
           if (mesh->SubStructSet[E_SS_NUM_MAT_PROPS] == FALSE)
             {
               resultObjPtr = Tcl_NewStringObj
            ("materials not initialized; use \"get -materials\" to initialize", 
                -1);
               result = TCL_ERROR; 
               goto done; 
             }
           fprintf(stderr, "ala\n"); 
           e_PutMaterials (mesh, filename);
           break;
         }
       case DecompIx:
         {
           if (mesh->SubStructSet[E_SS_NUM_DECOMP] == FALSE)
             {
               resultObjPtr = Tcl_NewStringObj
         ("decomp not initialized; use \"get -decomp\" to initialize", 
                -1);
               result = TCL_ERROR; 
               goto done; 
             }
           e_PutDecompData (mesh, filename);
           break;
         } 
       case StressesIx:
         {
           if (mesh->SubStructSet[E_SS_NUM_STRESS] == FALSE)
             {
               resultObjPtr = Tcl_NewStringObj
            ("stresses not initialized; use \"get -stresses\" to initialize", 
                -1);
               result = TCL_ERROR; 
               goto done; 
             }
           e_PutStresses (mesh, filename);
           break;
         }
       case MeshparamsIx:  
         {
           if (mesh->SubStructSet[E_SS_NUM_MESHPARAM] == FALSE)
             {
               resultObjPtr = Tcl_NewStringObj
          ("meshparams not initialized; use \"get -meshparams\" to initialize",
                -1);
               result = TCL_ERROR; 
               goto done; 
             }
           e_PutMeshParams (mesh, filename);
           break;
         } 
       case FeerrorsIx:
         {
           if (mesh->SubStructSet[E_SS_NUM_FEERROR] == FALSE)
             {
               resultObjPtr = Tcl_NewStringObj
            ("ferrors not initialized; use \"get -feerrors\" to initialize", 
                -1);
               result = TCL_ERROR; 
               goto done; 
             }
           e_PutFEErrors (mesh, filename);
           break;
         } 
       case GeomIx:  
         {
           if (mesh->SubStructSet[E_SS_NUM_GEOM] == FALSE)
             {
               resultObjPtr = Tcl_NewStringObj
            ("geom not initialized; use \"get -geom\" to initialize", 
                -1);
               result = TCL_ERROR; 
               goto done; 
             }
           e_PutGeomModel (mesh, filename);
           break;
         } 
     } /* end of switch(index) */
   
  done:

  /* initialize return structure */
      returnStructPtr->object = resultObjPtr;
      returnStructPtr->status = result;

  return returnStructPtr;
}

/* Modifications:
 * $Id: etliCmdMeshPut.c,v 1.1.1.1 2002/08/04 13:19:11 peteri Exp $
 */ 
