/********************************************************************
 etliCmdMeshSet.c --
      This file implements set subcommand for mesh objects. 
      
 ******************************************************************** 
** Copyright (C) 1999 Roman Putanowicz <romek@sect.mce.hw.ac.uk>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, 
** Boston, MA 02111-1307, USA.
 ***********************************************************************
*/


 /* $Id: etliCmdMeshSet.c,v 1.1.1.1 2002/08/04 13:19:11 peteri Exp $ */

#include <assert.h>
#include "etliInt.h"
#include "etliCmdMesh.h" /* for declaration of mesh internal procedures */
#include "mesh_supp.h" /* for additional mesh support (eg RP_MeshElementType)*/

/* Mesh set subcommand options */
static char *meshSetSubOpts[] = {
//               /* 1 */      "-Title",
//               /* 2 */      "-NNodes", 
//               /* 3 */      "-NMeshPoints",
//               /* 4 */      "-NElemsLink1",
//               /* 5 */      "-NElemsLink2",
//               /* 6 */      "-NElemsLink3",
//               /* 7 */      "-NElemsLink4",
//               /* 8 */      "-NElemsLink5",
//               /* 9 */      "-NElemsTriang1",
//               /* 10 */     "-NElemsTriang2",
//               /* 12 */     "-NElemsTriang3",
//               /* 13 */     "-NElemsTriang4",
//               /* 14 */     "-NElemsTriang5",
//               /* 15 */     "-NElemsQuad1",
//               /* 16 */     "-NElemsQuad3",
//               /* 17 */     "-NElemsTetrah1",
//               /* 18 */     "-NElemsBlock1",
//               /* 19 */     "-TotalNElems",
//               /* 20 */     "-NBCNodes",
//               /* 21 */     "-NLoadedNodes",
//               /* 22 */     "-NMats",
//               /* 23 */     "-NMdfMats",
//               /* 24 */     "-NCompMatsType1",
//               /* 25 */     "-NTimeSteps",
//               /* 26 */     "-NInternalTimeSteps",
//               /* 27 */     "-NExternalTimeSteps",
//               /* 28 */     "-FirstNetIndex",
//               /* 29 */     "-TimeStep",
//               /* 30 */     "-DampingFactor",
//               /* 31 */     "-Beta",
//               /* 32 */     "-Gamma",
//               /* 33 */     "-NStressPointsLink1",         
//               /* 34 */     "-NStressPointsLink2",         
//               /* 35 */     "-NStressPointsLink3",         
//               /* 36 */     "-NStressPointsLink4",         
//               /* 37 */     "-NStressPointsLink5",         
//               /* 38 */     "-NStressPointsTriang1",       
//               /* 39 */     "-NStressPointsTriang2",       
//               /* 40 */     "-NStressPointsTriang3",       
//               /* 41 */     "-NStressPointsTriang4",       
//               /* 42 */     "-NStressPointsTriang5",       
//               /* 43 */     "-NStressPointsQuad1",         
//               /* 44 */     "-NStressPointsQuad3",         
//               /* 45 */     "-NStressPointsTetrah1",       
//               /* 46 */     "-NStressPointsBlock1",        
//               /* 47 */     "-MeshPointCoords",       
//               /* 48 */     "-NodesLink1",               
//               /* 49 */     "-NodesLink2",               
//               /* 50 */     "-NodesLink3",               
//               /* 52 */     "-NodesLink4",               
//               /* 53 */     "-NodesLink5",               
//               /* 54 */     "-NodesTriang1",             
//               /* 55 */     "-NodesTriang2",             
//               /* 56 */     "-NodesTriang3",             
//               /* 57 */     "-NodesTriang4",             
//               /* 58 */     "-NodesTriang5",             
//               /* 59 */     "-NodesQuad1",               
//               /* 60 */     "-NodesQuad3",               
//               /* 61 */     "-NodesTetrah1",             
//               /* 62 */     "-NodesBlock1",              
//               /* 63 */     "-TotalNodalDOF",              
//               /* 64 */     "-NodalDOF",                  
//               /* 65 */     "-BCNodes",                  
//               /* 66 */     "-BCTypes",                  
//               /* 67 */     "-BCDispls",              
//               /* 68 */     "-BCSpringConsts",        
//               /* 69 */     "-LoadedNodes",               
//               /* 70 */     "-Loads",                 
//               /* 71 */     "-Mats",                    
//               /* 72 */     "-MatsLink1",               
//               /* 73 */     "-MatsLink2",               
//               /* 74 */     "-MatsLink3",               
//               /* 75 */     "-MatsLink4",               
//               /* 76 */     "-MatsLink5",               
//               /* 77 */     "-MatsTriang1",             
//               /* 78 */     "-CompMatsTriang2",     
///              /* 79 */     "-MatsTriang3",             
//               /* 80 */     "-MatsTriang4",             
//               /* 81 */     "-MatsTriang5",             
//               /* 82 */     "-MatsQuad1",               
//               /* 83 */     "-CompMatsQuad3",       
//               /* 84 */     "-MatsTetrah1",             
//               /* 85 */     "-MatsBlock1",              
//               /* 86 */     "-NodeTypes",                 
//               /* 87 */     "-NNurbsCurves",               
//               /* 88 */     "-NurbsCurveParams",      
//               /* 89 */     "-NurbsCurves",      
//               /* 90 */     "-GNIs",                      
//               /* 92 */     "-GEIs",                      
//               /* 93 */     "-GEIsLink1",                 
//               /* 94 */     "-GEIsLink2",                 
//               /* 95 */     "-GEIsLink3",                 
//               /* 96 */     "-GEIsLink4",                 
//               /* 97 */     "-GEIsLink5",                 
//               /* 98 */     "-GEIsTriang1",               
//               /* 99 */     "-GEIsTriang2",               
//               /* 100 */    "-GEIsTriang3",               
//               /* 101 */    "-GEIsTriang4",               
//               /* 102 */    "-GEIsTriang5",               
//               /* 103 */    "-GEIsQuad1",                 
//               /* 104 */    "-GEIsQuad3",                 
//               /* 105 */    "-GEIsTetrah1",               
//               /* 106 */    "-GEIsBlock1",                
//               /* 107 */    "-Geom",                      
//               /* 108 */    "-Props",                     
//               /* 119 */    "-CompPropsType1",       
//               /* 110 */    "-Decomp",                  
//               /* 111 */    "-MeshParam",            
//               /* 112 */    "-FEError",                
//               /* 113 */    "-Stresses",              
//               /* 114 */    "-KeywordSet",    
//               /* 115 */    "-SubStructSet",
   /* this are add/itional mesh poroperties which does not correspond 
    *  directly to members of MESH structure.
    */            
//              /* 116 */      "-BBox",
//              /* 117 */      "-element", 
//              /* 118 */      "-IsOfOneElementType",
//              /* 119 */      "-LinkElementsOnly",
//              /* 120 */      "-PlanarElementsOnly",        
//              /* 121 */      "-NSubDoms",
//              /* 122 */      "-SubDoms",        
//              /* 123 */      "-SubDomsLink1",               
//              /* 124 */      "-SubDomsLink2",               
//              /* 125 */      "-SubDomsLink3",               
//              /* 126 */      "-SubDomsLink4",               
//              /* 127 */      "-SubDomsLink5",               
//              /* 128 */      "-SubDomsTriang1",             
//              /* 129 */      "-SubDomsTriang2",             
//              /* 130 */      "-SubDomsTriang3",             
//              /* 131 */      "-SubDomsTriang4",             
//              /* 132 */      "-SubDomsTriang5",             
//              /* 133 */      "-SubDomsQuad1",               
//              /* 134 */      "-SubDomsQuad3",               
//              /* 135 */      "-SubDomsTetrah1",
//              /* 136 */      "-SubDomsBlock1",
                           NULL};
                 
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 SetTitle --     
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        static void 
SetTitle (ClientData clientData, Tcl_Interp *interp, 
               int objc, Tcl_Obj *CONST objv[], int *result,
               Tcl_Obj **resultObjPtr)
{
  MeshObj *meshObjPtr = (MeshObj *) clientData; 

  *resultObjPtr = Tcl_NewStringObj (meshObjPtr->mesh->Title, -1); 
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 SetFirstNetIndex --
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        static void
SetFirstNetIndex (ClientData clientData, Tcl_Interp *interp, 
               int objc, Tcl_Obj *CONST objv[], int *result,
               Tcl_Obj **resultObjPtr)
{
  char error[80];
  int elemType;

  MeshObj *meshObjPtr = (MeshObj *) clientData; 
  MESH *mesh = meshObjPtr->mesh; /* convinience variable */

  /* check if valid number of arguments is supplied for -FirstNetIndex */
  if (objc != 4)
    {
      Tcl_WrongNumArgs (interp, 1, objv, "set -FirstNetIndex elementtype");
      *result = TCL_ERROR;
      return; 
    }
 /* get element type form argument list */
  if (Tcl_GetIntFromObj (interp, objv[3], &elemType) != TCL_OK)
    {
      *result = TCL_ERROR;
      return;
    }
 /* check if element type is a valid type
  * (as specified in e_lib.h)   
  */
 if (elemType < 0 || elemType >= E_NELEM_TYPES)
   {
     sprintf (error, "wrong element type %d : expected integer 0-%d", 
                   elemType,  E_NELEM_TYPES-1); 
     *resultObjPtr = Tcl_NewStringObj (error, -1);
     *result = TCL_ERROR; 
     return;
   } 

 /* everytking is OK, return FirstNetIndex */
 *resultObjPtr = Tcl_NewIntObj (mesh->FirstNetIndex[elemType]);
 return; 
}    

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 SetNodalDOF --
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        static void
SetNodalDOF (ClientData clientData, Tcl_Interp *interp, 
               int objc, Tcl_Obj *CONST objv[], int *result,
               Tcl_Obj **resultObjPtr)
{
  int nodeNumber;
  char error[80];  

  MeshObj *meshObjPtr = (MeshObj *) clientData; 
  MESH *mesh = meshObjPtr->mesh; /* convinience variable */
  
 /* check if valid number of arguments is supplied for -NodalDOF */ 
  if (objc != 4)
    {
      Tcl_WrongNumArgs (interp, 1, objv, "set -NodalDOF nodenumber"); 
      *result = TCL_ERROR;
      return;       
    }

  /* get the node number from arguments list */
  if (Tcl_GetIntFromObj (interp, objv[3], &nodeNumber) != TCL_OK)
    {
      *result = TCL_ERROR;
      return;
    }

  /* check if node number is valid */
  if (nodeNumber<1 || nodeNumber>mesh->NNodes)
    {
      sprintf (error, "wrong node number %d : expected integer 1-%d", 
                    nodeNumber,  mesh->NNodes); 
      *resultObjPtr = Tcl_NewStringObj (error, -1);
      *result = TCL_ERROR; 
      return;
    }

  /* everything is OK, can retutn number of dofs */

  nodeNumber--; /* nodes are numbered from 1 but this is C */
 
  *resultObjPtr = Tcl_NewIntObj (mesh->NodalDOF[nodeNumber]);
  return; 
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 SetBCNodes --
          Return node index for boundary condition node nuber.
 Syntax:      
        etliNurbs  set -BCNodes number
        
 Description:       
       'number' = 1,...,NBCNodes
 Result:
        Return node index where boundary condition of 'number' is set
        or error message if 'number' is invalid.
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        static void
SetBCNodes (ClientData clientData, Tcl_Interp *interp, 
               int objc, Tcl_Obj *CONST objv[], int *result,
               Tcl_Obj **resultObjPtr)
{
  int bcNodeNumber;
  char error[80];  

  MeshObj *meshObjPtr = (MeshObj *) clientData; 
  MESH *mesh = meshObjPtr->mesh; /* convinience variable */
  
 /* check if valid number of arguments is supplied for -BCNodes */ 
  if (objc != 4)
    {
      Tcl_WrongNumArgs (interp, 1, objv, "set -BCNodes number"); 
      *result = TCL_ERROR;
      return;       
    }

  /* get the boundary condition node number from arguments list */
  if (Tcl_GetIntFromObj (interp, objv[3], &bcNodeNumber) != TCL_OK)
    {
      *result = TCL_ERROR;
      return;
    }

  /* check if boundary condition node number is valid */
  if (bcNodeNumber<1 || bcNodeNumber>mesh->NBCNodes)
    {
      sprintf (error, "wrong number %d : expected integer 1-%d", 
                    bcNodeNumber,  mesh->NBCNodes); 
      *resultObjPtr = Tcl_NewStringObj (error, -1);
      *result = TCL_ERROR; 
      return;
    }

  /* everything is OK, can return index of node */

  bcNodeNumber--; /* BC nodes are numbered from 1 but this is C */
 
  *resultObjPtr = Tcl_NewIntObj (mesh->BCNodes[bcNodeNumber]);
  return;    
}


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 SetBCTypes --
       Return boundary type for boundary condition of 'number'.
 Syntax:      
     etliNurbs set -BCTypes number ?-dof dofnumber?

 Description:       
      Return boundary condition type for all degrees of freedom
      or if number of degree of freedom is suplied only for that degree
      of freedom.      
       'number' = 1,...,NBCNodes  
       'dofnumber' = 1,...,L  where L is number of degrees of freedom 
                              at the node.
 Result:
        Return boundary condition type for boundary of 'number' 
        or error message if 'number' is invalid.
 Notes:
       Maximal number of dofs (which is 5 for QUAD3) is hardcoded
       in this function. When adding new element type this must be checked
       and updated when necessary. 
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        static void
SetBCTypes (ClientData clientData, Tcl_Interp *interp, 
               int objc, Tcl_Obj *CONST objv[], int *result,
               Tcl_Obj **resultObjPtr)
{
  int bcNodeNumber;
  int node, numberOfDofs; /* node number and number of DOFs at this node */
  char error[80];  

  static char *subOpts[] = {"-dof", NULL};
  enum subOptsIx   {AllIx, DofIx}; 
  int index;

  MeshObj *meshObjPtr = (MeshObj *) clientData; 
  MESH *mesh = meshObjPtr->mesh; /* convinience variable */
  
 /* check if valid number of arguments is supplied for -BCTypes */ 
  if (objc != 4 && objc != 6)
    {
      Tcl_WrongNumArgs (interp, 1, objv,
                      "set -BCTypes number ?-dof dofnumber?"); 
      *result = TCL_ERROR;
      return;       
    }

  /* get the boundary condition node number from arguments list */
  if (Tcl_GetIntFromObj (interp, objv[3], &bcNodeNumber) != TCL_OK)
    {
      *result = TCL_ERROR;
      return;
    }

  /* check if boundary condition node number is valid */
  if (bcNodeNumber<1 || bcNodeNumber>mesh->NBCNodes)
    {
      sprintf (error, "wrong number %d : expected integer 1-%d", 
                    bcNodeNumber,  mesh->NBCNodes); 
      *resultObjPtr = Tcl_NewStringObj (error, -1);
      *result = TCL_ERROR; 
      return;
    }

   bcNodeNumber--; /* BC nodes are numbered from 1 but this is C */

   node = mesh->BCNodes[bcNodeNumber]; /* get node number of BC node */
   numberOfDofs = mesh->NodalDOF[node-1]; /* get number of dofs for node*/

  /* switch either to case where we have to return list of dof types
   *  or only doftype for particular dof
   */ 

  if (objc == 4) /* returning list of dof types */
    {
      /* max number of dofs is 5 for nodes of QUAD3 */
      /* This must be changed if new element type is added */
      static int maxDofs = 5;
      Tcl_Obj *listPtrPtr[maxDofs]; /* for holding dofs type */
      int i;
      for (i=0; i<numberOfDofs; i++)
        {
          listPtrPtr[i] = Tcl_NewIntObj (*(mesh->BCTypes[bcNodeNumber]+i));
        }
      *resultObjPtr = Tcl_NewListObj (numberOfDofs, listPtrPtr);
      return;
    }
  else /* returning only type for given dof */
    {
      int dofNumber;
      /* check if proper suboption is suplied */
       if (Tcl_GetIndexFromObj (interp, objv[4], subOpts, "suboption", 
                              TCL_EXACT, &index) != TCL_OK)
        {
          *result = TCL_ERROR;
          return;
        }
      /* get dof number */ 
       if (Tcl_GetIntFromObj (interp, objv[5], &dofNumber) != TCL_OK)
         {
           *result = TCL_ERROR;
         }
      /* check if dof number is a valid one */
       if (dofNumber<1 || dofNumber>numberOfDofs)
         {
           sprintf (error, "wrong DOF number %d : expected integer 1-%d", 
                    dofNumber,  numberOfDofs); 
           *resultObjPtr = Tcl_NewStringObj (error, -1);
           *result = TCL_ERROR; 
           return;
         }
       dofNumber--; /* Dof are numbered from 1 but this is C */
      /* everything is ok, can return dof type */ 
       *resultObjPtr = Tcl_NewIntObj (*(mesh->BCTypes[bcNodeNumber]+dofNumber));
       return;
    }
}


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 SetBCDispls --
         Return initial nodal displacement for displacement boundary
         condition type
 Syntax:
      meshObj set -BCDispls bc_number -dof dof_number

 Description:
      Return value of initial displacement for displacement boundary
      condition of given dof at particular boundary condition.
      If specified degree of freedom is not subject to displacement type
      boundary condion  return an error message.
 Side effects:
      None.
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        static void
SetBCDispls (ClientData clientData, Tcl_Interp *interp, 
               int objc, Tcl_Obj *CONST objv[], int *result,
               Tcl_Obj **resultObjPtr)
{
  int bcNumber;
  int dofNumber;
  int node, numberOfDofs; /* node number and number of DOFs at this node */
  char error[80];  

  static char *subOpts[] = {"-dof", NULL};
  enum subOptsIx   {DofIx}; 
  int index;

  MeshObj *meshObjPtr = (MeshObj *) clientData; 
  MESH *mesh = meshObjPtr->mesh; /* convinience variable */
  
 /* check if valid number of arguments is supplied for -BCTypes */ 
  if (objc != 6)
    {
      Tcl_WrongNumArgs (interp, 1, objv,
                      "set -BCDispls number -dof dofnumber"); 
      *result = TCL_ERROR;
      return;       
    }

  /* get the boundary condition number from arguments list */
  if (Tcl_GetIntFromObj (interp, objv[3], &bcNumber) != TCL_OK)
    {
      *result = TCL_ERROR;
      return;
    }

  /* check if boundary condition number is valid */
  if (bcNumber<1 || bcNumber>mesh->NBCNodes)
    {
      sprintf (error, "wrong number %d : expected integer 1-%d", 
                    bcNumber,  mesh->NBCNodes); 
      *resultObjPtr = Tcl_NewStringObj (error, -1);
      *result = TCL_ERROR; 
      return;
    }

   /* check if proper suboption is suplied */
   if (Tcl_GetIndexFromObj (interp, objv[4], subOpts, "suboption", 
                         TCL_EXACT, &index) != TCL_OK)
     {
       *result = TCL_ERROR;
       return;
     }
   /* get dof number */ 
   if (Tcl_GetIntFromObj (interp, objv[5], &dofNumber) != TCL_OK)
     {
       *result = TCL_ERROR;
     }

   bcNumber--; /* boundary conditions are numbered from 1 but this is C */
   node = mesh->BCNodes[bcNumber]; /* get node number of BC node */
   numberOfDofs = mesh->NodalDOF[node-1]; /* get number of dofs for node*/

   /* check if dof number is a valid one */
   if (dofNumber<1 || dofNumber>numberOfDofs)
     {
       sprintf (error, "wrong DOF number %d : expected integer 1-%d", 
                 dofNumber,  numberOfDofs); 
       *resultObjPtr = Tcl_NewStringObj (error, -1);
       *result = TCL_ERROR; 
       return;
     }
   dofNumber--; /* Dof are numbered from 1 but this is C */

   /* check if given node and dof is subject to displacement condition */
   if ((*(mesh->BCTypes[bcNumber]+dofNumber)) != E_BC_NUM_FIXED)
     {
       sprintf (error, "Boundary condition is not of displacement type");
       *resultObjPtr = Tcl_NewStringObj (error, -1);
       *result = TCL_ERROR; 
       return;
     }

   /* everything is ok, can return initial displacement */ 
   *resultObjPtr = Tcl_NewIntObj (*(mesh->BCDispls[bcNumber]+dofNumber));
   return;
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 SetBCSpringConsts --
         Return spring constant for spring oundary condition type.
 Syntax:
      meshObj set -BCSpringConsts bc_node_number -dof dof_number

 Description:
      Return value of spring constant for spring boundary
      condition of given dof at particular boundary condition node.
      If specified degree of freedom is not subject to spring 
      boundary condion return an error message.
 Side effects:
      None.
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        static void
SetBCSpringConsts (ClientData clientData, Tcl_Interp *interp, 
               int objc, Tcl_Obj *CONST objv[], int *result,
               Tcl_Obj **resultObjPtr)
{
  int bcNodeNumber;
  int dofNumber;
  int node, numberOfDofs; /* node number and number of DOFs at this node */
  char error[80];  

  static char *subOpts[] = {"-dof", NULL};
  enum subOptsIx   {DofIx}; 
  int index;

  MeshObj *meshObjPtr = (MeshObj *) clientData; 
  MESH *mesh = meshObjPtr->mesh; /* convinience variable */
  
 /* check if valid number of arguments is supplied for -BCSpringConstants */ 
  if (objc != 6)
    {
      Tcl_WrongNumArgs (interp, 1, objv,
                      "set -BCSpringConstants number -dof dofnumber"); 
      *result = TCL_ERROR;
      return;       
    }

  /* get the boundary condition node number from arguments list */
  if (Tcl_GetIntFromObj (interp, objv[3], &bcNodeNumber) != TCL_OK)
    {
      *result = TCL_ERROR;
      return;
    }

  /* check if boundary condition node number is valid */
  if (bcNodeNumber<1 || bcNodeNumber>mesh->NBCNodes)
    {
      sprintf (error, "wrong number %d : expected integer 1-%d", 
                    bcNodeNumber,  mesh->NBCNodes); 
      *resultObjPtr = Tcl_NewStringObj (error, -1);
      *result = TCL_ERROR; 
      return;
    }

   /* check if proper suboption is suplied */
   if (Tcl_GetIndexFromObj (interp, objv[4], subOpts, "suboption", 
                         TCL_EXACT, &index) != TCL_OK)
     {
       *result = TCL_ERROR;
       return;
     }
   /* get dof number */ 
   if (Tcl_GetIntFromObj (interp, objv[5], &dofNumber) != TCL_OK)
     {
       *result = TCL_ERROR;
     }

   bcNodeNumber--; /* BC nodes are numbered from 1 but this is C */
   node = mesh->BCNodes[bcNodeNumber]; /* get node number of BC node */
   numberOfDofs = mesh->NodalDOF[node-1]; /* get number of dofs for node*/

   /* check if dof number is a valid one */
   if (dofNumber<1 || dofNumber>numberOfDofs)
     {
       sprintf (error, "wrong DOF number %d : expected integer 1-%d", 
                 dofNumber,  numberOfDofs); 
       *resultObjPtr = Tcl_NewStringObj (error, -1);
       *result = TCL_ERROR; 
       return;
     }
   dofNumber--; /* Dof are numbered from 1 but this is C */

   /* check if given node and dof is subject to spring condition */
   if ((*(mesh->BCTypes[bcNodeNumber]+dofNumber)) != E_BC_NUM_SPRING)
     {
       sprintf (error, "Boundary condition is not of spring type");
       *resultObjPtr = Tcl_NewStringObj (error, -1);
       *result = TCL_ERROR; 
       return;
     }
    
   /* everything is ok, can return spring constant */ 
   *resultObjPtr = Tcl_NewIntObj
           (*(mesh->BCSpringConsts[bcNodeNumber]+dofNumber));
   return;
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 SetLoadedNodes --
          Return node index for loaded node of given number. 
 Syntax:      
        etliNurbs  set -LoadedNodes number
        
 Description:       
       'number' = 1,...,NLoadedNodes
 Result:
        Return node index where load of 'number' is applied 
        or error message if 'number' is invalid.
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        static void
SetLoadedNodes (ClientData clientData, Tcl_Interp *interp, 
               int objc, Tcl_Obj *CONST objv[], int *result,
               Tcl_Obj **resultObjPtr)
{
  int loadNumber;
  char error[80];  

  MeshObj *meshObjPtr = (MeshObj *) clientData; 
  MESH *mesh = meshObjPtr->mesh; /* convinience variable */
  
 /* check if valid number of arguments is supplied for -LoadedNodes */ 
  if (objc != 4)
    {
      Tcl_WrongNumArgs (interp, 1, objv, "set -LoadedNodes number"); 
      *result = TCL_ERROR;
      return;       
    }

  /* get the load number from arguments list */
  if (Tcl_GetIntFromObj (interp, objv[3], &loadNumber) != TCL_OK)
    {
      *result = TCL_ERROR;
      return;
    }

  /* check if load number is valid */
  if (loadNumber<1 || loadNumber>mesh->NLoadedNodes)
    {
      sprintf (error, "wrong number %d : expected integer 1-%d", 
                    loadNumber,  mesh->NLoadedNodes); 
      *resultObjPtr = Tcl_NewStringObj (error, -1);
      *result = TCL_ERROR; 
      return;
    }

  /* everything is OK, can return index of node */

  loadNumber--; /* loads are numbered from 1 but this is C */
 
  *resultObjPtr = Tcl_NewIntObj (mesh->LoadedNodes[loadNumber]);
  return;    
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 SetLoads --
          Return list of node loads for given load number. 
 Syntax:      
        etliNurbs  set -Loads number
        
 Description:       
       'number' = 1,...,NLoadedNodes
 Result:
        Return list of nodal loads for  load of 'number'  
        or error message if 'number' is invalid.
        
 Notes:
       Maximal number of dofs (which is 5 for QUAD3) is hardcoded
       in this function. When adding new element type this must be checked
       and updated when necessary. 
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        static void
SetLoads (ClientData clientData, Tcl_Interp *interp, 
               int objc, Tcl_Obj *CONST objv[], int *result,
               Tcl_Obj **resultObjPtr)
{
  int loadNumber;
  char error[80];  

  /* max number of dofs is 5 for nodes of QUAD3 */
  /* This must be changed if new element type is added */
  static int maxDofs = 5;
  Tcl_Obj *listPtrPtr[maxDofs]; /* for holding dofs type */
  int i, node, numberOfDofs;

  MeshObj *meshObjPtr = (MeshObj *) clientData; 
  MESH *mesh = meshObjPtr->mesh; /* convinience variable */
  
 /* check if valid number of arguments is supplied for -Loads */ 
  if (objc != 4)
    {
      Tcl_WrongNumArgs (interp, 1, objv, "set -Loads number"); 
      *result = TCL_ERROR;
      return;       
    }

  /* get the load number from arguments list */
  if (Tcl_GetIntFromObj (interp, objv[3], &loadNumber) != TCL_OK)
    {
      *result = TCL_ERROR;
      return;
    }

  /* check if load number is valid */
  if (loadNumber<1 || loadNumber>mesh->NLoadedNodes)
    {
      sprintf (error, "wrong number %d : expected integer 1-%d", 
                    loadNumber,  mesh->NLoadedNodes); 
      *resultObjPtr = Tcl_NewStringObj (error, -1);
      *result = TCL_ERROR; 
      return;
    }

  /* everything is OK; can fo further */

  /*get the node index for load and number of degrees of freedom in that node*/
  loadNumber--; /* loads are numbered from 1 but this is C */
 
  node = mesh->LoadedNodes[loadNumber]; /* get node number of BC node */
  numberOfDofs = mesh->NodalDOF[node-1]; /* get number of dofs for node*/
   
  /* make list of nodal loads */
  for (i=0; i<numberOfDofs; i++)
    {
      listPtrPtr[i] = Tcl_NewDoubleObj (*(mesh->Loads[loadNumber]+i));
    }

  *resultObjPtr = Tcl_NewListObj (numberOfDofs, listPtrPtr);
  return;
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 SetMeshPointCoords --
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        static void
SetMeshPointCoords (ClientData clientData, Tcl_Interp *interp, 
               int objc, Tcl_Obj *CONST objv[], int *result,
               Tcl_Obj **resultObjPtr)
{
  int pointNumber;
  char error[80];
  Tcl_Obj *listPtrPtr[3]; /* for holding coords */
  int index=0; 
  static char *subOpts[] = {"-xyz", "-xy", "-x", "-y", "-z", NULL};
  enum subOptsIx {XYZIx, XYIx, XIx, YIx, ZIx};  

  MeshObj *meshObjPtr = (MeshObj *) clientData; 
  MESH *mesh = meshObjPtr->mesh; /* convinience variable */

  /* check if valid number of arguments is supplied for -MeshPointCoords */ 
  if (objc <4 || objc >5)
    {
      Tcl_WrongNumArgs (interp, 1, objv, 
                      "set -MeshPointCoords number ?option?"); 
      *result = TCL_ERROR;
      return;       
    }

  /* get the meshpoint number from arguments list */
  if (Tcl_GetIntFromObj (interp, objv[3], &pointNumber) != TCL_OK)
    {
      *result = TCL_ERROR;
      return;
    }
  /* check if mesh point number is valid */
  if (pointNumber<1 || pointNumber>mesh->NMeshPoints)
    {
      sprintf (error, "wrong point number %d : expected integer 1-%d", 
                    pointNumber,  mesh->NMeshPoints); 
      *resultObjPtr = Tcl_NewStringObj (error, -1);
      *result = TCL_ERROR; 
      return;
    }

  pointNumber--; /* mesh points are numbered from 1 but this is C */
  
  /* check if coord suboption is suplied and find option in the
   * list of valid suboptions 
   */
  index = 0; /* this is when no suboption is supplied */
  if (objc == 5) 
    {
      if (Tcl_GetIndexFromObj (interp, objv[4], subOpts, "suboption", 
                              TCL_EXACT, &index) != TCL_OK)
        {
          *result = TCL_ERROR;
          return;
        }
    }

  switch (index)
    {
      case XYZIx:
        {
          listPtrPtr[0] = 
             Tcl_NewDoubleObj (*(mesh->MeshPointCoords[pointNumber]+0)); 
          listPtrPtr[1] = 
             Tcl_NewDoubleObj (*(mesh->MeshPointCoords[pointNumber]+1)); 
          listPtrPtr[2] = 
             Tcl_NewDoubleObj (*(mesh->MeshPointCoords[pointNumber]+2)); 
          *resultObjPtr = Tcl_NewListObj(3, listPtrPtr); 
          return;
        }
      case XYIx:
        {
          listPtrPtr[0] = 
             Tcl_NewDoubleObj (*(mesh->MeshPointCoords[pointNumber]+0)); 
          listPtrPtr[1] = 
             Tcl_NewDoubleObj (*(mesh->MeshPointCoords[pointNumber]+1)); 
          *resultObjPtr = Tcl_NewListObj(2, listPtrPtr); 
          return;
        }
      case XIx: /* return only X coordinate */
        {
          listPtrPtr[0] = 
             Tcl_NewDoubleObj (*(mesh->MeshPointCoords[pointNumber]+0)); 
          *resultObjPtr = Tcl_NewListObj(1, listPtrPtr); 
          return;
        }  
      case YIx: /* return only Y coordinate */
        {
          listPtrPtr[0] = 
             Tcl_NewDoubleObj (*(mesh->MeshPointCoords[pointNumber]+1)); 
          *resultObjPtr = Tcl_NewListObj(1, listPtrPtr); 
          return;
        }
      case ZIx: /* retutn only Z coordinate */
        {
          listPtrPtr[0] = 
             Tcl_NewDoubleObj (*(mesh->MeshPointCoords[pointNumber]+2)); 
          *resultObjPtr = Tcl_NewListObj(1, listPtrPtr); 
          return;
        }
    } /* end of switch(index) */ 
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 SetIsOfOneElementType --
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        static void
SetIsOfOneElementType (ClientData clientData, Tcl_Interp * intetp,
        int objc, Tcl_Obj *CONST objv[], int *result,        
        Tcl_Obj **resultObjPtr)
{
  MeshObj *meshObjPtr = (MeshObj *) clientData; 
  *resultObjPtr = Tcl_NewIntObj (e_IsOfOneElementType (meshObjPtr->mesh));
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 SetLinkElementsOnly --
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        static void
SetLinkElementsOnly (ClientData clientData, Tcl_Interp * intetp,
        int objc, Tcl_Obj *CONST objv[], int *result,        
        Tcl_Obj **resultObjPtr)
{
  MeshObj *meshObjPtr = (MeshObj *) clientData; 
  *resultObjPtr = Tcl_NewIntObj (e_LinkElementsOnly (meshObjPtr->mesh));
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 SetPlanarElementsOnly --
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        static void
SetPlanarElementsOnly (ClientData clientData, Tcl_Interp * intetp,
        int objc, Tcl_Obj *CONST objv[], int *result,        
        Tcl_Obj **resultObjPtr)
{
  MeshObj *meshObjPtr = (MeshObj *) clientData; 
  *resultObjPtr = Tcl_NewIntObj (e_PlanarElementsOnly (meshObjPtr->mesh));
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 Macro for generation Set* procedures which acces int or 
 double mesh members 
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define SET_PROC_SIMPLE(MEMBER, TYPE) \
         static void \
Set##MEMBER (ClientData clientData, Tcl_Interp *interp, \
               int objc, Tcl_Obj *CONST objv[], int *result, \
               Tcl_Obj **resultObjPtr) \
{\
  MeshObj *meshObjPtr = (MeshObj *) clientData; \
  *resultObjPtr = Tcl_New##TYPE##Obj (meshObjPtr->mesh->##MEMBER); \
}
 
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 Generation of Set procedures 
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
 SET_PROC_SIMPLE(NNodes, Int);
 SET_PROC_SIMPLE(NMeshPoints, Int);
 SET_PROC_SIMPLE(NElemsLink1, Int);
 SET_PROC_SIMPLE(NElemsLink2, Int);
 SET_PROC_SIMPLE(NElemsLink3, Int);
 SET_PROC_SIMPLE(NElemsLink4, Int);
 SET_PROC_SIMPLE(NElemsLink5, Int);
 SET_PROC_SIMPLE(NElemsTriang1, Int);
 SET_PROC_SIMPLE(NElemsTriang2, Int);
 SET_PROC_SIMPLE(NElemsTriang3, Int);
 SET_PROC_SIMPLE(NElemsTriang4, Int);
 SET_PROC_SIMPLE(NElemsTriang5, Int);
 SET_PROC_SIMPLE(NElemsQuad1, Int);
 SET_PROC_SIMPLE(NElemsQuad3, Int);
 SET_PROC_SIMPLE(NElemsTetrah1, Int);
 SET_PROC_SIMPLE(NElemsBlock1, Int);
 SET_PROC_SIMPLE(TotalNElems, Int);
 SET_PROC_SIMPLE(NBCNodes, Int);
 SET_PROC_SIMPLE(NLoadedNodes, Int);
 SET_PROC_SIMPLE(NMats, Int);
 SET_PROC_SIMPLE(NMdfMats, Int);
 SET_PROC_SIMPLE(NCompMatsType1, Int);
 SET_PROC_SIMPLE(NTimeSteps, Int);
 SET_PROC_SIMPLE(NInternalTimeSteps, Int);
 SET_PROC_SIMPLE(NExternalTimeSteps, Int);
 
 SET_PROC_SIMPLE(TimeStep, Double);
 SET_PROC_SIMPLE(DampingFactor, Double);
 SET_PROC_SIMPLE(Beta, Double);
 SET_PROC_SIMPLE(Gamma, Double);
         
 SET_PROC_SIMPLE(NStressPointsLink1, Int);
 SET_PROC_SIMPLE(NStressPointsLink2, Int);
 SET_PROC_SIMPLE(NStressPointsLink3, Int);
 SET_PROC_SIMPLE(NStressPointsLink4, Int);
 SET_PROC_SIMPLE(NStressPointsLink5, Int);
 SET_PROC_SIMPLE(NStressPointsTriang1, Int);
 SET_PROC_SIMPLE(NStressPointsTriang2, Int);
 SET_PROC_SIMPLE(NStressPointsTriang3, Int);
 SET_PROC_SIMPLE(NStressPointsTriang4, Int);
 SET_PROC_SIMPLE(NStressPointsTriang5, Int);
 SET_PROC_SIMPLE(NStressPointsQuad1, Int);
 SET_PROC_SIMPLE(NStressPointsQuad3, Int);
 SET_PROC_SIMPLE(NStressPointsTetrah1, Int);
 SET_PROC_SIMPLE(NStressPointsBlock1, Int);
 SET_PROC_SIMPLE(TotalNodalDOF, Int);
#undef SET_PROC_SIMPLE 
        
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 Macro for generation Set* procedures which return element nodes list
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define SET_PROC_NODES(TYPE1, TYPE2, TYPENAME) \
        static void \
SetNodes##TYPE2 (ClientData clientData, Tcl_Interp *interp, \
               int objc, Tcl_Obj *CONST objv[], int *result, \
               Tcl_Obj **resultObjPtr) \
{ \
  int elemNumber; \
  char error[80]; \
  Tcl_Obj *listPtrPtr[E_NNPE_##TYPE1]; /* for holding nodes numbers */ \
  int i; \
  \
  MeshObj *meshObjPtr = (MeshObj *) clientData; \
  MESH *mesh = meshObjPtr->mesh; /* convinience variable */ \
  \
  /* check if valid number of arguments is supplied */ \
  if (objc != 4) \
    { \
      sprintf (error, "set -Nodes%s element", TYPENAME);\
      Tcl_WrongNumArgs (interp, 1, objv, error); \
      *result = TCL_ERROR; \
      return;  \
    } \
 /* check if there are Link1 elements in the mesh */ \
 if (mesh->NElems##TYPE2 < 1) \
   { \
    sprintf (error, "mesh does not contain %s elements", TYPENAME); \
      *resultObjPtr = Tcl_NewStringObj (error, -1); \
      *result = TCL_ERROR;  \
      return; \
   }   \
 /* get element number form argument list */ \
  if (Tcl_GetIntFromObj (interp, objv[3], &elemNumber) != TCL_OK) \
    { \
      *result = TCL_ERROR; \
      return; \
    } \
 /* check if element number is a valid number  
    By convention we assume that elements are numbered from 1.
  */ \
 if (elemNumber < 1 || elemNumber > mesh->NElems##TYPE2) \
   {\
    sprintf (error, "wrong element number %d : expected integer 1-%d", \
                    elemNumber,  mesh->NElems##TYPE2); \
      *resultObjPtr = Tcl_NewStringObj (error, -1); \
      *result = TCL_ERROR;  \
      return; \
   }  \
 elemNumber--; /* as we accesing C array */ \
 for (i=0; i<E_NNPE_##TYPE1; i++) \
    { \
      listPtrPtr[i] = Tcl_NewIntObj (*(mesh->Nodes##TYPE2[elemNumber]+i)); \
    } \
  *resultObjPtr = Tcl_NewListObj (E_NNPE_##TYPE1, listPtrPtr); \
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 Generation of Set procedures 
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
SET_PROC_NODES(LINK1, Link1, "Link1");
SET_PROC_NODES(LINK2, Link2, "Link2");
SET_PROC_NODES(LINK3, Link3, "Link3");
SET_PROC_NODES(LINK4, Link4, "Link4");
SET_PROC_NODES(LINK5, Link5, "Link5");
SET_PROC_NODES(TRIANG1, Triang1, "Triang1");
SET_PROC_NODES(TRIANG2, Triang2, "Triang2");
SET_PROC_NODES(TRIANG3, Triang3, "Triang3");
SET_PROC_NODES(TRIANG4, Triang4, "Triang4");
SET_PROC_NODES(TRIANG5, Triang5, "Triang5");
SET_PROC_NODES(QUAD1, Quad1, "Quad1");
SET_PROC_NODES(QUAD3, Quad3, "Quad3");
SET_PROC_NODES(TETRAH1, Tetrah1, "Tetrah1"); 
SET_PROC_NODES(BLOCK1, Block1, "Block1"); 


#undef SET_PROC_NODES        

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 SetBBox -- 
         Return mesh points bounding box (x1 y1 z1 x2 y2 z2).
 Syntax:
       meshObj set -BBox
       meshObj set -BBox -xyz
       meshObj set -BBox -xy
 Results:
      Depemding on the suboption returns list of four or six coordinates
      of the bounding box points.
 Side effects:
      None. 
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        static void
SetBBox (ClientData clientData, Tcl_Interp *interp, 
               int objc, Tcl_Obj *CONST objv[], int *result,
               Tcl_Obj **resultObjPtr)
{
  int j;
  Tcl_Obj *listPtrPtr[6]; /* for holding coords */
  int index=0; 
  static char *subOpts[] = {"-xyz", "-xy", NULL};
  enum subOptsIx {XYZIx, XYIx};  

  MeshObj *meshObjPtr = (MeshObj *) clientData; 

  /* check if valid number of arguments is supplied for -BBox */ 
  if (objc <3 || objc >4)
    {
      Tcl_WrongNumArgs (interp, 1, objv, "set -BBox ?option?"); 
      *result = TCL_ERROR;
      return;       
    }
  /* check if coord suboption is suplied and find option in the
   * list of valid suboptions 
   */
  index = 0; /* this is when no suboption is supplied */
  if (objc == 4) 
    {
      if (Tcl_GetIndexFromObj (interp, objv[3], subOpts, "suboption", 
                              TCL_EXACT, &index) != TCL_OK)
        {
          *result = TCL_ERROR;
          return;
        }
    }

  switch (index)
    {
      case XYZIx: /* all coordinates */
        {
          for (j=0; j<6; j++)
            {
              listPtrPtr[j] = Tcl_NewDoubleObj (meshObjPtr->bBox[j]); 
            }
          *resultObjPtr = Tcl_NewListObj(6, listPtrPtr); 
          return;
        }
      case XYIx: /* ommit Z coordinate */
        {
          listPtrPtr[0] = Tcl_NewDoubleObj (meshObjPtr->bBox[0]); 
          listPtrPtr[1] = Tcl_NewDoubleObj (meshObjPtr->bBox[1]); 
          listPtrPtr[2] = Tcl_NewDoubleObj (meshObjPtr->bBox[3]); 
          listPtrPtr[3] = Tcl_NewDoubleObj (meshObjPtr->bBox[4]); 
          *resultObjPtr = Tcl_NewListObj(4, listPtrPtr); 
          return;
        }
    } /* end of switch(index) */ 
}

/* declaration of element set low level functions */
static void SetElementNodes (MESH *mesh, int element, Tcl_Obj **resultObjPtr);
static void SetElementVertices (MESH *mesh, int element, Tcl_Obj **resultObjPtr);
static void SetElementNodesCoord (MESH *mesh, int element, Tcl_Obj **resultObjPtr);
static void SetElementVerticesCoord (MESH *mesh, int element, Tcl_Obj **resultObjPtr);

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 SetElement --
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        static void
SetElement (ClientData clientData, Tcl_Interp *interp, 
               int objc, Tcl_Obj *CONST objv[], int *result,
               Tcl_Obj **resultObjPtr)
{
  int elemNumber;
  char error[80];
  int index=0; 
  static char *subOpts[] = {"-type", "-nodes", "-vertices",
                            "-nodescoord", "-verticescoord", NULL};
  enum subOptsIx {TypeIx, NodesIx, VerticesIx, NodescoordIx, 
                  VerticescoordIx};  

                   
  MeshObj *meshObjPtr = (MeshObj *) clientData; 
  MESH *mesh = meshObjPtr->mesh; /* convinience variable */

  /* check if at least element number and suboption is supplied */
  if (objc < 5)
    {
      Tcl_WrongNumArgs (interp, 1, objv,
                 "set -element number -suboption ?args...?");
      *result = TCL_ERROR;
      return; 
    } 

  /* get element number and check if it is valid */ 
  if (Tcl_GetIntFromObj (interp, objv[3], &elemNumber) != TCL_OK)
    {
      *result = TCL_ERROR;
      return;
    }
  if (elemNumber<1 || elemNumber>mesh->TotalNElems)
    {
      sprintf (error, "wrong element number %d : expected integer 1-%d", 
                    elemNumber,  mesh->TotalNElems); 
      *resultObjPtr = Tcl_NewStringObj (error, -1);
      *result = TCL_ERROR; 
      return;
    }

  /* check if valid options is suplied */
  if (Tcl_GetIndexFromObj (interp, objv[4], subOpts, "suboption", 
                           TCL_EXACT, &index) != TCL_OK)
    {
      *result = TCL_ERROR; 
      return; 
    }

  switch (index)
    {
      case TypeIx:
        {        
          if (meshObjPtr->elemType == ETLI_MIXED ) /* mixed element type in
                                                      mesh */
            {
              *resultObjPtr = Tcl_NewIntObj(RP_MeshElementType(mesh,
                                      elemNumber));
            }
          else /* mesh of one element type */
            {
              *resultObjPtr = Tcl_NewIntObj(meshObjPtr->elemType); 
            }     
          break;
        }
      case NodesIx: 
        {
          SetElementNodes (mesh, elemNumber, resultObjPtr); 
          break;  
        }
      case VerticesIx: 
        {
          SetElementVertices (mesh, elemNumber, resultObjPtr); 
          break;  
        }
      case NodescoordIx:
        {
          SetElementNodesCoord (mesh, elemNumber, resultObjPtr); 
          break;  
        }
      case VerticescoordIx:          
        {
          SetElementVerticesCoord (mesh, elemNumber, resultObjPtr); 
          break;  
        }
      default:  /* program never should go here; it is only for safety check */
        {
          char error [100]; 
          snprintf (error, 99, 
                    "Bad suboption %s. Has no switch entry",
                     Tcl_GetStringFromObj (objv[index], NULL)); 
          *resultObjPtr = Tcl_NewStringObj (error, -1); 
          *result = TCL_ERROR; 
        }
    } /* end of switch(index) */ 
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 SetElementNodes --
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/* Maximal number of nodes per element is 9 for QUAD3 element */
#define _ETLI_MAX_NODES 9
        static void
SetElementNodes (MESH *mesh, int element, Tcl_Obj **resultObjPtr)
{
  Tcl_Obj *listPtrPtr[_ETLI_MAX_NODES]; /* for holding nodes numbers */
  int *nodesList; 
  int llength, i;

  RP_MeshElementNodes (mesh, element, &nodesList, &llength);  
  assert (llength <= _ETLI_MAX_NODES); 
  
  for (i=0; i<llength; i++)
    {
      listPtrPtr[i] = Tcl_NewIntObj (nodesList[i]);       
    }
  *resultObjPtr = Tcl_NewListObj (llength, listPtrPtr); 
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 SetElementNodesCoord --
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        static void
SetElementNodesCoord (MESH *mesh, int element, Tcl_Obj **resultObjPtr)
{
  Tcl_Obj *listPtrPtr[3*_ETLI_MAX_NODES]; /* for holding nodes coords,
                                           * 3*_ETLI_MAX_NODES because
                                           * of three coordinates
                                           */
  int *nodesList;
  int llength, i, node;

  RP_MeshElementNodes (mesh, element, &nodesList, &llength);  
  assert (llength <= 3*_ETLI_MAX_NODES); 
  
  for (i=0; i<llength; i++)
    {
      node = nodesList[i]-1; 
      listPtrPtr[3*i] = Tcl_NewDoubleObj (*mesh->MeshPointCoords[node]);       
      listPtrPtr[3*i+1] = Tcl_NewDoubleObj (*(mesh->MeshPointCoords[node]+1));
      listPtrPtr[3*i+2] = Tcl_NewDoubleObj (*(mesh->MeshPointCoords[node]+2));
    }
  *resultObjPtr = Tcl_NewListObj (2*llength, listPtrPtr); 
}
#undef _ETLI_MAX_NODES

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 SetElementVertices --
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/* Maximal number of vertices per element is 8 for BLOCK1 element */
#define _ETLI_MAX_VERTICES 8
        static void
SetElementVertices (MESH *mesh, int element, Tcl_Obj **resultObjPtr)
{
  Tcl_Obj *listPtrPtr[_ETLI_MAX_VERTICES]; /* for holding vertices numbers */
  int *verticesList; 
  int llength, i;

  RP_MeshElementVertices (mesh, element, &verticesList, &llength);  
  assert (llength <= _ETLI_MAX_VERTICES); 
  
  for (i=0; i<llength; i++)
    {
      listPtrPtr[i] = Tcl_NewIntObj (verticesList[i]);       
    }
  *resultObjPtr = Tcl_NewListObj (llength, listPtrPtr); 
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 SetElementVerticesCoord --
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        static void
SetElementVerticesCoord (MESH *mesh, int element, Tcl_Obj **resultObjPtr)
{
  Tcl_Obj *listPtrPtr[3*_ETLI_MAX_VERTICES]; /* for holding nodes coords,
                                           * 3*_ETLI_MAX_NODES because
                                           * of three coordinates
                                           */
  int *verticesList;
  int llength, i, vertex;

  RP_MeshElementVertices (mesh, element, &verticesList, &llength);  
  assert (llength <= 3*_ETLI_MAX_VERTICES); 
  
  for (i=0; i<llength; i++)
    {
      vertex = verticesList[i]; 
      listPtrPtr[3*i] = Tcl_NewDoubleObj ((*mesh->MeshPointCoords[vertex]));       
      listPtrPtr[3*i+1] = Tcl_NewDoubleObj (*(mesh->MeshPointCoords[vertex]+1)); 
      listPtrPtr[3*i+2] = Tcl_NewDoubleObj (*(mesh->MeshPointCoords[vertex]+2)); 
    }
  *resultObjPtr = Tcl_NewListObj (3*llength, listPtrPtr); 
}
#undef _ETLI_MAX_VERTICES

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 SetNSubDoms --
           Return number of subdomains.
 Syntax:
       meshObj set -NSubDoms
 Results:
       Normally return number of subdomains.
       If decomposition data has not been read return error code
       and print error message.
 Side effects:
       None. 
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        static void
SetNSubDoms (ClientData clientData, Tcl_Interp * intetp,
        int objc, Tcl_Obj *CONST objv[], int *result,        
        Tcl_Obj **resultObjPtr)
{
  MeshObj *meshObjPtr = (MeshObj *) clientData; 
  MESH *mesh = meshObjPtr->mesh; /* convinience variable */ 

  /* check if mesh decomposition structure has been initialized */  
  if (mesh->SubStructSet[E_SS_NUM_DECOMP] == FALSE)
    {
      *resultObjPtr = Tcl_NewStringObj
          ("decomp not initialized; use \"get -decomp\" to initialize", -1);
      *result = TCL_ERROR; 
      return; 
    }
  *resultObjPtr = Tcl_NewIntObj (mesh->Decomp.NSubDoms);
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 SetSubDoms --
            Return subdomain index for given element.
 Syntax:
        meshObj set -SubDoms elemNumber

 Results:       
        Normally return subdomain index.
        If element number is invalid or decomposition data has not 
        been read return error code and print an error message
 Side effects:
        None.1 
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        static void
SetSubDoms (ClientData clientData, Tcl_Interp *interp, 
               int objc, Tcl_Obj *CONST objv[], int *result,
               Tcl_Obj **resultObjPtr)
{
  int elemNumber;
  char error[80];
                   
  MeshObj *meshObjPtr = (MeshObj *) clientData; 
  MESH *mesh = meshObjPtr->mesh; /* convinience variable */

  /* check proper number of arguments supplied */
  if (objc != 4)
    {
      Tcl_WrongNumArgs (interp, 1, objv,
                 "set -SubDoms elemnumber");
      *result = TCL_ERROR;
      return; 
    } 

  /* get element number and check if it is valid */ 
  if (Tcl_GetIntFromObj (interp, objv[3], &elemNumber) != TCL_OK)
    {
      *result = TCL_ERROR;
      return;
    }
  if (elemNumber<1 || elemNumber>mesh->TotalNElems)
    {
      sprintf (error, "wrong element number %d : expected integer 1-%d", 
                    elemNumber,  mesh->TotalNElems); 
      *resultObjPtr = Tcl_NewStringObj (error, -1);
      *result = TCL_ERROR; 
      return;
    }
  /* check if mesh decomposition structure has been initialized */  
  if (mesh->SubStructSet[E_SS_NUM_DECOMP] == FALSE)
    {
      *resultObjPtr = Tcl_NewStringObj
          ("decomp not initialized; use \"get -decomp\" to initialize", -1);
      *result = TCL_ERROR; 
      return; 
    }
  
  /* everything OK can returnsubdomain index */
  *resultObjPtr = Tcl_NewIntObj (mesh->Decomp.SubDoms[elemNumber-1]);  
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 SetSubDoms#ELEMTYPE# --
            Return subdomain index for given element of particular type.
 Syntax:
        meshObj set -SubDomsLink1 elemNumber
        ....................................
        meshObj set -SubDomsLink5 elemNumber
        meshObj set -SubDomsTriang1 elemNumber
        ......................................
        meshObj set -SubDomsTriang5 elemNumber
        meshObj set -SubDomsQuad1 elemNumber
        meshObj set -SubDomsQuad3 elemNumber
        meshObj set -SubDomsTetrah1 elemNumber
        meshObj set -SubDomsBlock1 elemNumber

 Results:       
        Normally return subdomain index.
        If element number is invalid, decomposition data has not 
        been read, or mesh does not contain elements of partivular type
        return error code and print an error message

 Side effects:
        None. 
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/* Those functions described above are generated from the macro */

#define SET_PROC_SUBDOM(TYPE1, TYPE2, TYPENAME) /*TYPE1 is not used */ \
        static void \
SetSubDoms##TYPE2 (ClientData clientData, Tcl_Interp *interp, \
               int objc, Tcl_Obj *CONST objv[], int *result, \
               Tcl_Obj **resultObjPtr) \
{ \
  int elemNumber; \
  char error[80]; \
  \
  MeshObj *meshObjPtr = (MeshObj *) clientData; \
  MESH *mesh = meshObjPtr->mesh; /* convinience variable */ \
   \
  /* check if mesh contains particular type of elements */ \
  if ( mesh->NElems##TYPE2 < 1) \
    { \
      sprintf (error, "mesh does not contain elements of type %s", ##TYPENAME); \
      *resultObjPtr = Tcl_NewStringObj (error, -1); \
      *result = TCL_ERROR; \
      return; \
    } \
  /* check proper number of arguments supplied */ \
  if (objc != 4) \
    { \
      sprintf (error, "set -SubDoms%s elemnumber", ##TYPENAME); \
      Tcl_WrongNumArgs (interp, 1, objv, error); \
      *result = TCL_ERROR; \
      return; \
    } \
   \
  /* get element number and check if it is valid */ \
  if (Tcl_GetIntFromObj (interp, objv[3], &elemNumber) != TCL_OK) \
    { \
      *result = TCL_ERROR; \
      return; \
    } \
  if (elemNumber<1 || elemNumber>mesh->NElems##TYPE2) \
    { \
      sprintf (error, "wrong element number %d : expected integer 1-%d", \
                    elemNumber,  mesh->NElems##TYPE2); \
      *resultObjPtr = Tcl_NewStringObj (error, -1); \
      *result = TCL_ERROR;  \
      return; \
    } \
  /* check if mesh decomposition structure has been initialized */   \
  if (mesh->SubStructSet[E_SS_NUM_DECOMP] == FALSE) \
    { \
      *resultObjPtr = Tcl_NewStringObj \
          ("decomp not initialized; use \"get -decomp\" to initialize", -1); \
      *result = TCL_ERROR; \
      return; \
    } \
   \
  /* everything OK can returnsubdomain index */ \
  *resultObjPtr = Tcl_NewIntObj (mesh->Decomp.SubDoms##TYPE2[elemNumber-1]);  \
}

SET_PROC_SUBDOM(LINK1, Link1, "Link1");
SET_PROC_SUBDOM(LINK2, Link2, "Link2");
SET_PROC_SUBDOM(LINK3, Link3, "Link3");
SET_PROC_SUBDOM(LINK4, Link4, "Link4");
SET_PROC_SUBDOM(LINK5, Link5, "Link5");
SET_PROC_SUBDOM(TRIANG1, Triang1, "Triang1");
SET_PROC_SUBDOM(TRIANG2, Triang2, "Triang2");
SET_PROC_SUBDOM(TRIANG3, Triang3, "Triang3");
SET_PROC_SUBDOM(TRIANG4, Triang4, "Triang4");
SET_PROC_SUBDOM(TRIANG5, Triang5, "Triang5");
SET_PROC_SUBDOM(QUAD1, Quad1, "Quad1");
SET_PROC_SUBDOM(QUAD3, Quad3, "Quad3");
SET_PROC_SUBDOM(TETRAH1, Tetrah1, "Tetrah1"); 
SET_PROC_SUBDOM(BLOCK1, Block1, "Block1"); 

#undef SET_PROC_SUBDOM 

/* meshSetProc is an array of pointer to Set* functions */
static void (*meshSetProc[]) (ClientData clientData, 
               Tcl_Interp *interp, 
               int objc, 
               Tcl_Obj *CONST objv[],
               int *result,
               Tcl_Obj **resultObjPtr)={
//               /* 1 */      SetTitle,
//               /* 2 */      SetNNodes, 
//               /* 3 */      SetNMeshPoints,
//               /* 4 */      SetNElemsLink1,
//               /* 5 */      SetNElemsLink2,
//               /* 6 */      SetNElemsLink3,
//               /* 7 */      SetNElemsLink4,
//               /* 8 */      SetNElemsLink5,
//               /* 9 */      SetNElemsTriang1,
//               /* 10 */     SetNElemsTriang2,
//               /* 12 */     SetNElemsTriang3,
//               /* 13 */     SetNElemsTriang4,
//               /* 14 */     SetNElemsTriang5,
//               /* 15 */     SetNElemsQuad1,
//               /* 16 */     SetNElemsQuad3,
//               /* 17 */     SetNElemsTetrah1,
//               /* 18 */     SetNElemsBlock1,
//               /* 19 */     SetTotalNElems,
//               /* 20 */     SetNBCNodes,
//               /* 21 */     SetNLoadedNodes,
//               /* 22 */     SetNMats,
//               /* 23 */     SetNMdfMats,
//               /* 24 */     SetNCompMatsType1,
//               /* 25 */     SetNTimeSteps,
//               /* 26 */     SetNInternalTimeSteps,
//               /* 27 */     SetNExternalTimeSteps,
//               /* 28 */     SetFirstNetIndex,
//               /* 29 */     SetTimeStep,
//               /* 30 */     SetDampingFactor,
//               /* 31 */     SetBeta,
//               /* 32 */     SetGamma,
//               /* 33 */     SetNStressPointsLink1,         
//               /* 34 */     SetNStressPointsLink2,         
//               /* 35 */     SetNStressPointsLink3,         
//               /* 36 */     SetNStressPointsLink4,         
//               /* 37 */     SetNStressPointsLink5,         
//               /* 38 */     SetNStressPointsTriang1,       
//               /* 39 */     SetNStressPointsTriang2,       
//               /* 40 */     SetNStressPointsTriang3,       
//               /* 41 */     SetNStressPointsTriang4,       
//               /* 42 */     SetNStressPointsTriang5,       
//               /* 43 */     SetNStressPointsQuad1,         
//               /* 44 */     SetNStressPointsQuad3,         
//               /* 45 */     SetNStressPointsTetrah1,       
//               /* 46 */     SetNStressPointsBlock1,        
//               /* 47 */     SetMeshPointCoords,       
//               /* 48 */     SetNodesLink1,               
//               /* 49 */     SetNodesLink2,               
//               /* 50 */     SetNodesLink3,               
//               /* 52 */     SetNodesLink4,               
//               /* 53 */     SetNodesLink5,               
//               /* 54 */     SetNodesTriang1,             
//               /* 55 */     SetNodesTriang2,             
//               /* 56 */     SetNodesTriang3,             
//               /* 57 */     SetNodesTriang4,             
//               /* 58 */     SetNodesTriang5,             
//               /* 59 */     SetNodesQuad1,               
//               /* 60 */     SetNodesQuad3,               
//               /* 61 */     SetNodesTetrah1,             
//               /* 62 */     SetNodesBlock1,              
//               /* 63 */     SetTotalNodalDOF,              
//               /* 64 */     SetNodalDOF,                  
//               /* 65 */     SetBCNodes,                  
//               /* 66 */     SetBCTypes,                  
 //              /* 67 */     SetBCDispls,              
//               /* 68 */     SetBCSpringConsts,        
//               /* 69 */     SetLoadedNodes,               
//               /* 70 */     SetLoads,                 
//               /* 71 */     SetMats,                    
//               /* 72 */     SetMatsLink1,               
//               /* 73 */     SetMatsLink2,               
//               /* 74 */     SetMatsLink3,               
//               /* 75 */     SetMatsLink4,               
//               /* 76 */     SetMatsLink5,               
//               /* 77 */     SetMatsTriang1,             
//               /* 78 */     SetCompMatsTriang2,     
 //              /* 79 */     SetMatsTriang3,             
//               /* 80 */     SetMatsTriang4,             
//               /* 81 */     SetMatsTriang5,             
//               /* 82 */     SetMatsQuad1,               
//               /* 83 */     SetCompMatsQuad3,       
//               /* 84 */     SetMatsTetrah1,             
//               /* 85 */     SetMatsBlock1,              
//               /* 86 */     SetNodeTypes,                 
//               /* 87 */     SetNNurbsCurves,               
//               /* 88 */     SetNurbsCurveParams,      
//               /* 89 */     SetNurbsCurves,      
//               /* 90 */     SetGNIs,                      
//               /* 92 */     SetGEIs,                      
//               /* 93 */     SetGEIsLink1,                 
//               /* 94 */     SetGEIsLink2,                 
//               /* 95 */     SetGEIsLink3,                 
//               /* 96 */     SetGEIsLink4,                 
//               /* 97 */     SetGEIsLink5,                 
//               /* 98 */     SetGEIsTriang1,               
//               /* 99 */     SetGEIsTriang2,               
//               /* 100 */    SetGEIsTriang3,               
//               /* 101 */    SetGEIsTriang4,               
//               /* 102 */    SetGEIsTriang5,               
//               /* 103 */    SetGEIsQuad1,                 
//               /* 104 */    SetGEIsQuad3,                 
//               /* 105 */    SetGEIsTetrah1,               
//               /* 106 */    SetGEIsBlock1,                
//               /* 107 */    SetGeom,                      
//               /* 108 */    SetProps,                     
//               /* 119 */    SetCompPropsType1,       
//               /* 110 */    SetDecomp,                  
//               /* 111 */    SetMeshParam,            
//               /* 112 */    SetFEError,                
//               /* 113 */    SetStresses,              
//               /* 114 */    SetKeywordSet,    
//               /* 115 */    SetSubStructSet,
//               /* 116 */    SetBBox,
//               /* 117 */    SetElement, 
//               /* 118 */    SetIsOfOneElementType,
//               /* 119 */    SetLinkElementsOnly,
//               /* 120 */    SetPlanarElementsOnly,
//               /* 121 */    SetNSubDoms,
//               /* 122 */    SetSubDoms,        
//               /* 123 */    SetSubDomsLink1,               
//               /* 124 */    SetSubDomsLink2,               
//               /* 125 */    SetSubDomsLink3,               
//               /* 126 */    SetSubDomsLink4,               
//               /* 127 */    SetSubDomsLink5,               
//               /* 128 */    SetSubDomsTriang1,             
//               /* 129 */    SetSubDomsTriang2,             
//               /* 130 */    SetSubDomsTriang3,             
//               /* 131 */    SetSubDomsTriang4,             
//               /* 132 */    SetSubDomsTriang5,             
//               /* 133 */    SetSubDomsQuad1,               
//               /* 134 */    SetSubDomsQuad3,               
//               /* 135 */    SetSubDomsTetrah1,             
//               /* 135 */    SetSubDomsBlock1             
                           }; 

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 EtliMeshObjSet -- 
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        CmdReturn*
EtliMeshObjSet (ClientData clientData, Tcl_Interp *interp, 
              int objc, Tcl_Obj *CONST objv[])
{
  int index;
  int result = TCL_OK; 
  CmdReturn *returnStructPtr = NULL;
  Tcl_Obj *resultObjPtr = NULL;

  MeshObj *meshObjPtr = (MeshObj *) clientData; 
  MESH *mesh = meshObjPtr->mesh; /* convinience variable */

  /* allocate the space for the return structure */
  /* Use Tcl memory allocators to make possible monitor memory usage */ 
  returnStructPtr = (CmdReturn *)  Tcl_Alloc (sizeof (CmdReturn));


  /* check if mesh has been initialized */
  if (mesh == (MESH*) NULL)   
    {
      resultObjPtr = Tcl_NewStringObj 
          ("mesh not initialized; use get subcommand to initialize", -1); 
      result = TCL_ERROR; 
      goto done;
    }
  /* find option in the list of valid options for this subcommad */
  if (Tcl_GetIndexFromObj (interp, objv[2], meshSetSubOpts, 
                          "options", TCL_EXACT, &index) != TCL_OK)
    {
      result = TCL_ERROR;
      goto done;
    }

  /* calling appropriate set function */
  meshSetProc[index](clientData, interp, objc, objv, 
                      &result, &resultObjPtr); 
  done:
  /* initialize return structure */
     returnStructPtr->object = resultObjPtr;
     returnStructPtr->status = result;

  return returnStructPtr;
}

/* Modifications:
 * $Id: etliCmdMeshSet.c,v 1.1.1.1 2002/08/04 13:19:11 peteri Exp $
 */ 
