/********************************************************************
 etliInit.c - miscellaneous subroutines for Etli or Tk main() 
 ********************************************************************/

 /* $Id: etliInit.c,v 1.1.1.1 2002/08/04 13:19:11 peteri Exp $ */


#include "stdio.h"

#include "etliInt.h"

#define ETLI_VERSION   VERSION  /* VERSION is defined in configure.in */

#define NEED_TCL_MAJOR    8
#define NEED_TCL_MINOR    2

static char *etli_argv0 = "this program"; /* default program name */

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 Etli_Init --
 
      Etli package initialization.

 Results:     
      
      The return value is normally TCL_OK; If run with older
      version  of Tcl/Tk than required then TCL_ERROR is returned
      and an error message will be left in interp->result.
      
 Side effects:    

      Initialize package commands
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        int
Etli_Init (Tcl_Interp *interp)
{
  static int firstTime = 0; /* 0,1 valued flag */

  if (firstTime == 0)
    {
      int tcl_major = atoi (TCL_VERSION);
      char *dot = strchr (TCL_VERSION, '.');
      int tcl_minor = atoi(dot + 1);

      if (tcl_major < NEED_TCL_MAJOR ||
            (tcl_major == NEED_TCL_MAJOR && tcl_minor < NEED_TCL_MINOR))
         {
           sprintf (interp->result, 
              "%s compiled with Tcl %d.%d but needs at least Tcl %d.%d\n",
              etli_argv0, tcl_major, tcl_minor,
              NEED_TCL_MAJOR, NEED_TCL_MINOR); 
           return (TCL_ERROR);
         }            
      if (Tcl_PkgRequire (interp, "Tcl", TCL_VERSION, 0) == NULL)
        {
           return (TCL_ERROR); 
        }     
      if (Tcl_PkgProvide (interp, "Etli", ETLI_VERSION) != TCL_OK)
        {
          return (TCL_ERROR);
        }    
    }

  /* initialize commands */
  Etli_InitMeshCmd (interp); /* add mesh cmd to interpreter */
  Etli_InitMaterialCmd (interp); /* add material cmd to interpreter */
  Etli_InitCompositeCmd (interp); /* add composite cmd to interpretr */
  Etli_InitNurbsCmd (interp); /* add nurbs cmd to interpreter */
  Etli_InitGeomCmd (interp); /* add geom cmd to interpreter */
  //Etli_Init2DVecPoint (interp); /* add 2D vectors and points to interpreter */
  //Etli_Init3DVecPoint (interp); /* add 3D vectors and points to interpreter */
  Etli_InitElibInterCmd (interp); /* add commands to acces E_lib internals */
  
  firstTime = 1; 
  return (TCL_OK);
}    


/* Modifications:
 * $Id: etliInit.c,v 1.1.1.1 2002/08/04 13:19:11 peteri Exp $
 */ 
