/********************************************************************
 etliInt.h --
           Internal Etli header file 

 Roman Putanowicz
 SECT Research Group, Heriot-Watt University Edinburgh
 ********************************************************************/

/* $Id: etliInt.h,v 1.1.1.1 2002/08/04 13:19:11 peteri Exp $ */
 
#ifndef _ETLIINT_H
#define _ETLIINT_H

#include "e_lib.h" /* for E-Lib library */

#include "tcl.h"
#include "tk.h"
#include "etliPort.h" /* for EXPORT */


#define ETLI_MAX_NURBS_DEGREE 6 /* maximal degree of supported nurbs */


/*
 * This piece of code is stolen from "Tcl/Tk for Real Programmers"
 * by Clif Flynt, Chapter 12, 12.3.1
 *
 * the CmdReturn structure is used by the subroutines to
 * pass back success/failure code and a Tcl_Obj result
 *
 */

typedef struct cmd_return
{
  int status;
  Tcl_Obj *object;
} CmdReturn;  


/* The cmdDefinition  structure describes the minimum and 
 * maximum number of expected arguments for the subcommand
 * including cmd and subcomands names), and a usage message
 * to return if the argument iciunt is outside the expected
 * range
 */
typedef struct cmd_def
{
  char *usage;
  int minArgCnt;
  int maxArgCnt;
} cmdDefinition;


/*
 * nurbs object --
 */  
typedef struct nurbs_obj
{
  NURBSCURVE *nurbs; 
} NurbsObj; 


EXTERN EXPORT(int, Etli_InitMeshCmd) (Tcl_Interp *interp);
EXTERN EXPORT(int, Etli_InitElibInterCmd) (Tcl_Interp *interp);
EXTERN EXPORT(int, Etli_InitMaterialCmd) (Tcl_Interp *interp); 
EXTERN EXPORT(int, Etli_InitCompositeCmd) (Tcl_Interp *interp); 
EXTERN EXPORT(int, Etli_InitNurbsCmd) (Tcl_Interp *interp); 
EXTERN EXPORT(int, Etli_InitGeomCmd) (Tcl_Interp *interp); 
EXTERN EXPORT(int, Etli_Init2DVecPoint) (Tcl_Interp *interp); 
EXTERN EXPORT(int, Etli_Init3DVecPoint) (Tcl_Interp *interp); 

#endif /* _ETLIINT_H */
 
 
/* Modifications:
 * $Log: etliInt.h,v $
 * Revision 1.1.1.1  2002/08/04 13:19:11  peteri
 * Imported using TkCVS
 *
 */
