/********************************************************************
  mesh_supp.h  --
            MESH interface routines 
 
 Roman Putanowicz <romek@sect.mce.hw.ac.uk>
 SECT Research Group, Heriot-Watt University Edinburgh    
 ********************************************************************/

 /* $Id: mesh_supp.h,v 1.1.1.1 2002/08/04 13:19:11 peteri Exp $ */

#ifndef _MESH_SUPP_H
#define _MESH_SUPP_H

#include "e_lib.h" /* for MESH  */

/* For function documentation see the end of file */

/* Bounding box of for mesh points */
int RP_MeshPointsBBox (MESH *mesh, double *low, double *high); 

/* Element type for given element net index */
int RP_MeshElementType (MESH *mesh, int netIndex); 

/* List of element nodes */
int RP_MeshElementNodes (MESH *mesh, int netIndex, 
                            int **nodeList, int *llength);
/* List of element vertices */
int RP_MeshElementVertices (MESH *mesh, int netIndex, 
                            int **vertexList, int *llength);

#endif /* _MESH_SUPP_H */

/* Modifications:
 * $Log: mesh_supp.h,v $
 * Revision 1.1.1.1  2002/08/04 13:19:11  peteri
 * Imported using TkCVS
 *
 */ 


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 NAME
    RP_MeshPointsBBox - calculate bounding box of mesh points  

 PLATFORM
    Not tested 
 
 SYNOPIS
    #include "mesh_supp.h"
    int RP_MeshPointsBBox (MESH *mesh, double *low, double *high);

 DESCRIPTION
    Calculate coordinates of two extreme points on of bounding box
    (bounding cube in 3D) for a given mesh. Bounding box is related
    to mesh points.
    The arguments 'low' and 'high' should point to arrays large enough
    to hold 3 elements each (x,y and z coordiates). 
   
 RETURN VALUE
    Return 1 on succes, 0 if any error was encountered.
   
 SEE ALSO
    RP_BoundingBox2D(1), RP_BoundingBox3D(1)
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 NAME
    RP_MeshElementType - return element type for given net index  

 SYNOPIS
    #include "mesh_supp.h"
    int RP_MeshElementType (MESH *mesh, int netIndex);

 DESCRIPTION
    Find element type index (as specified  by macros 'E_ELEM_TYPE_*')
    by search over 'FirstNetIndex' array.
    
 RETURN VALUE
    Return element type index or -1 if 'netIndex' is out of range
    1 to 'TotalNElems'.
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 NAME
    RP_MeshElementNodes - return list of elemetn nodes and list lenght  
                          for given element netIndex

 SYNOPIS
    #include "mesh_supp.h"
    int RP_MeshElementType (MESH *mesh, int netIndex,
                            int **nodeList, int* length);

 DESCRIPTION
    Given element 'netIndex' this function  set 'nodeList'
    to point to list  of element nodes and  llength is set to
    list length. The memory for list is managed by mesh
    object and must not be modified or freed.
    
 RETURN VALUE
    Return 1 on success or -1 if 'netIndex' is out of range
    1 : 'TotalNElems'.

 SEE ALSO 
    RP_MeshElementVertices(1)   
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 NAME
    RP_MeshElementVertices - return list of elemetn nodes and list 
                  lenght for given element netIndex

 SYNOPIS
    #include "mesh_supp.h"
    int RP_MeshElementType (MESH *mesh, int netIndex,
                            int **vertexList, int* llength);

 DESCRIPTION
    Given element 'netIndex' this function  set 'nodeList'
    to point to list  of element vertices and  'llength' is set to
    list length. The memory for list is managed by mesh
    object and must not be modified or freed.

 RETURN VALUE
    Return 1 on success or -1 if 'netIndex' is out of range
    1 : 'TotalNElems'.

 SEE ALSO 
    RP_MeshElementNodes(1)   
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
