 /********************************************************************
 nurbs_supp.c 
            NURBSCURVE interface routines 
 
 Roman Putanowicz <romek@sect.mce.hw.ac.uk>
 SECT Research Group, Heriot-Watt University Edinburgh    
 ********************************************************************/

 /* $Id: nurbs_supp.c,v 1.1.1.1 2002/08/04 13:19:11 peteri Exp $ */

#include "nurbs_supp.h"
#include <assert.h>

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 RP_InitNurbs --
        Allocate memory for NURBSCURVE structure; 
        
 Results:       
        Pointer to allocated memory. 
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        NURBSCURVE*
RP_InitNurbs ()
{
   return ((NURBSCURVE*) e_calloc (1, sizeof (NURBSCURVE))); 
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 RP_EmptyNurbs --
       Allocates memory for members of NURBSCURVE of given degree.
       Allocated memory is continous i.e. the functions
       e_alloc_cont_*_matr are used. 

       At thr moment only members CntlPoints, Degree, EndPoint1
       and EndPoint2 of NURBSCURVE are considered. Usage of
       other members is currently not supported.

 Arguments:     
        nurbs - pointer NURBSCURVE structure
        degree - nurbs degree (actualy only degree=3 supported)

 Results:
        NONE:
        
 Side effects:
        Allocate memory for nurbs control points coords and weight.
        Set Degree of nurbs to degree.
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        void 
RP_EmptyNurbs (NURBSCURVE *nurbs, int degree)
{
 /* currently only nuebs of degree=3 supported */
  assert (degree == 3);     

  nurbs->Degree = degree; 
 /* allocate memory for degre+1 control point coordintates (x,y,z) and
    weight w */   
  nurbs->CntlPoints = e_alloc_cont_d_matr (degree+1, 4);   
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 RP_CleanNurbs --
       Free memory allocated for members of NURBSCURVE structure.

       At thr moment only members CntlPoints, Degree, EndPoint1
       and EndPoint2 of NURBSCURVE are considered. Usage of
       other members is currently not supported.

 Results:
       NONE

 Side effects:
       Free memory allocated for structure members. 
       Set CntlPoints to NULL; 
       Set Degree member to -1;
       Set EndPoint1 and EndPoint2 to -1; 
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        void 
RP_CleanNurbs (NURBSCURVE *nurbs)
{
    e_free_cont_d_matr (nurbs->Degree, nurbs->CntlPoints); 
    nurbs->CntlPoints = NULL; 
    nurbs->Degree = -1; 
    nurbs->EndPoint1 = -1; 
    nurbs->EndPoint2 = -1;         
}


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 RP_FreeNurbs --
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        void
RP_FreeNurbs (NURBSCURVE *nurbs)
{
  if (nurbs->CntlPoints != NULL)
    {
      RP_CleanNurbs (nurbs);
    }
  e_free ((void *) nurbs); 
}    


/* Modifications:
 * $Log: nurbs_supp.c,v $
 * Revision 1.1.1.1  2002/08/04 13:19:11  peteri
 * Imported using TkCVS
 *
 */ 
