#include "adv_mpar.h"
#include "adv_config.h"
#include "e_lib.h"
#include "mtrf.h"

double AdvGetMeshParam(double *pos, int ctrl, BG_DATA *bgd,
                       int nedge, EDGE *edges, double **coords,
                       double *mpr, double h)
{
  double   param, weight;
  int      i, j, n;
  double   xp, yp, dx, dy, d;
  MTRF     mtrf;

  param = 0.0;

  if(ctrl == CTRL_MPR)
  {
    weight = 0.0;
    for(i = 0; i < nedge; i++)
    {
      for(j = 0; j < edges[i].nen; j++)
      {
        n = edges[i].en[j]-1;

        xp = coords[n][0];
        yp = coords[n][1];

        dx = xp-pos[0];
        dy = yp-pos[1];

        if(E_ABS(dx) < CLOSE_NODE && E_ABS(dy) < CLOSE_NODE)
        {
          return(mpr[n]);
        }
        else
        {
          d = 1.0 / (dx*dx + dy*dy);
          param += d * mpr[n];
          weight += d;
        }
      }
    }

    param /= weight;
  }
  /* background control with possible sources */
  else if(ctrl == CTRL_BACKGROUND || ctrl == CTRL_SOURCES)
  {
    mtrf.used = FALSE;
    param = get_bg_mpr(bgd, pos, ctrl, &mtrf);
  }
  else if(ctrl == CTRL_UNIFORM)
  {
		param = h;
	}
  else
  {
    e_GeneralError("AdvGetMeshParam: invalid control type !");
  }

  /* check for safety */
  if(param < TOLERANCE)
  {
    e_GeneralError("AdvGetMeshParam: zero mesh parameter !");
  }

  return param;

}




