#include "clean.h"

void remove_unconnected_nodes(MESH *m)
{
  int *ch;                      /* checking table */
  int *tbl;                     /* translation table */
  int i, s;                     /* counters */
  int n;                        /* new node counter */
  int nn;                       /* number of new nodes */
  double **coords;              /* new coords matrix */
  double *mpr;                  /* new mpr vector */

  /* validation of mesh */
  if (!e_IsOfOneElementType(m))
    e_GeneralError("remove_unconnected_nodes: only TRIANG1 or QUAD1 meshes");
  if (m->NElemsTriang1 + m->NElemsQuad1 != m->TotalNElems)
    e_GeneralError("remove_unconnected_nodes: only TRIANG1 or QUAD1 meshes");

  /* allocation of tables */
  ch = e_alloc_i_vect(m->NNodes);
  tbl = e_alloc_i_vect(m->NNodes);

  /* initialization */
  for (i = 0; i < m->NNodes; i++)
    ch[i] = FALSE;

  /* building the tables */
  for (i = 0; i < m->NElemsTriang1; i++)
    for (s = 0; s < 3; s++)
      ch[m->NodesTriang1[i][s] - 1] = TRUE;
  for (i = 0; i < m->NElemsQuad1; i++)
    for (s = 0; s < 4; s++)
      ch[m->NodesQuad1[i][s] - 1] = TRUE;

  /* make translation table */
  n = 1;
  for (i = 0; i < m->NNodes; i++)
  {
    if (!ch[i])
      tbl[i] = 0;
    else
    {
      tbl[i] = n;
      n++;
    }
  }

  /* trivial exit, nothing to do */
  if (n - 1 == m->NMeshPoints)
  {
    e_free(tbl);
    e_free(ch);
    return;
  }

  /* clean up coords matrix */
  nn = n - 1;
  coords = e_alloc_cont_d_matr(nn, 3);
  for (i = 0; i < m->NNodes; i++)
    if (ch[i])
      for (s = 0; s < 3; s++)
        coords[tbl[i] - 1][s] = m->MeshPointCoords[i][s];
  e_free_cont_d_matr(m->NNodes, m->MeshPointCoords);
  m->MeshPointCoords = coords;
  e_free(m->NodalDOF);
  m->NodalDOF = e_alloc_i_vect(nn);
  for (i = 0; i < nn; i++)
    m->NodalDOF[i] = 3;
  m->TotalNodalDOF = 3 * nn;

  /* translate nodestriang1 */
  for (i = 0; i < m->NElemsTriang1; i++)
    for (s = 0; s < 3; s++)
      m->NodesTriang1[i][s] = tbl[m->NodesTriang1[i][s] - 1];

  /* translate nodesquad1 */
  for (i = 0; i < m->NElemsQuad1; i++)
    for (s = 0; s < 4; s++)
      m->NodesQuad1[i][s] = tbl[m->NodesQuad1[i][s] - 1];

  /* clean loads and BCs */
  for (i = 0; i < m->NLoadedNodes; i++)
    m->LoadedNodes[i] = tbl[m->LoadedNodes[i] - 1];
  for (i = 0; i < m->NBCNodes; i++)
    m->BCNodes[i] = tbl[m->BCNodes[i] - 1];

  /* clean up gni's */
  e_free(m->GNIs);
  m->GNIs = e_alloc_i_vect(nn);
  for (i = 0; i < nn; i++)
    m->GNIs[i] = i + 1;

  /* adjust nodal meshparams */
  if (m->SubStructSet[E_SS_NUM_MESHPARAM])
    if (m->MeshParam.NodalMeshParamsSet)
    {
      mpr = e_alloc_d_vect(m->NMeshPoints);
      for (i = 0; i < m->NMeshPoints; i++)
        mpr[tbl[i]] = m->MeshParam.NodalMeshParams[i];
      
      e_free(m->MeshParam.NodalMeshParams);
      m->MeshParam.NodalMeshParams=mpr;
    }
  
  /* last, set the number of points */
  m->NNodes = m->NMeshPoints = nn;

  /* free-up */
  e_free(tbl);
  e_free(ch);
  return;
}
