/********************************************************************
 cmb_lib.c 
        Set of combinatorics algorithms
 
 Roman Putanowicz <romek@sect.mce.hw.ac.uk>
 SECT Research Group, Heriot-Watt University, Edinburgh    
 ********************************************************************/

/* $Id: cmb.c,v 1.1 2001/04/19 16:03:22 peteri Exp $ */

#include "cmb.h"
#include "e_lib.h"
#include <assert.h>

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 cmb_NextKSubsetLex --
        Return next k-subset of n-set in lexicographic order. 

 Arguments:
        n - number of elements in the set.
        k - number of elements in desired subset
        next - to be set to 0 before the first call for an new sequence
               and to 1 otherwise
        a - subset element array (must be allocated by user);       

 Return value:
        1 if the subset is not the last one
        0 otherwise        
 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
        int
cmb_NextKSubsetLex (int n, int k, int next, int *a)
{
  static int h;
  static int m;
  int j;

  assert (a != NULL); 
  if (n<k || k<=0)
  {
    e_GeneralError ("cmb_NextKSubsetLex : Invalid input n=%d or k=%d\n",
                    n, k);
  }
          
  if (next == 0)
  {
    h=k;
    m=0; 
  }
  else 
  {
    if (m < n-h)
    {
      h=0; 
    }
    h++;
    m = a[k-h] + 1; 
  }
  for (j=0; j<h; j++)
  {
    a[k+j-h] = m+j;
  }

  return (a[0] != n-k); 
} /* end of cmb_NextKSubset */


/* Modifications:
 * $Log: cmb.c,v $
 * Revision 1.1  2001/04/19 16:03:22  peteri
 * Combinatorial functions.
 *
 * Revision 1.1.1.1  2001/04/09 22:52:27  peteri
 * Imported using TkCVS
 *
 * Revision 1.1  2001/02/14 17:45:11  peteri
 * Initial revision
 *
 */ 
            
