/* ////////////////////////////////////////////////////////////////////////// */
/*                                                                            */
/*                      Control Mesh Parameter Functions                      */
/*                                (Biao Cheng)                                */
/*                                                                            */
/* ////////////////////////////////////////////////////////////////////////// */

#include "e_lib.h"
#include "cntlprm.h"

extern int GetKeyword(char *line, char *word, int *LineNumber,
                      char *FullPathName, FILE *fp);

static int 	g_LineNumber;
static char *g_FullPathName;
static FILE *g_fp;

#define CNTL_KW_NCONTROL_SETS    "NCONTROL_SETS"
#define CNTL_KW_NCONTROL_POINTS  "NCONTROL_POINTS"
#define CNTL_KW_COORDINATES      "COORDINATES"
#define CNTL_KW_INNER_RADIUS     "INNER_RADIUS"
#define CNTL_KW_OUTER_RADIUS     "OUTER_RADIUS"
#define CNTL_KW_MESH_PARAMETER   "MESH_PARAMETER"

#define CNTL_EXTN  "ctl"

static int WordMatchesCntlParamKeyword(char *Keyword, char *word);
static int IsACntlParamKeyword(char *word);
static void GetNControlSets(PARAMCNTLSETS *ParamCntlSets, char *line);
static void GetNControlPoints(PARAMCNTL *ParamCntls, char *line);
static void GetInnerRadius(PARAMCNTL *ParamCntls, int j, char *line);
static void GetOuterRadius(PARAMCNTL *ParamCntls, int j, char *line);
static void GetCoordinates(PARAMCNTL *ParamCntls, int j, char *line);
static void GetParameter(PARAMCNTL *ParamCntls, int j, char *line);
static char *OpenInputFile(char *FileName, char *extension, FILE **fp);

static int C_GetKeyword(char *line, char *word)
{
  return GetKeyword(line, word, &g_LineNumber, g_FullPathName, g_fp);
}

/* ////////////////////////////////////////////////////////////////////////// */

void GetCntlMeshParams(PARAMCNTLSETS **ParamCntlSets, char *CntlFile)
{
   int j,k;

   char line[E_LINE_MAX + 1];
   char word[E_LINE_MAX + 1];


   g_LineNumber = 0;
   g_FullPathName = OpenInputFile(CntlFile,CNTL_EXTN,&g_fp);

   *ParamCntlSets = (PARAMCNTLSETS *)e_calloc(1,sizeof(PARAMCNTLSETS));

   C_GetKeyword(line,word);
   WordMatchesCntlParamKeyword(CNTL_KW_NCONTROL_SETS,word);
   GetNControlSets(*ParamCntlSets,line);

   (*ParamCntlSets)->ParamCntls
       = (PARAMCNTL *)e_calloc((*ParamCntlSets)->NCntlSets,sizeof(PARAMCNTL));

   for (k = 0; k < (*ParamCntlSets)->NCntlSets; k++)
   {
      C_GetKeyword(line,word);
      WordMatchesCntlParamKeyword(CNTL_KW_NCONTROL_POINTS,word);
      GetNControlPoints(&(*ParamCntlSets)->ParamCntls[k],line);

      (*ParamCntlSets)->ParamCntls[k].InnerRadius
          = e_alloc_d_vect((*ParamCntlSets)->ParamCntls[k].NCntlPoints);

      (*ParamCntlSets)->ParamCntls[k].OuterRadius
          = e_alloc_d_vect((*ParamCntlSets)->ParamCntls[k].NCntlPoints);

      (*ParamCntlSets)->ParamCntls[k].Coords
          = e_alloc_frag_d_matr
            ((*ParamCntlSets)->ParamCntls[k].NCntlPoints, E_NDIMS);

      (*ParamCntlSets)->ParamCntls[k].Param
          = e_alloc_d_vect((*ParamCntlSets)->ParamCntls[k].NCntlPoints);

      for (j = 0; j < (*ParamCntlSets)->ParamCntls[k].NCntlPoints; j++)
      {
         C_GetKeyword(line,word);
         WordMatchesCntlParamKeyword(CNTL_KW_COORDINATES,word);
         GetCoordinates(&(*ParamCntlSets)->ParamCntls[k],j,line);

         C_GetKeyword(line,word);
         WordMatchesCntlParamKeyword(CNTL_KW_INNER_RADIUS,word);
         GetInnerRadius(&(*ParamCntlSets)->ParamCntls[k], j, line);

         C_GetKeyword(line,word);
         WordMatchesCntlParamKeyword(CNTL_KW_OUTER_RADIUS,word);
         GetOuterRadius(&(*ParamCntlSets)->ParamCntls[k],j,line);

         C_GetKeyword(line,word);
         WordMatchesCntlParamKeyword(CNTL_KW_MESH_PARAMETER,word);
         GetParameter(&(*ParamCntlSets)->ParamCntls[k],j,line);
      }

   }

   if (e_GetLine(line,E_LINE_MAX,&g_LineNumber,g_fp) != EOF)
      e_FileLineError(g_LineNumber,g_FullPathName,"undefined keyword");

   fclose(g_fp);
   e_free(g_FullPathName);
}


/* ////////////////////////////////////////////////////////////////////////// */


static void GetNControlSets(PARAMCNTLSETS *ParamCntlSets, char *line)
{
   if (sscanf(line,"%*s %d", &(ParamCntlSets->NCntlSets)) == 1)
   {
      if (ParamCntlSets->NCntlSets < 1)
         e_FileLineError(g_LineNumber,g_FullPathName,
                         "ill-defined number of parameter control sets");
   }
   else
   {
      e_FileLineError(g_LineNumber,g_FullPathName,
                      "cannot read number of parameter control sets");
   }
}


/* ////////////////////////////////////////////////////////////////////////// */


static void GetNControlPoints(PARAMCNTL *ParamCntls, char *line)
{
   if (sscanf(line,"%*s %d",&(ParamCntls->NCntlPoints)) == 1)
   {
      if (ParamCntls->NCntlPoints < 1)
         e_FileLineError(g_LineNumber, g_FullPathName,
                         "ill-defined number of parameter control points");
   }
   else
   {
      e_FileLineError(g_LineNumber, g_FullPathName,
                      "cannot read number of parameter control points");
   }
}


/* ////////////////////////////////////////////////////////////////////////// */


static void GetInnerRadius(PARAMCNTL *ParamCntls, int j, char *line)
{
   if (sscanf(line,"%*s %lf",&(ParamCntls->InnerRadius[j])) == 1)
   {
      if(ParamCntls->InnerRadius[j] < 0.0)
         e_FileLineError(g_LineNumber,g_FullPathName,
                                  "ill-defined inner radius");
   }
   else
   {
      e_FileLineError(g_LineNumber,g_FullPathName,
                      "cannot read inner radius");
   }
}


/* ////////////////////////////////////////////////////////////////////////// */


static void GetOuterRadius(PARAMCNTL *ParamCntls, int j, char *line)
{
   if (sscanf(line,"%*s %lf",&ParamCntls->OuterRadius[j]) == 1)
   {
      if(!(ParamCntls->OuterRadius[j] > ParamCntls->InnerRadius[j]))
         e_FileLineError(g_LineNumber,g_FullPathName,
                         "ill-defined outer radius");
   }
   else
   {
      e_FileLineError(g_LineNumber,g_FullPathName,
                      "cannot read outer radius");
   }
}


/* ////////////////////////////////////////////////////////////////////////// */


static void GetCoordinates(PARAMCNTL *ParamCntls, int j, char *line)
{
   if (sscanf(line,"%*s %lf %lf %lf",&(ParamCntls->Coords[j][0]), 
                                 &(ParamCntls->Coords[j][1]), 
                                 &(ParamCntls->Coords[j][2]) ) != 3)
         e_FileLineError(g_LineNumber,g_FullPathName,
                            "cannot read coordinates");
  }


/* ////////////////////////////////////////////////////////////////////////// */


static void GetParameter(PARAMCNTL *ParamCntls, int j, char *line)
{
   if (sscanf(line,"%*s %lf",&(ParamCntls->Param[j])) == 1)
   {
      if(!(ParamCntls->Param[j] > 0.0))
         e_FileLineError(g_LineNumber,g_FullPathName,
                         "ill-defined mesh parameter");
   }
   else
   {
      e_FileLineError(g_LineNumber,g_FullPathName,
                      "cannot read mesh parameter");
   }
}


/* ////////////////////////////////////////////////////////////////////////// */


static int WordMatchesCntlParamKeyword(char *Keyword, char *word)
{
   if (strcmp(Keyword,word) != 0)
   {
      if (IsACntlParamKeyword(word))
         e_FileLineError(g_LineNumber,g_FullPathName,"keyword out of sequence");
      else
         e_FileLineError(g_LineNumber,g_FullPathName,"undefined keyword");

      return 0;
   }
   else
      return 1;
}


/* ////////////////////////////////////////////////////////////////////////// */


static int IsACntlParamKeyword(char *word)
{
   if (   !strcmp(CNTL_KW_NCONTROL_SETS,word)
       || !strcmp(CNTL_KW_NCONTROL_POINTS,word)
       || !strcmp(CNTL_KW_COORDINATES,word)
       || !strcmp(CNTL_KW_INNER_RADIUS,word)
       || !strcmp(CNTL_KW_OUTER_RADIUS,word)
       || !strcmp(CNTL_KW_MESH_PARAMETER,word) )
   {
      return TRUE;
   }
   else
   {
      return FALSE;
   }
}



/* ////////////////////////////////////////////////////////////////////////// */

void FreeCntlMeshParams(PARAMCNTLSETS *ParamCntlSets)
{
   int i;


   if (ParamCntlSets == NULL)
      return;

   for(i = 0; i < ParamCntlSets->NCntlSets; i++)
   {
      e_free(ParamCntlSets->ParamCntls[i].InnerRadius);
      e_free(ParamCntlSets->ParamCntls[i].OuterRadius);
      e_free(ParamCntlSets->ParamCntls[i].Param);
      e_free_frag_d_matr(ParamCntlSets->ParamCntls[i].NCntlPoints, 
                         ParamCntlSets->ParamCntls[i].Coords      );
   }

   e_free(ParamCntlSets);
}

/* ////////////////////////////////////////////////////////////////////////// */
/* OpenInputFile: To open an input file. The extension "extension" is added
   to the filename argument.                                                  */

/* The return value is a pointer to the full file pathname if the file is
   opened successfully, else NULL.                                            */


static char *OpenInputFile(char *FileName, char *extension, FILE **fp)
{
   int FullPathNameLength;
   char *FullPathName;


   FullPathNameLength = strlen(FileName) + strlen(extension) + 1;
   FullPathName = e_alloc_c_vect(FullPathNameLength + 1);

   if (FullPathName == NULL)
      return NULL;

   strcpy(FullPathName,FileName);
   strcat(FullPathName,".");
   strcat(FullPathName,extension);

   if ((*fp = fopen(FullPathName,"r")) == NULL)
   {
      e_FileError(FullPathName,"cannot open file");
      e_free(FullPathName);
      return NULL;
   }
   else
      return FullPathName;
}


