/* ////////////////////////////////////////////////////////////////////////// */
/*                                                                            */
/*                  Quadrilateral Mesh Enhancement Functions                  */
/*                                                                            */
/* ////////////////////////////////////////////////////////////////////////// */
#include <e_lib.h>
#include "boundary.h"
#include "nodelem.h"


/* FIXME: there is memory overrun bug in the code,
 * In Q_Adjust the NumNodes+Q_Work_Space size is used to increment vectors,
 * however NumNodes can increase as well, so memcpy in e_incr_i_vect overruns,
 * but it is not harmful if the number of new points is not bigger than
 * the memory increment.
 */

/* ////////////////////////////////////////////////////////////////////////// */

#define Q_SMALL_MEM_INCREM          20
#define Q_LARGE_MEM_INCREM          200

#define ACT_NODES_MEM_INCREM        Q_LARGE_MEM_INCREM
#define INTER_NODES_MEM_INCREM      Q_LARGE_MEM_INCREM

#define CornerNodes                 5
#define Q_Tolerance_10              1.0e-10
#define Q_Nurbs_Tole                0.001
#define Numer_C_Nodes               5
#define Q_FAILED_FRONTS_MEM_INCREM  Q_SMALL_MEM_INCREM
#define Q_Max_Number                1.0e+10
#define Q_Node_Relax_Fac            1.5
#define APEX_NODES_MEM_INCREM       Q_LARGE_MEM_INCREM
#define Q_Boundary_Elem_Relax_Fac   1.2
#define Q_Max_Angel_BoundaryElems   2.88
#define Q_Tolerance_2               1.0e-2
#define INTERSECT_NODES_MEM_INCREM  Q_SMALL_MEM_INCREM
#define Q_Pn_Node_Relax_Fac         0.2
#define Q_ELEMENTS_MEM_INCREM       Q_LARGE_MEM_INCREM
#define Q_Max_Neg_Number            -1.0e+10
#define Q_Mesh_Gene_Relax_Fac       0.55
#define Q_Tolerance_5               1.0e-5
#define Max_Elems_Per_Node          15
#define Q_Node_WorkSpace            100
#define Q_Flag_Check_Neg_Number     -1.0e+19
#define Q_EN_ELEMENTS_MEM_INCREM    Q_ELEMENTS_MEM_INCREM
#define Q_EN_NODES_MEM_INCREM       ACT_NODES_MEM_INCREM
#define P_E_MEM_INCREM              Q_LARGE_MEM_INCREM
#define Q_Max_Side_Angle_Re_Ang     160.0
#define Q_PI_Degree                 180.0
#define Q_Max_Mid_Angle_Re_Ang      100.0
#define Q_Flag_Neg_Number           -1.0e+20
#define Q_Angle_Scale               100.0
#define Q_Max_Angle_Flat_Elem       177.0

/* ////////////////////////////////////////////////////////////////////////// */

struct a_node
{
  int n;
  double x;
  double y;
  int type;
};


struct b_node
{
  int n;
  double x;
  double y;
  double a;
};


struct front
{
  int n;
  int fn;
  int bn;
};


struct q_element
{
  int n;
  int fn;
  int sn;
  int tn;
  int ft;
};


/* struct to terminate the advanced fronts */
struct l_front
{
  int n;
  int fn;
  int bn;
  struct l_front *next;
  struct l_front *prev;
};


typedef struct l_front *listptr;


struct sub_front
{
  int n;
  struct l_front *front;
  struct sub_front *next;
  int aapx;
};

typedef struct sub_front *subptr;


struct p_node
{
  int n;
  double x;
};


struct p_e
{
  int point_elem[Max_Elems_Per_Node];
};

/* ////////////////////////////////////////////////////////////////////////// */

static void Q_ReadMesh(MESH *SubMesh, struct a_node **Quad_Nodes,
                       struct q_element **Quad_Elements,
                       int *NumNodes, int *NumQuads);
static int Q_En_boundary(int i, struct p_e *pe,
                         struct q_element *Quad_Elements);
static int Q_En_check_bound_number(int **ang_num, int le);
static void Q_reduce(struct a_node *Quad_Nodes, struct q_element
                     *Quad_Elements, int NumNodes, int NumQuads);
static void Q_setele(struct p_e *pe, int temp_size,
                     struct q_element *Quad_Elements,
                     struct a_node *Quad_Nodes, int NumQuads, int NumNodes);
static void Q_check(struct p_e *pe, struct a_node *Quad_Nodes, int NumNodes);
static void Q_adjust(int *m1, struct a_node **Quad_Nodes, struct q_element
                     **Quad_Elements, int **new_n, int **old, int *NumNodes,
                     int *NumQuads, int temp_size);
static int Q_p_elmt(int i, struct p_e *pe, struct a_node *Quad_Nodes,
                    struct q_element
                    *Quad_Elements, int NumQuads, int NumNodes,
                    int temp_size);
static int Q_e_elmt(int j, int fn, int bn, struct p_e *pe,
                    struct q_element *Quad_Elements,
                    struct a_node *Quad_Nodes, int NumQuads, int NumNodes,
                    int temp_size);
static int Q_lenpe(struct p_e *ptr);
static void Q_insertpe(int se, struct p_e *ptr);
static void Q_q_refine(int i, double *x, double *y, struct p_e *pe,
                       struct q_element *Quad_Elements,
                       struct a_node *Quad_Nodes);
static void Q_iso_refine(int i, double *x, double *y, struct p_e *pe,
                         struct q_element *Quad_Elements,
                         struct a_node *Quad_Nodes);
static void Q_re_angle(int i, double *x, double *y, struct p_e *pe,
                       struct q_element *Quad_Elements,
                       struct a_node *Quad_Nodes);
static void Q_get_number(int i, int i2, int *fn, int *tn, int *ft,
                         struct q_element *Quad_Elements);
static void Q_divide_point(double x1, double y1, double x2, double y2,
                           double x3, double y3, double *xx, double *yy);
static void Q_center_point(double x1, double y1, double x2, double y2,
                           double *x, double *y, double *a);
static void Q_new_q(int n, int bn, int fn, struct q_element *Quad_Elements);
static double Q_g_angle(double x1, double y1, double x2, double y2, double x3,
                        double y3, double x6, double y6);
static double Q_cos_angle(double x1, double y1, double x2, double y2,
                          double x3, double y3);
static void Q_flat_elem(int i, struct p_e *pe,
                        struct q_element **Quad_Elements,
                        struct a_node **Quad_Nodes, int *NumNodes,
                        int *NumQuads, int Q_NumBoundaryNodes, int temp_size);
static int Q_opposite_a(int i, int id, struct q_element *Quad_Elements);
static void Q_En_check_right(struct a_node *Quad_Nodes,
                             struct q_element *Quad_Elements, int NumQuads);
static void Q_ConvertRefinedMesh(MESH *RfndMesh, int NBoundaryNodes,
                                 struct a_node *Quad_Nodes,
                                 struct q_element *Quad_Elements,
                                 int NumNodes, int NumQuads);
static int Q_OptiPoint(int elem, int id2, int id3, struct p_e *pe,
                       struct q_element *Quad_Elements);
static int Q_right(double x1,double y1,double x2,double y2,double x3,double y3);
static void Q_Creat_Node(int i, double x,double y,struct a_node **act_nodes);
void Q_Creat_Quad(int i,int first,int second,int third,int fourth, 
                  struct q_element **q_elements);
static struct a_node *b_incr_act_node_vect(unsigned int length, struct a_node *vector,
                                           unsigned int MemIncrem);
static struct q_element *b_incr_q_element_vect(unsigned int length, 
                                               struct q_element *vector, 
                                               unsigned int MemIncrem);
static void seglen(double x1,double y1, double x2, double y2, double *pL);
static void ArrangeBoundaryNodes(MESH *mesh);



/* ////////////////////////////////////////////////////////////////////////// */

void QuadEnhancement(MESH *RfndMesh)
{

  int NBoundaryNodes = 0;
  int i, j, k, e2, itn, number, m1, i1;
  double xc, yc, x, y;
  int fn, bn, sn, tn, ft;

  int Q_Refine_Number_Iteration = 5;

  struct q_element *Quad_Elements;
  struct a_node *Quad_Nodes;
  int NumQuads, NumNodes;
  struct p_e *pe = 0;
  int *new_n, *old;
  int temp_size;

  if(RfndMesh->NElemsQuad1 != RfndMesh->TotalNElems)
  {
    printf("\nQuad enhancement works with quad elements only!\n");
    return;
  }

  ArrangeBoundaryNodes(RfndMesh);
  
  Q_ReadMesh(RfndMesh, &Quad_Nodes, &Quad_Elements, &NumNodes, &NumQuads);

  temp_size = NumNodes + Q_Node_WorkSpace;
  pe = (struct p_e *) e_calloc(temp_size, sizeof(struct p_e));
  old = e_alloc_i_vect(temp_size);
  new_n = e_alloc_i_vect(temp_size);

  Q_reduce(Quad_Nodes, Quad_Elements, NumNodes, NumQuads);
  Q_setele(pe, temp_size, Quad_Elements, Quad_Nodes, NumQuads, NumNodes);

  for (k = 0; k < RfndMesh->NMeshPoints; k++)
  {
    if (Q_En_boundary(k + 1, pe, Quad_Elements))
      NBoundaryNodes++;
  }

  for (itn = 1; itn < Q_Refine_Number_Iteration; itn++)
  {

    Q_reduce(Quad_Nodes, Quad_Elements, NumNodes, NumQuads);
    Q_setele(pe, temp_size, Quad_Elements, Quad_Nodes, NumQuads, NumNodes);

    for (i = 1; i <= NumNodes; i++)
      if (i > NBoundaryNodes)
        if (Q_lenpe(pe + i) == 2)
          j =
            Q_p_elmt(i, pe, Quad_Nodes, Quad_Elements, NumQuads, NumNodes,
                     temp_size);

    Q_adjust(&m1, &Quad_Nodes, &Quad_Elements, &new_n, &old, &NumNodes,
             &NumQuads, temp_size);
    Q_setele(pe, temp_size, Quad_Elements, Quad_Nodes, NumQuads, NumNodes);

    for (i = 1; i <= NumQuads; i++)
    {
      if (Quad_Elements[i].fn > 0)
      {
        j = 1;
        fn = Quad_Elements[i].fn;
        bn = Quad_Elements[i].tn;
        if (fn > NBoundaryNodes && bn > NBoundaryNodes)
          j = Q_e_elmt(i, fn, bn, pe, Quad_Elements, Quad_Nodes,
                       NumQuads, NumNodes, temp_size);
        if (j == 1)
        {
          fn = Quad_Elements[i].sn;
          bn = Quad_Elements[i].ft;
          if (fn > NBoundaryNodes && bn > NBoundaryNodes)
            j = Q_e_elmt(i, fn, bn, pe, Quad_Elements, Quad_Nodes,
                         NumQuads, NumNodes, temp_size);
        }
      }
    }


    for (i = 1; i <= NumNodes; i++)
      if (i > NBoundaryNodes)
        if (Q_lenpe(pe + i) == 2)
          j =
            Q_p_elmt(i, pe, Quad_Nodes, Quad_Elements, NumQuads, NumNodes,
                     temp_size);

    Q_adjust(&m1, &Quad_Nodes, &Quad_Elements, &new_n, &old, &NumNodes,
             &NumQuads, temp_size);
    Q_setele(pe, temp_size, Quad_Elements, Quad_Nodes, NumQuads, NumNodes);



    for (j = 0; j < 2; j++)
    {
      for (i = 1; i <= NumNodes; i++)
      {
        if (i > NBoundaryNodes)
        {
          if (Quad_Nodes[i].x > Q_Flag_Check_Neg_Number)
          {
            Q_q_refine(i, &xc, &yc, pe, Quad_Elements, Quad_Nodes);
            Quad_Nodes[i].x = xc;
            Quad_Nodes[i].y = yc;
          }
        }
      }
    }


    for (i = 1; i <= NumNodes; i++)
    {
      if (i > NBoundaryNodes)
      {
        if (Quad_Nodes[i].x > Q_Flag_Check_Neg_Number)
        {
          Q_iso_refine(i, &xc, &yc, pe, Quad_Elements, Quad_Nodes);
          Quad_Nodes[i].x = xc;
          Quad_Nodes[i].y = yc;
        }
      }
    }


    for (i = NBoundaryNodes + 1; i <= NumNodes; i++)
    {
      number = Q_lenpe(pe + i);
      if (number == 3 && Quad_Nodes[i].x > Q_Flag_Check_Neg_Number)
      {
        Q_re_angle(i, &x, &y, pe, Quad_Elements, Quad_Nodes);
        Quad_Nodes[i].x = x;
        Quad_Nodes[i].y = y;
        if (x < Q_Flag_Check_Neg_Number)
        {
          Quad_Nodes[i].x = x;
          Quad_Nodes[i].y = y;
          Q_setele(pe, temp_size, Quad_Elements, Quad_Nodes, NumQuads,
                   NumNodes);
        }
        else
        {
          Quad_Nodes[i].x = (x + Quad_Nodes[i].x) / 2;
          Quad_Nodes[i].y = (Quad_Nodes[i].y + y) / 2;
        }
      }
    }


    for (i = 1; i <= NBoundaryNodes; i++)
    {
      if (Q_lenpe(pe + i) == 1)
      {
        j = (pe + i)->point_elem[0];
        Q_flat_elem(j, pe, &Quad_Elements, &Quad_Nodes, &NumNodes,
                    &NumQuads, NBoundaryNodes, temp_size);

/* The following smoothing is added in by Biao in 26/03/98. If two flat elements are together and need refine, it will generate two same points on the same line if the new element is still with straight interior line. */

        for (i1 = 1; i1 <= NumNodes; i1++)
        {
          if (i1 > NBoundaryNodes)
          {
            if (Quad_Nodes[i1].x > Q_Flag_Check_Neg_Number)
            {
              Q_iso_refine(i1, &xc, &yc, pe, Quad_Elements, Quad_Nodes);
              Quad_Nodes[i1].x = xc;
              Quad_Nodes[i1].y = yc;
            }
          }
        }


      }
    }



    e2 = 0;

    for (i = 1; i <= NumQuads; i++)
    {
      if (Quad_Elements[i].fn > 0)
      {
        e2++;
        fn = Quad_Elements[i].fn;
        sn = Quad_Elements[i].sn;
        tn = Quad_Elements[i].tn;
        ft = Quad_Elements[i].ft;
        if (Quad_Nodes[fn].x < Q_Flag_Check_Neg_Number ||
            Quad_Nodes[sn].x < Q_Flag_Check_Neg_Number ||
            Quad_Nodes[tn].x < Q_Flag_Check_Neg_Number ||
            Quad_Nodes[ft].x < Q_Flag_Check_Neg_Number)
          e_GeneralError("error in function refine");
      }
    }

    Q_adjust(&m1, &Quad_Nodes, &Quad_Elements, &new_n, &old, &NumNodes,
             &NumQuads, temp_size);
    Q_setele(pe, temp_size, Quad_Elements, Quad_Nodes, NumQuads, NumNodes);
    Q_En_check_right(Quad_Nodes, Quad_Elements, NumQuads);

  }

  e_free(pe);
  e_free(old);
  e_free(new_n);

  Q_ConvertRefinedMesh(RfndMesh, NBoundaryNodes, Quad_Nodes,
                       Quad_Elements, NumNodes, NumQuads);

  e_free(Quad_Nodes);
  e_free(Quad_Elements);

}


/* ////////////////////////////////////////////////////////////////////////// */


static void Q_ReadMesh(MESH *SubMesh, struct a_node **Quad_Nodes,
                       struct q_element **Quad_Elements,
                       int *NumNodes, int *NumQuads)
{

  int i, j;


  (*Quad_Elements) =
    (struct q_element *) e_calloc(SubMesh->NElemsQuad1 +
                                  Q_EN_ELEMENTS_MEM_INCREM,
                                  sizeof(struct q_element));

  (*Quad_Nodes) =
    (struct a_node *) e_calloc(SubMesh->NMeshPoints + Q_EN_NODES_MEM_INCREM,
                               sizeof(struct a_node));

  *NumQuads = SubMesh->NElemsQuad1;
  *NumNodes = SubMesh->NMeshPoints;


  for (i = 1; i <= *NumQuads; i++)
  {
    (*Quad_Elements)[i].n = i;
    (*Quad_Elements)[i].fn = SubMesh->NodesQuad1[i - 1][0];
    (*Quad_Elements)[i].sn = SubMesh->NodesQuad1[i - 1][1];
    (*Quad_Elements)[i].tn = SubMesh->NodesQuad1[i - 1][2];
    (*Quad_Elements)[i].ft = SubMesh->NodesQuad1[i - 1][3];
  }

  for (j = 1; j <= *NumNodes; j++)
  {
    (*Quad_Nodes)[j].n = j;
    (*Quad_Nodes)[j].x = SubMesh->MeshPointCoords[j - 1][0];
    (*Quad_Nodes)[j].y = SubMesh->MeshPointCoords[j - 1][1];
  }
}


/* ////////////////////////////////////////////////////////////////////////// */
/* angle of the boundary nodes 1 for boundary node */


static int Q_En_boundary(int i, struct p_e *pe,
                         struct q_element *Quad_Elements)
{
  int le, jj, w, p1, p2;
  int **ang_num;

  le = Q_lenpe((pe + i));

  if (le < 2)
    return 1;

  ang_num = e_alloc_cont_i_matr(le, 2);

  for (jj = 0; jj < le; jj++)
  {
    w = (pe + i)->point_elem[jj];

    if (Quad_Elements[w].fn == i)
    {
      p1 = Quad_Elements[w].sn;
      p2 = Quad_Elements[w].ft;
    }
    else if (Quad_Elements[w].sn == i)
    {
      p1 = Quad_Elements[w].fn;
      p2 = Quad_Elements[w].tn;
    }
    else if (Quad_Elements[w].tn == i)
    {
      p1 = Quad_Elements[w].sn;
      p2 = Quad_Elements[w].ft;
    }
    else if (Quad_Elements[w].ft == i)
    {
      p1 = Quad_Elements[w].fn;
      p2 = Quad_Elements[w].tn;
    }
    else
      e_GeneralError("wrong data in the Q_En_count_Angle");
    ang_num[jj][0] = p1;
    ang_num[jj][1] = p2;
  }

  if (Q_En_check_bound_number(ang_num, le))
  {
    e_free_cont_i_matr(le, ang_num);
    return 1;                   /* for boundary nodes */
  }
  else
  {
    e_free_cont_i_matr(le, ang_num);
    return 0;                   /* for interior nodes */
  }
}


/* ////////////////////////////////////////////////////////////////////////// */


static int Q_En_check_bound_number(int **ang_num, int le)
{
  int j, i, jj, id = 0;

  for (j = 0; j < le; j++)
  {
    for (jj = 0; jj < 2; jj++)
    {
      for (i = 0; i < le; i++)
      {
        if (i != j)
        {
          if (ang_num[i][0] == ang_num[j][jj]
              || ang_num[i][1] == ang_num[j][jj])
            id++;
        }
      }
    }
  }

  if (id == 2 * le)
    return 0;
  else
    return 1;
}


/* ////////////////////////////////////////////////////////////////////////// */
/* replace one node with another in case of they are too close each other */


static void Q_reduce(struct a_node *Quad_Nodes, struct q_element
                     *Quad_Elements, int NumNodes, int NumQuads)
{
  int i, j, k;
  double x, y;
  for (i = 1; i <= NumNodes; i++)
  {
    x = Quad_Nodes[i].x;
    y = Quad_Nodes[i].y;
    for (j = i + 1; j <= NumNodes; j++)
    {
      if (Quad_Nodes[j].x > Q_Flag_Check_Neg_Number)
      {
        if (fabs(Quad_Nodes[j].x - x) < Q_Tolerance_10 &&
            fabs(Quad_Nodes[j].y - y) < Q_Tolerance_10)
        {
          Quad_Nodes[j].x = Q_Flag_Neg_Number;

          for (k = 1; k <= NumQuads; k++)
          {
            if (Quad_Elements[k].fn == j)
              Quad_Elements[k].fn = i;
            if (Quad_Elements[k].sn == j)
              Quad_Elements[k].sn = i;
            if (Quad_Elements[k].tn == j)
              Quad_Elements[k].tn = i;
            if (Quad_Elements[k].ft == j)
              Quad_Elements[k].ft = i;
          }
        }
      }
    }
  }

}


/* ////////////////////////////////////////////////////////////////////////// */
/* function to setup the chain of the points and elements'relationship */


static void Q_setele(struct p_e *pe, int temp_size,
                     struct q_element *Quad_Elements,
                     struct a_node *Quad_Nodes, int NumQuads, int NumNodes)
{
  int i, j, n;
  int fn, sn, tn, ft;


  if (NumNodes > temp_size)
    e_GeneralError("out of size for pe structure");

  for (i = 0; i < temp_size; i++)
  {
    for (j = 0; j < Max_Elems_Per_Node; j++)
    {
      (pe + i)->point_elem[j] = 0;
    }
  }

  for (i = 1; i <= NumQuads; i++)
  {
    if (Quad_Elements[i].fn > 0)
    {
      n = Quad_Elements[i].n;
      fn = Quad_Elements[i].fn;
      sn = Quad_Elements[i].sn;
      tn = Quad_Elements[i].tn;
      ft = Quad_Elements[i].ft;

      Q_insertpe(n, (pe + fn));
      Q_insertpe(n, (pe + sn));
      Q_insertpe(n, (pe + tn));
      Q_insertpe(n, (pe + ft));
    }
  }
  Q_check(pe, Quad_Nodes, NumNodes);
}


/* ////////////////////////////////////////////////////////////////////////// */
/* function to check the error in the process of the enhancement */


static void Q_check(struct p_e *pe, struct a_node *Quad_Nodes, int NumNodes)
{
  int i, n;

  for (i = 1; i <= NumNodes; i++)
  {
    n = Q_lenpe(pe + i);
    if (n == 0 && Quad_Nodes[i].x > Q_Flag_Check_Neg_Number)
    {

      e_GeneralError("quad enhancement: a point has no elements connected it");
    }
  }

}


/* ////////////////////////////////////////////////////////////////////////// */
/* this function adjust the element and node topology after enhancement and
some active nodes x values are -1, and q_elements fn are -1. all these
elements and nodes are damped and new topology must reodered */


static void Q_adjust(int *m1, struct a_node **Quad_Nodes, struct q_element
                     **Quad_Elements, int **new_n, int **old, int *NumNodes,
                     int *NumQuads, int temp_size)
{
  int i, k = 0, fn, sn, tn, ft;


  *old = e_incr_i_vect(*NumNodes + Q_Node_WorkSpace, *old, P_E_MEM_INCREM);
  *new_n =
    e_incr_i_vect(*NumNodes + Q_Node_WorkSpace, *new_n, P_E_MEM_INCREM);

  for (i = 0; i < temp_size; i++)
    (*old)[i] = (*new_n)[i] = 0;

  for (i = 1; i <= *NumNodes; i++)
  {
    (*old)[i] = i;

/* if((*Quad_Nodes)[i].x<0.0) (*new_n)[i]=0; */

    if ((*Quad_Nodes)[i].x < Q_Flag_Check_Neg_Number)
      (*new_n)[i] = 0;
    else
    {
      k++;
      (*new_n)[i] = k;
    }
  }
  *m1 = k;
  k = 0;

  for (i = 1; i <= *NumNodes; i++)
  {

    if ((*new_n)[i] > 0)
    {
      k++;
      (*Quad_Nodes)[k].n = k;
      (*Quad_Nodes)[k].x = (*Quad_Nodes)[(*old)[i]].x;
      (*Quad_Nodes)[k].y = (*Quad_Nodes)[(*old)[i]].y;
      (*Quad_Nodes)[k].type = (*Quad_Nodes)[(*old)[i]].type;
    }
  }

  *NumNodes = k;

  for (i = 1; i <= *NumQuads; i++)
  {
    fn = (*Quad_Elements)[i].fn;
    sn = (*Quad_Elements)[i].sn;
    tn = (*Quad_Elements)[i].tn;
    ft = (*Quad_Elements)[i].ft;
    if (fn > 0)
    {
      (*Quad_Elements)[i].fn = (*new_n)[fn];
      (*Quad_Elements)[i].sn = (*new_n)[sn];
      (*Quad_Elements)[i].tn = (*new_n)[tn];
      (*Quad_Elements)[i].ft = (*new_n)[ft];
    }
  }

  k = 0;
  for (i = 1; i <= *NumQuads; i++)
  {
    (*old)[i] = i;
    if ((*Quad_Elements)[i].fn == -1)
      (*new_n)[i] = 0;
    else
    {
      k++;
      (*new_n)[i] = k;
    }
  }

  k = 0;

  for (i = 1; i <= *NumQuads; i++)
  {
    if ((*new_n)[i] > 0)
    {
      k++;
      (*Quad_Elements)[k].n = k;
      (*Quad_Elements)[k].fn = (*Quad_Elements)[(*old)[i]].fn;
      (*Quad_Elements)[k].sn = (*Quad_Elements)[(*old)[i]].sn;
      (*Quad_Elements)[k].tn = (*Quad_Elements)[(*old)[i]].tn;
      (*Quad_Elements)[k].ft = (*Quad_Elements)[(*old)[i]].ft;
    }
  }

  *NumQuads = k;

}



/* ////////////////////////////////////////////////////////////////////////// */
/* function to eliminate the point if a point have two elements connected
with it and this point is not a boundary point */


static int Q_p_elmt(int i, struct p_e *pe, struct a_node *Quad_Nodes,
                    struct q_element
                    *Quad_Elements, int NumQuads, int NumNodes, int temp_size)
{
  int fn, sn, tn, ft;
  int id;
  int i2, i3;

  id = 0;
  i2 = (pe + i)->point_elem[0];
  if (Quad_Elements[i2].fn == i)
  {
    fn = Quad_Elements[i2].sn;
    sn = Quad_Elements[i2].tn;
    tn = Quad_Elements[i2].ft;
  }
  else if (Quad_Elements[i2].sn == i)
  {
    fn = Quad_Elements[i2].fn;
    sn = Quad_Elements[i2].ft;
    tn = Quad_Elements[i2].tn;
  }
  else if (Quad_Elements[i2].tn == i)
  {
    fn = Quad_Elements[i2].sn;
    sn = Quad_Elements[i2].fn;
    tn = Quad_Elements[i2].ft;
  }
  else if (Quad_Elements[i2].ft == i)
  {
    fn = Quad_Elements[i2].fn;
    sn = Quad_Elements[i2].sn;
    tn = Quad_Elements[i2].tn;
  }
  else
    e_GeneralError("no point was found common in function p_elem");


  i3 = (pe + i)->point_elem[1];

  if (Quad_Elements[i3].fn == i)
    ft = Quad_Elements[i3].tn;
  else if (Quad_Elements[i3].sn == i)
    ft = Quad_Elements[i3].ft;
  else if (Quad_Elements[i3].tn == i)
    ft = Quad_Elements[i3].fn;
  else if (Quad_Elements[i3].ft == i)
    ft = Quad_Elements[i3].sn;
  else
    e_GeneralError("no point was found common in function p_elem");

  if (ft == fn || ft == sn || ft == tn)
    return 1;

  Quad_Elements[i2].fn = fn;
  Quad_Elements[i2].sn = sn;
  Quad_Elements[i2].tn = tn;
  Quad_Elements[i2].ft = ft;

  Quad_Elements[i3].fn = -1;
  Quad_Nodes[i].x = Q_Flag_Neg_Number;

  Q_setele(pe, temp_size, Quad_Elements, Quad_Nodes, NumQuads, NumNodes);
  return id;
}



/* ////////////////////////////////////////////////////////////////////////// */
/* element elimination if a element have two opposite point having three
elements connected with them */


static int Q_e_elmt(int j, int fn, int bn, struct p_e *pe, struct q_element
                    *Quad_Elements, struct a_node *Quad_Nodes,
                    int NumQuads, int NumNodes, int temp_size)
{
  int n1, n2, count = 0;
  int n, id;
  int i;
  double x, y;

  n1 = Q_lenpe(pe + fn);
  n2 = Q_lenpe(pe + bn);

  if (n1 == 3 && n2 == 3)
  {
    x = (Quad_Nodes[fn].x + Quad_Nodes[bn].x) / 2;
    y = (Quad_Nodes[fn].y + Quad_Nodes[bn].y) / 2;
    Quad_Nodes[fn].x = x;
    Quad_Nodes[fn].y = y;

    for (i = 0; i < 3; i++)
    {
      n = (pe + bn)->point_elem[i];
      if (n != j)
        Q_new_q(n, bn, fn, Quad_Elements);
    }

    Quad_Elements[j].fn = -1;
    Quad_Nodes[bn].x = Q_Flag_Neg_Number;

    Q_setele(pe, temp_size, Quad_Elements, Quad_Nodes, NumQuads, NumNodes);
    id = 0;
    count++;
    return id;
  }
  id = 1;
  return id;
}


/* ////////////////////////////////////////////////////////////////////////// */
/* function to determinate the number of the elements which relate to the
point */


static int Q_lenpe(struct p_e *ptr)
{
  int i = 0, n;
  n = ptr->point_elem[0];
  if (n == 0)
  {
    i = 0;
    return i;
  }
  while (i < Max_Elems_Per_Node)
  {
    n = ptr->point_elem[i];
    if (n == 0)
      return i;
    i++;
    if (i == Max_Elems_Per_Node)
      e_GeneralError("out of size for pe (15)");
  }
  return 0;
}



/* ////////////////////////////////////////////////////////////////////////// */
/* function to insert the number of the element which relate to the point */


static void Q_insertpe(int se, struct p_e *ptr)
{
  int i;
  i = Q_lenpe(ptr);
  ptr->point_elem[i] = se;
}


/* ////////////////////////////////////////////////////////////////////////// */
/* function to enhance the points appliing the general method */


static void Q_q_refine(int i, double *x, double *y, struct p_e *pe,
                       struct q_element *Quad_Elements,
                       struct a_node *Quad_Nodes)
{
  int j = 0, w, jj, jjj;
  int fn, sn, tn;


  *x = 0.0;
  *y = 0.0;

  jjj = Q_lenpe((pe + i));
  for (jj = 0; jj < jjj; jj++)
  {
    w = (pe + i)->point_elem[jj];

    if (Quad_Elements[w].fn == i)
    {
      fn = Quad_Elements[w].sn;
      sn = Quad_Elements[w].tn;
      tn = Quad_Elements[w].ft;
    }
    else if (Quad_Elements[w].sn == i)
    {
      fn = Quad_Elements[w].fn;
      sn = Quad_Elements[w].ft;
      tn = Quad_Elements[w].tn;
    }
    else if (Quad_Elements[w].tn == i)
    {
      fn = Quad_Elements[w].sn;
      sn = Quad_Elements[w].fn;
      tn = Quad_Elements[w].ft;
    }
    else if (Quad_Elements[w].ft == i)
    {
      fn = Quad_Elements[w].fn;
      sn = Quad_Elements[w].sn;
      tn = Quad_Elements[w].tn;
    }
    else
      e_GeneralError("no point was found common in function q_refine");

    *x = *x + Quad_Nodes[fn].x + 2 * Quad_Nodes[sn].x + Quad_Nodes[tn].x;
    *y = *y + Quad_Nodes[fn].y + 2 * Quad_Nodes[sn].y + Quad_Nodes[tn].y;
    j = j + 4;
  }

  *x = *x / j;
  *y = *y / j;

}


/* ////////////////////////////////////////////////////////////////////////// */
/* function to enhance the inter points according to special method which
the new position is not the centerior of polygon */


static void Q_iso_refine(int i, double *x, double *y, struct p_e *pe, struct
                         q_element *Quad_Elements, struct a_node *Quad_Nodes)
{
  int w, jj, jjj;
  int fn, sn, tn;


  *x = 0.0;
  *y = 0.0;

  jjj = Q_lenpe((pe + i));


  for (jj = 0; jj < jjj; jj++)
  {
    w = (pe + i)->point_elem[jj];

    if (Quad_Elements[w].fn == i)
    {
      fn = Quad_Elements[w].sn;
      sn = Quad_Elements[w].tn;
      tn = Quad_Elements[w].ft;
    }
    else if (Quad_Elements[w].sn == i)
    {
      fn = Quad_Elements[w].fn;
      sn = Quad_Elements[w].ft;
      tn = Quad_Elements[w].tn;
    }
    else if (Quad_Elements[w].tn == i)
    {
      fn = Quad_Elements[w].sn;
      sn = Quad_Elements[w].fn;
      tn = Quad_Elements[w].ft;
    }
    else if (Quad_Elements[w].ft == i)
    {
      fn = Quad_Elements[w].fn;
      sn = Quad_Elements[w].sn;
      tn = Quad_Elements[w].tn;
    }
    else
      e_GeneralError("no common point was found in function iso_refine");

    *x = *x + Quad_Nodes[fn].x - 0.5 * Quad_Nodes[sn].x + Quad_Nodes[tn].x;
    *y = *y + Quad_Nodes[fn].y - 0.5 * Quad_Nodes[sn].y + Quad_Nodes[tn].y;

  }

  *x = *x / jjj / 1.5;
  *y = *y / jjj / 1.5;


}



/* ////////////////////////////////////////////////////////////////////////// */
/* this function perform the enhancement of the points which have three
elements connected with those point. There are two way to refine them,
one to reposition it, another is merged three elements to two */


static void Q_re_angle(int i, double *x, double *y, struct p_e *pe,
                       struct q_element *Quad_Elements,
                       struct a_node *Quad_Nodes)
{
  int id, j1, j2, j3, fn1, tn1, fn2, tn2, n, fn3, tn3, i1, i2, i3;
  int i4, it, i14, i24, i34;
  double x1, y1, x2, y2, x3, y3, x4, y4;
  double a120, a100;
  double xx, yy;
  double angle, angle1, angle2, angle3;

/* control the angles which are formed after conbine three elems are not
large than 160 degree */
  a120 = M_PI * Q_Max_Side_Angle_Re_Ang / Q_PI_Degree;
/* control wheather or not to combine three elems or reposition*/
  a100 = M_PI * Q_Max_Mid_Angle_Re_Ang / Q_PI_Degree;



  j1 = (pe + i)->point_elem[0];
  Q_get_number(i, j1, &fn1, &tn1, &i14, Quad_Elements);
  j2 = (pe + i)->point_elem[1];
  Q_get_number(i, j2, &fn2, &tn2, &i24, Quad_Elements);
  j3 = (pe + i)->point_elem[2];
  Q_get_number(i, j3, &fn3, &tn3, &i34, Quad_Elements);
  x2 = Quad_Nodes[fn1].x;
  y2 = Quad_Nodes[fn1].y;
  x1 = Quad_Nodes[i].x;
  y1 = Quad_Nodes[i].y;
  x3 = Quad_Nodes[tn1].x;
  y3 = Quad_Nodes[tn1].y;
  x4 = Quad_Nodes[i14].x;
  y4 = Quad_Nodes[i14].y;

  if (Q_right(x1, y1, x3, y3, x2, y2))
  {
    i1 = fn1;
    i2 = tn1;
  }
  else
  {
    i1 = tn1;
    i2 = fn1;
  }

  if (i2 == fn2)
  {
    i3 = tn2;
  }
  else if (i2 == tn2)
  {
    i3 = fn2;
  }
  else if (i2 == fn3)
  {
    i4 = j2;
    j2 = j3;
    j3 = i4;
    it = i24;
    i24 = i34;
    i34 = it;
    i3 = tn3;
  }
  else if (i2 == tn3)
  {
    i4 = j2;
    j2 = j3;
    j3 = i4;
    it = i24;
    i24 = i34;
    i34 = it;
    i3 = fn3;
  }
  else
    e_GeneralError("no point was found common in function re_angle");

  x1 = Quad_Nodes[i1].x;
  y1 = Quad_Nodes[i1].y;
  x2 = Quad_Nodes[i2].x;
  y2 = Quad_Nodes[i2].y;
  x3 = Quad_Nodes[i3].x;
  y3 = Quad_Nodes[i3].y;
  x4 = Quad_Nodes[i].x;
  y4 = Quad_Nodes[i].y;

  angle1 = Q_g_angle(x3, y3, x1, y1, x2, y2, x4, y4);
  angle2 = Q_g_angle(x1, y1, x3, y3, x2, y2, x4, y4);
  angle3 = Q_g_angle(x2, y2, x1, y1, x3, y3, x4, y4);

  angle = angle1;
  id = 1;
  if (angle2 > angle)
  {
    angle = angle2;
    id = 2;
  }
  if (angle3 > angle)
  {
    angle = angle3;
    id = 3;
  }

  if (angle < a100)
  {
/* move the middle point to the center of polygon */
    Q_divide_point(x1, y1, x2, y2, x3, y3, &xx, &yy);
  }
  else
  {
    xx = Quad_Nodes[i].x;
    yy = Quad_Nodes[i].y;

    if (id == 1)
    {
      x1 = Quad_Nodes[i2].x;
      y1 = Quad_Nodes[i2].y;
      x2 = Quad_Nodes[i14].x;
      y2 = Quad_Nodes[i14].y;
      x3 = Quad_Nodes[i24].x;
      y3 = Quad_Nodes[i24].y;

      angle1 = Q_g_angle(x1, y1, x2, y2, x3, y3, x4, y4);
      x1 = Quad_Nodes[i1].x;
      y1 = Quad_Nodes[i1].y;
      x3 = Quad_Nodes[i34].x;
      y3 = Quad_Nodes[i34].y;

      angle2 = Q_g_angle(x1, y1, x2, y2, x3, y3, x4, y4);
      if (angle1 < a120 && angle2 < a120)
      {
        n = j2;
        i4 = i14;
        Quad_Elements[j1].fn = -1;
        Q_new_q(n, i, i14, Quad_Elements);
        n = j3;
        Q_new_q(n, i, i14, Quad_Elements);
        xx = Q_Flag_Neg_Number;
        yy = Quad_Nodes[i4].y;
      }
    }
    if (id == 2)
    {
      x1 = Quad_Nodes[i2].x;
      y1 = Quad_Nodes[i2].y;
      x2 = Quad_Nodes[i14].x;
      y2 = Quad_Nodes[i14].y;
      x3 = Quad_Nodes[i24].x;
      y3 = Quad_Nodes[i24].y;

      angle1 = Q_g_angle(x1, y1, x2, y2, x3, y3, x4, y4);
      x1 = Quad_Nodes[i3].x;
      y1 = Quad_Nodes[i3].y;
      x2 = Quad_Nodes[i34].x;
      y2 = Quad_Nodes[i34].y;

      angle2 = Q_g_angle(x1, y1, x2, y2, x3, y3, x4, y4);
      if (angle1 < a120 && angle2 < a120)
      {
        i4 = i24;
        Quad_Elements[j2].fn = -1;
        n = j1;
        Q_new_q(n, i, i24, Quad_Elements);
        n = j3;
        Q_new_q(n, i, i24, Quad_Elements);
        xx = Q_Flag_Neg_Number;
        yy = Quad_Nodes[i4].y;
      }
    }

    if (id == 3)
    {
      x1 = Quad_Nodes[i3].x;
      y1 = Quad_Nodes[i3].y;
      x2 = Quad_Nodes[i24].x;
      y2 = Quad_Nodes[i24].y;
      x3 = Quad_Nodes[i34].x;
      y3 = Quad_Nodes[i34].y;

      angle1 = Q_g_angle(x1, y1, x2, y2, x3, y3, x4, y4);
      x1 = Quad_Nodes[i1].x;
      y1 = Quad_Nodes[i1].y;
      x2 = Quad_Nodes[i14].x;
      y2 = Quad_Nodes[i14].y;

      angle2 = Q_g_angle(x1, y1, x2, y2, x3, y3, x4, y4);
      if (angle1 < a120 && angle2 < a120)
      {
        i4 = i34;
        Quad_Elements[j3].fn = -1;
        n = j1;
        Q_new_q(n, i, i34, Quad_Elements);
        n = j2;
        Q_new_q(n, i, i34, Quad_Elements);
        xx = Q_Flag_Neg_Number;
        yy = Quad_Nodes[i4].y;
      }
    }
  }
  *x = xx;
  *y = yy;

}


/* ////////////////////////////////////////////////////////////////////////// */
/* function perform getting rest of number of element i2 except point i */


static void Q_get_number(int i, int i2, int *fn, int *tn, int *ft,
                         struct q_element *Quad_Elements)
{

  if (Quad_Elements[i2].fn == i)
  {
    *fn = Quad_Elements[i2].sn;
    *tn = Quad_Elements[i2].ft;
    *ft = Quad_Elements[i2].tn;
  }
  else if (Quad_Elements[i2].sn == i)
  {
    *fn = Quad_Elements[i2].fn;
    *tn = Quad_Elements[i2].tn;
    *ft = Quad_Elements[i2].ft;
  }
  else if (Quad_Elements[i2].tn == i)
  {
    *fn = Quad_Elements[i2].sn;
    *tn = Quad_Elements[i2].ft;
    *ft = Quad_Elements[i2].fn;
  }
  else if (Quad_Elements[i2].ft == i)
  {
    *fn = Quad_Elements[i2].fn;
    *tn = Quad_Elements[i2].tn;
    *ft = Quad_Elements[i2].sn;
  }
  else
    e_GeneralError("no common point was found in function get_number");

}

/* after the merge three elements, modify the topology of new
point-element relationship */
void Q_modify(int i, int j1, int j2, int j3, struct p_e *pe)
{

  if ((pe + i)->point_elem[0] == j1)
    (pe + i)->point_elem[0] = j2;
  if ((pe + i)->point_elem[1] == j1)
    (pe + i)->point_elem[1] = j2;
  if ((pe + i)->point_elem[2] == j1)
    (pe + i)->point_elem[2] = j2;
  Q_insertpe(j3, (pe + i));
}


/* ////////////////////////////////////////////////////////////////////////// */
/* this function is a sub-function of re_angle. It reposition the point
to make three interior angle most equal to 120 */


static void Q_divide_point(double x1, double y1, double x2, double y2,
                           double x3, double y3, double *xx, double *yy)
{
  double cx1, cx2, cy1, cy2, a1, a2;
  double sine, cosine, l1, l2, l, s1, s2, s3, s4, s5;
  double d1, d2;

  Q_center_point(x1, y1, x2, y2, &cx1, &cy1, &a1);
  Q_center_point(x2, y2, x3, y3, &cx2, &cy2, &a2);

  l1 = E_SQUARE(cx1 - cx2);
  l2 = E_SQUARE(cy1 - cy2);
  l = sqrt(l1 + l2);
  if (l <= Q_Tolerance_10)
    e_GeneralError("zero length in function divide_point");

  sine = (cy2 - cy1) / l;
  cosine = (cx2 - cx1) / l;

  seglen(cx1, cy1, cx2, cy2, &s1);
  seglen(cx1, cy1, x2, y2, &s2);
  seglen(x2, y2, cx2, cy2, &s3);


  s4 = (s1 + s2 + s3) / 2;
  s5 = sqrt(s4 * (s4 - s1) * (s4 - s2) * (s4 - s3));
  d1 = 2 * s5 / l;
  if ((a1 * a1 - d1 * d1) < Q_Tolerance_10)
    e_GeneralError("error in function divide_point");

  d2 = sqrt(a1 * a1 - d1 * d1);
  *xx = cx1 + d2 * cosine - (-d1) * sine;
  *yy = cy1 + d2 * sine + (-d1) * cosine;

  if (fabs(*xx - x2) < Q_Tolerance_5 && fabs(*yy - y2) < Q_Tolerance_5)
  {
    *xx = cx1 + d2 * cosine - d1 * sine;
    *yy = cy1 + d2 * sine + d1 * cosine;
    *xx = (x1 + x2 + x3) / 3.0;
    *yy = (y1 + y2 + y3) / 3.0;
  }

  if (!Q_right(cx2, cy2, cx1, cy1, *xx, *yy))
    e_GeneralError("error  in function divide_point");

}


/* ////////////////////////////////////////////////////////////////////////// */
/* function to get the the centr point of circles which intersect at
the new position of relevent point */

static void Q_center_point(double x1, double y1, double x2, double y2,
                           double *x, double *y, double *a)
{
  double rsin, rcos;
  double l, x3, y3;


  seglen(x1, y1, x2, y2, &l);

  if (l < Q_Tolerance_10)
    e_GeneralError("zero length in function divide_point");

  rsin = (y2 - y1) / l;
  rcos = (x2 - x1) / l;

  *x = x1 + l * rcos / 2 - (l / 2 * tan(M_PI / 6)) * rsin;
  *y = y1 + l * rsin / 2 + (l / 2 * tan(M_PI / 6)) * rcos;

  x3 = *x;
  y3 = *y;

/* *a is the radous of the circle */
  *a = l / 2 / sin(M_PI / 3);
  if (!Q_right(x1, y1, x2, y2, x3, y3))
    e_GeneralError("error  in function center_point");

}


/* ////////////////////////////////////////////////////////////////////////// */
/* this function is a sub-function e_elmt to replace a point with a new
point */


static void Q_new_q(int n, int bn, int fn, struct q_element *Quad_Elements)
{
  int i;
  i = bn;
  if (Quad_Elements[n].fn == i)
  {
    Quad_Elements[n].fn = fn;
  }
  else if (Quad_Elements[n].sn == i)
  {
    Quad_Elements[n].sn = fn;
  }
  else if (Quad_Elements[n].tn == i)
  {
    Quad_Elements[n].tn = fn;
  }
  else if (Quad_Elements[n].ft == i)
  {
    Quad_Elements[n].ft = fn;
  }
  else
    e_GeneralError("no point was found common in function new_p");

}


/* ////////////////////////////////////////////////////////////////////////// */
/* x1 y1 are the middle node coordinates */


static double Q_g_angle(double x1, double y1, double x2, double y2, double x3,
                        double y3, double x6, double y6)
{
  double a, a1, a2;

  x1 = Q_Angle_Scale * x1;
  y1 = Q_Angle_Scale * y1;
  x2 = Q_Angle_Scale * x2;
  y2 = Q_Angle_Scale * y2;
  x3 = Q_Angle_Scale * x3;
  y3 = Q_Angle_Scale * y3;
  x6 = Q_Angle_Scale * x6;
  y6 = Q_Angle_Scale * y6;
  a1 = Q_cos_angle(x1, y1, x6, y6, x3, y3);
  a2 = Q_cos_angle(x1, y1, x6, y6, x2, y2);
  a = a1 + a2;

  return a;
}


/* ////////////////////////////////////////////////////////////////////////// */
/* this function is a fragile function and be damped at this monent */


static double Q_cos_angle(double x1, double y1, double x2, double y2,
                          double x3, double y3)
{
  double s1, s2, s3;
  double ss;

/* using the area coordinate for searching algorithm
first I used the area farmula but the acurary too poor now I
used a2=c2+b2-2*a*bcosa */
  seglen(x1, y1, x2, y2, &s1);
  seglen(x2, y2, x3, y3, &s3);
  seglen(x1, y1, x3, y3, &s2);


  ss = (s1 * s1 + s2 * s2 - s3 * s3);
  ss = ss / 2 / s1 / s2;
  if (fabs(ss - 1) < Q_Tolerance_10)
    ss = 0.0;
  else
    ss = acos(ss);
  return ss;
}


/* ////////////////////////////////////////////////////////////////////////// */


static void Q_flat_elem(int i, struct p_e *pe,
                        struct q_element **Quad_Elements,
                        struct a_node **Quad_Nodes, int *NumNodes,
                        int *NumQuads, int Q_NumBoundaryNodes, int temp_size)
{
  int j, id, id1, id2, id3, id4, id55, jj, fn, sn, tn, ft;
  int id33, id22, temp;
  double x1, x2, x3, y1, y2, y3;
  double angle, angle1, angle2, angle3, angle4;
  double tole, x4, y4;
  tole = M_PI * Q_Max_Angle_Flat_Elem / Q_PI_Degree;


  fn = (*Quad_Elements)[i].fn;
  sn = (*Quad_Elements)[i].sn;
  tn = (*Quad_Elements)[i].tn;
  ft = (*Quad_Elements)[i].ft;
  /* remember the middle point is the x1,y1*/
  /* Following line have been changed at 26/11/96 to make sure only boundary angles
     have been checked */

  if (sn > Q_NumBoundaryNodes)
    angle1 = 0.0;
  else
  {
    x1 = (*Quad_Nodes)[sn].x;
    y1 = (*Quad_Nodes)[sn].y;
    x2 = (*Quad_Nodes)[fn].x;
    y2 = (*Quad_Nodes)[fn].y;
    x3 = (*Quad_Nodes)[tn].x;
    y3 = (*Quad_Nodes)[tn].y;
    x4 = (*Quad_Nodes)[ft].x;
    y4 = (*Quad_Nodes)[ft].y;
    angle1 = Q_g_angle(x1, y1, x2, y2, x3, y3, x4, y4);
  }

  if (tn > Q_NumBoundaryNodes)
    angle2 = 0.0;
  else
  {
    x1 = (*Quad_Nodes)[tn].x;
    y1 = (*Quad_Nodes)[tn].y;
    x2 = (*Quad_Nodes)[sn].x;
    y2 = (*Quad_Nodes)[sn].y;
    x3 = (*Quad_Nodes)[ft].x;
    y3 = (*Quad_Nodes)[ft].y;
    x4 = (*Quad_Nodes)[fn].x;
    y4 = (*Quad_Nodes)[fn].y;
    angle2 = Q_g_angle(x1, y1, x2, y2, x3, y3, x4, y4);
  }

  if (ft > Q_NumBoundaryNodes)
    angle3 = 0.0;
  else
  {
    x1 = (*Quad_Nodes)[ft].x;
    y1 = (*Quad_Nodes)[ft].y;
    x2 = (*Quad_Nodes)[tn].x;
    y2 = (*Quad_Nodes)[tn].y;
    x3 = (*Quad_Nodes)[fn].x;
    y3 = (*Quad_Nodes)[fn].y;
    x4 = (*Quad_Nodes)[sn].x;
    y4 = (*Quad_Nodes)[sn].y;
    angle3 = Q_g_angle(x1, y1, x2, y2, x3, y3, x4, y4);
  }

  if (fn > Q_NumBoundaryNodes)
    angle4 = 0.0;
  else
  {
    x1 = (*Quad_Nodes)[fn].x;
    y1 = (*Quad_Nodes)[fn].y;
    x2 = (*Quad_Nodes)[ft].x;
    y2 = (*Quad_Nodes)[ft].y;
    x3 = (*Quad_Nodes)[sn].x;
    y3 = (*Quad_Nodes)[sn].y;
    x4 = (*Quad_Nodes)[tn].x;
    y4 = (*Quad_Nodes)[tn].y;
    angle4 = Q_g_angle(x1, y1, x2, y2, x3, y3, x4, y4);
  }

  angle = angle1;
  id1 = sn;

  if (angle2 < angle4)
    id2 = tn;
  else
    id2 = fn;
  if (angle2 > angle)
  {
    angle = angle2;
    id1 = tn;

    if (angle1 < angle3)
      id2 = sn;
    else
      id2 = ft;
  }
  if (angle3 > angle)
  {
    angle = angle3;
    id1 = ft;

    if (angle2 < angle4)
      id2 = tn;
    else
      id2 = fn;
  }
  if (angle4 > angle)
  {
    angle = angle4;
    id1 = fn;

    if (angle1 < angle3)
      id2 = sn;
    else
      id2 = ft;
  }
  if (angle > tole)
  {
/*printf("\n tole =%lf angle =%lf",tole,angle*180.0/M_PI); */
    id3 = Q_opposite_a(i, id1, (*Quad_Elements));
    id4 = Q_opposite_a(i, id2, (*Quad_Elements));
/*following line was added by Biao on 14/11/96 */
    if (!Q_OptiPoint(i, id2, id3, pe, (*Quad_Elements)))
    {
      temp = id2;
      id2 = id4;
      id4 = temp;
    }

/*********************************************/

    jj = Q_lenpe((pe + id3));
    if (jj == 2)
    {
      if (i == (pe + id3)->point_elem[0])
        id = (pe + id3)->point_elem[1];
      else if (i == (pe + id3)->point_elem[1])
        id = (pe + id3)->point_elem[0];
      else
        jj = jj;

      if (((*Quad_Elements)[id].fn == id4) || ((*Quad_Elements)[id].sn == id4)
          || ((*Quad_Elements)[id].tn == id4)
          || ((*Quad_Elements)[id].ft == id4))
      {
        id55 = id2;
        id2 = id4;
        id4 = id55;
      }
      id33 = Q_opposite_a(id, id3, (*Quad_Elements));
      id22 = Q_opposite_a(id, id2, (*Quad_Elements));
    }
    else
    {
      for (j = 0; j < jj; j++)
      {
        id = (pe + id3)->point_elem[j];
        if (((*Quad_Elements)[id].fn == id2 || (*Quad_Elements)[id].sn == id2
             || (*Quad_Elements)[id].tn == id2
             || (*Quad_Elements)[id].ft == id2) && id != i)
          break;
      }
      id33 = Q_opposite_a(id, id3, *Quad_Elements);
      id22 = Q_opposite_a(id, id2, *Quad_Elements);
    }

    *NumNodes = *NumNodes + 1;
    Q_Creat_Node(*NumNodes, ((*Quad_Nodes)[id2].x + (*Quad_Nodes)[id3].x) / 2,
                 ((*Quad_Nodes)[id2].y + (*Quad_Nodes)[id3].y) / 2,
                 Quad_Nodes);

    (*Quad_Elements)[i].fn = id3;
    (*Quad_Elements)[i].sn = id4;
    (*Quad_Elements)[i].tn = id1;
    (*Quad_Elements)[i].ft = *NumNodes;

    *NumNodes = *NumNodes + 1;
    Q_Creat_Node(*NumNodes,
                 ((*Quad_Nodes)[id2].x + (*Quad_Nodes)[id22].x) / 2,
                 ((*Quad_Nodes)[id2].y + (*Quad_Nodes)[id22].y) / 2,
                 Quad_Nodes);


    (*Quad_Elements)[id].fn = id3;
    (*Quad_Elements)[id].sn = id22;
    (*Quad_Elements)[id].tn = *NumNodes;
    (*Quad_Elements)[id].ft = *NumNodes - 1;

    *NumQuads = *NumQuads + 1;
    Q_Creat_Quad(*NumQuads, id1, *NumNodes - 1, *NumNodes, id2,
                 Quad_Elements);
    *NumQuads = *NumQuads + 1;
    Q_Creat_Quad(*NumQuads, id2, *NumNodes, id22, id33, Quad_Elements);

    Q_setele(pe, temp_size, *Quad_Elements, *Quad_Nodes, *NumQuads,
             *NumNodes);
  }


}



/* Function to check if a node is suitable to enhance the boundary elem */

static int Q_OptiPoint(int elem, int id2, int id3,
                       struct p_e *pe, struct q_element *Quad_Elements)
{

  int j, id;


  if (Q_lenpe(pe + id2) == 1)
    return 0;
  for (j = 0; j < Q_lenpe(pe + id2); j++)
  {
    id = (pe + id2)->point_elem[j];
    if (id != elem)
    {
      if (Quad_Elements[id].fn == id3 || Quad_Elements[id].sn == id3 ||
          Quad_Elements[id].tn == id3 || Quad_Elements[id].ft == id3)
        return 1;
    }
  }

  return 0;

}


/* ////////////////////////////////////////////////////////////////////////// */
/* function to find the opposite node in i element */


static int Q_opposite_a(int i, int id, struct q_element *Quad_Elements)
{
  int j;

  if (Quad_Elements[i].fn == id)
  {
    j = Quad_Elements[i].tn;
  }
  else if (Quad_Elements[i].sn == id)
  {
    j = Quad_Elements[i].ft;
  }
  else if (Quad_Elements[i].tn == id)
  {
    j = Quad_Elements[i].fn;
  }
  else if (Quad_Elements[i].ft == id)
  {
    j = Quad_Elements[i].sn;
  }
  else
    e_GeneralError("id is not a node for element i in function Q_opposite_a");

  return j;
}


/* ////////////////////////////////////////////////////////////////////////// */


static void Q_En_check_right(struct a_node *Quad_Nodes,
                             struct q_element *Quad_Elements, int NumQuads)
{
  int i, fn, sn, tn, ft;
  double x1, x2, x3, y1, y2, y3;

  for (i = 1; i <= NumQuads; i++)
  {
    fn = Quad_Elements[i].fn;
    sn = Quad_Elements[i].sn;
    tn = Quad_Elements[i].tn;
    ft = Quad_Elements[i].ft;
    x1 = Quad_Nodes[fn].x;
    y1 = Quad_Nodes[fn].y;
    x2 = Quad_Nodes[sn].x;
    y2 = Quad_Nodes[sn].y;
    x3 = Quad_Nodes[tn].x;
    y3 = Quad_Nodes[tn].y;
    if (!Q_right(x1, y1, x2, y2, x3, y3))
    {
      Quad_Elements[i].sn = ft;
      Quad_Elements[i].ft = sn;
    }
  }

}


/* ////////////////////////////////////////////////////////////////////////// */
/* Q_ConvertRefinedMesh: To convert the refined MESH structure to one
   corresponding to the enhanced mesh.                                        */


static void Q_ConvertRefinedMesh(MESH *RfndMesh, int NBoundaryNodes,
                                 struct a_node *Quad_Nodes, struct q_element
                                 *Quad_Elements, int NumNodes, int NumQuads)
{
  int i, j, k;
  double **pMeshPoints;

  pMeshPoints = e_alloc_cont_d_matr(NumNodes, E_NDIMS);
  memcpy(pMeshPoints[0], RfndMesh->MeshPointCoords[0],
         NBoundaryNodes * E_NDIMS * sizeof(double));
  e_free_cont_d_matr(RfndMesh->NMeshPoints, RfndMesh->MeshPointCoords);
  RfndMesh->MeshPointCoords = pMeshPoints;

  e_free_cont_i_matr(RfndMesh->NElemsQuad1, RfndMesh->NodesQuad1);
  RfndMesh->NodesQuad1 = e_alloc_cont_i_matr(NumQuads, E_NNPE_QUAD1);
  RfndMesh->NMeshPoints = NumNodes;
  RfndMesh->NNodes = NumNodes;
  RfndMesh->NElemsQuad1 = NumQuads;
  RfndMesh->TotalNElems = NumQuads;

  for (i = NBoundaryNodes; i < RfndMesh->NMeshPoints; i++)
  {
    RfndMesh->MeshPointCoords[i][E_XDIR] = Quad_Nodes[i + 1].x;
    RfndMesh->MeshPointCoords[i][E_YDIR] = Quad_Nodes[i + 1].y;
    RfndMesh->MeshPointCoords[i][E_ZDIR] = 0.0;
  }

  for (i = 0; i < RfndMesh->TotalNElems; i++)
  {
    RfndMesh->NodesQuad1[i][0] = Quad_Elements[i + 1].fn;
    RfndMesh->NodesQuad1[i][1] = Quad_Elements[i + 1].sn;
    RfndMesh->NodesQuad1[i][2] = Quad_Elements[i + 1].tn;
    RfndMesh->NodesQuad1[i][3] = Quad_Elements[i + 1].ft;
  }

  e_free(RfndMesh->NodalDOF);
  e_free(RfndMesh->GNIs);
  e_free(RfndMesh->GEIs);

  RfndMesh->NodalDOF = e_alloc_i_vect(RfndMesh->NNodes);
  RfndMesh->GNIs = e_alloc_i_vect(RfndMesh->NNodes);
  RfndMesh->GEIs = e_alloc_i_vect(RfndMesh->NElemsQuad1);

  RfndMesh->GEIsQuad1 = RfndMesh->GEIs;
  RfndMesh->TotalNodalDOF = 0;

  for (i = 0; i < RfndMesh->NElemsQuad1; i++)
  {
    RfndMesh->GEIs[i] = i + 1;

    for (j = 0; j < E_NNPE_QUAD1; j++)
    {
      k = RfndMesh->NodesQuad1[i][j] - 1;

      if (RfndMesh->NodalDOF[k] == 0)
      {
        RfndMesh->TotalNodalDOF += E_VERTEX_NODE_DOF_QUAD1;
        RfndMesh->NodalDOF[k] = E_VERTEX_NODE_DOF_QUAD1;
      }
    }
  }

  for (k = 0; k < RfndMesh->NNodes; k++)
    RfndMesh->GNIs[k] = k + 1;
}

void Q_Creat_Quad(int i,int first,int second,int third,int fourth, struct q_element **q_elements)
{

 *q_elements = b_incr_q_element_vect(i,*q_elements,Q_ELEMENTS_MEM_INCREM);

	(*q_elements)[i].n=i;
	(*q_elements)[i].fn=first;
	(*q_elements)[i].sn=second;
	(*q_elements)[i].tn=third;
	(*q_elements)[i].ft=fourth;
}


/* ////////////////////////////////////////////////////////////////////////// */

static void Q_Creat_Node(int i, double x,double y,struct a_node **act_nodes)
{

       *act_nodes = b_incr_act_node_vect(i,*act_nodes,ACT_NODES_MEM_INCREM);
       (*act_nodes)[i].n=i;
       (*act_nodes)[i].x=x;
       (*act_nodes)[i].y=y;
       (*act_nodes)[i].type = E_RM_NUM_INTERIOR;
}


/* ////////////////////////////////////////////////////////////////////////// */


static struct a_node *b_incr_act_node_vect(unsigned int length, struct a_node *vector,
                                           unsigned int MemIncrem)
{
   struct a_node *TempVector;

   if (length % MemIncrem == 0)
   {
     TempVector = (struct a_node *)
         e_calloc(length + MemIncrem,sizeof(struct a_node));

     if (TempVector == NULL)
     {
       e_free(vector);
       return NULL;
     }
     
     memcpy(TempVector,vector,length*sizeof(struct a_node));
     e_free(vector);
     return TempVector;
   }
   else
   {
     return vector;
   }
}


/* ////////////////////////////////////////////////////////////////////////// */


static struct q_element *b_incr_q_element_vect(unsigned int length, struct q_element
                                        *vector, unsigned int MemIncrem      )
{
   struct q_element *TempVector;

   if (length % MemIncrem == 0)
   {
      TempVector = (struct q_element *)
                   e_calloc(length + MemIncrem,sizeof(struct q_element));

      if (TempVector == NULL)
      {
         e_free(vector);
         return NULL;
      }

      memcpy(TempVector,vector,length*sizeof(struct q_element));
      e_free(vector);
      return TempVector;
   }
   else      return vector;
}


/* ////////////////////////////////////////////////////////////////////////// */


static int Q_right(double x1,double y1,double x2,double y2,double x3,double y3)
{
  double D;
  D=x3*(y1-y2)+y3*(x2-x1)+(x1*y2-y1*x2);
  if(D>Q_Tolerance_10)
      return 1;
  else
      return 0;
}


/* ////////////////////////////////////////////////////////////////////////// */
/* Function to calculate the length of a segment */


static void seglen(double x1,double y1, double x2, double y2, double *pL)
{
  double L;
  
  L=sqrt(E_SQUARE(x2-x1) + E_SQUARE(y2-y1));
  *pL = L;
}



/* ////////////////////////////////////////////////////////////////////////// */
/* ArrangeBoundaryNodes: rearrange the mesh so that the boundary coincides
   with the start of the coords array */
static void ArrangeBoundaryNodes(MESH *mesh)
{
  int nb;
  int **ne;
  int *nne;
  BND *bnd;
  double **new_coords;
  int *table;
  int *taken;
  int i,n,s;
  int *allbnd;
  int  NBoundaryNodes;
  
  /* get adjacency */
  nne = e_alloc_i_vect(mesh->NMeshPoints);
  ne  = e_init_frag_i_matr(mesh->NMeshPoints);
  e_InitMeshPointElemConnection(mesh, nne, ne);
  
  /* get boundary */
  Gen2DBoundary(mesh, nne, ne, &nb, &bnd);
  if (nb!=1)
    e_GeneralError("ArrangeBoundaryNodes:"
                   " this mesh generator only accepts single domain meshes");
    
  NBoundaryNodes = bnd->oe.nen;
  for (i = 0;i < bnd->nie; i++)
  {
    NBoundaryNodes += bnd->ie[i].nen;
  }

  /* copy all bnd nodes into temp vector */
  allbnd = e_alloc_i_vect(NBoundaryNodes);
  memcpy(&(allbnd[0]),&(bnd->oe.en[0]),bnd->oe.nen*sizeof(int));
  NBoundaryNodes = bnd->oe.nen;
  for (i = 0;i < bnd->nie; i++)
  {
    memcpy(&(allbnd[NBoundaryNodes]),
           &(bnd->ie[i].en[0]),
           bnd->ie[i].nen*sizeof(int));
    NBoundaryNodes += bnd->ie[i].nen;
  }
  
  /* rearrange mesh definition */
  table = e_alloc_i_vect(mesh->NMeshPoints);
  taken = e_alloc_i_vect(mesh->NMeshPoints);
  new_coords = e_alloc_cont_d_matr(mesh->NMeshPoints,3);
  for (i = 0; i < NBoundaryNodes; i++)
  {
    table[allbnd[i]-1]= i;
    taken[allbnd[i]-1]= TRUE;
  }
  n=i;
  for (i=0;i<mesh->NMeshPoints;i++)
  {
    if (!taken[i])
    {
      table[i]=n;
      n++;
    }
  }
  
  for (i=0;i<mesh->NMeshPoints;i++)
    for (s=0;s<3;s++)
      new_coords[table[i]][s]=mesh->MeshPointCoords[i][s];

  for (i=0;i<mesh->NElemsTriang1;i++)
    for (s=0;s<3;s++)
      mesh->NodesTriang1[i][s]=table[mesh->NodesTriang1[i][s]-1]+1;

  for (i=0;i<mesh->NElemsQuad1;i++)
    for (s=0;s<4;s++)
      mesh->NodesQuad1[i][s]=table[mesh->NodesQuad1[i][s]-1]+1;

  e_free_cont_d_matr(mesh->NMeshPoints,mesh->MeshPointCoords);
  mesh->MeshPointCoords=new_coords;
  e_free(table);
  e_free(taken);

  /* free-up adjacency */
  e_free(nne);
  e_free_frag_i_matr(mesh->NMeshPoints,ne);

  /* free boundary */
  Free2DBoundary(nb,bnd);
  e_free(allbnd);
}

