#include "fusion.h"
#include "mtrf.h"
#include "mgn_aux.h"
#include "mgn_graphics.h"
#include "pav_main.h"

extern int ShowGraphics;

#define ALLOC_SIZE 64

static int j_Check_CCW_Quad(MESH *mesh, int *points);
static int j_FindFirstTriang(MESH *mesh, int *Degree, int **AdjElems,
                             int *ActiveList);
static int j_FindSecondTriang(MESH *mesh, int *Degree, int **AdjElems,
                              int *ActiveList, int NLeftTriangs,
                              int FirstTriang, int *point1, int *point2,
                              int *PointFirst, int *PointSecond, int fission);
static double j_CalculateDiag(MESH *mesh, int FirstTriang, int SecondTriang,
                              int *point1, int *point2, int *PointFirst,
                              int *PointSecond);
static double j_Com_Distance(MESH *mesh, int point1, int point2);
static void j_Com_Creat_Quad(MESH *mesh, int point1, int point2, int point3,
                             int point4);
static void j_FissionElems(MESH *mesh, int *Degree, int **AdjElems,
                           int *ActiveList, int *NLeftTriangs,
                           int FirstTriang, int SecondTriang);
static void j_Com_Average(MESH *mesh, int Elem1, int Node);
static void j_Com_Update_List(MESH *mesh, int *Degree, int **AdjElems,
                              int *ActiveList, int DeletedElem);
static void j_CombineTwoTriangs(MESH *mesh, int *Degree, int **AdjElems,
                                int *ActiveList, int *NLeftTriangs,
                                int FirstTriang, int SecondTriang, int point1,
                                int point2, int PointFirst, int PointSecond);
static int j_ElemPairedRegions(MESH *mesh, int NTriangElems, int IgnoreElem1,
                               int IgnoreElem2, int *Degree, int **AdjElems);
static int j_GetInitFrontElem(MESH *mesh, int *Degree, char *ElemFound);
static void j_AdvanceFront(int *Degree, int **AdjElems, int *NFrontElements,
                           int **FrontElements, char *ElemFound,
                           int **WorkSpace1, int **WorkSpace2);
static void j_FindApexPoints(MESH *mesh, int FirstTriang, int SecondTriang,
                             int *point1, int *point2, int *PointFirst,
                             int *PointSecond);
static void j_AdjustTriangQuadMesh(MESH *mesh, int NKeepTriangle, int *KeepTriangle);



void fuse2quad(MESH *mesh, int fission)
{
  /* based on combine.c */
  int  *Degree;
  int **AdjElems;
  int   FirstTriang, SecondTriang;
  int   point1, point2, PointFirst, PointSecond;
  int   temp1, temp2;
  int  *ActiveList;
  int   InitNMeshPoints;
  int   NLeftTriangs;
  int   i, even;
  int   NKeepTriangle;
  int  *KeepTriangle;
  
  if(mesh->NElemsTriang1 % 2 && fission)
  {
    printf("\nFusion is possible only with even number of elements !");
    printf("\nTry Rank method !\n");
    return;
  }

  /* initialize adjacency */
  e_SetAdjacency(mesh, &Degree, &AdjElems);

  /* initialize */
  NLeftTriangs = mesh->NElemsTriang1;
  InitNMeshPoints = mesh->NMeshPoints;
  ActiveList = e_alloc_i_vect(mesh->NElemsTriang1);
  for (i = 0; i < mesh->NElemsTriang1; i++)
    ActiveList[i] = 1;

  NKeepTriangle = 0;
  KeepTriangle  = NULL;
  
  /* main loop */
  while (TRUE)
  {
    /* find first triangle to fuse */
    FirstTriang = j_FindFirstTriang(mesh, Degree, AdjElems, ActiveList);
    if (!FirstTriang)
      break;

    /* find second triangle to fuse */
    SecondTriang =
      j_FindSecondTriang(mesh, Degree, AdjElems, ActiveList, NLeftTriangs,
                         FirstTriang, &point1, &point2, &PointFirst,
                         &PointSecond, fission);

    /* could not find second triangle */
    if (!SecondTriang)
    {
      /* no fission, no problem,
         just mark the first element as non active any more and keep it in the mesh */
      if(!fission)
      {
        ActiveList[FirstTriang - 1] = 0;
        KeepTriangle = e_incr_i_vect(NKeepTriangle, KeepTriangle, ALLOC_SIZE);
        KeepTriangle[NKeepTriangle] = FirstTriang;
        NKeepTriangle++;
        continue;
      }
      /* with fission, this is not possible */
      else
      {
        e_GeneralError("Can not find the second triangle");
      }
    }

    /* use fission elements */
    if(fission)
    {
      /* checkout adjacency situation */
      temp1 = Degree[FirstTriang - 1];
      temp2 = Degree[SecondTriang - 1];

      Degree[FirstTriang - 1] = 0;
      Degree[SecondTriang - 1] = 0;
      
      /* check the number of elements in the subregion is even */
      even = j_ElemPairedRegions(mesh, NLeftTriangs, 
                                 FirstTriang, SecondTriang, 
                                 Degree, AdjElems);
      if(even)
      {
        /* if even number of elements remains, merge elements */
        Degree[FirstTriang - 1] = temp1;
        Degree[SecondTriang - 1] = temp2;
        j_CombineTwoTriangs(mesh, Degree, AdjElems, ActiveList, &NLeftTriangs,
                            FirstTriang, SecondTriang, point1, point2,
                            PointFirst, PointSecond);
      }
      else
      {
        /* if odd number of elements, do fission to get even number of element */
        Degree[FirstTriang - 1] = temp1;
        Degree[SecondTriang - 1] = temp2;
        j_FissionElems(mesh, Degree, AdjElems, ActiveList, &NLeftTriangs,
                       FirstTriang, SecondTriang);
      }
    }
    /* no fission element just merge the two selected */
    else
    {
      j_CombineTwoTriangs(mesh, Degree, AdjElems, ActiveList, &NLeftTriangs,
                          FirstTriang, SecondTriang, point1, point2,
                          PointFirst, PointSecond);
    }
  }

  /* free-up */
  e_FreeAdjacency(mesh, Degree, AdjElems);
  e_free(ActiveList);

  /* get the quads right */
  j_AdjustTriangQuadMesh(mesh, NKeepTriangle, KeepTriangle);

  /* ensure that the boundary nodes are the first nodes in the list */
  /* ArrangeBoundaryNodes(mesh, &NBoundaryNodes); */
  
}

/* 
 * Find the first triangle that could be used in the fusion process.
 * The element has minimal number of adjacent triangles or
 * it has only one adjacent element and the adjacent element has minimal
 * number of adjacent elements.
 */
static int j_FindFirstTriang(MESH *mesh, int *Degree, int **AdjElems,
                             int *ActiveList)
{
  int i, elem = 0, FirstMin = 4, SecondMin = 4;
  int Temp_SecondElem;

  for (i = 0; i < mesh->NElemsTriang1; i++)
  {
    /* if it has less adjacent element and still active */
    if (Degree[i] <= FirstMin && ActiveList[i] != 0)
    {
      /* only one side left for this triangle */
      if (Degree[i] == 1)
      {
        /* this is an adjacent element */
        Temp_SecondElem = AdjElems[i][0];
        /* again, check that it has less active adjacent element */
        if (Degree[Temp_SecondElem - 1] < SecondMin)
        {
          elem = i + 1;
          FirstMin = 1;
          SecondMin = Degree[Temp_SecondElem - 1];
        }
      }
      /* store it as a candidate */
      else
      {
        elem = i + 1;
        FirstMin = Degree[i];
      }
    }
  }
  return (elem);
}

/* 
 * Find the second triangle to use in a fusion process.
 * it tries to find a triangle which is adjacent, the distance between 
 * the two apex points is the shortest and if the number of adjacent 
 * triangles of the first triangle are larger than one then check that
 * the remaining number of elements is always even in the formed subregion.
 */
static int j_FindSecondTriang(MESH *mesh, int *Degree, int **AdjElems,
                              int *ActiveList, int NLeftTriangs,
                              int FirstTriang, int *point1, int *point2,
                              int *PointFirst, int *PointSecond, int fission)
{

  int i, TempPoints[4];
  int SecondElem = 0, Temp_SecondElem = 0;
  double min = 1.0e+5, Distance;
  int temp1, temp2;

  for (i = 0; i < Degree[FirstTriang - 1]; i++)
  {
    Temp_SecondElem = AdjElems[FirstTriang - 1][i];
    
    if (ActiveList[Temp_SecondElem - 1] == 0)
      continue;
    
    /* only if there are more than one possibilities, degrees of freedom */
    if (Degree[FirstTriang - 1] > 1)
    {
      temp1 = Degree[FirstTriang - 1];
      temp2 = Degree[Temp_SecondElem - 1];

      Degree[FirstTriang - 1] = 0;
      Degree[Temp_SecondElem - 1] = 0;
      
      /* check even number of elements in each subregion if we use fission */
      if (   fission
          && !j_ElemPairedRegions(mesh, NLeftTriangs, FirstTriang, 
                                  Temp_SecondElem, Degree, AdjElems) )
      {
        Degree[FirstTriang - 1] = temp1;
        Degree[Temp_SecondElem - 1] = temp2;
        continue;
      }

      Degree[FirstTriang - 1] = temp1;
      Degree[Temp_SecondElem - 1] = temp2;
    }

    /* calculate the distance between the two apex point */
    Distance =
      j_CalculateDiag(mesh, FirstTriang, Temp_SecondElem, &TempPoints[0],
                      &TempPoints[1], &TempPoints[2], &TempPoints[3]);

    /* FIXME:
       Here quality of the elements should be also checked !!! */
    if (Distance < min)
    {
      SecondElem = Temp_SecondElem;
      min = Distance;
      *point1 = TempPoints[0];
      *point2 = TempPoints[1];
      *PointFirst = TempPoints[2];
      *PointSecond = TempPoints[3];
    }
  }
  return (SecondElem);
}


/* find shared and non-shared nodes and 
   calculate the distance between the apex points */
static double j_CalculateDiag(MESH *mesh, int FirstTriang, int SecondTriang,
                              int *point1, int *point2, int *PointFirst,
                              int *PointSecond)
{
  j_FindApexPoints(mesh, FirstTriang, SecondTriang, point1, point2,
                   PointFirst, PointSecond);

  return (j_Com_Distance(mesh, *point1, *point2));
}

/* distance between to points */
static double j_Com_Distance(MESH *mesh, int point1, int point2)
{
  double Distance;

  Distance = sqrt(E_SQUARE(mesh->MeshPointCoords[point1 - 1][0] -
                           mesh->MeshPointCoords[point2 - 1][0]   ) +
                  E_SQUARE(mesh->MeshPointCoords[point1 - 1][1] -
                           mesh->MeshPointCoords[point2 - 1][1]   )   );

  return (Distance);
}


/* checks that the remaining fronts contain even number of nodes */
static int j_ElemPairedRegions(MESH *mesh, int NTriangElems, int IgnoreElem1,
                               int IgnoreElem2, int *Degree, int **AdjElems)
{
  int ElemCount;
  int *FrontElements;
  int InitFrontElem;
  int NElemsChecked;
  int NFrontElements;
  int *WorkSpace1;
  int *WorkSpace2;

  char *ElemFound;

  WorkSpace1 = e_alloc_i_vect(mesh->TotalNElems);
  WorkSpace2 = e_alloc_i_vect(mesh->TotalNElems);
  ElemFound = e_alloc_c_vect(mesh->TotalNElems);

  ElemFound[IgnoreElem1 - 1] = TRUE;
  ElemFound[IgnoreElem2 - 1] = TRUE;
  NTriangElems -= 2;
  NElemsChecked = 0;
  FrontElements = WorkSpace1;

  while (NElemsChecked < NTriangElems)
  {
    InitFrontElem = j_GetInitFrontElem(mesh, Degree, ElemFound);

    if (InitFrontElem > 0)
    {
      ElemCount = 1;
      NFrontElements = 1;
      FrontElements[0] = InitFrontElem;
      ElemFound[InitFrontElem - 1] = TRUE;

      while (NFrontElements > 0)
      {
        j_AdvanceFront(Degree, AdjElems, &NFrontElements, &FrontElements,
                       ElemFound, &WorkSpace1, &WorkSpace2);
        ElemCount += NFrontElements;
      }

      if (ElemCount % 2 == 1)
      {
        e_free(WorkSpace1);
        e_free(WorkSpace2);
        e_free(ElemFound);
        return 0;
      }

      NElemsChecked += ElemCount;
    }
  }

  e_free(WorkSpace1);
  e_free(WorkSpace2);
  e_free(ElemFound);

  return 1;
}


/* ////////////////////////////////////////////////////////////////////////// */
/* GetInitFrontElem: To return the index of the first element in a new
   advancing element front. If no such element exists, the return value is 0. */
static int j_GetInitFrontElem(MESH *mesh, int *Degree, char *ElemFound)
{
  int i;

  for(i = 0; i < mesh->TotalNElems; i++)
  {
    if(Degree[i] == 0)
      continue;

    if(!ElemFound[i])
      return i + 1;
  }

  return 0;
}


/* ////////////////////////////////////////////////////////////////////////// */
/* AdvanceFront: To advance the element front by one layer. */
static void j_AdvanceFront(int *Degree, int **AdjElems, int *NFrontElements,
                           int **FrontElements, char *ElemFound,
                           int **WorkSpace1, int **WorkSpace2)
{
  int i, j, k, l;
  int NNewFrontElements;
  int *Temp;

  NNewFrontElements = 0;

  for (k = 0; k < *NFrontElements; k++)
  {
    i = (*FrontElements)[k] - 1;

    for (j = 0; j < Degree[i]; j++)
    {
      l = AdjElems[i][j] - 1;

      if (!ElemFound[l])
      {
        ElemFound[l] = TRUE;
        (*WorkSpace2)[NNewFrontElements++] = l + 1;
      }
    }
  }

  Temp = *WorkSpace1;
  *WorkSpace1 = *WorkSpace2;
  *WorkSpace2 = Temp;

  *NFrontElements = NNewFrontElements;
  *FrontElements = *WorkSpace1;
}




/* find the shared and non-shared apex points */
static void j_FindApexPoints(MESH *mesh, int FirstTriang, int SecondTriang,
                             int *point1, int *point2, int *PointFirst,
                             int *PointSecond)
{
  int i, j;
  int point, temp, count1 = 0, count2 = 0;
  int CheckCommon = 0;
  int nodes1[3], nodes2[3];

  for (i = 0; i < 3; i++)
  {
    nodes1[i] = mesh->NodesTriang1[FirstTriang - 1][i];
    nodes2[i] = mesh->NodesTriang1[SecondTriang - 1][i];
  }

  /* determine the two common nodes, shared side */
  for (i = 0; i < 3; i++)
  {
    point = nodes1[i];
    count1 = 0;

    for (j = 0; j < 3; j++)
      if (nodes2[j] == point)
      {
        count1++;
        CheckCommon++;
      }

    if (count1 == 0)
    {
      temp = point;
      if(i == 0)
      {
        *PointFirst = nodes1[2];
        *PointSecond = nodes1[1];
      }
      else if(i == 2)
      {
        *PointFirst = nodes1[1];
        *PointSecond = nodes1[0];
      }
      else
      {
        *PointFirst = nodes1[0];
        *PointSecond = nodes1[2];
      }
    }
  }

  if (CheckCommon != 2)
    e_GeneralError("No common nodes with two checking elements");

  /* first non-shared node */
  *point1 = temp;

  /* second non-shared node */
  for (i = 0; i < 3; i++)
  {
    point = nodes2[i];
    count2 = 0;
    for (j = 0; j < 3; j++)
      if (nodes1[j] == point)
        count2++;
    if (count2 == 0)
      break;
  }

  *point2 = point;
}

/* combine two triangles into a quad, update relevant data structures */
static void j_CombineTwoTriangs(MESH *mesh, int *Degree, int **AdjElems,
                                int *ActiveList, int *NLeftTriangs,
                                int FirstTriang, int SecondTriang, int point1,
                                int point2, int PointFirst, int PointSecond)
{
  j_Com_Creat_Quad(mesh, point1, PointFirst, point2, PointSecond);

  (*NLeftTriangs) = (*NLeftTriangs) - 2;
  ActiveList[FirstTriang - 1] = 0;
  ActiveList[SecondTriang - 1] = 0;

  j_Com_Update_List(mesh, Degree, AdjElems, ActiveList, FirstTriang);
  j_Com_Update_List(mesh, Degree, AdjElems, ActiveList, SecondTriang);
  Degree[FirstTriang - 1] = 0;
  Degree[SecondTriang - 1] = 0;
}

/* check whether the quad is defined counter-clockwise */
static int j_Check_CCW_Quad(MESH *mesh, int *points)
{
  MTRF mtrf;
  int  ccw, cc, c, valid;
  int  i, prev, next;

  mtrf.used = FALSE;
  cc = 0;
  c  = 0;
  for(i = 0; i < 4; i++)
  {
    prev = (i+3) % 4;
    next = (i+1) % 4;
    ccw = IsCCW(mesh->MeshPointCoords[points[prev]-1], 
                mesh->MeshPointCoords[points[i]-1], 
                mesh->MeshPointCoords[points[next]-1], &mtrf, &valid);

    if(valid == TRUE)
    {
      if(ccw == TRUE)
      {
        cc++;
      }
      else
      {
        c++;
      }
    }
  }

  if(cc == 0 && c == 0)
  {
    e_GeneralError("j_Check_CCW_Quad: invalid quad element !");
  }

  if(cc > c)
    return TRUE;
  else
    return FALSE;
}

/* create a merged quad element */
static void j_Com_Creat_Quad(MESH *mesh, int point1, int point2, int point3,
                             int point4)
{

  int  p[4];

  p[0] = point1;
  p[1] = point2;
  p[2] = point3;
  p[3] = point4;
  mesh->NodesQuad1 = e_incr_cont_i_matr(mesh->NElemsQuad1, E_NNPE_QUAD1, 
                                        mesh->NodesQuad1, ALLOC_SIZE);
  /* check counter clock-wiseness */
  if(!j_Check_CCW_Quad(mesh, p))
  {
    p[1] = point4;
    p[3] = point2;
  }

  mesh->NodesQuad1[mesh->NElemsQuad1][0] = p[0];
  mesh->NodesQuad1[mesh->NElemsQuad1][1] = p[1];
  mesh->NodesQuad1[mesh->NElemsQuad1][2] = p[2];
  mesh->NodesQuad1[mesh->NElemsQuad1][3] = p[3];
  mesh->NElemsQuad1++;

  if(ShowGraphics)
  {
    mgnDrawAll(mesh->NMeshPoints, mesh->MeshPointCoords,
               0, NULL, mesh->NElemsQuad1, mesh->NodesQuad1, -1, 
               NULL, NULL, NULL);
  }


}

/* do element fission */
static void j_FissionElems(MESH *mesh, int *Degree, int **AdjElems,
                           int *ActiveList, int *NLeftTriangs,
                           int FirstTriang, int SecondTriang)
{
  int i, j, temp = 0;
  int point1, point2, SpecialNode = 0;
  int node1[2], node2[2];

  /* new point at the centre of the triangle */
  mesh->MeshPointCoords = e_incr_cont_d_matr(mesh->NMeshPoints, 3, 
                                             mesh->MeshPointCoords, 1);
  j_Com_Average(mesh, SecondTriang, mesh->NMeshPoints);
  mesh->NMeshPoints++;

  /* form first quad by the first triangle + the traingle defined by
     the shared nodes and the new centre node */
  j_FindApexPoints(mesh, FirstTriang, SecondTriang, &point1, &point2,
                   &node1[0], &node1[1]);
  j_Com_Creat_Quad(mesh, point1, node1[0], mesh->NMeshPoints, node1[1]);

  /* find a triangle adjacent to the second triangle, but not the first */
  for(i = 0; i < Degree[SecondTriang - 1]; i++)
  {
    temp = AdjElems[SecondTriang - 1][i];
    if (temp != FirstTriang)
      break;
  }
  if(temp == 0)
    e_GeneralError("j_FissionElems: internal error 1 !");

  /* again find shared and apex nodes */
  j_FindApexPoints(mesh, SecondTriang, temp, &point1, &point2, 
                   &node2[0], &node2[1]);
  /* create a quad by the adjacent triangle and the triangle defined
     by the shared nodes and the new centre node */
  j_Com_Creat_Quad(mesh, point2, node2[0], mesh->NMeshPoints, node2[1]);

  /* which node was shared by the two quad creation */
  for(i = 0; i < 2; i++)
  {
    for(j = 0; j < 2; j++)
    {
      if(node1[i] == node2[j])
      {
        SpecialNode = node1[i];
      }
    }
  }

  if(SpecialNode == 0)
    e_GeneralError("j_FissionElems: internal error 2 !");

  /* change the triangle configuration */
  for(i = 0; i < 3; i++)
  {
    if (mesh->NodesTriang1[SecondTriang - 1][i] == SpecialNode)
    {
      mesh->NodesTriang1[SecondTriang - 1][i] = mesh->NMeshPoints;
      break;
    }
  }
  
  /* update relevant data structures */
  (*NLeftTriangs) = (*NLeftTriangs) - 2;
  ActiveList[FirstTriang - 1] = 0;
  ActiveList[temp - 1] = 0;

  j_Com_Update_List(mesh, Degree, AdjElems, ActiveList, FirstTriang);
  j_Com_Update_List(mesh, Degree, AdjElems, ActiveList, temp);
  Degree[FirstTriang - 1] = 0;
  Degree[temp - 1] = 0;
}

/* determine the centre of the triangle */
static void j_Com_Average(MESH *mesh, int Elem1, int Node)
{
  int i, j;
  double average[3];

  average[0] = average[1] = average[2] = 0.0;
  for (i = 0; i < 3; i++)
  {
    for (j = 0; j < 3; j++)
    {
      average[i] = average[i] + 
                   mesh->MeshPointCoords[mesh->NodesTriang1[Elem1-1][j]-1][i];
    }
  }
  for (i = 0; i < 3; i++)
  {
    mesh->MeshPointCoords[Node][i] = average[i] / 3.0;
  }
}

/* update relevant data structures, adjacency, active list */
static void j_Com_Update_List(MESH *mesh, int *Degree, int **AdjElems,
                              int *ActiveList, int DeletedElem)
{
  int i, j, k, re_elem;

  for(i = 0; i < Degree[DeletedElem - 1]; i++)
  {
    re_elem = AdjElems[DeletedElem - 1][i];
    if(ActiveList[re_elem - 1] == 0)
      continue;

    if(Degree[re_elem - 1] == 0)
    {
      e_GeneralError("j_Com_Update_List: internal error 1 !");
    }
    
    if(Degree[re_elem - 1] == 1)
    {
      if(AdjElems[re_elem - 1][0] != DeletedElem)
      {
        e_GeneralError("j_Com_Update_List: internal error 2 !");
      }
    }
    else
    {
      for(j = 0; j < Degree[re_elem - 1]; j++)
      {
        if(AdjElems[re_elem - 1][j] == DeletedElem)
        {
          for(k = j; k < Degree[re_elem - 1] - 1; k++)
          {
            AdjElems[re_elem - 1][k] = AdjElems[re_elem - 1][k + 1];
          }

          Degree[re_elem - 1]--;

          if(Degree[re_elem - 1] == 0)
            e_GeneralError("j_Com_Update_List: internal error 3 !");
        }
      }
    }
  }
}


/* ////////////////////////////////////////////////////////////////////////// */
static void j_AdjustTriangQuadMesh(MESH *mesh, int NKeepTriangle, int *KeepTriangle)
{
  int   i, k;
  int **triang;
  

  mesh->NNodes        = mesh->NMeshPoints;
  mesh->TotalNodalDOF = 0;
  e_free(mesh->NodalDOF);
  mesh->NodalDOF = e_alloc_i_vect(mesh->NNodes);

  e_free(mesh->GNIs);
  mesh->GNIs = e_alloc_i_vect(mesh->NNodes);
  for (k = 0; k < mesh->NMeshPoints; k++)
  {
    mesh->GNIs[k] = k + 1;
    mesh->TotalNodalDOF += E_VERTEX_NODE_DOF_QUAD1;
    mesh->NodalDOF[k] = E_VERTEX_NODE_DOF_QUAD1;
  }

  if(NKeepTriangle == 0)
  {
    e_free_cont_i_matr(mesh->NElemsTriang1, mesh->NodesTriang1);
    mesh->NodesTriang1 = NULL;
    mesh->NElemsTriang1 = 0;

    mesh->FirstNetIndex[E_ELEM_TYPE_TRIANG1] = 0;
    mesh->FirstNetIndex[E_ELEM_TYPE_QUAD1] = 1;
  }
  else
  {

    triang = e_alloc_cont_i_matr(NKeepTriangle, 3);
    for(k = 0; k < NKeepTriangle; k++)
    {
      for(i = 0; i < 3; i++)
      {
        triang[k][i] = mesh->NodesTriang1[KeepTriangle[k]-1][i];
      }
    }
    e_free_cont_i_matr(mesh->NElemsTriang1, mesh->NodesTriang1);
    mesh->NodesTriang1 = triang;
    mesh->NElemsTriang1 = NKeepTriangle;
    
    mesh->FirstNetIndex[E_ELEM_TYPE_TRIANG1] = 1;
    mesh->FirstNetIndex[E_ELEM_TYPE_QUAD1] = mesh->NElemsTriang1+1;
  }


  mesh->TotalNElems = mesh->NElemsQuad1 + mesh->NElemsTriang1;
  mesh->GEIs = e_alloc_i_vect(mesh->TotalNElems);
  for (i = 0; i < mesh->TotalNElems; i++)
  {
    mesh->GEIs[i] = i + 1;
  }
  
  if(mesh->NElemsTriang1 > 0)
  {
    mesh->GEIsTriang1 = mesh->GEIs + mesh->FirstNetIndex[E_ELEM_TYPE_TRIANG1] - 1;
    mesh->KeywordSet[E_KW_NUM_NELEMS_TRIANG1] = TRUE;
    mesh->KeywordSet[E_KW_NUM_NODES_TRIANG1] = TRUE;
  }
  else
  {
    mesh->GEIsTriang1 = NULL;
    mesh->KeywordSet[E_KW_NUM_NELEMS_TRIANG1] = FALSE;
    mesh->KeywordSet[E_KW_NUM_NODES_TRIANG1] = FALSE;
  }
  
  
  if(mesh->NElemsQuad1 > 0)
  {
    mesh->GEIsQuad1 = mesh->GEIs + mesh->FirstNetIndex[E_ELEM_TYPE_QUAD1] - 1;
    mesh->KeywordSet[E_KW_NUM_NELEMS_QUAD1] = TRUE;
    mesh->KeywordSet[E_KW_NUM_NODES_QUAD1] = TRUE;
  }
  else
  {
    mesh->GEIsQuad1 = NULL;
    mesh->KeywordSet[E_KW_NUM_NELEMS_QUAD1] = TRUE;
    mesh->KeywordSet[E_KW_NUM_NODES_QUAD1] = TRUE;
  }
}




