#ifndef DTR_H
#define DTR_H


/* include files */
#include "e_lib.h"
#include "mgni.h"
#include "bgdata.h"
#include "mtrf.h"

/* the structure for new generated node */
typedef struct
{
  int np;                       /* number of points in the set */
  int tap;                      /* number of allocated spaces in cr */
  double **cr;                  /* the coordinates of the points and the mpr value in [3] */
}
RFS;

/* mapping transformation structures and functions */
#include "mtrf.h"

/* from dtr_main.c */
void boundary_refinement_and_recovery(double ***coords, int *np, int *tp,
                                      int ctrltype, int geomset, MESH *mesh,
                                      BG_DATA *bgd, double **mpr, double h,
                                      BND *bnd, int ***tpnl,
                                      int ***triangles, int *ntriangles,
                                      int *ttriangles, int **convhull,
                                      int *nhullelems, int force_even);

/* from dtr_pp.c */
void smooth_by_nodal_averaging(double **coords, int np, int **tr, int ntr,
                               int **tpnl, int sm);

/* from dtr_del.c */
int trianglesearch(double *i, double **coords, int **tr, int starttr,
                   int *ft, int *onside, MTRF * mtrf);
int exh_tr_search(double *i, double **coords, int **tr, int ntr,
                  int *ft, int *onside, MTRF * mtrf);
void insert_node(int i, int t, int *pNtr, int **tr, int **efwl,
                 int *pnwl, int *ptwl, int onside, int **hull, int *nhull,
                 int **tpnl);
void update_hull(int **hull, int *nhull, int ossi, int osei, int i);
void connect_outside_node(int i, double **coords, int *pNtr, int **tr,
                          int **efwl, int *pnwl, int *ptwl, int **hull,
                          int *pNhull, int **tpnl, MTRF * mtrf);
void edge_flipping(int i, double **coords, int **efwl, int *pnwl, int *ptwl,
                   int **tr, int **tpnl, MTRF * mtrf);
void add_new_nodes(double **coords, int np, int nnp, int stnp, int ***triangles,
                   int *ntriangles, int *ttriangles, int **convhull,
                   int *nhullelems, int **tpnl, MTRF *mtrf);
int delaunay_check(int **tr, int ntr, double **coords, int np);

/* from dtr_io.c */
void parse_initial_mesh(MESH *mesh, double ***coords, int *np);
void parse_boundary_of_mesh(MESH *mesh, double ***coords, int *np);
void parse_full_mesh(MESH * bgmesh, double ***bgcoords, int *bgnp,
                     int ***bgtr, int *bgntr, int relax_adj);
void create_outputmesh(double **coords, int np, int **triangles,
                       int ntriangles, MESH *outputmesh);
void parse_mesh(int *task, int meth, MESH *mesh, double ***coords, int *np,
                int ***tr, int *ntr, int *ttr, int ***tpnl, int *nhull,
                int **convhull, BND ** bnd);

/* from dtr_bnd.c */
void makehalfwaynode(double ***coords, int *np, int *tp,
                     int nd, int nxnd, int ***tpnl, int ctrltype,
                     double **crsnmpr);
int edgechecked(int **tpnl, int **triangles, int nd, int nxnd);
int make_boundary_nodes(double ***coords, int *np, int *tp, int *stnp,
                        int ctrltype, int geomset, MESH *mesh,
                        BG_DATA *bgd, double **mpr, double h, BND *bnd,
                        int ***tpnl, int force_even);
void setup_2D_bnd(MESH *mesh, BND **bnd);
int recover_boundaries(double ***coords, int *np, int *tp, int *stnp,
                       int **tr, int ntr, int ***tpnl,
                       BND *bnd, int ctrltype, double **crsnmpr,
                       MTRF *mtrf);
void updatebnd(EDGE *edge, int node, int nnp, int startnp);
void setareacode(BND *bnd, int **tpnl, int **tr, int ntr);
int leftandrighttr(int **tr, int **tpnl, int fn, int sn, int *lt, int *rt);
EDGE *incr_edge_vect(unsigned int length, EDGE * vector,
                     unsigned int increment);
int recover_bnd_by_swap(double ***coords, int *np, int *tp, int *stnp,
                        int **tr, int ntr, int ***tpnl, BND *bnd,
                        int ctrltype, double **crsnmpr, MTRF *mtrf);
void check_and_swap_edge(double **coords, int np, int **tr, int ntr, int nd,
                         int nx, int **tpnl, MTRF *mtrf, int *nswaps);
void swap2triangles(int **tr, int t1, int t2, int **tpnl);
int refinesegment(double **coords, int fn, int sn, RFS *refset, int ctrltype,
                  int geomset, MESH *mesh, BG_DATA *bgd, double *mpr, double h,
                  int mode);
void addrefsettocoords(double ***coords, int *np, int *tp, RFS *refset,
                       int ***tpnl, int reversed);
void updatempr(double **mpr, int np, int *tp, RFS *refset, int reversed);

/* from dtr_tpn */
void addtr2tpnlist(int **tr, int t, int **tpnl);
void rmvtrftpnlist(int t, int n, int **tpnl);
void inittpnl(int ***tpnl, int np);
void createtpnl(int ***tpnl, int **tr, int ntr, int np);
void freetpnl(int **tpnl, int np);

/* from dtr_patch */
void patchnonconv(double **coords, int ***tr, int *ntr, int *ttr, int **tpnl,
                  int nh, int *hull, BND *bnd);
void patchholes(double **coords, int ***tr, int *ntr, int *ttr, int **tpnl,
                EDGE ie);

/* from dtr_ebe.c */
void setup_edges(int **tr, int ntr, EDGE ** edgs);
void refine_edges(double ***coords, int *np, int *tp,
                  int ctrltype, int geomset, MESH * mesh,
                  BG_DATA *bgd, double **mpr, double h, int ***tpnl, int **tr,
                  int ntr, EDGE * edgs, int force_even);
void ebe_refine(MGNI *mgni, double ***coords, int *np, int *tp, int geomset,
                MESH *mesh, BG_DATA *bgd, double **mpr, int ***tpnl,
                int ***triangles, int *ntriangles, int *ttriangles,
                EDGE *edgs, int mtrfused);

/* from dtr_f.c */
#ifndef WIN32
void f_randomize_mesh(MESH * mesh);
#endif
int f_compute_convex_hull(double **coords, int nnodes, int **h, MTRF *mtrf,
                          int meth);
void f_connect_mesh(double **coords, int np, int ***triangles,
                    int *ntriangles, int **convhull, int *nhullelems,
                    int **tpnl, MTRF *mtrf);
void f_connect_bnd_nodes(double **coords, int np, BND *bnd, int ***triangles,
                         int *ntriangles, int **convhull, int *nhullelems,
                         int **tpnl, MTRF *mtrf);
void f_refine_del_mesh(double ***coords, int *np, int *tcs, int ***triangles,
                       int *ntriangles, int *tts, int **convhull,
                       int *nhullelems, int ctrl, double alfa,
                       double beta, BG_DATA *bgd, double **mpr, double h,
                       int ***tpnl, BND *bnd, int dln, MTRF *mtrf);


#endif
