#include "mgn.h"
#include "mgn_graphics.h"
#include "dtr.h"

void setup_edges(int **tr, int ntr, EDGE **edgs)
{
  int t, s;

  (*edgs) = (EDGE *) e_calloc(ntr, sizeof(EDGE));
  for (t = 0; t < ntr; t++)
  {
    (*edgs)[t].nen = 3;
    (*edgs)[t].en = e_alloc_i_vect(EDGE_INCR);
    for (s = 0; s < 3; s++)
      (*edgs)[t].en[s] = tr[t][s];
  }
}
void refine_edges(double ***coords, int *np, int *tp,
                  int ctrltype, int geomset, MESH *mesh,
                  BG_DATA *bgd, double **mpr, double h, int ***tpnl, int **tr,
                  int ntr, EDGE *edgs, int force_even)
{
  int t, s, b, p;
  int oldtp;
  int nsp;                      /* number of inserted nodes per segment */
  int fn;                       /* first node of the segment */
  int sn;                       /* second node of the segment */
  RFS *refset;                  /* the set of refined nodes to be inserted */
  int reversed;                 /* is the refset reversed */
  int i1, i2;                   /* index of start and end of sequence in neighbours edgs entry */

  refset = NULL;
  refset = (RFS *) e_calloc(1, sizeof(RFS));
  refset->np = 0;
  refset->tap = INCR_SIZE;
  refset->cr = NULL;
  refset->cr = e_alloc_cont_d_matr(refset->tap, 4);

  for (t = 0; t < ntr; t++)
  {
    b = 0;
    for (s = 0; s < 3; s++)
    {
      if (tr[t][s + 3] != -1)
        if (edgs[tr[t][s + 3]].nen > 3)
        {
          /* first find out the orientation of the shared edge */
          if ((tr[t][s]      ==tr[tr[t][s+3]][(tr[t][s+6]-3+1)%3])&&
            	(tr[t][(s+1)%3]==tr[tr[t][s+3]][ tr[t][s+6]-3     ])  )
            reversed=FALSE;
          else if ((tr[t][(s+1)%3]==tr[tr[t][s+3]][(tr[t][s+6]-3+1)%3])&&
            			 (tr[t][s]      ==tr[tr[t][s+3]][ tr[t][s+6]-3     ])  )
            reversed=TRUE;
          else
            e_GeneralError("neighbours don't match");
          /* copy the refined nodes from the neighbour node */
          i1 = 0;
          while (edgs[tr[t][s + 3]].en[i1] != tr[t][(s + 1) % 3])
            i1++;
          i2 = i1;
          while (edgs[tr[t][s + 3]].en[i2 % edgs[tr[t][s + 3]].nen] !=
                 tr[t][s])
            i2++;
          if (!reversed)
            nsp = (i2 - i1) - 1;
          else
            nsp = edgs[tr[t][s + 3]].nen - (i2 - i1);
          for (p = 0; p < nsp; p++)
            edgs[t].en =
              e_incr_i_vect(edgs[t].nen + p, edgs[t].en, EDGE_INCR);
          for (p = edgs[t].nen - 1; p > b; p--)
            edgs[t].en[nsp + p] = edgs[t].en[p];
          for (p = 0; p < nsp; p++)
          {
            if (!reversed)
              edgs[t].en[b + 1 + p] = edgs[tr[t][s + 3]].en[(i2 - 1 - p)];
            else
              edgs[t].en[b + 1 + p] = edgs[tr[t][s + 3]].en[(i2 + 1 + p)%edgs[tr[t][s + 3]].nen];
            edgs[t].nen++;
          }
          b += nsp + 1;
          continue;
        }
      fn = tr[t][s];
      sn = tr[t][(s + 1) % 3];
      refset->np = 0;
      reversed = refinesegment((*coords), fn, sn, refset, ctrltype, geomset, mesh,
                        bgd, *mpr, h, (tr[t][s + 3]==-1)?E_SEGREF_ODD:E_SEGREF_NONE);
      updatebnd(&(edgs[t]), b, refset->np, (*np));
      b += refset->np + 1;
      oldtp = (*tp);
      addrefsettocoords(coords, np, tp, refset, tpnl, reversed);
      if (ctrltype == CTRL_MPR)
      {
        updatempr(mpr, (*np), &oldtp, refset, reversed);
        if (oldtp != (*tp))
          e_GeneralError("unexpected error in make_boundary_nodes");
      }
      (*np) += refset->np;
    }
    if (DebugOutput)
      for (s = 0; s < edgs[t].nen; s++)
        printf("triangle[%d] edge[%d]=%d\n", t + 1, s + 1, edgs[t].en[s] + 1);
  }
}

void ebe_refine(MGNI *mgni, double ***coords, int *np, int *tp, int geomset,
                MESH *mesh, BG_DATA *bgd, double **mpr, int ***tpnl,
                int ***triangles, int *ntriangles, int *ttriangles,
                EDGE *edgs, int usemtrf)
{
  int t;                        /* the element being refined */
  int s;                        /* side counter */
  int **etr;                    /* triangle matrix for current element */
  int netr;                     /* number of tr's in etr */
  int tetr;                     /* allocated length of etr */
  int *hull;                    /* convex hull of etr */
  int nh;                       /* number of elems in hull */
  int **etpnl;                  /* tpnl for etr */
  int p, e;                     /* other counters */

  int **crstr;                  /* the coarse triangle matrix */
  int ncrstr;                   /* no of tr's in crstr */

  MTRF mtrf;                    /* mapping transformation data */

  /* make a dummy of the coarse triangle matrix */
  crstr = e_alloc_cont_i_matr(*ntriangles, 3);
  ncrstr = *ntriangles;
  for (t = 0; t < *ntriangles; t++)
    for (s = 0; s < 3; s++)
      crstr[t][s] = (*triangles)[t][s];

  /* reset the new triangle matrix and tpnl */
  *ntriangles = 0;
  e_free_cont_i_matr(*ttriangles, *triangles);
  *ttriangles = 1;
  *triangles = e_alloc_cont_i_matr(*ttriangles, 11);
  freetpnl(*tpnl, *tp);

  for (t = 0; t < ncrstr; t++)
  {
    if (PrintProgress)
      printf("\nebe refinement: refining coarse triangle %d\n", t + 1);

    /* setup the triangle matrix for the element */
    etr = e_alloc_cont_i_matr(1, 11);
    netr = 1;
    tetr = 1;
    hull = e_alloc_i_vect(INCR_SIZE);
    nh = 3;
    for (s = 0; s < 3; s++)
    {
      etr[0][s] = crstr[t][s];
      etr[0][3 + s] = -1;
      hull[s] = crstr[t][s];
    }
    etr[0][9] = 1;
    etr[0][10] = 1;
    inittpnl(&etpnl, *tp);
    addtr2tpnlist(etr, 0, etpnl);

    /* setup the mapping transformation data */
    if (usemtrf)
    {
      if (!get_mtrf(*coords, etr[0], &mtrf))
        e_GeneralError("triangle %d can't be mapped", t + 1);
      mtrf.used = TRUE;
    }
    else
      mtrf.used = FALSE;

    /* add the boundary refinement nodes */
    for (e = 0; e < edgs[t].nen; e++)
    {
      for (s = 0; s < 3; s++)
        if (edgs[t].en[e] == crstr[t][s])
          break;
      if (s < 3)
        continue;
      add_new_nodes(*coords, *np, 1, edgs[t].en[e], &etr, &netr, &tetr,
                    &hull, &nh, etpnl, &mtrf);
      if (DebugOutput && ShowGraphics)
      {
        mgnDrawAll(*np, *coords, netr, etr, 0, NULL, 0,
                   NULL, NULL, "triangulation");
      }
    }

    /* perform the element refinement */
    f_refine_del_mesh(coords, np, tp, &etr, &netr, &tetr, &hull, &nh,
                      mgni->ctrl, mgni->alfa, mgni->beta, bgd, mpr,
                      mgni->h, &etpnl, NULL, mgni->dln, &mtrf);

    /* copy the resulting refinement triangles into the
     * global triangle matrix */
    while ((*ttriangles) - (*ntriangles) < netr)
    {
      (*triangles) =
        e_incr_cont_i_matr((*ttriangles), 11, (*triangles), (*ttriangles));
      (*ttriangles) *= 2;
    }
    for (p = 0; p < netr; p++)
    {
      for (s = 0; s < 6; s++)
        (*triangles)[(*ntriangles)][s] = etr[p][s];
      /* watch out the adjacency info is copied but doesn't span
       * more tahn one element! */
      (*triangles)[(*ntriangles)][10] = +1;
      (*ntriangles)++;
    }
    e_free_cont_i_matr(tetr, etr);
    freetpnl(etpnl, *tp);
    e_free(hull);
  }

  /* get the tpnl right otherwise smoothing won't work */
  inittpnl(tpnl, *tp);
  for (p = 0; p < *ntriangles; p++)
    addtr2tpnlist((*triangles), p, *tpnl);
}
