#include "eg_lib.h"
#include "mgn.h"
#include "dtr.h"
#include "dtr_surf.h"

#define MGNI_ALFA	1.20
#define MGNI_BETA	1.0
#define MGNI_DLN	DLN_CENTR
#define MGNI_SMTYPE	SM_NODAL_AVG
#define MGNI_SMITER	5

/*static int convex_hull_of_nodes(double **coords, int *nodes, int nnodes,
                                int **hull, MTRF *mtrf, int meth);*/
static double area2Dsurfin3D(double **coords, int *nodes, int nnodes,
                             MTRF *mtrf);

void refine_surface(double ***coords, int *np, int *tp, double h,
                    int nedgs, int **edgs, int nsurfs, SURF *surfs,
                    int ***tr, int *ntr)
{
  EDGE *refedgs;                /* the refined edges */
  RFS *refset;                  /* the set of refined nodes to be inserted */
  int reversed;                 /* is the refset reversed */
  int e;                        /* edge counter */
  int oldtp;

  int s;                        /* the surface being refined */
  int **etr;                    /* triangle matrix for current surface */
  int netr;                     /* number of tr's in etr */
  int tetr;                     /* allocated length of etr */
  int *hull;                    /* convex hull of etr */
  int nh;                       /* number of elems in hull */
  int **etpnl;                  /* tpnl for etr */
  int n, i;                     /* other counters */

  MTRF mtrf;                    /* mapping transformation data */
  int starttr[3];               /* triangle that determines mtrf */
  BND *bnd;                     /* the boundary structure for the surface */
  int nct;                      /* for triangle counting */

  double area;                  /* for CCW determination */
  int cn;
  int j, b;
  int *tempe;

  /* dummy variables */
  int **tpnl;
  int stnp, nnp;

  tpnl = NULL;
  inittpnl(&tpnl, *tp);

  /* setup the structure for the refined edges */
  refedgs = (EDGE *) e_calloc(nedgs, sizeof(EDGE));
  for (e = 0; e < nedgs; e++)
  {
    refedgs[e].nen = 2;
    refedgs[e].en = e_alloc_i_vect(EDGE_INCR);
    refedgs[e].en[0] = edgs[e][0];
    refedgs[e].en[1] = edgs[e][1];
  }

  /* setup the refset */
  refset = NULL;
  refset = (RFS *) e_calloc(1, sizeof(RFS));
  refset->tap = INCR_SIZE;
  refset->cr = NULL;
  refset->cr = e_alloc_cont_d_matr(refset->tap, 4);

  /* first refine all edges */
  for (e = 0; e < nedgs; e++)
  {
    refset->np = 0;
    reversed =
      refinesegment((*coords), edgs[e][0], edgs[e][1], refset, CTRL_UNIFORM,
                    FALSE, NULL, NULL, NULL, h, E_SEGREF_NONE);
    oldtp = (*tp);
    addrefsettocoords(coords, np, tp, refset, &tpnl, reversed);
    if (oldtp != (*tp))
      e_GeneralError("unexpected error in make_boundary_nodes");
    for (n = 0; n < refset->np; n++)
    {
      refedgs[e].en = e_incr_i_vect(refedgs[e].nen, refedgs[e].en, EDGE_INCR);
      refedgs[e].en[1 + n] = (*np) + n;
      refedgs[e].nen++;
    }
    (*np) += refset->np;
    refedgs[e].en[1 + n] = edgs[e][1];

    /* printf("refedge[%d]=",e+1);
       for (i=0;i<refedgs[e].nen;i++)
       printf("%d ",refedgs[e].en[i]);
       printf("\n"); */
  }

  freetpnl(tpnl, *tp);

  /* surface refinement */
  *ntr = 0;
  *tr = NULL;

  for (s = 0; s < nsurfs; s++)
  {
    if (PrintProgress)
      printf("\nsurface refinement: refining surface %d\n", s + 1);

    /* compile the coarse boundary for the surface */
    bnd = (BND *) e_calloc(1, sizeof(BND));
    bnd->oe.nen = 0;
    for (e = 0; e < surfs[s].ol.ne; e++)
      bnd->oe.nen += refedgs[surfs[s].ol.e[e]].nen - 1;
    bnd->oe.en = e_alloc_i_vect(bnd->oe.nen);
    bnd->oe.nen = 0;
    for (e = 0; e < surfs[s].ol.ne; e++)
    {
      if (e == 0)
      {
        if ((edgs[surfs[s].ol.e[0]][1] == edgs[surfs[s].ol.e[1]][0]) ||
            (edgs[surfs[s].ol.e[0]][1] == edgs[surfs[s].ol.e[1]][1]))
          reversed = FALSE;
        else
          reversed = TRUE;
      }
      else
      {
        if ((edgs[surfs[s].ol.e[e]][1] == edgs[surfs[s].ol.e[e - 1]][0]) ||
            (edgs[surfs[s].ol.e[e]][1] == edgs[surfs[s].ol.e[e - 1]][1]))
          reversed = TRUE;
        else
          reversed = FALSE;
      }
      for (n = 0; n < refedgs[surfs[s].ol.e[e]].nen - 1; n++)
      {
        if (!reversed)
          bnd->oe.en[bnd->oe.nen] = refedgs[surfs[s].ol.e[e]].en[n];
        else
          bnd->oe.en[bnd->oe.nen] =
            refedgs[surfs[s].ol.e[e]].en[refedgs[surfs[s].ol.e[e]].nen - n -
                                         1];
        bnd->oe.nen++;
      }
    }

    /* for (i=0;i<bnd->oe.nen;i++)
       printf("bnd surf %d oe[%d]=%d\n",s,i,bnd->oe.en[i]); */

    bnd->nie = surfs[s].nil;
    if (bnd->nie > 0)
      bnd->ie = (EDGE *) e_calloc(bnd->nie, sizeof(EDGE));
    for (i = 0; i < bnd->nie; i++)
    {
      bnd->ie[i].nen = 0;
      for (e = 0; e < surfs[s].il[i].ne; e++)
        bnd->ie[i].nen += refedgs[surfs[s].il[i].e[e]].nen - 1;
      bnd->ie[i].en = e_alloc_i_vect(bnd->ie[i].nen);
      bnd->ie[i].nen = 0;
      for (e = 0; e < surfs[s].il[i].ne; e++)
      {
        if (e == 0)
        {
          if ((edgs[surfs[s].il[i].e[0]][1] == edgs[surfs[s].il[i].e[1]][0])
              || (edgs[surfs[s].il[i].e[0]][1] ==
                  edgs[surfs[s].il[i].e[1]][1]))
            reversed = FALSE;
          else
            reversed = TRUE;
        }
        else
        {
          if ((edgs[surfs[s].il[i].e[e]][1] ==
               edgs[surfs[s].il[i].e[e - 1]][0])
              || (edgs[surfs[s].il[i].e[e]][1] ==
                  edgs[surfs[s].il[i].e[e - 1]][1]))
            reversed = TRUE;
          else
            reversed = FALSE;
        }
        for (n = 0; n < refedgs[surfs[s].il[i].e[e]].nen - 1; n++)
        {
          if (!reversed)
            bnd->ie[i].en[bnd->ie[i].nen] =
              refedgs[surfs[s].il[i].e[e]].en[n];
          else
            bnd->ie[i].en[bnd->ie[i].nen] =
              refedgs[surfs[s].il[i].e[e]].en[refedgs[surfs[s].il[i].e[e]].
                                              nen - n - 1];
          bnd->ie[i].nen++;
        }
      }
      /* for (j=0;j<bnd->ie[i].nen;j++)
         printf("bnd surf %d ie[%d][%d]=%d\n",s,i,j,bnd->ie[i].en[j]); */

    }

    /* setup the triangle matrix for the surface */
    etr = NULL;
    netr = 0;
    hull = NULL;
    nh = 0;
    etpnl = NULL;
    inittpnl(&etpnl, *tp);

    /* get start triangle for init of mtrf *
     * is unsafe with non-convex surfaces because it might not correpond to direction of bnd */
    starttr[0] = edgs[surfs[s].ol.e[0]][0];
    starttr[1] = edgs[surfs[s].ol.e[0]][1];

    for (e = 1; e < surfs[s].ol.ne; e++)
    {
      cn = edgs[surfs[s].ol.e[e]][0];
      if ((cn != starttr[0]) && (cn != starttr[1]))
      {
        area=eg_TriangArea3D((*coords)[starttr[0]],
                             (*coords)[starttr[1]],
                             (*coords)[cn]);
        if (E_ABS(area) > 1e-6)
          break;
      }
      cn = edgs[surfs[s].ol.e[e]][1];
      if ((cn == starttr[0]) || (cn == starttr[1]))
        continue;
      area=eg_TriangArea3D((*coords)[starttr[0]],
                           (*coords)[starttr[1]],
                           (*coords)[cn]);
      if (E_ABS(area) > 1e-6)
        break;
    }

    starttr[2] = cn;

    if (e == surfs[s].ol.ne)
      e_GeneralError("can't find starting triangle for mtrf");

    printf("starttr for mtrf=(%d %d %d)\n",
           starttr[0], starttr[1], starttr[2]);

    /* get preliminary mtrf */
    if (!get_mtrf(*coords, starttr, &mtrf))
      e_GeneralError("surface %d can't be mapped to 2D", s + 1);
    mtrf.used = TRUE;

    /* then detrmine convex hull of surface
       nh=convex_hull_of_nodes(*coords, bnd->oe.en,bnd->oe.nen, &hull, &mtrf, 1);

       starttr[0]=hull[0];
       starttr[1]=hull[1];
       cn=2;
       for (i=0;i<3;i++)
       middle[i]=((*coords)[starttr[0]][i]+(*coords)[starttr[0]][i])/2.0;
       while (E_ABS(orient3d((*coords)[starttr[0]],(*coords)[starttr[1]],middle,(*coords)[hull[cn]]))<1e-4)
       {
       printf("starttr %d,%d,%d val=%1.2e\n",starttr[0],starttr[1],hull[cn],
       E_ABS(orient3d((*coords)[starttr[0]],(*coords)[starttr[1]],middle,(*coords)[hull[cn]])));
       cn++;
       }
       starttr[2]=hull[cn];
       e_free(hull);
       nh=0; */

    /* check if with this mtrf the bnd is mapped as CCW */
    area = area2Dsurfin3D(*coords, bnd->oe.en, bnd->oe.nen, &mtrf);
    if (area < 0.0)             /* boundary is CW instead of CCW */
    {
      j = starttr[1];
      starttr[1] = starttr[2];
      starttr[2] = j;
      /* and modify the mtrf */
      if (!get_mtrf(*coords, starttr, &mtrf))
        e_GeneralError("surface %d can't be mapped to 2D", s + 1);
      mtrf.used = TRUE;
    }

    /* if there are internal boundaries (holes)
     * check their orientation */
    for (b = 0; b < bnd->nie; b++)
    {
      area = area2Dsurfin3D(*coords, bnd->ie[b].en, bnd->ie[b].nen, &mtrf);
      if (area > 0.0)           /* boundary is CCW instead of CW */
      {
        /* revert boundary definition */
        printf("\nsurf %d internal bnd %d direction reversed\n", s + 1,
               b + 1);
        tempe = e_alloc_i_vect(bnd->ie[b].nen);
        memcpy((void *) tempe, (void *) (bnd->ie[b].en),
               bnd->ie[b].nen * sizeof(int));
        for (j = 0; j < bnd->ie[b].nen; j++)
          bnd->ie[b].en[j] = tempe[bnd->ie[b].nen - 1 - j];
        e_free(tempe);
        /* for (j=0;j<bnd->ie[b].nen;j++)
           printf("bnd surf %d ie[%d][%d]=%d\n",s,b,j,bnd->ie[b].en[j]); */
      }
    }

    /* connect boundary nodes */
    f_connect_bnd_nodes(*coords, *np, bnd, &etr, &netr, &hull,
                        &nh, etpnl, &mtrf);
    tetr = netr;

    /* boundary refinement and recovery by halfwaynode method */

    nnp = recover_boundaries(coords, np, tp, &stnp, etr, netr, &etpnl,
                             bnd, CTRL_UNIFORM, NULL, &mtrf);

    if (nnp != 0)
      e_GeneralError("boundary recovery failed");

    /* setting the areacodes */
    setareacode(bnd, etpnl, etr, netr);

    /* refine the interior of the domain */
    f_refine_del_mesh(coords, np, tp, &etr, &netr,
                      &tetr, &hull, &nh,
                      CTRL_UNIFORM, MGNI_ALFA, MGNI_BETA, NULL, NULL, h,
                      &etpnl, NULL, MGNI_DLN, &mtrf);
    /* smoothing by nodal averaging before output */
    if (MGNI_SMTYPE == SM_NODAL_AVG)
      smooth_by_nodal_averaging(*coords, *np, etr, netr, etpnl, MGNI_SMITER);

    /* count how many triangles to copy */

    /* for (i=0;i<netr;i++)
       etr[i][10]=1; */


    nct = 0;
    for (i = 0; i < netr; i++)
      if (etr[i][10] != -1)
        nct++;

    /* copy the surface triangles into the global triangle matrix */

    (*tr) = e_ext_cont_i_matr(*ntr, 3, *tr, nct);
    for (i = 0; i < netr; i++)
      if (etr[i][10] != -1)
      {
        for (n = 0; n < 3; n++)
          (*tr)[*ntr][n] = etr[i][n];
        (*ntr)++;
      }

    /* freeup all memory used for this surface */
    e_free_cont_i_matr(tetr, etr);
    freetpnl(etpnl, *tp);
    e_free(hull);
    e_free(bnd->oe.en);
    for (i = 0; i < bnd->nie; i++)
      e_free(bnd->ie[i].en);
  }

  /* freeup more bits'n pieces */
  for (i = 0; i < nedgs; i++)
    e_free(refedgs[i].en);
  e_free_cont_d_matr(refset->tap, refset->cr);
  e_free(refset);
}

/*static int convex_hull_of_nodes(double **coords, int *nodes, int nnodes,
                                int **hull, MTRF * mtrf, int meth)
{
  double **crds;
  int i, s;
  int nh;

  crds = e_alloc_cont_d_matr(nnodes, 3);
  for (i = 0; i < nnodes; i++)
    for (s = 0; s < 3; s++)
      crds[i][s] = coords[nodes[i]][s];
  nh = f_compute_convex_hull(crds, nnodes, hull, mtrf, meth);
  e_free_cont_d_matr(nnodes, crds);
  return nh;
}*/

static double area2Dsurfin3D(double **coords, int *nodes, int nnodes,
                             MTRF * mtrf)
{
  double mapp0[3], mappj[3], mappjp[3];
  double x, y, x1, y1;
  double area;
  int j;

  if (!mtrf_map(coords[nodes[0]], mtrf, mapp0))
    e_GeneralError("area2Dsurfin3D: mapping error 0");
  area = 0.0;
  for (j = 0; j < nnodes - 1; j++)
  {
    if (!mtrf_map(coords[nodes[j]], mtrf, mappj))
      e_GeneralError("area2Dsurfin3D: mapping error j");
    if (!mtrf_map(coords[nodes[j + 1]], mtrf, mappjp))
      e_GeneralError("area2Dsurfin3D: mapping error jp");
    x = mappj[0] - mapp0[0];
    y = mappj[1] - mapp0[1];
    x1 = mappjp[0] - mappj[0];
    y1 = mappjp[1] - mappj[1];
    area += x * y1 - y * x1;
  }

  return area;
}
