/* the boundary structures */
#define EDGE_INCR	10
typedef struct
{
  int ne;                       /* number of edges in the loop */
  int *e;                       /* the edge indices */
}
ELOOP;

typedef struct
{
  ELOOP ol;                     /* the outer loop */
  int nil;                      /* number of inner loops (for holes) */
  ELOOP *il;                    /* the inner loops */
}
SURF;

void refine_surface(double ***coords,
                    int *np,
                    int *tp,
                    double h,
                    int nedgs,
                    int **edgs,
                    int nsurfs, SURF * surfs, int ***tr, int *ntr);

/* The surface refinement function
 *
 *  coords	pointer to double** array of coordinates of all vertices used
 *  np		pointer to the number of points specified in coords
 *  tp		pointer to the number of allocated rows in coords (always>=np)
 *	h     uniform mesh size
 *  nedgs	number of edges in edgs matrix
 *  edgs	edgs matrix. two columns. nedgs rows. contains node indices.
 *	  			therefore starting from 0
 *  nsurfs	number of surfaces defined in surfs
 *  surfs	the surfaces are defined in a matrix of SURF structures
 *	  			care should be taken that the ol is oriented in a different
 *				direction than the il.
 *  tr		pointer to triangle matrix which will be allocated by the mgn
 *	ntr		number of triangles which will be generated and stored in tr
 */
