/* FILE mgn.c
 *
 * The new mesh generation program for the SECT Group!
 *
 */

#include "e_lib.h"
#include "timing.h"
#include "mgn.h"
#include "mgn_smsh.h"

int main(int argc, char *argv[])
{
  /* variable declarations */
  MGNI *mgni;                   /* the mgn info structure */
  int np, ntr;                  /* statistics numbers */

  if (argc < 2)
  {
    show_usage();
    exit(1);
  }

  /* start the timing */
  p_StartTimer();

  /* parse command line options */
  if (argc > 2)
    get_cmdline_options(argc, argv);

  /* parse the mgn batch file */
  if (!parse_mgn_into_mgni(&mgni, argv[1]))
    e_GeneralError("Problems while parsing the mgn file.\n");
  /* and warn for not implemented options */
  mgni_check(mgni);

  /* globals set from command line options */
  mgni->pps = TRUE;
  mgni->doa = DebugOutput;
  mgni->gra = ShowGraphics;

  /* main function */
  mgni_loadfiles(mgni);
  mgni_main(mgni);

  /* output to file !!! */
  if (mgni->outf == OUTF_E_LIB)
  {
    e_PutMeshData(mgni->outputmesh, mgni->fname[MGJ_FI_RES]);
    /* only in this it is possible to output mesh parameters */
    if(   (mgni->meth == METH_DELAUNAY)
       && (mgni->ctrl == CTRL_MPR)
       && (mgni->opt  == CONV_OPT_NONE))
      e_PutMeshParams(mgni->outputmesh, mgni->fname[MGJ_FI_RES]);
  }
  else if (mgni->outf == OUTF_SMSH)
  {
    smsh_add_surfelem(mgni->outputmesh);
    smsh_output(mgni->outputmesh, mgni->fname[MGJ_FI_RES]);
  }

  /* free up all consumed memory */
  np = (mgni->outputmesh)->NNodes;
  ntr = (mgni->outputmesh)->TotalNElems;
  
  mgni_free(mgni);
  e_free(mgni);

  /* print some statistics */
  consumedtime = p_Timer();
  printf("\nExecution completed succesfully!");
  printf("\nTotal execution time was:    %1.1f sec.",
         ((double) consumedtime) / 1000);
  printf("\nNumber of nodes in mesh:     %d", np);
  printf("\nNumber of elements in mesh:  %d\n\n", ntr);

  return 0;
}

/* end of file */
