#include "mgn.h"
#include "bgdata.h"

void show_usage(void)
{
  char c;

  /* show the usage of the program */
  printf
    ("\nmgn v%s (%s %s)\nSECT Research Group, Heriot-Watt University Edinburgh\n",
     VERSION, __DATE__, __TIME__);
  printf("usage: mgn filename [-options]\n");
  printf("   filename : name of mesh generation batch file without extension\n");
  printf("   -g	      : activates the graphical interface\n\n");
  printf("******************************************************************\n");
  printf("would you like to see the mesh generation batch file syntax? [y/n]");
  fflush(stdout);
  c = getchar();
  if ((c == 'y') || (c == 'Y'))
  {
    printf("\n\n");
    printf("*.mgn batch file specification\n");
    printf("\n");
    printf("TASK  \"REMESH\"      # remesh an existing mesh, keeping boundary specification\n");
    printf("      \"REFINE\"      # refine an existing coarse mesh, keeping internal nodes\n");
    printf("      \"EBE_REFINE\"  # refine an existing coarse mesh, on elem by elem basis\n");
    printf("      \"POSTPROC\"    # perform post-processing on the mesh\n");
    printf("\n");
    printf("METHOD  \"ADV_FRONT\"         # advancing front method\n");
    printf("        \"DELAUNAY\"          # Delaunay triangulation\n");
    printf("        \"PAVING\"            # paving method\n");
    printf("        \"REG_GRID\"          # regular grid method\n");
    printf("        \"NONE\"              # use with post-processing only \n");
    printf("\n");
    printf("ALFA    d                     # alfa parameter for Delaunay point insertion\n");
    printf("                              # for \"CENTROID\"  alfa>1.0\n");
    printf("                              # for \"CENTR_RAD\" 0.5>=alfa>=2.0\n");
    printf("                              # for \"RGR_GRID\" alfa = gprm\n");
    printf("BETA    d                     # beta parameter for Delaunay point insertion\n");
    printf("                              # for \"RGR_GRID\" beta = rprm\n");
    printf("\n");
    printf("DLN_OPTION  \"CENTROID\"      # use centroid insertion based on edge lengths\n");
    printf("            \"CENTR_RAD\"	    # use centroid insertion based on radial connections\n");
    printf("            \"PROJECTION\"    # use projective point placement insertion\n");
    printf("            \"CHEW1\"         # use Chew 1 as refinement method\n");
    printf("\n");
    printf("CONTROL_TYPE  \"BACKGROUND\"  # control mesh size with a background mesh when remeshing\n");
    printf("              \"MPR\"         # control mesh size on an element basis when refining\n");
    printf("              \"SOURCES\"     # control mesh size with local sources\n");
    printf("              \"UNIFORM\"     # uniform mesh size\n");
    printf("\n");
    printf("MESH_OPTION\n");
    printf("  \"FISSION_TO_QUAD\"         # generate quads using fusion of triangles with fission elements\n");
    printf("  \"RANK_TO_QUAD\"            # generate quads using the Rank method\n");
    printf("  \"HALF_TO_QUAD\"            # generate quads by halfing all edges\n");
    printf("  \"SPLIT_TO_TRIANG\"         # generate triangles from a quad mesh by splitting\n");
    printf("  \"NONE\"                    # no conversions\n");
    printf("\n");
    printf("GEOM_MODEL_FILE   \"*\"       # geometric model\n");
    printf("COARSE_MESH_FILE  \"*\"       # coarse mesh for refining or just any mesh for remeshing\n");
    printf("RESULT_MESH_FILE  \"*\"       # resulting mesh file\n");
    printf("\n");
    printf("BKGRND_MESH_FILE  \"*\"       # background mesh file\n");
    printf("BKGRND_MESH_PARAMS_FILE \"*\" # background mesh parameters file\n");
    printf("\n");
    printf("COARSE_MESH_PARAMS_FILE \"*\" # coarse mesh parameters file\n");
    printf("\n");
    printf("LOCAL_MESH_CTRL_FILE  \"*\"   # local mesh density control file (sources definition)\n");
    printf("\n");
    printf("UNIFORM_MESH_PARAM    d     # uniform mesh parameter\n");
    printf("\n");
    printf("SMOOTHING_TYPE  \"LAPLACIAN\" # optimal Laplacian smoothing applied to resulting mesh\n");
    printf("  \"LAPLACIAN_NDE\"	          # same as LAPLACIAN but without diagonal exchange\n");
    printf("  \"NODAL_AVG\"               # plain nodal averaging\n");
    printf("  \"PAVING\"                  # paving smoothing\n");
    printf("  \"NONE\"                    # no smoothing\n");
    printf("SMOOTHING_ITER  n             # number of smoothing iterations\n");
    printf("\n");
    printf("OUTPUT_FORMAT \"E_LIB\"       # E_Library output format\n");
    printf("OUTPUT_FORMAT \"SMSH\"        # smsh output format\n\n");
  }
}

void get_cmdline_options(int argc, char **argv)
{
  int i;

  for (i = 2; i < argc; i++)
  {
    if (argv[i][0] == '-')
      switch (argv[i][1])
      {
      case 'g':
#ifdef HAVE_LIBG2
        ShowGraphics = TRUE;
        printf("\nGraphics activated!\n");
#else
        printf("\nGraphics is not available!\n");
#endif
        break;
      case 'd':
        DebugOutput = TRUE;
        printf("\nDebug Output Active\n");
        break;
      default:
        e_GeneralError("unknown option");
      }
    else
      e_GeneralError("unknown option");
  }
}

int load_check_crsmesh(MGNI * mgni, MESH ** mesh)
{
  *mesh = NULL;
  if (!e_GetMeshData(mesh, mgni->fname[MGJ_FI_COARSE]))
  {
    printf("\nInput mesh file contains faults");
    return FALSE;
  }
  if (!e_IsOfOneElementType(*mesh))
  {
    printf("Input mesh file contains more than one element type");
    return FALSE;
  }
  if (mgni->meth == METH_DELAUNAY)
  {
    if (e_GetElementType(*mesh) != E_ELEM_TYPE_TRIANG1)
    {
      printf("\nThe Delaunay method accepts only Triang1 meshes");
      return FALSE;
    }
  }
  else if (mgni->meth == METH_ADV_FRONT)
  {
    if ((e_GetElementType(*mesh) != E_ELEM_TYPE_TRIANG1) &&
        (e_GetElementType(*mesh) != E_ELEM_TYPE_QUAD1))
    {
      printf
        ("\nThe Advancing Front method accepts only Triang1 or Quad1 meshes");
      return FALSE;
    }
  }
  else if (mgni->meth == METH_PAVING)
  {
    if ((e_GetElementType(*mesh) != E_ELEM_TYPE_TRIANG1) &&
        (e_GetElementType(*mesh) != E_ELEM_TYPE_QUAD1))
    {
      printf("\nThe Paving method accepts only Triang1 or Quad1 meshes");
      return FALSE;
    }
  }
  else if (mgni->meth == METH_REG_GRID)
  {
    if ((e_GetElementType(*mesh) != E_ELEM_TYPE_TRIANG1) &&
        (e_GetElementType(*mesh) != E_ELEM_TYPE_QUAD1))
    {
      printf("\nThe regular grid  method accepts only Triang1 meshes");
      return FALSE;
    }
  }
  return TRUE;
}

void parse_meshsize_ctrl(MGNI *mgni, BG_DATA **bgd, double **crsmpr)
{
  if ((mgni->ctrl == CTRL_BACKGROUND) || (mgni->ctrl == CTRL_SOURCES))
  {
    get_bg_data(bgd, mgni->bgmesh, mgni->pcs);
    *crsmpr=NULL;
  }
  else if (mgni->ctrl == CTRL_MPR)
  {
    if (!mgni->mesh->SubStructSet[E_SS_NUM_MESHPARAM])
      e_GeneralError("parse_meshsize_ctrl: mesh parameters not set");
    if (!mgni->mesh->MeshParam.NodalMeshParamsSet)
      e_GeneralError("parse_meshsize_ctrl: only nodal mesh parameters are accepted");
    *crsmpr=e_alloc_d_vect(mgni->mesh->NMeshPoints);
    memcpy(&((*crsmpr)[0]),&(mgni->mesh->MeshParam.NodalMeshParams[0]),
      		 mgni->mesh->NMeshPoints*sizeof(double));
    *bgd=NULL;
  }
}
