#include "pav_main.h"

#define LEFT  1
#define RIGHT 2

static void CloseWithoutNodeIns(PAVEDMESH *mesh, NODE *head);
static void CloseWith2End(PAVEDMESH *mesh, LOOP *loop, 
                          int nsides, int *sides, NODE **endnode);
static int InsertTriangCentre(PAVEDMESH *mesh, int n1, int n2, int n3);
static void Close321Comb(PAVEDMESH *mesh, NODE *node, int symm);
static void Close411Comb(PAVEDMESH *mesh, NODE *node, int cn);
static void CloseWith3End(PAVEDMESH *mesh, LOOP *loop, 
                          int nsides, int *sides, NODE **endnode);
static int InsertQuadCenre(PAVEDMESH *mesh, int n1, int n2, int n3, int n4);
static void Insert2QuadNode(PAVEDMESH *mesh, int n1, int n2, int n3, int n4,
                            int *nn1, int *nn2);
static void Close3111Comb(PAVEDMESH *mesh, NODE *node, int nn1, int nn2);
static void Close2211Comb(PAVEDMESH *mesh, NODE *node1, NODE *node2, int nn1);
static void CloseWith4End(PAVEDMESH *mesh, LOOP *loop, 
                          int nsides, int *sides, NODE **endnode);


/* ######################################################################### */
/* shape with 1 side  */
static void CloseWithoutNodeIns(PAVEDMESH *mesh, NODE *head)
{
  int   n1, n2, n3, n4;

  n4   = head->Node;
  n3   = head->Prev->Node;
  n2   = head->Prev->Prev->Node;
  n1   = head->Prev->Prev->Prev->Node;

  CreateElement(n1, n2, n3, n4, mesh, FALSE);

  n2 = n4;
  n3 = head->Next->Node;
  n4 = head->Next->Next->Node;

  CreateElement(n1, n2, n3, n4, mesh, FALSE);

}

/* ######################################################################### */
/* shape with 2 sides */
static void CloseWith2End(PAVEDMESH *mesh, LOOP *loop, 
                          int nsides, int *sides, NODE **endnode)
{
  if(nsides != 2)
  {
    e_GeneralError("CloseWith2End: internal error 1 !");
  }

  if(   (sides[0] == 3)
     && (sides[1] == 3) )
  {
    CloseWithoutNodeIns(mesh, endnode[0]->Next);
  }
  else if(   (sides[0] == 4)
          && (sides[1] == 2) )
  {
    CloseWithoutNodeIns(mesh, endnode[1]->Next);
  }
  else if(   (sides[1] == 4)
          && (sides[0] == 2) )
  {
    CloseWithoutNodeIns(mesh, endnode[0]->Next);
  }
  else if(   (sides[1] == 5)
          && (sides[0] == 1) )
  {
    CloseWithoutNodeIns(mesh, endnode[0]->Next);
  }
  else if(   (sides[0] == 5)
          && (sides[1] == 1) )
  {
    CloseWithoutNodeIns(mesh, endnode[1]->Next);
  }
  else
  {
    e_GeneralError("CloseWith2End: internal error 2 !");
  }

}

/* ######################################################################### */
/* shape with 3 sides */
/* insert a node at the centre of a triangle shape */
static int InsertTriangCentre(PAVEDMESH *mesh, int n1, int n2, int n3)
{
  double x1, y1, x2, y2, x3, y3, x, y;

  x1 = mesh->NodeCoords[n1][0];
  y1 = mesh->NodeCoords[n1][1];
  x2 = mesh->NodeCoords[n2][0];
  y2 = mesh->NodeCoords[n2][1];
  x3 = mesh->NodeCoords[n3][0];
  y3 = mesh->NodeCoords[n3][1];
  
  x = (x1 + x2 + x3) / 3.0;
  y = (y1 + y2 + y3) / 3.0;

  CreateNode(x, y, NODE_INTERNAL, mesh, FALSE);
  return(mesh->NNodes-1);
}


static void Close321Comb(PAVEDMESH *mesh, NODE *node, int symm)
{
  int n1, n2, n3, n4;

  if(symm == RIGHT)
  {
    n1 = node->Node;
    n2 = node->Next->Node;
    n3 = node->Prev->Prev->Node;
    n4 = node->Prev->Node;
  
    CreateElement(n1, n2, n3, n4, mesh, FALSE);
 
    n1 = node->Next->Node;
    n2 = node->Next->Next->Node;
    n3 = node->Next->Next->Next->Node;
    n4 = node->Prev->Prev->Node;
  
    CreateElement(n1, n2, n3, n4, mesh, FALSE);
  }
  else if(symm == LEFT)
  {
    n1 = node->Node;
    n2 = node->Next->Node;
    n3 = node->Next->Next->Node;
    n4 = node->Prev->Node;
  
    CreateElement(n1, n2, n3, n4, mesh, FALSE);

    n1 = node->Next->Next->Node; 
    n2 = node->Next->Next->Next->Node;
    n3 = node->Prev->Prev->Node;
    n4 = node->Prev->Node;

    CreateElement(n1, n2, n3, n4, mesh, FALSE);
  }
  else
  {
    e_GeneralError("Close321Comb: internal error !");
  }
}


static void Close411Comb(PAVEDMESH *mesh, NODE *node, int cn)
{
  int n1, n2, n3, n4;

  n1 = node->Node;
  n2 = node->Next->Node;
  n3 = cn;
  n4 = node->Prev->Node;

  CreateElement(n1, n2, n3, n4, mesh, FALSE);

  n1 = node->Next->Node;
  n2 = node->Next->Next->Node;
  n3 = node->Next->Next->Next->Node;
  n4 = cn;
  
  CreateElement(n1, n2, n3, n4, mesh, FALSE);

  n1 = node->Next->Next->Next->Next->Node;
  n2 = node->Prev->Node;
  n3 = cn;
  n4 = node->Next->Next->Next->Node;
  
  CreateElement(n1, n2, n3, n4, mesh, FALSE);
}


static void CloseWith3End(PAVEDMESH *mesh, LOOP *loop, 
                          int nsides, int *sides, NODE **endnode)
{
  int i, n1, n2, n3, n4, newn, symm;

  if(nsides != 3)
  {
    e_GeneralError("CloseWith3End: internal error 1 !");
  }

  if(   (sides[0] == 2)
     && (sides[1] == 2)
     && (sides[2] == 2) )
  {
    n1 = endnode[0]->Node;
    n2 = endnode[1]->Node;
    n3 = endnode[2]->Node;
    
    newn = InsertTriangCentre(mesh, n1, n2, n3);
    
    for(i = 0; i < 3; i ++)
    {
      n1 = endnode[i]->Node;
      n2 = endnode[i]->Next->Node;
      n3 = newn;
      n4 = endnode[i]->Prev->Node;
      
      CreateElement(n1, n2, n3, n4, mesh, FALSE);
    }
  }
  else if(sides[0] == 3)
  {
    if(sides[1] == 2)
      symm = RIGHT;
    else
      symm = LEFT;

    Close321Comb(mesh, endnode[0], symm);
  }
  else if(sides[1] == 3)
  {
    if(sides[2] == 2)
      symm = RIGHT;
    else
      symm = LEFT;

    Close321Comb(mesh, endnode[1], symm);
  }
  else if(sides[2] == 3)
  {
    if(sides[0] == 2)
      symm = RIGHT;
    else
      symm = LEFT;

    Close321Comb(mesh, endnode[2], symm);
  }
  else if(sides[0] == 4)
  {
    n1 = endnode[0]->Node;
    n2 = endnode[1]->Node;
    n3 = endnode[2]->Node;

    newn = InsertTriangCentre(mesh, n1, n2, n3);
    
    Close411Comb(mesh, endnode[0], newn);
  }
  else if(sides[1] == 4)
  {
    n1 = endnode[0]->Node;
    n2 = endnode[1]->Node;
    n3 = endnode[2]->Node;

    newn = InsertTriangCentre(mesh, n1, n2, n3);
    
    Close411Comb(mesh, endnode[1], newn);
  }
  else if(sides[2] == 4)
  {
    n1 = endnode[0]->Node;
    n2 = endnode[1]->Node;
    n3 = endnode[2]->Node;

    newn = InsertTriangCentre(mesh, n1, n2, n3);
    
    Close411Comb(mesh, endnode[2], newn);
  }
  else
  {
    e_GeneralError("CloseWith3End: internal error 2 !");
  }
}


/* ######################################################################### */
/* shape with 4 sides */

static int InsertQuadCenre(PAVEDMESH *mesh, int n1, int n2, int n3, int n4)
{
  double x1, y1, x2, y2, x3, y3, x4, y4, x, y;

  x1 = mesh->NodeCoords[n1][0];
  y1 = mesh->NodeCoords[n1][1];
  x2 = mesh->NodeCoords[n2][0];
  y2 = mesh->NodeCoords[n2][1];
  x3 = mesh->NodeCoords[n3][0];
  y3 = mesh->NodeCoords[n3][1];
  x4 = mesh->NodeCoords[n4][0];
  y4 = mesh->NodeCoords[n4][1];
  
  x = (x1 + x2 + x3 + x4) / 4.0;
  y = (y1 + y2 + y3 + y4) / 4.0;

  CreateNode(x, y, NODE_INTERNAL, mesh, FALSE);

  return(mesh->NNodes-1);
}


static void Insert2QuadNode(PAVEDMESH *mesh, int n1, int n2, int n3, int n4,
                            int *nn1, int *nn2)
{
  double x1, y1, x2, y2, x3, y3, x4, y4, x, y;

  x1 = mesh->NodeCoords[n1][0];
  y1 = mesh->NodeCoords[n1][1];
  x2 = mesh->NodeCoords[n2][0];
  y2 = mesh->NodeCoords[n2][1];
  x3 = mesh->NodeCoords[n3][0];
  y3 = mesh->NodeCoords[n3][1];
  x4 = mesh->NodeCoords[n4][0];
  y4 = mesh->NodeCoords[n4][1];

  /* aligned along the x axis, 
     but it should be parallel with the side with 3 segments,
     smoothing should take care of it */
  y = (y1 + y2 + y3 + y4) / 4.0;

  x = ( ((x1 + x4)) + ((x2 + x3) / 2.0) ) / 3.0;
  CreateNode(x, y, NODE_INTERNAL, mesh, FALSE);
  *nn1 = mesh->NNodes-1;

  x = ( ((x1 + x4) / 2.0) + ((x2 + x3)) ) / 3.0;
  CreateNode(x, y, NODE_INTERNAL, mesh, FALSE);
  *nn2 = mesh->NNodes-1;
}


static void Close3111Comb(PAVEDMESH *mesh, NODE *node, int nn1, int nn2)
{
  int n1, n2, n3, n4;

  n1 = node->Node;
  n2 = node->Next->Node;
  n3 = nn1;
  n4 = node->Prev->Node;

  CreateElement(n1, n2, n3, n4, mesh, FALSE);

  n1 = node->Next->Node;
  n2 = node->Next->Next->Node;
  n3 = nn2;
  n4 = nn1;
  
  CreateElement(n1, n2, n3, n4, mesh, FALSE);

  n1 = node->Prev->Node;
  n2 = nn1;
  n3 = nn2;
  n4 = node->Prev->Prev->Node;
  
  CreateElement(n1, n2, n3, n4, mesh, FALSE);

  n1 = node->Next->Next->Node;
  n2 = node->Next->Next->Next->Node;
  n3 = node->Prev->Prev->Node;
  n4 = nn2;

  CreateElement(n1, n2, n3, n4, mesh, FALSE);

}


static void Close2211Comb(PAVEDMESH *mesh, NODE *node1, NODE *node2, int nn1)
{
  int n1, n2, n3, n4;

  n1 = node1->Node;
  n2 = node1->Next->Node;
  n3 = nn1;
  n4 = node1->Prev->Node;

  CreateElement(n1, n2, n3, n4, mesh, FALSE);

  n1 = node2->Prev->Node;
  n2 = node2->Node;
  n3 = node2->Next->Node;
  n4 = nn1;
  
  CreateElement(n1, n2, n3, n4, mesh, FALSE);

  n1 = nn1;
  n2 = node2->Next->Node;
  n3 = node2->Next->Next->Node;
  n4 = node1->Prev->Node;
  
  CreateElement(n1, n2, n3, n4, mesh, FALSE);
}


static void CloseWith4End(PAVEDMESH *mesh, LOOP *loop, 
                          int nsides, int *sides, NODE **endnode)
{
  int n1, n2, n3, n4, nn1, nn2;

  if(nsides != 4)
  {
    e_GeneralError("CloseWith4End: internal error 1 !");
  }
  
  /* 3-1-1-1 combination */
  if(sides[0] == 3)
  {
    n1 = endnode[0]->Node;
    n2 = endnode[1]->Node;
    n3 = endnode[2]->Node;
    n4 = endnode[3]->Node;
    
    Insert2QuadNode(mesh, n1, n2, n3, n4, &nn1, &nn2);
    Close3111Comb(mesh, endnode[0], nn1, nn2);
  }
  else if(sides[1] == 3)
  {
    n1 = endnode[1]->Node;
    n2 = endnode[2]->Node;
    n3 = endnode[3]->Node;
    n4 = endnode[0]->Node;
    
    Insert2QuadNode(mesh, n1, n2, n3, n4, &nn1, &nn2);
    Close3111Comb(mesh, endnode[1], nn1, nn2);
  }
  else if(sides[2] == 3)
  {
    n1 = endnode[2]->Node;
    n2 = endnode[3]->Node;
    n3 = endnode[0]->Node;
    n4 = endnode[1]->Node;
    
    Insert2QuadNode(mesh, n1, n2, n3, n4, &nn1, &nn2);
    Close3111Comb(mesh, endnode[2], nn1, nn2);
  }
  else if(sides[3] == 3)
  {
    n1 = endnode[3]->Node;
    n2 = endnode[0]->Node;
    n3 = endnode[1]->Node;
    n4 = endnode[2]->Node;
    
    Insert2QuadNode(mesh, n1, n2, n3, n4, &nn1, &nn2);
    Close3111Comb(mesh, endnode[3], nn1, nn2);
  }
  /* 2-1-2-1 combination */
  else if(   (sides[0] == 2)
          && (sides[1] == 1) )
  {
    CloseWithoutNodeIns(mesh, endnode[0]->Next);
  }
  else if(   (sides[0] == 1)
          && (sides[1] == 2) )
  {
    CloseWithoutNodeIns(mesh, endnode[1]->Next);
  }
  /* 2-2-1-1 combination */
  else if(   (sides[0] == 2)
          && (sides[1] == 2) )
  {
    n1 = endnode[0]->Node;
    n2 = endnode[1]->Node;
    n3 = endnode[2]->Node;
    n4 = endnode[3]->Node;
    
    nn1 = InsertQuadCenre(mesh, n1, n2, n3, n4);
    
    Close2211Comb(mesh, endnode[0], endnode[1], nn1);
  }
  else if(   (sides[1] == 2)
          && (sides[2] == 2) )
  {
    n1 = endnode[0]->Node;
    n2 = endnode[1]->Node;
    n3 = endnode[2]->Node;
    n4 = endnode[3]->Node;
    
    nn1 = InsertQuadCenre(mesh, n1, n2, n3, n4);
    
    Close2211Comb(mesh, endnode[1], endnode[2], nn1);
  }
  else if(   (sides[2] == 2)
          && (sides[3] == 2) )
  {
    n1 = endnode[0]->Node;
    n2 = endnode[1]->Node;
    n3 = endnode[2]->Node;
    n4 = endnode[3]->Node;
    
    nn1 = InsertQuadCenre(mesh, n1, n2, n3, n4);
    
    Close2211Comb(mesh, endnode[2], endnode[3], nn1);
  }
  else if(   (sides[3] == 2)
          && (sides[0] == 2) )
  {
    n1 = endnode[0]->Node;
    n2 = endnode[1]->Node;
    n3 = endnode[2]->Node;
    n4 = endnode[3]->Node;
    
    nn1 = InsertQuadCenre(mesh, n1, n2, n3, n4);
    
    Close2211Comb(mesh, endnode[3], endnode[0], nn1);
  }
  else
  {
    e_GeneralError("CloseWith4End: internal error 2 !");
  }

}


/* ######################################################################### */
/* close the loop according to the number of nodes and shape */
void LoopClosure(PAVEDMESH *mesh, LOOP *loop)
{
  int   n1, n2, n3, n4, i, nnode;
  NODE *head;
  int   nsides;
  int   sides[6];
  NODE *endnode[6];

  /* simply remove the nodes, nothing to be done */
  if(loop->NNodes == 2)
  {
    ;
  }
  /* insert one last quad and remove nodes */
  else if(loop->NNodes == 4)
  {
    head = loop->Nodes;
    n1   = head->Node;
    n2   = head->Next->Node;
    n3   = head->Next->Next->Node;
    n4   = head->Prev->Node;
  
    CreateElement(n1, n2, n3, n4, mesh, FALSE);
  }
  /* closure formula */
  else if(loop->NNodes == 6)
  {
    /* classify the loop */
    ClassifyALoop(mesh->NodeCoords, loop);
    FinalNodeTypeInLoop(mesh, loop);
    /* zero out values */
    nsides = 0;
    for(i = 0; i < 6; i++)
    {
      sides[i] = 0;
    }
    /* go to after an end node, if there is any */
    for(i = 0; i < loop->NNodes; i++)
    {
      if(CHK_BIT(loop->Nodes->Flag, NODE_END))
      {
        endnode[nsides] = loop->Nodes;
        break;
      }
      
      StepForward(loop, FALSE);
    }
    StepForward(loop, FALSE);
    /* count sides */
    for(i = 0; i < loop->NNodes; i++)
    {
      sides[nsides]++;
      if(CHK_BIT(loop->Nodes->Flag, NODE_END))
      {
        nsides++;
        endnode[nsides] = loop->Nodes;
      }
      StepForward(loop, FALSE);
    }
    /* differentiate between special cases */
    switch(nsides)
    {
      case 4:
        CloseWith4End(mesh, loop, nsides, sides, endnode);
        break;
      case 3:
        CloseWith3End(mesh, loop, nsides, sides, endnode);
        break;
      case 2:
        CloseWith2End(mesh, loop, nsides, sides, endnode);
        break;
      case 1:
        CloseWithoutNodeIns(mesh, endnode[0]);
        break;
      case 0:
      case 5:
      case 6:
        CloseWithoutNodeIns(mesh, loop->Nodes);
        break;
      default:
        e_GeneralError("LoopClosure: unknown loop pattern to close !");
        break;
    }
  }
  else
  {
    e_GeneralError("LoopClosure: internal error !");
  }
  
  nnode = loop->NNodes;
  for(i = 0; i < nnode; i++)
  {
    RemoveBehindFrontNode(mesh, loop, FALSE);
  }
  
  loop->Nodes = NULL;
  
  /* remove the loop from the mesh */
  RemoveLoop(mesh, loop, FALSE);

}


/* ######################################################################### */
/*                               Mesh cleaning                               */
/* ######################################################################### */
void MeshClosure(PAVEDMESH *mesh)
{
  int i, quad, n, smooth;

  do
  {
    smooth = FALSE;
    /* check at all nodes, whether there is only two quads */
    for(i = 0; i < mesh->NNodes; i++)
    {
      if((mesh->NNodeElem[i] == 2) && !CHK_BND(mesh->NodeFlag[i]))
      {
#ifdef PAVING_DEBUG
        e_GeneralWarning("Two elements should be merged at node: %d", i+1);
#endif
        quad = mesh->NodeElem[i][1];
        n = FindNodeInQuad(mesh->QuadNodes[quad], i, 2);
        /* delete the element */
        DeleteElement(quad, mesh, FALSE);
        /* change the other element */
        quad = mesh->NodeElem[i][0];
        ChangeElementNode(quad, i, n, mesh, FALSE);
        
        smooth = TRUE;
      }
    }
    
    if(smooth)
    {
      SmoothInterior(mesh, NULL, N_SMOOTH_ITERATION, FALSE);
    }
    
  }
  while(smooth == TRUE);

#ifdef PAVING_DEBUG
  for(i = 0; i < mesh->NNodes; i++)
  {
    if(CHK_FRONT(mesh->NodeFlag[i]))
    {
      e_GeneralWarning("MeshClosure: Node %d is still on the front", i+1);
    }
  }
#endif

}




