#include "mgn.h"
#include "rgr.h"
#include "dtr.h" 
#include "eg_lib.h"
#include "mgn_aux.h"

/*****************************************************************************/

int simple_refine(double ***coords, int *np, int *tp,
                  int *stnp, double **mpr, int **tr, int ntr,
                  int ***tpnl, double rprm, int proj)
{
  int t;                        /* triangle */
  double l;                     /* length */
  int s, i;                     /* side */
  int nnp;                      /* number of new nodes */
  double pos[2];                /* centroid */
  double nmpr;                  /* the local mpr of the centroid */
  double allowed_mpr;
  int n;                        /* node counter */
  double a, area;               /* area's of triangles */
  double v1[3], v2[3], v3[3];
  int ct;                       /* current triangle */
  int onside;                   /* necessary for triangle search */
  int sres;                     /* result of the triangle search */
  int nsides_ok;
  int sides_ok[3];
  double ds;                    /* perpendicular projection distance */

  MTRF mtrf;
  mtrf.used = FALSE;

  (*stnp) = (*np);
  nnp = 0;
  for (t = 0; t < ntr; t++)
  {
    if ((tr[t][10] == -1) || (tr[t][9] == 0))
      continue;
    if (!proj)                  /* centroid point insertion */
    {
      for (s = 0; s < 3; s++)
      {
        l = eg_PointDist2D((*coords)[tr[t][s]],
                           (*coords)[tr[t][(s + 1) % 3]]);
        allowed_mpr =
          rprm * E_MIN((*mpr)[tr[t][s]], (*mpr)[tr[t][(s + 1) % 3]]);
        if (l > allowed_mpr)
        {
          pos[0] = 0.0;
          pos[1] = 0.0;
          nmpr = 0.0;
          for (i = 0; i < 3; i++)
          {
            pos[0] += (*coords)[tr[t][i]][0];
            pos[1] += (*coords)[tr[t][i]][1];
            nmpr += (*mpr)[tr[t][i]];
          }
          pos[0] /= 3.0;
          pos[1] /= 3.0;
          nmpr /= 3.0;
          include_grid_node(pos, coords, np, tp, mpr, &nnp, tpnl, nmpr);
          break;
        }
      }
      if (s == 3)
      {
        tr[t][9] = 0;
        continue;
      }
    }
    else                        /* projective point insertion */
    {
      for (s = 0; s < 3; s++)
      {
        l = eg_PointDist2D((*coords)[tr[t][s]],
                           (*coords)[tr[t][(s + 1) % 3]]);
        allowed_mpr =
          rprm * E_MIN((*mpr)[tr[t][s]], (*mpr)[tr[t][(s + 1) % 3]]);
        if (l > allowed_mpr)
          break;
      }
      if (s == 3)
      {
        tr[t][9] = 0;
        continue;
      }
      nsides_ok = 0;
      for (s = 0; s < 3; s++)
        sides_ok[s] = FALSE;
      for (s = 0; s < 3; s++)
        if (tr[t][3 + s] == -1) /* side is boundary */
        {
          sides_ok[s] = TRUE;
          nsides_ok++;
        }
        else if (tr[tr[t][3 + s]][10] == -1) /* side is shared by non-active triangle */
        {
          sides_ok[s] = TRUE;
          nsides_ok++;
        }
        else if (tr[tr[t][3 + s]][9] == 0) /* side contains a switched off element */
        {
          sides_ok[s] = TRUE;
          nsides_ok++;
        }
      if (nsides_ok == 3)
        continue;
      pos[0] = 0.0;
      pos[1] = 0.0;
      for (s = 0; s < 3; s++)
        if (sides_ok[s])
        {
          ds =
            eg_PointDist2D((*coords)[tr[t][s]],
                           (*coords)[tr[t][(s + 1) % 3]]);
          v1[0] = (*coords)[tr[t][(s + 1) % 3]][0] - (*coords)[tr[t][s]][0];
          v1[1] = (*coords)[tr[t][(s + 1) % 3]][1] - (*coords)[tr[t][s]][1];
          v1[2] = 0.0;
          v2[0] = 0.0;
          v2[1] = 0.0;
          v2[2] = 1.0;
          e_VectCrossVect(v1, v2, v3);
          l = 0.0;
          for (i = 0; i < 3; i++)
            l += v3[i] * v3[i];
          for (i = 0; i < 3; i++)
            v3[i] /= sqrt(l);
          pos[0] +=
            ((*coords)[tr[t][(s + 1) % 3]][0] + (*coords)[tr[t][s]][0]) / 2.0;
          pos[0] -= ds * v3[0] * 0.866;
          pos[1] +=
            ((*coords)[tr[t][(s + 1) % 3]][1] + (*coords)[tr[t][s]][1]) / 2.0;
          pos[1] -= ds * v3[1] * 0.866;
        }
      if (nsides_ok > 0)
      {
        pos[0] /= nsides_ok;
        pos[1] /= nsides_ok;
      }
      else
      {
        pos[0] = 0.0;
        pos[1] = 0.0;
        nmpr = 0.0;
        for (i = 0; i < 3; i++)
        {
          pos[0] += (*coords)[tr[t][i]][0];
          pos[1] += (*coords)[tr[t][i]][1];
          nmpr += (*mpr)[tr[t][i]];
        }
        pos[0] /= 3.0;
        pos[1] /= 3.0;
        nmpr /= 3.0;
      }
      sres = trianglesearch(pos, *coords, tr, t, &ct, &onside, &mtrf);
      if (sres == 0)            /* the optimal projection fell outside */
      {
        e_GeneralWarning("optimal projection failed, centroid assumed");
        pos[0] = 0.0;
        pos[1] = 0.0;
        nmpr = 0.0;
        for (i = 0; i < 3; i++)
        {
          pos[0] += (*coords)[tr[t][i]][0];
          pos[1] += (*coords)[tr[t][i]][1];
          nmpr += (*mpr)[tr[t][i]];
        }
        pos[0] /= 3.0;
        pos[1] /= 3.0;
        nmpr /= 3.0;
        ct = t;
      }
      else if (sres == -1)      /* duplicated element */
      {
        continue;
      }
      else                      /* normal case */
      {
        area = 0.0;
        nmpr = 0.0;
        for (i = 0; i < 3; i++)
        {
          a =
            eg_TriangArea2D((*coords)[tr[ct][(i + 1) % 3]],
                            (*coords)[tr[ct][(i + 2) % 3]], pos);
          area += a;
          nmpr += (*mpr)[tr[ct][i]] * a;
        }
        nmpr /= area;
        /* further testing against slivers */
        for (s = 0; s < 3; s++)
        {
          l = eg_PointDist2D(pos, (*coords)[tr[ct][s]]);
          if (l < rprm * 0.4 * E_MIN(nmpr, (*mpr)[tr[ct][s]]))
            break;
        }
        if (s < 3)
          continue;
      }

      include_grid_node(pos, coords, np, tp, mpr, &nnp, tpnl, nmpr);
    }
    /* second round: BETA=2.0  */
    for (n = (*stnp); n < (*np) - 1; n++)
    {
      l = eg_PointDist2D((*coords)[n], pos);
      allowed_mpr = E_MIN((*mpr)[n], nmpr);
      if (l < allowed_mpr / 2.0)
      {
        (*np)--;
        nnp--;
        e_free((*tpnl)[(*np)]);
      }
    }
  }
  return nnp;
}

int simple_dual_refine(double ***coords, int *np, int *tp,
                       int *stnp, double **mpr, int **tr, int ntr,
                       int ***tpnl, double rprm, int use_star)
{
  int t;                        /* triangle */
  double l;                     /* length */
  int s, i;		                 	/* side */
  int nnp;                      /* number of new nodes */
  double pos[3];                /* the candidate for insertion */
  double nmpr;                  /* the mpr of the candidate for insertion */
  double allowed_mpr;

  int stb[3];                   /* side too big */
  int nstb;                     /* no of sides too big */
  int ng_nstb;                  /* no of sides too big with neighbour */
  int ngnstb[3];                /* vector of ng_nstb for three neighbours */
  int cont;                     /* continue? */

  int sx, tx;                   /* equivqlent of s,t for the neighbouring nstb=1 triangle */
  int star;                     /* are we in the star case? */
  int nt;                       /* neighbouring triangle */

  int k;                        /* counter */

  int stb1, stb2;
  double pos1[3], pos2[3];
  double d1, d2;
  double nmpr1, nmpr2;
  int nb;

  MTRF mtrf;
  int vt;

  mtrf.used = FALSE;

  (*stnp) = (*np);
  nnp = 0;

  pos[2] = 0.0;
  nmpr = 0.0;

  for (t = 0; t < ntr; t++)
    tr[t][9] = 1;

  for (t = 0; t < ntr; t++)
  {
    if (tr[t][10] == -1)
      continue;
    if (tr[t][9] < 1)
      continue;
    nstb = 0;
    for (i = 0; i < 3; i++)
      stb[i] = FALSE;

    for (s = 0; s < 3; s++)
    {
      l = eg_PointDist2D((*coords)[tr[t][s]], (*coords)[tr[t][(s + 1) % 3]]);
      allowed_mpr =
        rprm * E_MIN((*mpr)[tr[t][s]], (*mpr)[tr[t][(s + 1) % 3]]);

      if (l > allowed_mpr)
      {
        if (tr[t][3 + s] != -1) /* we assume boundaries can't be too big */
        {
          if (tr[tr[t][3 + s]][10] != -1)
          {
            stb[s] = TRUE;
            nstb++;
          }
        }
      }
    }
    if (nstb == 0)              /* class 0 */
      continue;

    if (nstb == 3)
    {
      pos[0] = 0.0;
      pos[1] = 0.0;
      nmpr = 0.0;
      for (i = 0; i < 3; i++)
      {
        pos[0] += (*coords)[tr[t][i]][0];
        pos[1] += (*coords)[tr[t][i]][1];
        nmpr += (*mpr)[tr[t][i]];
      }
      pos[0] /= 3.0;
      pos[1] /= 3.0;
      nmpr /= 3.0;
      include_grid_node(pos, coords, np, tp, mpr, &nnp, tpnl, nmpr);
      (*coords)[(*np) - 1][2] = -20.0;
      for (i = 0; i < 3; i++)
        if (tr[tr[t][3 + i]][9] > 0)
          tr[tr[t][3 + i]][9] = 0;
      tr[t][9] = 0;
    }
    else if (nstb == 1)         /* class 1 */
    {
      for (s = 0; s < 3; s++)
        if (stb[s])
          break;
      /* priority to nstb==2 's */
      ng_nstb = 0;
      for (i = 0; i < 3; i++)
      {
        l = eg_PointDist2D((*coords)[tr[tr[t][3 + s]][i]],
                           (*coords)[tr[tr[t][3 + s]][(i + 1) % 3]]);
        allowed_mpr =
          rprm * E_MIN((*mpr)[tr[tr[t][3 + s]][i]],
                       (*mpr)[tr[tr[t][3 + s]][(i + 1) % 3]]);
        if (l > allowed_mpr)
          if (tr[tr[t][3 + s]][3 + i] != -1)
            if (tr[tr[tr[t][3 + s]][3 + i]][10] != -1)
              ng_nstb++;
      }

      if (ng_nstb > 1)
        continue;

      if (tr[3 + s][9] == 0)
        continue;

      pos[0] = (*coords)[tr[t][s]][0] + (*coords)[tr[t][(s + 1) % 3]][0];
      pos[1] = (*coords)[tr[t][s]][1] + (*coords)[tr[t][(s + 1) % 3]][1];
      nmpr = (*mpr)[tr[t][s]] + (*mpr)[tr[t][(s + 1) % 3]];
      pos[0] /= 2.0;
      pos[1] /= 2.0;
      nmpr /= 2.0;

      star = FALSE;
      if (use_star)
      {
        /* star detection to prevent propagation */
        /* along the sides of the current triangle */
        for (i = 1; i < 3; i++)
        {
          nt = tr[t][3 + (s + i) % 3];
          if (nt != -1)
            if ((tr[nt][10] != -1) && (tr[nt][9] > 0))
            {
              if (!IsCCW((*coords)[tr[nt][0]],
                         (*coords)[tr[nt][1]],
                         (*coords)[tr[nt][2]], &mtrf, &vt))
                e_GeneralError("not CCW");
              if (j_incircle((*coords)[tr[nt][0]],
                             (*coords)[tr[nt][1]],
                             (*coords)[tr[nt][2]], pos, &mtrf))
              {
                l =
                  eg_PointDist2D((*coords)[tr[t][(s + i) % 3]],
                                 (*coords)[tr[t][(s + i + 1) % 3]]);
                allowed_mpr =
                  E_MIN((*mpr)[tr[t][(s + i) % 3]],
                        (*mpr)[tr[t][(s + i + 1) % 3]]);
                if (l > (0.999 * allowed_mpr))
                {
                  pos[0] =
                    ((*coords)[tr[t][(s + i) % 3]][0] +
                     (*coords)[tr[t][(s + i + 1) % 3]][0]) / 2.0;
                  pos[1] =
                    ((*coords)[tr[t][(s + i) % 3]][1] +
                     (*coords)[tr[t][(s + i + 1) % 3]][1]) / 2.0;
                  nmpr =
                    ((*mpr)[tr[t][(s + i) % 3]] +
                     (*mpr)[tr[t][(s + i + 1) % 3]]) / 2.0;
                  tr[nt][9] = -4; /* green: the violating triangle */
                  tr[t][9] = -19; /* red: the current triangle */
                  tr[tr[t][3 + s]][9] = -3; /* blue: the original neighbour */
                  star = TRUE;
                  break;
                }
              }
            }
        }
        /* along the sides of the neighbouring nstb=1 triangle */
        if (!star)
        {
          tx = tr[t][3 + s];
          sx = tr[t][6 + s] - 3;
          for (i = 1; i < 3; i++)
          {
            nt = tr[tx][3 + (sx + i) % 3];
            if (nt != -1)
              if ((tr[nt][10] != -1) && (tr[nt][9] > 0))
              {
                if (!IsCCW((*coords)[tr[nt][0]],
                           (*coords)[tr[nt][1]],
                           (*coords)[tr[nt][2]], &mtrf, &vt))
                  e_GeneralError("not CCW");
                if (j_incircle((*coords)[tr[nt][0]],
                               (*coords)[tr[nt][1]],
                               (*coords)[tr[nt][2]], pos, &mtrf))
                {
                  l =
                    eg_PointDist2D((*coords)[tr[tx][(sx + i) % 3]],
                                   (*coords)[tr[tx][(sx + i + 1) % 3]]);
                  allowed_mpr =
                    E_MIN((*mpr)[tr[tx][(sx + i) % 3]],
                          (*mpr)[tr[tx][(sx + i + 1) % 3]]);
                  if (l > (0.999 * allowed_mpr))
                  {
                    pos[0] =
                      ((*coords)[tr[tx][(sx + i) % 3]][0] +
                       (*coords)[tr[tx][(sx + i + 1) % 3]][0]) / 2.0;
                    pos[1] =
                      ((*coords)[tr[tx][(sx + i) % 3]][1] +
                       (*coords)[tr[tx][(sx + i + 1) % 3]][1]) / 2.0;
                    nmpr =
                      ((*mpr)[tr[tx][(sx + i) % 3]] +
                       (*mpr)[tr[tx][(sx + i + 1) % 3]]) / 2.0;
                    tr[nt][9] = -7; /* green: the violating triangle */
                    tr[t][9] = -20; /* red: the current triangle */
                    tr[tx][9] = -9; /* blue: the original neighbour */
                    star = TRUE;
                    break;
                  }
                }
              }
          }
        }
      }

      if (!star)
        tr[tr[t][3 + s]][9] = 0;
      if (!star)
        tr[t][9] = 0;
      include_grid_node(pos, coords, np, tp, mpr, &nnp, tpnl, nmpr);

    }
    else if (nstb == 2)         /* class 2 */
    {
      cont = FALSE;
      for (s = 0; s < 3; s++)
      {
        ngnstb[s] = 0;
        if (stb[s])
        {
          ng_nstb = 0;
          for (i = 0; i < 3; i++)
          {
            l = eg_PointDist2D((*coords)[tr[tr[t][3 + s]][i]],
                               (*coords)[tr[tr[t][3 + s]][(i + 1) % 3]]);
            allowed_mpr =
              rprm * E_MIN((*mpr)[tr[tr[t][3 + s]][i]],
                           (*mpr)[tr[tr[t][3 + s]][(i + 1) % 3]]);
            if (l > allowed_mpr)
              if (tr[tr[t][3 + s]][3 + i] != -1)
                if (tr[tr[tr[t][3 + s]][3 + i]][10] != -1)
                  ng_nstb++;
          }
          if (ng_nstb > 2)
          {
            cont = TRUE;
            break;
          }
          ngnstb[s] = ng_nstb;
        }
      }
      if (cont)
        continue;
      pos[0] = 0.0;
      pos[1] = 0.0;
      nmpr = 0.0;

#ifdef TOO_COMPLICATED
      /* latest implementation */
      cpos[0] = 0.0;
      cpos[1] = 0.0;
      cnmpr = 0.0;
      for (s = 0; s < 3; s++)
        if (stb[s])
        {
          cpos[0] +=
            ((*coords)[tr[t][s]][0] + (*coords)[tr[t][(s + 1) % 3]][0]) / 2.0;
          cpos[1] +=
            ((*coords)[tr[t][s]][1] + (*coords)[tr[t][(s + 1) % 3]][1]) / 2.0;
          cnmpr += ((*mpr)[tr[t][s]] + (*mpr)[tr[t][(s + 1) % 3]]) / 2.0;
        }
      cpos[0] /= 2.0;
      cpos[1] /= 2.0;
      cnmpr /= 2.0;

      for (s = 0; s < 3; s++)
      {
        if (stb[s])
        {
          if ((ngnstb[s] == 2) && (tr[tr[t][3 + s]][9] > 0))
          {
            pos[0] +=
              ((*coords)[tr[t][s]][0] +
               (*coords)[tr[t][(s + 1) % 3]][0]) / 2.0;
            pos[1] +=
              ((*coords)[tr[t][s]][1] +
               (*coords)[tr[t][(s + 1) % 3]][1]) / 2.0;
            nmpr += ((*mpr)[tr[t][s]] + (*mpr)[tr[t][(s + 1) % 3]]) / 2.0;
            d =
              eg_PointDist2D(pos,
                             (*coords)[tr[tr[t][3 + s]]
                                       [(tr[t][6 + s] - 1) % 3]]);
            cd =
              eg_PointDist2D(cpos,
                             (*coords)[tr[tr[t][3 + s]]
                                       [(tr[t][6 + s] - 1) % 3]]);
#ifdef XXX
            if (d < cd)
            {
              if (d <
                  rprm * E_MIN(nmpr,
                               (*mpr)[tr[tr[t][3 + s]]
                                      [(tr[t][6 + s] - 1) % 3]]))
              {
                include_grid_node(pos, coords, np, tp, mpr, &nnp, tpnl, nmpr);
                printf(" case 1: d=%1.2lf cd=%1.2lf\n", d, cd);
                tr[t][9] = 0;
                for (k = 0; k < 3; k++)
                  if (stb[k])
                    if (tr[tr[t][3 + k]][9] > 0)
                      tr[tr[t][3 + k]][9] = 0;
                break;
              }
              else
              {
                include_grid_node(cpos, coords, np, tp, mpr, &nnp, tpnl,
                                  cnmpr);
                printf(" case 2: d=%1.2lf cd=%1.2lf\n", d, cd);
                tr[t][9] = 0;
                for (k = 0; k < 3; k++)
                  if (stb[k])
                    if (tr[tr[t][3 + k]][9] > 0)
                      tr[tr[t][3 + k]][9] = 0;
                break;
              }
            }
            else                /* cd<d */
            {
              if (cd <
                  rprm * E_MIN(nmpr,
                               (*mpr)[tr[tr[t][3 + s]]
                                      [(tr[t][6 + s] - 1) % 3]]))
              {
                include_grid_node(cpos, coords, np, tp, mpr, &nnp, tpnl,
                                  cnmpr);
                printf(" case 3: d=%1.2lf cd=%1.2lf\n", d, cd);
                tr[t][9] = 0;
                for (k = 0; k < 3; k++)
                  if (stb[k])
                    if (tr[tr[t][3 + k]][9] > 0)
                      tr[tr[t][3 + k]][9] = 0;
                break;
              }
              else
              {
#endif
                include_grid_node(pos, coords, np, tp, mpr, &nnp, tpnl, nmpr);
                printf(" case 4: d=%1.2lf cd=%1.2lf\n", d, cd);
                tr[t][9] = 0;
                for (k = 0; k < 3; k++)
                  if (stb[k])
                    if (tr[tr[t][3 + k]][9] > 0)
                      tr[tr[t][3 + k]][9] = 0;
                break;
#ifdef XXX
              }
            }
#endif
          }
          else if ((ngnstb[s] == 2) && (tr[tr[t][3 + s]][9] < 1))
            continue;
          else if ((ngnstb[s] == 1) && (tr[tr[t][3 + s]][9] < 1))
            continue;
          else if ((ngnstb[s] == 1) && (tr[tr[t][3 + s]][9] > 0))
          {
            for (cs = s; cs < s + 3; cs++)
              if (stb[cs % 3])
                break;
            if ((ngnstb[cs] == 2) && (tr[tr[t][3 + cs]][9] > 0))
              continue;
            pos[0] +=
              ((*coords)[tr[t][s]][0] +
               (*coords)[tr[t][(s + 1) % 3]][0]) / 2.0;
            pos[1] +=
              ((*coords)[tr[t][s]][1] +
               (*coords)[tr[t][(s + 1) % 3]][1]) / 2.0;
            nmpr += ((*mpr)[tr[t][s]] + (*mpr)[tr[t][(s + 1) % 3]]) / 2.0;
            include_grid_node(pos, coords, np, tp, mpr, &nnp, tpnl, nmpr);
            tr[t][9] = 0;
            for (k = 0; k < 3; k++)
              if (stb[k])
                if (tr[tr[t][3 + k]][9] > 0)
                  tr[tr[t][3 + k]][9] = 0;
            break;
          }
        }                       /* end if stb */
      }                         /* end for all sides */

      if (s == 3)
      {
        include_grid_node(cpos, coords, np, tp, mpr, &nnp, tpnl, cnmpr);
        tr[t][9] = 0;
        for (k = 0; k < 3; k++)
          if (stb[k])
            if (tr[tr[t][3 + k]][9] > 0)
              tr[tr[t][3 + k]][9] = 0;
      }
      (*coords)[(*np) - 1][2] = -1.0;
#else
      /* new method based on diagonal comparison */
      stb1 = -1;
      for (s = 0; s < 3; s++)
        if (stb[s])
        {
          if (stb1 < 0)
            stb1 = s;
          else
            stb2 = s;
        }
      pos1[0] =
        ((*coords)[tr[t][stb1]][0] +
         (*coords)[tr[t][(stb1 + 1) % 3]][0]) / 2.0;
      pos1[1] =
        ((*coords)[tr[t][stb1]][1] +
         (*coords)[tr[t][(stb1 + 1) % 3]][1]) / 2.0;
      nmpr1 = ((*mpr)[tr[t][stb1]] + (*mpr)[tr[t][(stb1 + 1) % 3]]) / 2.0;
      pos2[0] =
        ((*coords)[tr[t][stb2]][0] +
         (*coords)[tr[t][(stb2 + 1) % 3]][0]) / 2.0;
      pos2[1] =
        ((*coords)[tr[t][stb2]][1] +
         (*coords)[tr[t][(stb2 + 1) % 3]][1]) / 2.0;
      nmpr2 = ((*mpr)[tr[t][stb2]] + (*mpr)[tr[t][(stb2 + 1) % 3]]) / 2.0;
      d1 = eg_PointDist2D(pos1, (*coords)[tr[t][(stb1 + 2) % 3]]);
      d2 = eg_PointDist2D(pos2, (*coords)[tr[t][(stb2 + 2) % 3]]);

      if (d1 < d2)
      {
        pos[0] = pos1[0];
        pos[1] = pos1[1];
        nmpr = nmpr1;
        s = stb1;
      }
      else
      {
        pos[0] = pos2[0];
        pos[1] = pos2[1];
        nmpr = nmpr2;
        s = stb2;
      }

      if (tr[tr[t][3 + s]][9] > 0) /* shorter diag neighb not notified */
      {
        if (ngnstb[s] == 1)
        {
          include_grid_node(pos, coords, np, tp, mpr, &nnp, tpnl, nmpr);
          (*coords)[(*np) - 1][2] = -1.0;
          tr[t][9] = 0;
          tr[tr[t][3 + s]][9] = 0;
        }
        else                    /* ngnstb[s]==2 */
        {
          /* check the neighbours diag situation */
          for (k = tr[t][6 + s] - 2; k < tr[t][6 + s]; k++)
          {
            l = eg_PointDist2D((*coords)[tr[tr[t][3 + s]][k % 3]],
                               (*coords)[tr[tr[t][3 + s]][(k + 1) % 3]]);
            allowed_mpr =
              rprm * E_MIN((*mpr)[tr[tr[t][3 + s]][k % 3]],
                           (*mpr)[tr[tr[t][3 + s]][(k + 1) % 3]]);
            if (l > allowed_mpr) /* found the neighb 's other too big side */
            {
              pos1[0] =
                ((*coords)[tr[tr[t][3 + s]][k % 3]][0] +
                 (*coords)[tr[tr[t][3 + s]][(k + 1) % 3]][0]) / 2.0;
              pos1[1] =
                ((*coords)[tr[tr[t][3 + s]][k % 3]][1] +
                 (*coords)[tr[tr[t][3 + s]][(k + 1) % 3]][1]) / 2.0;
              break;
            }
          }
          nb = tr[t][3 + s];
          d1 = eg_PointDist2D(pos1, (*coords)[tr[nb][(k + 2) % 3]]);
          d2 = eg_PointDist2D(pos, (*coords)[tr[nb][(tr[t][s + 6] + 2) % 3]]);
          if (d2 <= d1)
          {
            include_grid_node(pos, coords, np, tp, mpr, &nnp, tpnl, nmpr);
            (*coords)[(*np) - 1][2] = -1.0;
            tr[t][9] = 0;
            tr[tr[t][3 + s]][9] = 0;
          }
          else
            continue;
        }
      }
      else                      /* shorter diag neighb IS notified */
      {
        if (ngnstb[s] == 1)
          e_GeneralError("oops");
        else                    /* ngnstb[s]==2 */
        {
          /* check the neighbours diag situation */
          for (k = tr[t][6 + s] - 2; k < tr[t][6 + s]; k++)
          {
            l = eg_PointDist2D((*coords)[tr[tr[t][3 + s]][k % 3]],
                               (*coords)[tr[tr[t][3 + s]][(k + 1) % 3]]);
            allowed_mpr =
              rprm * E_MIN((*mpr)[tr[tr[t][3 + s]][k % 3]],
                           (*mpr)[tr[tr[t][3 + s]][(k + 1) % 3]]);
            if (l > allowed_mpr) /* found the neighb 's other too big side */
            {
              pos1[0] =
                ((*coords)[tr[tr[t][3 + s]][k % 3]][0] +
                 (*coords)[tr[tr[t][3 + s]][(k + 1) % 3]][0]) / 2.0;
              pos1[1] =
                ((*coords)[tr[tr[t][3 + s]][k % 3]][1] +
                 (*coords)[tr[tr[t][3 + s]][(k + 1) % 3]][1]) / 2.0;
              break;
            }
          }
          nb = tr[t][3 + s];
          d1 = eg_PointDist2D(pos1, (*coords)[tr[nb][(k + 2) % 3]]);
          d2 = eg_PointDist2D(pos, (*coords)[tr[nb][(tr[t][s + 6] + 2) % 3]]);
          if (d2 <= d1)
          {
            include_grid_node(pos, coords, np, tp, mpr, &nnp, tpnl, nmpr);
            (*coords)[(*np) - 1][2] = -1.0;
            tr[t][9] = 0;
            tr[tr[t][3 + s]][9] = 0;
          }
          else
            continue;
        }
      }
#endif
    }                           /* end class  */
  }                             /* end for all triangles */
  return nnp;
}

/* end of file */
