#include "e_lib.h"

#define SHAPE   1
#define SKEW    2
#define TAPER   3

static int compare_val(const void *l1, const void *l2);



int main(int argc, char *argv[])
{
  MESH   *m;
  int	  option;
  double *vals;
  int     i, j;
  int     n1, n2, n3;
  double  dx1, dx2, dy1, dy2, dz1, dz2, le1, le2, cx, cy, cz;
  double  co1, sa, t, area;
  
  if(argc < 3)
  {
    printf("quadstat  file -option\n");
    printf("for QUAD1 mdf files only !\n");
    printf(" -s  shape\n");
    printf(" -k  skew\n");
    printf(" -t  taper\n");
    exit(1);
  }
    
  if (argv[2][0] != '-')
  {
    printf("run quadstat to find out the options\n\n");
    exit(1);
  }
    
  switch (argv[2][1])
  {
    case 's':
        option=SHAPE;
        break;
    case 'k':
        option=SKEW;
        break;
    case 't':
        option=TAPER;
        break;
    default:
        e_GeneralError("unknown stats option");
  }
    
  e_GetMeshData(&m, argv[1]);
  if(m->NElemsQuad1 != m->TotalNElems)
  {
    e_GeneralError("Only QUAD1 element can be in the mesh");
  }
  

  vals = e_alloc_d_vect(m->NElemsQuad1);

  /* calculate shape measure */
  for(i = 0;i < m->NElemsQuad1; i++)
  {
    switch(option)
    {
      case SHAPE:
      {
        vals[i] = 0.0;
        for(j = 0; j < 4; j++)
        {
          n1 = m->NodesQuad1[i][j]-1;
          n2 = m->NodesQuad1[i][(j+3) % 4]-1;
          n3 = m->NodesQuad1[i][(j+1) % 4]-1;
          
          dx1 = m->MeshPointCoords[n2][0] - m->MeshPointCoords[n1][0];
          dy1 = m->MeshPointCoords[n2][1] - m->MeshPointCoords[n1][1];
          dz1 = m->MeshPointCoords[n2][2] - m->MeshPointCoords[n1][2];

          dx2 = m->MeshPointCoords[n3][0] - m->MeshPointCoords[n1][0];
          dy2 = m->MeshPointCoords[n3][1] - m->MeshPointCoords[n1][1];
          dz2 = m->MeshPointCoords[n3][2] - m->MeshPointCoords[n1][2];
          
          le1 = sqrt(dx1*dx1 + dy1*dy1 + dz1*dz1);
          le2 = sqrt(dx2*dx2 + dy2*dy2 + dz2*dz2);
          /* scalar product */
          vals[i] += E_ABS((dx1 * dx2 + dy1 * dy2 + dz1 * dz1 ) / (le1 * le2));
        }
        vals[i] = 1.0 - vals[i] / 4.0;
        break;
      }
      case SKEW:
      {
        vals[i] = 0.0;
        for(j = 0; j < 4; j++)
        {
          n1 = m->NodesQuad1[i][j]-1;
          n2 = m->NodesQuad1[i][(j+3) % 4]-1;
          n3 = m->NodesQuad1[i][(j+1) % 4]-1;
          
          dx1 = m->MeshPointCoords[n2][0] - m->MeshPointCoords[n1][0];
          dy1 = m->MeshPointCoords[n2][1] - m->MeshPointCoords[n1][1];
          dz1 = m->MeshPointCoords[n2][2] - m->MeshPointCoords[n1][2];

          dx2 = m->MeshPointCoords[n3][0] - m->MeshPointCoords[n1][0];
          dy2 = m->MeshPointCoords[n3][1] - m->MeshPointCoords[n1][1];
          dz2 = m->MeshPointCoords[n3][2] - m->MeshPointCoords[n1][2];
          
          le1 = dx1*dx1 + dy1*dy1 + dz1*dz1;
          le2 = dx2*dx2 + dy2*dy2 + dz2*dz2;

          co1 = dx1*dx2 + dy1*dy2 + dz1*dz2;
          /* sin(alpha) */
          sa = co1 / (sqrt(le1) * sqrt(le2));
          if(vals[i] < sa)
          {
            vals[i] = sa;
          }
        }
        break;
      }
      case TAPER:
      {
        vals[i] = 0.0;
        area = 0.0;
        /* center of the quad */
        for(j = 0; j < 4; j++)
        {
          n1 = m->NodesQuad1[i][j]-1;
          cx += m->MeshPointCoords[n1][0];
          cy += m->MeshPointCoords[n1][1];
          cz += m->MeshPointCoords[n1][2];
        }
        cx /= 4.0;
        cy /= 4.0;
        cz /= 4.0;
        for(j = 0; j < 4; j++)
        {
          n1 = m->NodesQuad1[i][j]-1;
          n2 = m->NodesQuad1[i][(j+1) % 4]-1;

          dx1 = m->MeshPointCoords[n1][0] - cx;
          dy1 = m->MeshPointCoords[n1][1] - cy;
          dz1 = m->MeshPointCoords[n1][2] - cz;

          dx2 = m->MeshPointCoords[n2][0] - cx;
          dy2 = m->MeshPointCoords[n2][1] - cy;
          dz2 = m->MeshPointCoords[n2][1] - cz;

          le1 = dx1*dx1 + dy1*dy1 + dz1*dz1;
          le2 = dx2*dx2 + dy2*dy2 + dz2*dz2;

          co1 = dx1*dx2 + dy1*dy2 + dz1*dz2;

          sa = sqrt(le1 * le2 - co1 * co1) * 0.5;
          area += sa;
          
          if(j != 0)
          {
            if(vals[i] > sa)
            {
              vals[i] = sa;
            }
          }
          else
          {
            vals[i] = sa;
          }
        }
        vals[i] = vals[i] / area * 4;
        
        break;
      }
      default:
      {
        e_GeneralError("Invalid shape measure");
      }
    }
  }

  /* print shape measure */
  qsort((void *)vals, m->NElemsQuad1,sizeof(double),compare_val);
  printf("#\n#\n");
  switch(option)
  {
    case SHAPE:
    {
      printf("# Shape\n");
      break;
    }
    case SKEW:
    {
      printf("# Skew\n");
      break;
    }
    case TAPER:
    {
      printf("# Taper\n");
      break;
    }
  }
  for(i = 0; i < m->NElemsQuad1; i++)
      printf("%1.6lf\n", vals[i]);
  
  e_FreeMeshData(m);
  e_free(vals);

  return 0;
}




static int compare_val(const void *l1, const void *l2)
{
  double *ld1, *ld2;
    
  ld1=(double*)l1;
  ld2=(double*)l2;
    
  if (*ld1>*ld2)
      return 1;
  if (*ld1<*ld2)
      return -1;
  return 0;
}
