#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "e_lib.h"

#define ASPECT	0
#define	SANGLE	1
#define RATIO	2
#define DEV_ANG 3
#define DEV_LNG 4
#define	AANGLE	5
#define	XDEV_ANG 6

int compare_val(const void *l1, const void *l2);
double dist(double *A, double *B);


int main(int argc, char *argv[])
{
  MESH *m;
  int	option;
  double *vals;
    
  int i, s, k;
  double l, d;
  int xd=0;
    
  /* aspect ratio vars */
  double dif1[2], dif2[2];
  double Sk, pk, hmax;
  /* smallest angle vars */
  double lengths[3];
  double sides[3][2];
  double sma, a, surf;
  /* length ratio vars */
  double minl,avl;
  /* standard deviation vars */
  double dev=0.0;
     
    
  if (argc<3)
  {
    printf("stats file -option\n");
    printf("for TRAING1 mdf files only !\n");
    printf(" -a  aspect ratio\n");
    printf(" -s  smallest angle\n");
    printf(" -g  any angle\n");
    printf(" -r  length ratio\n");
    printf(" -d  standard deviation of angles\n");
    printf(" -l  standard deviation of lengths\n\n");
    printf(" -x  standard deviation of non regular triangle angles\n\n");
    exit(1);
  }
    
  if (argv[2][0] != '-')
  {
    printf("run stats to find out the options\n\n");
    exit(1);
  }
    
  switch (argv[2][1])
  {
    case 'a':
        option=ASPECT;
        break;
    case 's':
        option=SANGLE;
        break;
    case 'g':
        option=AANGLE;
        break;
    case 'r':
        option=RATIO;
        break;
    case 'd':
        option=DEV_ANG;
        break;
    case 'l':
        option=DEV_LNG;
        break;
    case 'x':
        option=XDEV_ANG;
        break;
    default:
        e_GeneralError("unknown stats option");
  }
    
  e_GetMeshData(&m, argv[1]);
  if(m->NElemsTriang1 != m->TotalNElems)
  {
    e_GeneralError("Only TRIANG1 element can be in the mesh");
  }

  if (option==AANGLE)
      vals=e_alloc_d_vect(3*m->NElemsTriang1);
  else
      vals=e_alloc_d_vect(m->NElemsTriang1);
    
  for (i=0;i<m->NElemsTriang1;i++)
  {
    if (option==ASPECT)
    {
      pk=0.0;
      hmax=0.0;
      for (s=0;s<3;s++)
      {
        l=dist(m->MeshPointCoords[m->NodesTriang1[i][s]-1], 
               m->MeshPointCoords[m->NodesTriang1[i][(s+1)%3]-1]);
        if (l>hmax)
            hmax=l;
        pk+=l;
      }
      pk*=0.5;
      Sk=(m->MeshPointCoords[m->NodesTriang1[i][1]-1][0]-
          m->MeshPointCoords[m->NodesTriang1[i][0]-1][0] )*
          (m->MeshPointCoords[m->NodesTriang1[i][2]-1][1]-
           m->MeshPointCoords[m->NodesTriang1[i][0]-1][1] )-
          (m->MeshPointCoords[m->NodesTriang1[i][2]-1][0]-
           m->MeshPointCoords[m->NodesTriang1[i][0]-1][0] )*
          (m->MeshPointCoords[m->NodesTriang1[i][1]-1][1]-
           m->MeshPointCoords[m->NodesTriang1[i][0]-1][1] );
      Sk*=0.5;	
      vals[i]=(sqrt(3.0)/6.0)*((hmax*pk)/Sk);
    }
    else if (option==SANGLE)
    {
      sma=10.0;
      for (s=0;s<3;s++)
      {
        lengths[s]=dist(m->MeshPointCoords[m->NodesTriang1[i][s]-1], 
                        m->MeshPointCoords[m->NodesTriang1[i][(s+1)%3]-1]);
        for (k=0;k<2;k++)
            sides[s][k]=m->MeshPointCoords[m->NodesTriang1[i][(s+1)%3]-1][k]-
                m->MeshPointCoords[m->NodesTriang1[i][s]-1][k];
      }
      for (s=0;s<3;s++)
      {
        surf=e_VectDotProd(2, sides[s], sides[(s+2)%3]);
        a=asin(surf/(lengths[s]*lengths[(s+2)%3]));
        if (a<M_PI/2.0)
            a+=M_PI/2.0;
        if (a<sma)
            sma=E_ABS(a);
      }
      vals[i]=180.0*sma/M_PI;
    }
    else if (option==AANGLE)
    {
      sma=10.0;
      for (s=0;s<3;s++)
      {
        lengths[s]=dist(m->MeshPointCoords[m->NodesTriang1[i][s]-1], 
                        m->MeshPointCoords[m->NodesTriang1[i][(s+1)%3]-1]);
        for (k=0;k<2;k++)
            sides[s][k]=m->MeshPointCoords[m->NodesTriang1[i][(s+1)%3]-1][k]-
                m->MeshPointCoords[m->NodesTriang1[i][s]-1][k];
      }
      for (s=0;s<3;s++)
      {
        surf=e_VectDotProd(2, sides[s], sides[(s+2)%3]);
        a=asin(surf/(lengths[s]*lengths[(s+2)%3]));
        if (a<M_PI/2.0)
            a+=M_PI/2.0;
        vals[3*i+s]=180.0*a/M_PI;    
      }
	    
    }
    else if (option==RATIO)
    {
      minl=1e9;
      avl=0.0;
      for (s=0;s<3;s++)
      {
        l=dist(m->MeshPointCoords[m->NodesTriang1[i][s]-1], 
               m->MeshPointCoords[m->NodesTriang1[i][(s+1)%3]-1]);
        if (l<minl)
            minl=l;
        avl+=l;
      }
      avl/=3.0;
      vals[i]=minl/avl;
    }
    else if (option==DEV_ANG)
    {
      for (s=0;s<3;s++)
      {
        lengths[s]=dist(m->MeshPointCoords[m->NodesTriang1[i][s]-1], 
                        m->MeshPointCoords[m->NodesTriang1[i][(s+1)%3]-1]);
        for (k=0;k<2;k++)
            sides[s][k]=m->MeshPointCoords[m->NodesTriang1[i][(s+1)%3]-1][k]-
                m->MeshPointCoords[m->NodesTriang1[i][s]-1][k];
      }
      for (s=0;s<3;s++)
      {
        surf=e_VectDotProd(2, sides[s], sides[(s+2)%3]);
        a=asin(surf/(lengths[s]*lengths[(s+2)%3]));
        if (a<M_PI/2.0)
            a+=M_PI/2.0;
        a*=180.0/M_PI;
        dev+=(a-60.0)*(a-60.0);
      }
    }
    else if (option==XDEV_ANG)
    {
      for (s=0;s<3;s++)
      {
        lengths[s]=dist(m->MeshPointCoords[m->NodesTriang1[i][s]-1], 
                        m->MeshPointCoords[m->NodesTriang1[i][(s+1)%3]-1]);
        for (k=0;k<2;k++)
            sides[s][k]=m->MeshPointCoords[m->NodesTriang1[i][(s+1)%3]-1][k]-
                m->MeshPointCoords[m->NodesTriang1[i][s]-1][k];
      }
      for (s=0;s<3;s++)
      {
        surf=e_VectDotProd(2, sides[s], sides[(s+2)%3]);
        a=asin(surf/(lengths[s]*lengths[(s+2)%3]));
        if (a<M_PI/2.0)
            a+=M_PI/2.0;
        a*=180.0/M_PI;
        if (E_ABS(a-60.0)>0.5)
        {
          dev+=(a-60.0)*(a-60.0);
          xd++;
        }
      }
    }
    else if (option==DEV_LNG)
    {
      avl=0.0;
      for (s=0;s<3;s++)
      {
        lengths[s]=dist(m->MeshPointCoords[m->NodesTriang1[i][s]-1], 
                        m->MeshPointCoords[m->NodesTriang1[i][(s+1)%3]-1]);
        avl+=lengths[s];
      }
      avl/=3.0;
      for (s=0;s<3;s++)
          dev+=(lengths[s]-avl)*(lengths[s]-avl);
    }
  }
    
  if ((option==DEV_ANG)||(option==DEV_LNG))
      dev=sqrt(dev/(3.0*m->NElemsTriang1));
    	
  if (option==XDEV_ANG)
      dev=sqrt(dev/(double)xd);
    
  if ((option!=DEV_ANG)&&(option!=DEV_LNG)&&(option!=XDEV_ANG))
  {
    printf("#\n#\n");
    if (option==AANGLE)
    {
      qsort((void *)vals,3*m->NElemsTriang1,sizeof(double),compare_val);
      printf("# Distribution of the angles\n");
      for (i=0;i<3*m->NElemsTriang1;i++)
          printf("%1.6lf\n", vals[i]);
    }
    else
    {
      switch(option)
      {
        case ASPECT:
            printf("# Distribution of the aspect ratio\n");
            break;
        case SANGLE:
            printf("# Distribution of the smallest angle\n");
            break;
        case RATIO:
            printf("# Distribution of the length ratio\n");
            break;
        default:
        {
          e_GeneralError("Invalid statistics!");
        }
      }
      
      qsort((void *)vals,m->NElemsTriang1,sizeof(double),compare_val);
      for (i=0;i<m->NElemsTriang1;i++)
          printf("%1.6lf\n", vals[i]);
    }
  }
  else
      printf("%1.6lf\n",dev);
    	
  e_FreeMeshData(m);
  e_free(vals);

  return 0;
}

int compare_val(const void *l1, const void *l2)
{
  double *ld1, *ld2;
    
  ld1=(double*)l1;
  ld2=(double*)l2;
    
  if (*ld1>*ld2)
      return 1;
  if (*ld1<*ld2)
      return -1;
  return 0;
}



double dist(double *A, double *B)
{
  /* returns the positive distance between two points */

  double dx2, dy2;
    
  dx2=(A[0]-B[0]);
  dx2=dx2*dx2;
  dy2=(A[1]-B[1]);
  dy2=dy2*dy2;
    
  return sqrt(dx2+dy2);
}
