# -- DoNothing --
# 
# Arguments:
#   NONE
# Results
#   Returns NULL
# 
proc DoNothing {} {
   return 
}

# -- EnterDefaultMode --
#    Sets global data for default mode. Sets global bindings.
#
# Arguments:
#   NONE
#   
# Results:
#   Returns NULL.
#   Sets the default bindings for all  events.
#   Calls - SetErrorMessage, DefaultBindings.
#   
proc EnterDefaultMode {} {
    global drawArea
    set drawArea(mode) "None"    
    set drawArea(modefun) "DefaultBindings"
    set drawArea(modeleave) "DoNothing"
     if { [string compare $drawArea(currentpanel) ""] != 0} {
        pack forget .nvWork.right.$drawArea(currentpanel)
     }
     set drawArea(currentpanel) {}     
     SetErrorMessage {}
     $drawArea(cv) delete "zoom_window"
     DefaultBindings
}

# -- EnterZoomWindowMode --
# 
# Initiate window zooming mode.
#
# Arguments:
#    NONE
# Results:
#    Retuens NULL.
#    Before entering zoom mode enters the default mode.
#    Sets mode name and initialtes window zooming bindings.
#    Calls - EnterDefaultMode, MakeZoomWindowBindings.
#
proc EnterZoomWindowMode {} {
    global drawArea

    EnterDefaultMode
    set drawArea(mode) "Zoom window"
    set drawArea(modefun) "MakeZoomWindowBindings"
    set drawArea(modeleave) "BreakZoomWindowMode"
    MakeZoomWindowBindings
    return  
} 

# -- BreakZoomWindowMode --
#
# Terminates the zoom window mode
#
# Arguments:
#   NONE
# Results:
#   Returns NULL.
#   Enters the default mode
#   Sets zoomMode(active) to -1 -> zoom mode is switched off
#   Calls - EnterDefaultMode
#
proc BreakZoomWindowMode {} {
    global zoomMode
    set zoomMode(active) -1
    EnterDefaultMode
}

# -- MakeZoomWindowBindings --
#
# Sets bindings for mouse events related to zoom window mode
#
# Arguments:
#   NONE
# Results:
#   Returns NULL.
#   Calls: EnterDefaultMode, Zoom, DrawZoomWindow, BreakZoomWindowMode.   
#
proc MakeZoomWindowBindings {} {
    global zoomMode
    global drawArea

    bind $drawArea(cv) <ButtonPress-1> {
        global zoomMode
        global drawArea
        if { $zoomMode(active) == 1 } then {
            set zoomMode(active) 0 
            Zoom $zoomMode(x) $zoomMode(y) %x %y
        } else {
            set zoomMode(active) 1
            set $zoomMode(x) %x
            set $zoomMode(y) %y
            DrawZoomWindow %x %y
        }
    }

    bind $drawArea(cv) <ButtonPress-3> {
        BreakZoomWindowMode        
    }
    return        
}

# -- EnterZoomRegiomMode --
# 
# Initiate region zooming mode.
#
# Arguments:
#    NONE
# Results:
#    Returns NULL.
#    Before entering zoom mode enters the default mode.
#    Sets mode name and initialtes region zooming bindings.
#    Draw zoom region in top left corner of the canvas.
#
proc EnterZoomRegionMode {} {
    global drawArea

    EnterDefaultMode
    set drawArea(mode) "Zoom region"
    set drawArea(modefun) "MakeZoomRegionBindings"
    set drawArea(modeleave) "BreakZoomRegionMode"
    SetRegionSize
    DrawZoomRegionWindow 2 2
    MakeZoomRegionBindings
    return  
} 



# -- BreakZoomRegionMode 
#    
# Argyments:
#    NONE
# Restults:
#    Retrurns NULL.
#    Deletes zoom window, enters the default mode.
#    Calls - EnterDefaultMode.
proc BreakZoomRegionMode {} {
    global drawArea
   
    $drawArea(cv) delete "zoom_window"
    EnterDefaultMode
    return 
}

# -- MakeZoomRegionBindings
#    Set up the bindings for make zoom region.
# Arguments:
#    NONE
# Restults:
#    Returns NULL.
#    Binds - ButtonPress-1, ButtonPress-3, 
#    Calls - Zoom, ZoomRegionImage, DrawZoomRegionWindow,
#            BreakZoomRegionMode.
proc MakeZoomRegionBindings {} {
    global drawArea
    global zoomMode

    bind $drawArea(cv) <ButtonPress-1> {
        global drawArea
        global zoomMode
        eval Zoom $zoomMode(regioncoords) 
        eval ZoomRegionImage $zoomMode(regioncoords)
        DrawZoomRegionWindow %x %y
    }

    bind $drawArea(cv) <ButtonPress-3> {
        BreakZoomRegionMode        
    }
    return        
}

# -- EnterDrawNurbsMode --
#
# Enter nurbs drawing mode.
#
# Arguments:
#   NONE
# Results:
#   Returns NULL.
#   Sets the mode title. Make bindings for NURBS drawing mode
#   Before entering NURBS drawing mode enters the default mode.
#   Calls - EnterDefaultMode, MakeDrawNurbsBindings, SetErrorMessage
#
proc EnterDrawNurbsMode {} {
    global Mesh
    global drawArea

    EnterDefaultMode
# check if there are any mesh nodes, if not set error 
    if { $Mesh(nnodes) < 1 } then {
       SetErrorMessage "Mesh nodes are not defined"
       return 
    }
    set drawArea(mode) "Draw NURBS"
    set drawArea(modefun) "MakeDrawNurbsBindings"
    set drawArea(modeleave) "BreakDrawNurbsMode"
    MakeDrawNurbsBindings
    return  
}    



# -- BreakDrawNurbsMode --
#
# Terminates NURBS drawing mode.
# 
# Arguments:
#   NONE
# Results:
#   Returns NULL.
#   Removes any undinished nurbs. Enters the default mode.
#   Calls - InitNurbs, EnterDefaultMode.
#
proc BreakDrawNurbsMode {} {
    global Nurbs
    global drawArea

    for {set i 1} {$i <= $Nurbs(nnodes)} {incr i} {
        $drawArea(cv) delete nurbsNode$Nurbs(id).$i
    }
    $drawArea(cv) delete ns$Nurbs(id)
    InitNurbs
    EnterDefaultMode
    return
}

# -- MakeDrawNurbsBindings --
#
# Sets the mouse bindings for NURBS drawing mode
#
# Arguments:
#   NONE
# Results:
#   Returns NULL. 
#   Sets the bindings for moues events.
#   Binds - ButtonPress-1, ButtonPress-3
#   Calls - DrawNurbsNode, DrawNurgsSkel, DrawNurbs, InitNurbs, 
#           BreakDrawNurbsMode.
#
proc MakeDrawNurbsBindings {} {
    global Nurbs
    global drawArea

    bind $drawArea(cv) <ButtonPress-1> {
         ConstructNurbs %x %y
    }
    bind $drawArea(cv) <ButtonPress-3> {
        BreakDrawNurbsMode
    }
    return
}

# -- EnterEditNurbsMode --
#
# Enter nurbs editing mode
#
# Arguments:
#   NONE
# Results:
#   Returns NULL.
#   Sets the mode title. Make bindings for NURBS editing mode
#   Before entering NURBS drawing mode enters the default mode.
#   Calls -  EndterDefaultMode, MakeEditNurbsBindings, ShowNurbsEditPanel, 
#            SetInfoMessage.
#
proc EnterEditNurbsMode {} {
    global Nurbs
    global drawArea
    EnterDefaultMode
    set drawArea(mode)      "Edit Nurbs"
    set drawArea(modefun)   "MakeEditNurbsBindings"
    set drawArea(modeleave) "BreakEditNurbsMode"
    MakeEditNurbsBindings
    ShowNurbsEditPanel 1
    SetInfoMessage "Use mouse button 2 to sellect or drag control points"
    return  
}    

# -- BreakEditNurbsMode --
#
# Terminates nurbs editing mode.
#
# Arguments:
#   NONE
# Results:
#   Returns NULL.
#   Sets the mode title. Make bindings for NURBS editing mode.
#   Before entering NURBS drawing mode enters the default mode.
#   Calls - ShowNurbsEditPanel, SetInfoMessage, EnterDefaultMode.
#
proc BreakEditNurbsMode {} {
    global Nurbs
    global drawArea    

    set Nurbs(curnurbs) -1
    set Nurbs(curnode) -1

    ShowNurbsEditPanel 0
    SetInfoMessage ""
    EnterDefaultMode
    return
}

# -- MakeEditNurbsBindings --
#
# Sets the mouse events bindings for NURBS editing mode.
#
# Arguments:
#   NONE
# Results:
#   Returns NULL.
#   Calls - GiveOverlappingItems, SetNurbsEditInfo, ShowNurbsEditPanel,
#           UpdateWeightDisplay, SetWarningMessage, SetNurbsCoords,
#           InitEditNurbsBox, ShowEditNurbsBox, GiveNurbsNode, SetInfoMessage, 
#           MoveAxes, SetCoordsInfo, BreakEditNurbsMode.
#
proc MakeEditNurbsBindings {} {
    global Nurbs
    global drawArea

    bind $drawArea(cv) <ButtonPress-2> {
        set n [GiveNurbsNode %x %y]
        set nurbs [lindex $n 0]
        set node  [lindex $n 1]
        if { $nurbs != -1 } {
            if { [winfo exist .nurbsnodebox] == 1} then {
                InitEditNurbsBox $nurbs $node
            } else {
                ShowEditNurbsBox $nurbs $node 
            }
        }
    }

    bind $drawArea(cv) <ButtonPress-1> {
        global drawArea
        global Nurbs
        
        set items [GiveOverlappingItems $drawArea(cv) \
                NURBSNODE \
                %x %y \
                $drawArea(searchwidth) $drawArea(searchheight)]
        if { [llength $items] > 0} {
            set nodeId [lindex $items 0]
            set tagList [$drawArea(cv) itemcget $nodeId -tags]
            set node [GetNumberFromTag $tagList "nurbsNode" ]
            set node [split $node "."]
            set Nurbs(curnurbs) [lindex $node 0]
            set Nurbs(curnode) [lindex $node 1]
            SetNurbsEditInfo $Nurbs(curnurbs) $Nurbs(curnode)
            ShowNurbsEditPanel 1 
            SetInfoMessage \
                    "Selected NURBS: $Nurbs(curnurbs) at node $Nurbs(curnode)"
            UpdateWeightDisplay
        }
        if { [llength $items] > 1 } {
            SetWarningMessage \
                   "Warning: more than one node selected - taking the top one" 
        }
    }
    bind $drawArea(cv) <B1-Motion> {
         MoveAxes %x %y 
         set pp [list %x %y]    
         set pp [Canv2Viewport $pp]    
         SetCoordsInfo $pp
         if {$Nurbs(curnurbs) != -1 && $Nurbs(curnode) != -1} {
            if {$Nurbs(curnode) != 1 && $Nurbs(curnode) != 4} {
                set pp [list %x %y]
                set pp [Canv2Viewport $pp]    
                set x [lindex $pp 0]
                set y [lindex $pp 1]
                SetNurbsCoords $Nurbs(curnurbs) $Nurbs(curnode) $x $y
            }
         }
    }
    bind $drawArea(cv) <ButtonPress-3> {
        BreakEditNurbsMode
    }
    return 
}

# -- EnterDeleteNurbsMode --
#
# Enter nurbs deleting mode.
#
# Arguments:
#   NONE
# Results:
#   Returns NULL.
#   Sets the mode title. Make bindings for NURBS deleting mode.
#   Before entering NURBS deleting mode enters the default mode.
#   Calls - EndterDefaultMode, MakeDeleteNurbsBindings, SetInfoMessage.
#
proc EnterDeleteNurbsMode {} {
    global Nurbs
    global drawArea
    EnterDefaultMode
    set drawArea(mode) "Delete NURBS"
    set drawArea(modefun) "MakeDeleteNurbsBindings"
    set drawArea(modeleave) "BreakDeleteNurbsMode"
    MakeDeleteNurbsBindings
    SetInfoMessage "To delete NURBS click mouse button 1 over NURBS curve"
    return  
}    

# -- MakeDeleteNurbsBindings --
#
# Sets the mouse events bindings for NURBS deleting mode.
#
# Arguments:
#   NONE
# Results:
#   Returns NULL.
#   Calls - GiveOverlappingItems, GetNumberFromTag, BreakDeleteNurbsMode,
#           SetInfoMessage, SetWarningMessage, DeleteNurbs.
#
proc MakeDeleteNurbsBindings {} {
    global Nurbs
    global drawArea

    bind $drawArea(cv) <ButtonPress-1> {
        global drawArea
        global Nurbs
        
        set items [GiveOverlappingItems $drawArea(cv) \
                NURBSCURVE \
                %x %y \
                $drawArea(searchwidth) $drawArea(searchheight)]
        if { [llength $items] > 0} {
            set nurbsId [lindex $items 0]
            set tagList [$drawArea(cv) itemcget $nurbsId -tags]
            set id [GetNumberFromTag $tagList "nurbsCurve" ]
            SetInfoMessage \
                    "Deleted NURBS number: $id"
        }
        if { [llength $items] > 1 } {
            SetWarningMessage \
                   "Warning: more than one node selected - taking the top one" 
        }
        DeleteNurbs $id
    }
    bind $drawArea(cv) <ButtonPress-3> {
        BreakDeleteNurbsMode
    }
    return 
}

# -- BreakDeleteNurbsMode --
#
# Terminates nurbs deleting mode.
#
# Arguments:
#   NONE
# Results:
#   Returns NULL.
#   Resets the mode to default mode. Cleans info message.
#   Calls - SetInfoMessage, EnterDefaultMode.
#
proc BreakDeleteNurbsMode {} {
    global Nurbs
    global drawArea    

    set Nurbs(curnurbs) -1
    set Nurbs(curnode) -1

    SetInfoMessage ""
    EnterDefaultMode
    return
}



#--------------------------- ELEMENTS
# Enter element delete mode
proc EnterDelElemMode {nvertex} {
    global drawArea
    global Mesh    

    EnterDefaultMode
    if { $Mesh(nelems$nvertex) < 1 } then {
       SetErrorMessage "Type of elements are not defined"
       return 
    }
    set drawArea(mode) "DelElem ($nvertex)"
    set drawArea(modefun) "MakeDelElemBindings"
    set drawArea(modeleave) "BreakDelElemMode"
    set Mesh(nvertex) $nvertex
    
    MakeDelElemBindings 
}

proc BreakDelElemMode {} {
    EnterDefaultMode
    
    return
}

proc MakeDelElemBindings {} {
    global Mesh
    global drawArea

    bind $drawArea(cv) <ButtonPress-1> {
        global drawArea
        global Mesh

        if { $Mesh(nvertex) == 2 } {
          set itemtag LINK
        }
        if { $Mesh(nvertex) == 3 } {
          set itemtag TRIANG
        }
        if { $Mesh(nvertex) == 4 } {
          set itemtag QUAD
        }

        set items [GiveOverlappingItems $drawArea(cv) \
                $itemtag \
                %x %y 0 0]

        if { [llength $items] > 0} { 
            set elem [lindex $items 0]
            set elem1 [$drawArea(cv) itemcget $elem -tags]
            set elem1 [GetNumberFromTag $elem1 "e"]

            DeleteElement $elem1
        }
    }

    bind $drawArea(cv) <ButtonPress-3> {
        BreakDelElemMode
    }
}

# ----------------------------------

# Enter elements drawing mode
proc EnterElementsMode {nvertex} {
    global drawArea
    global Mesh    


    EnterDefaultMode
# check if there are any mesh nodes, if not set error 
    if { $Mesh(nnodes) < 1 } then {
       SetErrorMessage "Mesh nodes are not defined"
       return 
    }

    set drawArea(mode) "Elements ($nvertex)"
    set drawArea(modefun) "MakeDrawElemsBindings"
    set drawArea(modeleave) "BreakDrawElemsMode"
    set Mesh(nvertex) $nvertex
    
    MakeDrawElemsBindings 
}

proc BreakDrawElemsMode {} {
    ClearNodeSelect
    EnterDefaultMode
    
    return
}

proc MakeDrawElemsBindings {} {
    global Mesh
    global drawArea
    bind $drawArea(cv) <ButtonPress-1> {
        ConstructElement %x %y
    }
    bind $drawArea(cv) <ButtonPress-3> {
        BreakDrawElemsMode
    }
}

proc ConstructElement {x y} {
        global drawArea
        global Mesh     
        set items [GiveOverlappingItems $drawArea(cv) \
               MESHNODE \
                $x $y \
                $drawArea(searchwidth) $drawArea(searchheight)]
        if { [llength $items] > 0} { 
            set node [lindex $items 0]
            set node1 [$drawArea(cv) itemcget $node -tags]
            set node1 [GetNumberFromTag $node1 "meshNode"]
            lappend Mesh(selnodnum) $node1 
            set node1 [expr $node1 -1]
            lappend Mesh(selnodes) \
                [Viewport2Canv [lindex $Mesh(nodecoords) $node1]]
            lappend Mesh(selnodid) $node    
            $drawArea(cv) itemconfigure $node -fill $drawArea(nodeselectcolor) 
        }
        if { [llength $Mesh(selnodes)] == $Mesh(nvertex) } {
           incr Mesh(nelems$Mesh(nvertex)) 
           incr Mesh(totnelems) 
           DrawElement $Mesh(nelems$Mesh(nvertex))
           UpdateMesh
           ClearNodeSelect
        } 
}

#--------------------------- NODES
# Enter nodes drawing mode
proc EnterDrawMeshNodeMode {} {
    global drawArea
    EnterDefaultMode
    set drawArea(mode) "Draw nodes"
    set drawArea(modefun) "MakeDrawMeshNodeBindings" 
    set drawArea(modeleave) "BreakDrawMeshNodeMode"
    MakeDrawMeshNodeBindings    
}

proc BreakDrawMeshNodeMode {} {
    EnterDefaultMode
    return
}

# clear node selection 
proc ClearNodeSelect {} {
    global drawArea
    global Mesh

    foreach i $Mesh(selnodid) {
         $drawArea(cv) itemconfigure $i -fill $drawArea(ncolor) 
    }
    set Mesh(selnodes) {}
    set Mesh(selnodid) {}
    set Mesh(selnodnum) {}
}

# make node bindings
proc MakeDrawMeshNodeBindings {} {
    global Mesh
    global drawArea
    
    bind $drawArea(cv) <ButtonPress-1> {
        CreateMeshNode %x %y 0
    }
    bind $drawArea(cv) <ButtonPress-3> {
        BreakDrawMeshNodeMode
    }
    return
}

# Enter nodes drawing mode
proc EnterEditMeshNodeMode {} {
    global drawArea
    EnterDefaultMode
    set drawArea(mode) "Edit nodes"
    set drawArea(modefun) "MakeEditMeshNodeBindings" 
    set drawArea(modeleave) "BreakEditMeshNodeMode"
    ShowEditMeshNodePanel 1
    MakeEditMeshNodeBindings    
}

proc BreaEditMeshNodeMode {} {
    EnterDefaultMode
    return
}

# make node bindings
proc MakeEditMeshNodeBindings {} {
    global Mesh
    global drawArea
    
    bind $drawArea(cv) <ButtonPress-1> {
        set a [GiveMeshNode %x %y]
        if {[llength $a] > 0} then {
            set node [lindex [GiveMeshNode %x %y] 0]
            set drawArea(nodeselected) $node
            if { $node != 0 } {
                if { [winfo exist .meshnodebox] == 1} then {
                   InitEditMeshNodeBox  $node
                } else {
                   ShowEditMeshNodeBox $node 
                }
            }
            eval DrawGhostVertex %x %y 
        }
    }
    bind $drawArea(cv) <B1-Motion> {
         MoveAxes %x %y 
         set pp [list %x %y]    
         set pp [Canv2Viewport $pp]    
         SetCoordsInfo $pp
         MoveGhostVertex %x %y 
         %W raise GHOSTVERTEX
    }
    bind $drawArea(cv) <ButtonRelease-1> {
         DeleteGhostVertex 
         set pc [list %x %y]
         set pp [Canv2Viewport $pc]
         eval MoveMeshNode $drawArea(nodeselected) $pp
    }
    
    bind $drawArea(cv) <ButtonPress-3> {
        ShowEditMeshNodePanel 0
        BreakDrawMeshNodeMode
    }
    return
}

# -- DefaultBindings --
#
# Sets the default bindings
#
# Arguments:
#   NONE
# Results:
#   Returns NULL
# Calls:
#   MoveAxes, InitCursorPos, SetCoordsInfo, EnterDefaultMode
#   
proc DefaultBindings {} {
    global drawArea
    global ImageProp
    global Nurbs

#window resizing 
    bind $drawArea(cv) <Configure> {
      set drawArea(height) %h
      set ImageProp(yins) %h
      set ImageProp(ydisp) %h
      set drawArea(width) %w
      set drawArea(canv) [list 0 0 %w %h]
         
      MoveAxes [lindex $drawArea(pos) 0] [lindex $drawArea(pos) 1] 
      SetInfoMessage "Canvas size %w by %h"
#
      CleanCanvas
      ZoomLimitsRegionImage
      RedrawMesh
      RedrawNurbs

    }

    bind $drawArea(cv) <ButtonPress-1> {}
    bind $drawArea(cv) <ButtonPress-2> {}
    bind $drawArea(cv) <ButtonRelease-1> {}
    bind $drawArea(cv) <ButtonRelease-2> {}
    bind $drawArea(cv) <ButtonPress-3> {
        EnterDefaultMode
    }
    bind $drawArea(cv) <B2-Motion> {
        MoveAxes %x %y 
    }
    bind $drawArea(cv) <Motion> {
        MoveAxes %x %y 
        set pp [list %x %y]    
        set pp [Canv2Viewport $pp]    
        SetCoordsInfo $pp
    }
    bind  $drawArea(cv) <Enter> {
        InitCursorPos %x %y 
        RestoreCurrentBindings
    }
    bind $drawArea(cv) <Leave> {
        SuspendCurrentBindings
    }
   
    bind .nvCommand.commandEntry <Leave> {
        .nvCommand.commandEntry configure -state disabled
        bind .nvCommand.commandEntry <KeyPress-Return> {}    
    }
    bind .nvCommand.commandEntry <Enter> {
        .nvCommand.commandEntry configure -state normal 
         bind .nvCommand.commandEntry <KeyPress-Return> ExecuteCommand
    }

    bind all <Key-Escape> {
      EscapeBinding
   }
    return
}

proc EscapeBinding {} {
   global drawArea 
   $drawArea(modeleave)
   .nvCommand.commandEntry delete 0 end 
   .nvCommand.commandEntry icursor 0
   SetInfoMessage {}
   EnterDefaultMode
}

proc RestoreCurrentBindings {} {
    global drawArea
    $drawArea(modefun)
}

proc SuspendCurrentBindings {} {
    global drawArea
    bind $drawArea(cv) <B1-Motion> {}
    bind $drawArea(cv) <B2-Motion> {}
    bind $drawArea(cv) <B3-Motion> {}
    bind $drawArea(cv) <ButtonPress-1> {}
    bind $drawArea(cv) <ButtonPress-2> {}
    bind $drawArea(cv) <ButtonPress-3> {}
}
#EOF
