#------------ NURBS DRAWING

proc EditNurbsAction {} {
    global Entries
    global Mesh
    global Nurbs
    set status [expr $Entries(nodenum) * $Entries(nurbsnum) \                                  * $Entries(xcoord) * $Entries(ycoord) \
                    * $Entries(weight)]

    if {$status != 1} {
        SetErrorMessage "Nurbs parameters not specified correctly"
        return
    }
    set nurbsweight [.nurbsnodebox.input.nurbsweightent get] 
    set x [.nurbsnodebox.input.xcoordent get] 
    set y [.nurbsnodebox.input.ycoordent get] 
    set nurbsnodenum [.nurbsnodebox.input.nurbsnodenument get] 
    set nurbsnum [.nurbsnodebox.input.nurbsnument get] 
    set Nurbs(curnurbs) $nurbsnum
    set Nurbs(curnode) $nurbsnodenum

    etliNurbs$nurbsnum set -ctrlpoint $nurbsnodenum -weight $nurbsweight 
    SetNurbsEditInfo $Nurbs(curnurbs) $Nurbs(curnode)
    UpdateWeightDisplay

    if {$nurbsnodenum !=1 && $nurbsnodenum != 4} then {
       etliNurbs$nurbsnum set -ctrlpoint $nurbsnodenum -coords $x $y 0.0
    } else {
       SetErrorMessage "To move endpoints you have to move mesh node"
       return
    }
    UpdateNurbs $nurbsnum    
    UpdateNurbsNodes $nurbsnum $nurbsnodenum
    UpdateNurbsSkel $nurbsnum $nurbsnodenum
    return 
}

# -- InitNurbs --
#
# Initializes NURBS drawing related variables
#n 
# Arguments:
#   NONE
# Results:
#   Returns NULL.
#
proc InitNurbs {} {
    global Nurbs
    set Nurbs(nnodes) 0
    set Nurbs(def) {}
    set Nurbs(skel) {}
    set Nurbs(startNode) 0
    set Nurbs(endNode) 0
    return    
}


proc ConstructNurbs {x y {weight 1.0}} {
    global Nurbs
    global drawArea

    if {$Nurbs(nnodes) == 0 || $Nurbs(nnodes) == 3} {
       if {$Nurbs(nnodes) == 0} then {
           set Nurbs(startNode)
       }
       set meshNode [GiveMeshNode $x $y]
       set nodenum [lindex $meshNode 0]
       if { $nodenum == 0} then {
          SetErrorMessage "Firs and last nurbs node must be a mesh node"
          return -1 
       }
       set pp [lindex $meshNode 1]
       set pc [Viewport2Canv $pp]
       set x [lindex $pc 0]
       set y [lindex $pc 1]
       lappend Nurbs(skel) [Viewport2Canv $pp]

       if {$Nurbs(nnodes) == 0} then {
           set Nurbs(startNode) $nodenum
       } else {
           set Nurbs(endNode) $nodenum
       }
    } else {
        set pp [list $x $y] 
        lappend Nurbs(skel) $pp
        set pp [Canv2Viewport $pp]
    }
    lappend pp 0.0 $weight 
    lappend Nurbs(def) $pp
    incr Nurbs(nnodes)
    DrawNurbsNode $x $y    
    DrawNurbsSkel $Nurbs(id)  
    if {$Nurbs(nnodes) == 4} {
       set def [join $Nurbs(def) " "]     
       set n [eval etli_nurbs  3 $Nurbs(startNode) $Nurbs(endNode) "$def"]
       AddNurbsToNode $Nurbs(startNode) 0 $Nurbs(id)
       AddNurbsToNode $Nurbs(endNode) 1 $Nurbs(id)
       lappend Nurbs(nurbslist)  [GetNumberFromTag $n etliNurbs]
       DrawNurbs $drawArea(cv) $n 30 $Nurbs(id) 
       DrawNurbsBase $Nurbs(id)
       ShowNurbsNumbers $Nurbs(shownnum)
       InitNurbs 
       incr Nurbs(id) 
    }
    return $Nurbs(nnodes)
} 

# user command for creatind nurbs 
proc nurbs {x y {w 1.0}} {
    set pv [Viewport2Canv [list $x $y]]
    set pp [list $x $y]
    set x [lindex $pv 0]
    set y [lindex $pv 1]
    set i [ConstructNurbs $x $y $w]
    if { $i > 0 && $i < 4} {
     incr i
      SetInfoMessage "Specify NURBS control point number $i:"
      .nvCommand.commandEntry delete 0 end
      .nvCommand.commandEntry icursor 0
      .nvCommand.commandEntry insert 0 {nurbs }
    } elseif {$i == 0} {
      .nvCommand.commandEntry delete 0 end
      .nvCommand.commandEntry icursor 0
      SetInfoMessage {}
    } else {
      SetInfoMessage "First and last nurbs node must be placed over a mesh node"
    }
}


# -- DrawNurbs --
#
# Draws NURBS curve
#
# Arguments:
#   canv -- canvas widget
#   nurb -- the etliNurbs command object
#   resolution -- number of line segment on NURBS curve
#   id -- nurbs unique id 
# Results:
#   Returns NULL.
# 
proc DrawNurbs {canv nurb resolution id} {
    global Nurbs
    set t 0.0
    set dt [expr 1.0 / $resolution]    
    set pp [$nurb info -coords $t]
    set pp [lrange $pp 0 1]
    set pp [Viewport2Canv $pp]
    set nurbsTag  [list nurbsCurve$id NURBSCURVE]
    for {set i 0} {$i < $resolution} {incr i} {
        set t [expr $t + $dt]
        set pk [$nurb info -coords $t]
        set pk [lrange $pk  0 1]
        set pk [Viewport2Canv $pk]
        lappend pp [lindex $pk 0]
        lappend pp [lindex $pk 1]
    } 
    eval $canv create line $pp -tags {$nurbsTag} -fill $Nurbs(lcolor) \
             -width $Nurbs(nurbswidth)
    return
}

proc DrawNurbsBase {id} {
    global Nurbs
    global drawArea    
    if {$Nurbs(showbase) == 1} {
       set color $Nurbs(nbasecolor)
    } else {
       set color $drawArea(bg)
    }
    set nurbsInfo [etliNurbs$id info -all]
    set nodeTag  [list nurbsBase$id NURBSBASE]
    set startNodeCoords  [Viewport2Canv [lrange $nurbsInfo 3 4]]
    set endNodeCoords  [Viewport2Canv [lrange $nurbsInfo 15 16]]
    
    eval $drawArea(cv) create line $startNodeCoords $endNodeCoords \
         -tags {$nodeTag} -fill $color -width 2
    $drawArea(cv) lower nurbsBase$id 
    if {$drawArea(baseimage) != ""} {
       $drawArea(cv) raise nurbsBase$id IMAGE
    }
    if {[llength [$drawArea(cv) find withtag ELEMENT]] > 0} then {
         $drawArea(cv) raise nurbsBase$id ELEMENT
    }
     
 
    return
}
# -- DrawNurbsNode --
#
# Draws nurbs node circle
#
# Arguments:
#    x, y -- node coordinates
# Results:
#    Returns NULL.
#
proc DrawNurbsNode {x y} {
    global Nurbs
    global drawArea    
    set nodeTag  [list nurbsNode$Nurbs(id).$Nurbs(nnodes) NURBSNODE]
    if {$Nurbs(shownodes) == 1} {
       set color $Nurbs(ncurcolor)
    } else {
       set color $drawArea(bg)
    }
    DrawVertex $drawArea(cv) $x $y $Nurbs(nradius) $color \
            $nodeTag
    return
}

proc RedrawNurbsNodes {id} {
    global Nurbs
    global drawArea    
    if {$Nurbs(shownodes) == 1} {
       set color $Nurbs(ncurcolor)
    } else {
       set color $drawArea(bg)
    }
    set nurbsInfo [etliNurbs$id info -all]
    set nodeTag  [list nurbsNode$id.1 NURBSNODE]
    set nodeCoords  [Viewport2Canv [lrange $nurbsInfo 3 4]]
    eval DrawVertex $drawArea(cv) $nodeCoords $Nurbs(nradius) $color {$nodeTag}
    $drawArea(cv) lower nurbsNode$id.1 
        if {$drawArea(baseimage) != ""} {
              $drawArea(cv) raise nurbsNode$id.1 IMAGE 
        }

    set nodeTag  [list nurbsNode$id.2 NURBSNODE]
    set nodeCoords  [Viewport2Canv [lrange $nurbsInfo 7 8]]
    eval DrawVertex $drawArea(cv) $nodeCoords $Nurbs(nradius) $color {$nodeTag}
    $drawArea(cv) lower nurbsNode$id.2 
        if {$drawArea(baseimage) != ""} {
              $drawArea(cv) raise nurbsNode$id.2 IMAGE 
        }

    set nodeTag  [list nurbsNode$id.3 NURBSNODE]
    set nodeCoords  [Viewport2Canv [lrange $nurbsInfo 11 12]]
    eval DrawVertex $drawArea(cv) $nodeCoords $Nurbs(nradius) $color {$nodeTag}
    $drawArea(cv) lower nurbsNode$id.3 
        if {$drawArea(baseimage) != ""} {
              $drawArea(cv) raise nurbsNode$id.3 IMAGE 
        }

    set nodeTag  [list nurbsNode$id.4 NURBSNODE]
    set nodeCoords  [Viewport2Canv [lrange $nurbsInfo 15 16]]
    eval DrawVertex $drawArea(cv) $nodeCoords $Nurbs(nradius) $color {$nodeTag}
    $drawArea(cv) lower nurbsNode$id.4 
        if {$drawArea(baseimage) != ""} {
              $drawArea(cv) raise nurbsNode$id.4 IMAGE 
        }
 
    return
}

proc DrawNurbsSkel {id} {
    global Nurbs
    global drawArea
    if {$Nurbs(showskel) == 1} {
       set color $Nurbs(scolor)
    } else {
       set color $drawArea(bg)
    }

    if {$Nurbs(nnodes) == 2} {
       set skelCoords [join $Nurbs(skel) " "]    
       set skelTags [list NSKEL ns$id]    
       eval $drawArea(cv) create line $skelCoords \
                -fill $color \
                -tags {$skelTags} 
        $drawArea(cv) lower NSKEL 
        if {$drawArea(baseimage) != ""} {
              $drawArea(cv) raise NSKEL IMAGE 
        }
        if {$Nurbs(showskel) == 1} {
          catch { $drawArea(cv) raise NSKEL ELEMENT }
        }
    } elseif {$Nurbs(nnodes) > 2} {
        set skelCoords [join $Nurbs(skel) " "]    
        eval $drawArea(cv) coords ns$id  $skelCoords 
        $drawArea(cv) itemconfigure ns$id -fill $color         
        $drawArea(cv) lower NSKEL 
        if {$drawArea(baseimage) != ""} {
              $drawArea(cv) raise NSKEL IMAGE 
        }
        if {$Nurbs(showskel) == 1} {
          catch { $drawArea(cv) raise NSKEL ELEMENT }
        }

    }
    return 
}

proc RedrawNurbsSkel {id} {
    global drawArea
    global Nurbs
    if {$Nurbs(showskel) == 1} {
       set color $Nurbs(scolor)
    } else {
       set color $drawArea(bg)
    }
    set nurbsInfo [etliNurbs$id info -all]
    set skelCoords {} 
    set skelCoords [concat $skelCoords [Viewport2Canv [lrange $nurbsInfo 3 4]]] 
    set skelCoords [concat $skelCoords [Viewport2Canv [lrange $nurbsInfo 7 8]]]
   set skelCoords [concat $skelCoords [Viewport2Canv [lrange $nurbsInfo 11 12]]]
   set skelCoords [concat $skelCoords [Viewport2Canv [lrange $nurbsInfo 15 16]]]
    
    set skelTags [list NSKEL ns$id]    
    eval $drawArea(cv) create line $skelCoords \
                -fill $color \
                -tags {$skelTags} 
    $drawArea(cv) lower NSKEL 
    if {$drawArea(baseimage) != ""} {
         $drawArea(cv) raise NSKEL IMAGE 
    }
    if {$Nurbs(showskel) == 1} {
          catch {$drawArea(cv) raise NSKEL ELEMENT}
        }

    return 
}

proc ShowNurbsNodes {flag} {
    global drawArea
    global Nurbs

    if {$flag == 1} {
        $drawArea(cv) itemconfigure NURBSNODE -fill $Nurbs(ncurcolor) \
                -outline $Nurbs(ncurcolor)
        $drawArea(cv) raise NURBSNODE
    } else {
        $drawArea(cv) itemconfigure NURBSNODE -fill $drawArea(bg) \
                -outline $drawArea(bg)
        $drawArea(cv) lower NURBSNODE
    }
    update idletask
    return
}

proc ShowNurbsNodesNumbers {flag} {
    global drawArea
    global Nurbs
    if {$Nurbs(shownodesnum) == 1} {
        set color $drawArea(numberscolor)
    } else {
        set color $drawArea(bg)
    }
    if {$flag == 1} {
       foreach nurbs $Nurbs(nurbslist) {
           for {set i 1} {$i<=4} {incr i} {
              set pp [lrange [etliNurbs$nurbs info -ctrlpoint $i] 0 1]
              set pp [Viewport2Canv $pp]
              set x [expr [lindex $pp 0] + $Nurbs(numoffset)]
              set y [expr [lindex $pp 1] + $Nurbs(numoffset)]
              set number $nurbs
              set tags [list NURBSNODENUM  nnu$number.$i]
              eval $drawArea(cv) create text $x $y \
                      -fill $color \
                      -text $i -tags {$tags} 
           }
       }
    } else {
        eval $drawArea(cv) delete NURBSNODENUM
    }
    update idletask
    return
}   

proc ShowNurbsNumbers {flag} {
    global drawArea
    global Nurbs

    if {$flag == 1} {
       foreach nurbs $Nurbs(nurbslist) {
           set pp [lrange [etliNurbs$nurbs info -coords 0.5] 0 1]
           set pp [Viewport2Canv $pp]
           set tagList [list NURBSNUM nurbsNum$nurbs]
           set x [ expr [lindex $pp 0] + 5]
           set y [ expr [lindex $pp 1] + 5] 
           eval $drawArea(cv) create text $x $y -text $nurbs -tags {$tagList}
       }
    } else {
        eval $drawArea(cv) delete NURBSNUM
    }
    return
}




proc ShowNurbsSkel {flag} {
    global drawArea
    global Nurbs

    if {$flag == 1} {
        $drawArea(cv) itemconfigure NSKEL -fill $Nurbs(scolor) 
        $drawArea(cv) raise NSKEL 
    } else {
        $drawArea(cv) itemconfigure NSKEL -fill $drawArea(bg) 
        $drawArea(cv) lower NSKEL
    }
    return
}

proc UpdateNurbs {id} {
    global Nurbs
    global drawArea
    $drawArea(cv) delete nurbsCurve$id
    DrawNurbs $drawArea(cv) etliNurbs$id 40 $id
    return
}

proc UpdateNurbsNodes {id node} {
    global Nurbs
    global drawArea

    $drawArea(cv) delete nurbsNode$id.$node
    set pp [lrange [etliNurbs$id info -ctrlpoint $node] 0 1]
    set pp [Viewport2Canv $pp]
    set x [lindex $pp 0]
    set y [lindex $pp 1]
    set nodeTag  [list nurbsNode$id.$node NURBSNODE]
    if {$Nurbs(shownodes) == 1} {
       set color $Nurbs(ncurcolor)
    } else {
       set color $drawArea(bg)
    }
    DrawVertex $drawArea(cv) $x $y $Nurbs(nradius) $color \
            $nodeTag
    if {$Nurbs(shownodesnum) == 1} {
        set x [expr $x + $Nurbs(numoffset)]
        set y [expr $y + $Nurbs(numoffset)]
        $drawArea(cv) coords nnu$id.$node $x $y
    }        
    UpdateNurbsNumber $Nurbs(shownnum) $id
}

proc UpdateNurbsSkel {id node} {
    global drawArea    

    set coords [$drawArea(cv) coords ns$id]
    set pp [lrange [etliNurbs$id info -ctrlpoint $node] 0 1]
    set pp [Viewport2Canv $pp]
    set x [lindex $pp 0]
    set y [lindex $pp 1]
    set ii [expr 2*($node-1)]
    set iii [expr $ii + 1]
    set coords [lreplace $coords $ii $iii $x $y]
    eval $drawArea(cv) coords ns$id $coords
    return 
}


proc UpdateNurbsNumber {flag id} {
    global drawArea
    global Nurbs

    if {$flag == 1} {
        set pp [lrange [etliNurbs$id info -coords 0.5] 0 1]
        set pp [Viewport2Canv $pp]
        set x [ expr [lindex $pp 0] + 5]
        set y [ expr [lindex $pp 1] + 5] 
        eval $drawArea(cv) coords nurbsNum$id $x $y
    } 
    return
}

proc RedrawNurbs {} {
    global Nurbs
    foreach id $Nurbs(nurbslist) {
        RedrawNurbsSkel $id      
        RedrawNurbsNodes $id
        UpdateNurbs $id
        ShowNurbsNodesNumbers $Nurbs(shownodesnum)
        ShowNurbsNumbers $Nurbs(shownnum)

        DrawNurbsBase $id
    }
    return
}

proc ShowNurbsBase {flag} {
    global drawArea
    global Nurbs

    if {$flag == 1} {
       catch {
        $drawArea(cv) itemconfigure NURBSBASE -fill $Nurbs(nbasecolor) 
        $drawArea(cv) raise NURBSBASE ELEMENT
      }
    } else {
     catch {
        $drawArea(cv) itemconfigure NURBSBASE -fill $drawArea(bg) 
        $drawArea(cv) lower NURBSBASE
      }
    }
    return
}


#------------------ MESH NODES

proc EditMeshNodeAction {} {
    global Entries
    global Mesh
    set status [expr $Entries(nodenum) * $Entries(xcoord) * $Entries(ycoord) \
                    * $Entries(meshparam)]

    if {$status != 1} {
        SetErrorMessage "Node parameters not specified correctly"
        return
    }
    set meshparam [.meshnodebox.input.meshparament get] 
    set x [.meshnodebox.input.xcoordent get] 
    set y [.meshnodebox.input.ycoordent get] 

    if {[string equal [winfo class .meshnodebox.input.nodenument] "Entry"]} {
       set nodenum [.meshnodebox.input.nodenument get] 
    } else {
      set nodenum [.meshnodebox.input.nodenument cget -text]
    }


    set nodeindex [expr $nodenum -1]
    set Mesh(nodemeshparam) [lreplace $Mesh(nodemeshparam) \
             $nodeindex $nodeindex $meshparam]
    MoveMeshNode $nodenum $x $y      
}

# create mesh node -- add it to mesh and draw it on the canvas
# returns number of created node
proc CreateMeshNode {x y {view_or_canv 1}} {
    global Mesh
    if {$view_or_canv == 0} then {
       set pp [Canv2Viewport [list $x $y]]
    } else {
      set pv [Viewport2Canv [list $x $y]]
      set pp [list $x $y]
      set x [lindex $pv 0]
      set y [lindex $pv 1]
    }
    lappend  Mesh(nodecoords) $pp
    lappend  Mesh(nodemeshparam) 1.0
    lappend  Mesh(nodenurbs) {}
    incr Mesh(nnodes)   
    DrawMeshNode $x $y $Mesh(nnodes)
  return $Mesh(nnodes)
}

# User subroutine for creating new node 
proc node {x y {mesh_param 1.0}} {
    global Mesh
    set pv [Viewport2Canv [list $x $y]]
    set pp [list $x $y]
    set x [lindex $pv 0]
    set y [lindex $pv 1]
    lappend  Mesh(nodecoords) $pp
    lappend  Mesh(nodemeshparam) $mesh_param 
    lappend  Mesh(nodenurbs) {}
    incr Mesh(nnodes)   
    DrawMeshNode $x $y $Mesh(nnodes)

    .nvCommand.commandEntry delete 0 end
    .nvCommand.commandEntry icursor 0
  return $Mesh(nnodes)
}


# draw node
proc DrawMeshNode {x y nodeNum} {
    global Mesh
    global drawArea    
      
    set nodeTag meshNode$nodeNum 
    set radius $drawArea(nradius)
    if { $Mesh(shownodes) == 1 } then { 
       set color $drawArea(ncolor)
    } else { 
       set color $drawArea(bg)
    }
    DrawMeshNodeNumber $x $y $nodeNum $nodeTag
    lappend nodeTag MESHNODE
    DrawVertex  $drawArea(cv) $x $y  $radius $color \
            $nodeTag   
    return
}

proc RedrawMeshNodes {} {
    global Mesh
    global drawArea

    set i 0
    foreach pp $Mesh(nodecoords) {
       set pc [Viewport2Canv $pp]
       incr i
       eval DrawMeshNode $pc $i
    }
    return 
}

# draw node number
proc DrawMeshNodeNumber {atX atY number tagList} {
   global drawArea
   global Mesh 
   set tx [expr $atX + $drawArea(ntxo)]   
   set ty [expr $atY + $drawArea(ntyo)]   
   if {$Mesh(shownodesnum) == 1} {
        set color $drawArea(numberscolor)
   } else {
        set color $drawArea(bg)
   }
   lappend tagList MESHNODENUM
   $drawArea(cv) create text $tx $ty \
            -text $number \
            -font $drawArea(tfont) \
            -fill $color \
            -tag $tagList\
            -anchor nw
}

proc MoveMeshNode {nodeNum x y} {
    global Mesh
    set nodeIndex [expr $nodeNum -1]
    set nurbsList [lindex $Mesh(nodenurbs) $nodeIndex]
    foreach nurbs $nurbsList {
         set nurbsNum [lindex $nurbs 0]
         set nurbsNode [expr 3*[lindex $nurbs 1] + 1]
         SetNurbsCoords $nurbsNum $nurbsNode $x $y
    } 
    set p [list $x $y]
    set Mesh(nodecoords) [lreplace $Mesh(nodecoords) $nodeIndex $nodeIndex \
                 $p]
    RedrawCanvas
    return
}

proc ShowMeshNodes {flag} {
    global drawArea

    if {$flag == 1} {
       catch {
        $drawArea(cv) itemconfigure MESHNODE -fill $drawArea(ncolor) \
                -outline $drawArea(ncolor)
        $drawArea(cv) raise MESHNODE ELEMENT
      }
    } else {
     catch {
        $drawArea(cv) itemconfigure MESHNODE -fill $drawArea(bg) \
                -outline $drawArea(bg)
        $drawArea(cv) lower MESHNODE
      }
    }
    return
}

proc ShowMeshNodesNumbers {flag} {
    global drawArea
    global Mesh
    if {$flag == 1} {
      catch {
        $drawArea(cv) itemconfigure MESHNODENUM -fill $drawArea(numberscolor) 
        $drawArea(cv) raise MESHNODENUM 
    }
    } else {
      catch {
        $drawArea(cv) itemconfigure MESHNODENUM -fill $drawArea(bg) 
        $drawArea(cv) lower MESHNODENUM
       }
    }
    return
} 

#------------------  ELEMENTS
proc DrawElement {id} {
    global drawArea
    global Mesh

    set elemTag e$id
    lappend elemTag "ELEMENT"
    
    if {$Mesh(nvertex) != 2} {
        set el [DrawTorQElem $Mesh(selnodes) $elemTag]
        $drawArea(cv) lower $el
        if {$drawArea(baseimage) != ""} {
            $drawArea(cv) raise $el IMAGE
        }
    } else {
       set  el [DrawLinkElem $Mesh(selnodes) $elemTag]
       $drawArea(cv) lower $el
        if {$drawArea(baseimage) != ""} {
            $drawArea(cv) raise $el IMAGE
        }
    }
    set center [CalcGravityCenter $Mesh(selnodes)]
    set cx [lindex $center 0]
    set cy [lindex $center 1]
    
    DrawElementNumber $cx $cy $id $elemTag 
}

proc DrawLinkElem {pointList elemTag} {
    global drawArea
    global Mesh
    lappend elemTag "LINK"
    set pointList [join $pointList " "]
    if { $Mesh(showelems) == 0} then {
        set color $drawArea(linkcolor)
    } else {
         set color $drawArea(bg)
    }
    set  el [eval $drawArea(cv) create line $pointList\
        -fill $drawArea(linkcolor) \
        -tag [list $elemTag] \
        -width $drawArea(linkwidth) ]
    return $el
}

proc DrawTorQElem {pointList elemTag} {
    global drawArea
    lappend elemTag "PLANAR"
    if { [llength $pointList] == 4} then {
       lappend elemTag "QUAD"
    } else {
       lappend elemTag "TRIANG"
    }
    set pointList [join $pointList " "]
    set el [eval $drawArea(cv) create polygon $pointList \
        -fill $drawArea(elcolor) \
        -tag [list $elemTag] \
        -outline $drawArea(elemoutline)]

    return $el
}

proc RedrawElements {} {
#    redraw link elements 
     RedrawElemGeneric 2
#    redraw triang elements
     RedrawElemGeneric 3
#    redraw quad elements
     RedrawElemGeneric 4
    return
}

proc RedrawElemGeneric {type} {
    global Mesh
    set i 0
    set Mesh(nvertex) $type
    foreach elem $Mesh(Nodes$type) {
      incr i
      set $Mesh(selnodnum) $elem
      foreach node $elem {
         set n [expr $node -1]
         lappend Mesh(selnodes) \
                [Viewport2Canv [lindex $Mesh(nodecoords) $n]]
      }
      DrawElement $i
      set Mesh(selnodes) {}
      set Mesh(selnodnum) {}
    }
    return
}


# draw element number
proc DrawElementNumber {atX atY number tagList} {
   global drawArea
   lappend tagList ELEMNUM
   $drawArea(cv) create text $atX $atY \
            -text $number \
            -font $drawArea(tfont) \
            -fill $drawArea(numberscolor) \
            -tag $tagList\
            -anchor c 
}

proc ShowMeshElems {flag} {
    global drawArea

    if {$flag == 1} {
       catch {
        $drawArea(cv) itemconfigure LINK  -fill $drawArea(linkcolor) 
        $drawArea(cv) itemconfigure TRIANG  -fill $drawArea(elcolor) \
                -outline $drawArea(elemoutline) 
        $drawArea(cv) itemconfigure QUAD  -fill $drawArea(elcolor) \
                -outline $drawArea(elemoutline)
        $drawArea(cv) raise  QUAD MESHNODE
        $drawArea(cv) raise  TRIANG QUAD
        $drawArea(cv) raise  LINK TRIANG
      }
    } else {
     catch {
        $drawArea(cv) itemconfigure LINK  -fill $drawArea(bg) 
        $drawArea(cv) itemconfigure TRIANG  -fill $drawArea(bg) \
                -outline  $drawArea(bg)
        $drawArea(cv) itemconfigure QUAD  -fill $drawArea(bg) \
                -outline  $drawArea(bg)

        $drawArea(cv) lower ELEMENT 
        if {$drawArea(baseimage) != "" } {
           $drawArea(cv) raise ELEMENT IMAGE
        }
      }
    }
    return
}

proc ShowMeshElemsNumbers {flag} {
    global drawArea

    if {$flag == 1} {
        $drawArea(cv) itemconfigure ELEMNUM -fill $drawArea(numberscolor) 
        $drawArea(cv) raise ELEMNUM 
    } else {
        $drawArea(cv) itemconfigure ELEMNUM -fill $drawArea(bg) 
        $drawArea(cv) lower ELEMNUM 
    }
    return
} 

#------------------ MESH
proc RedrawMesh {} {
    global Mesh

# quick hack to preserve element mode
    set nodemode $Mesh(nvertex)
    RedrawMeshNodes
    RedrawElements
    ShowMeshElems $Mesh(showelems)
    ShowMeshElemsNumbers $Mesh(showelemsnum)
    ShowMeshNodes $Mesh(shownodes)
    ShowMeshNodesNumbers $Mesh(shownodesnum)
    set Mesh(nvertex) $nodemode
}
#------------------  ZOOM
proc DrawZoomWindow {x y} {
    global drawArea
    global zoomMode
    set pp [list $x $y]
    set zoomMode(x) $x
    set zoomMode(y) $y     
    eval $drawArea(cv) create line $pp $pp $pp $pp $pp -fill gray \
                       -tags "zoom_window" -width 1
    bind $drawArea(cv) <Motion> {+
            UpdateZoomWindow %x %y      
    }
    return
}

proc UpdateZoomWindow {x y} {
    global drawArea
    global zoomMode
 
    set p1 [list $zoomMode(x) $zoomMode(y)]
    set p2 [list $x $zoomMode(y)]
    set p3 [list $x $y]
    set p4 [list $zoomMode(x) $y]
    eval $drawArea(cv) coords zoom_window $p1 $p2 $p3 $p4 $p1
    return
}

proc DrawZoomRegionWindow {x y} {
    global drawArea
    global zoomMode
#    if {$x > [expr $drawArea(regionWidth) * $drawArea(currentRegionCol)]} {
#        incr drawArea(currentRegionCol)
#    }
#    if {$x < [expr $drawArea(regionWidth) *($drawArea(currentRegionCol)-1)]} {
#        incr drawArea(currentRegionCol) -1
#    }
#    if {$y > [expr $drawArea(regionHeight) * $drawArea(currentRegionRow)]} {
#        incr drawArea(currentRegionRow)
#    }
#    if {$y < [expr $drawArea(regionHeight) *($drawArea(currentRegionRow)-1)]} {
#        incr drawArea(currentRegionRow) -1
#    }

    
    set xl  [expr int($x - $drawArea(regionWidth) * 0.5) ]
    set xh  [expr int($x + $drawArea(regionWidth) * 0.5) ] 
    set yl  [expr int($y - $drawArea(regionHeight) * 0.5) ]
    set yh  [expr int($y + $drawArea(regionHeight) * 0.5)]

    if { $xl < 0 } { 
      set xl 0 
      set xh $drawArea(regionWidth)
    }
    if { $yl < 0 } { 
      set yl 0
      set yh $drawArea(regionHeight)
    }
    if { $xh > $drawArea(width) } { 
      set xl [expr $drawArea(width) - $drawArea(regionWidth)] 
      set xh $drawArea(width)
    }
    if { $yh > $drawArea(height) } { 
      set yl [expr  $drawArea(height) - $drawArea(regionHeight)]
      set yh $drawArea(height)
    }


    set p1 [list $xl $yl]
    set p2 [list $xh $yl]
    set p3 [list $xh $yh]
    set p4 [list $xl $yh]

    eval $drawArea(cv) create line $p1 $p2 $p3 $p4 $p1 -fill gray \
                       -tags "zoom_window" -width 1

    set zoomMode(regioncoords) [list $xl $yl $xh $yh] 

    bind $drawArea(cv) <Motion> {+
            MoveZoomRegionWindow %x %y      
    }
    return
}

#----------- CURSOR POSITION
proc InitCursorPos {x y} {
    global drawArea
    set drawArea(pos) [list [$drawArea(cv) canvasx $x $drawArea(xsnap)] \
             [$drawArea(cv) canvasy $y $drawArea(ysnap)]]
    return
}

#----------- AXES
proc MoveAxes {x y} {
    global drawArea
    set Pos [list [$drawArea(cv) canvasx $x $drawArea(xsnap)] \
             [$drawArea(cv) canvasy $y $drawArea(ysnap)]]
    set XPos [lindex $Pos 0] 
    set YPos [lindex $Pos 1]
    set XOldPos [lindex $drawArea(pos) 0]
    set YOldPos [lindex $drawArea(pos) 1]
    set XPosDiff [expr abs($XPos - $XOldPos)]
    set YPosDiff [expr abs($YPos - $YOldPos)]
    if { $YPosDiff >= $drawArea(ysnap) } {
        set drawArea(pos) [list [lindex $drawArea(pos) 0] $YPos] 
        set p1 [list [lindex $drawArea(canv) 0] $YPos]
        set p2 [list [lindex $drawArea(canv) 2] $YPos]
        eval $drawArea(cv) coords xaxis $p1 $p2
        $drawArea(cv) raise xaxis
    }
    if { $XPosDiff >= $drawArea(xsnap) } {
        set drawArea(pos) [list $XPos [lindex $drawArea(pos) 1]] 
        set p1 [list $XPos [lindex $drawArea(canv) 1]]
        set p2 [list $XPos [lindex $drawArea(canv) 3]]
        eval $drawArea(cv) coords yaxis $p1 $p2
        $drawArea(cv) raise yaxis
    }
    return
}

proc DrawAxes {} {
    global drawArea 
    if {$drawArea(showaxes) == 1} {
    set x1 [list [lindex $drawArea(canv) 0] $drawArea(height)]
    set x2 [list [lindex $drawArea(canv) 2] $drawArea(height)]
    set y1 [list 0 [lindex $drawArea(canv) 1]]
    set y2 [list 0 [lindex $drawArea(canv) 3]]
     
    eval $drawArea(cv) create line $x1 $x2 -fill \
              $drawArea(axescolor)  -tags ax \
            -tags "xaxis" -width 1
    eval $drawArea(cv) create line $y1 $y2 -fill \
            $drawArea(axescolor) -tags ax \
            -tags "yaxis" -width 1
     update
    } else {
       $drawArea(cv) delete  "xaxis"
       $drawArea(cv) delete  "yaxis"
    }
    
    return
}

# DeleteAxes  -- delete axes from canvas
# Arguments:
#    NONE
# Return value:
#    NONE
proc DeleteAxes {} {
    global drawArea
    $drawArea(cv) delete ax
    $drawArea(cv) delete yaxis 
    $drawArea(cv) delete xaxis 
    return
}



# Procedure: CalcCircle
#       Return circle bounding box coordinates
proc CalcCircle { atX atY radius} {
    set left   [expr $atX - $radius]
    set top    [expr $atY + $radius]
    set right  [expr $atX + $radius]
    set bottom [expr $atY - $radius]

    return "$left $top $right $bottom"
}


#----------- CANVAS MARKS

# Procedure: MreateMark
proc CreateMark {} {
    image create bitmap MARK -data "    
       #define mark_width 7
       #define mark_height 7
       static unsigned char mark_bits[] = {
       0x08, 0x08, 0x08, 0x7f, 0x08, 0x08, 0x08}; "
    return
}


# Procedure: DrawMark
proc DrawMark { canvName atX atY tagList} {
    lappend tagList Mark 
    $canvName create image $atX $atY -image MARK -tags $tagList 
    return
}


# Procedure: DrawVertex
proc DrawVertex { canvName atX atY radius color tagList} {
    set left   [expr $atX - $radius]
    set top    [expr $atY + $radius]
    set right  [expr $atX + $radius]
    set bottom [expr $atY - $radius]
    lappend tagList VERTEX 
    $canvName create oval $left $bottom $right $top \
            -fill $color \
            -outline $color \
            -tag $tagList 
    return
}

# Procedure: DrawGhostVertex
proc DrawGhostVertex {atX atY} {
    global drawArea
    set radius 5
    set left   [expr $atX - $radius]
    set top    [expr $atY + $radius]
    set right  [expr $atX + $radius]
    set bottom [expr $atY - $radius]
    $drawArea(cv) create oval $left $bottom $right $top \
            -outline black \
            -tag GHOSTVERTEX 
    return
}

proc MoveGhostVertex {atX atY} {
   global drawArea
   set radius 5
    set left   [expr $atX - $radius]
    set top    [expr $atY + $radius]
    set right  [expr $atX + $radius]
    set bottom [expr $atY - $radius]
   $drawArea(cv) coords GHOSTVERTEX $left $bottom $right $top 
}

proc DeleteGhostVertex {} {
   global drawArea
   $drawArea(cv) delete GHOSTVERTEX
}

# procedure: MoveVertex 
proc MoveVertex {number x y} {
   global drawArea
   global Mesh
   set n [expr $number - 1]
   set ncoords [lindex $Mesh(nodecoords) $n]
   set nx [lindex $ncoords 0]
   set ny [lindex $ncoords 1]
   set nx [expr $nx + $x]
   set ny [expr $ny + $y]
   set Mesh(nodecoords) [lreplace $Mesh(nodecoords) $n $n [list $nx $ny]]
   $drawArea(cv) move meshNode$number $x $y
}


# Procedure: GiveOverlappingItems
proc GiveOverlappingItems { canvName itemTag x y dx dy} {
    set x1 [expr $x - $dx]
    set x2 [expr $x + $dx]
    set y1 [expr $y - $dy]
    set y2 [expr $y + $dy]
    set overlappingItems {}
    set items [$canvName find overlapping $x1 $y1 $x2 $y2]
    foreach i $items {
        set tags [$canvName gettags $i]
        if {[lsearch $tags $itemTag] != -1} {
            lappend overlappingItems $i
        }    
    }
    return $overlappingItems
}

#-------------- GLOBAL CANVAS OPERATIONS
# Procedure: redrawCanvas
proc RedrawCanvas {} {
    global drawArea
    $drawArea(cv) delete Mark
    CleanCanvas
    RedrawMesh
    RedrawNurbs
}

# cleanCanvas
proc CleanCanvas {} {
    global drawArea
    $drawArea(cv) delete all
    DrawAxes
    DrawImage
}

proc DeleteMesh {} {
    global Mesh
    set Mesh(nnodes) 0
    set Mesh(nelems2) 0
    set Mesh(nelems3) 0
    set Mesh(nelems4) 0
    set Mesh(totnelems) 0
    set Mesh(Nodes2) {}
    set Mesh(Nodes3) {}
    set Mesh(Nodes4) {}
    set Mesh(nodecoords) {}
    set Mesh(nodenurbs) {}
    set Mesh(nvertex) -1
    set Mesh(nodemeshparam) {}
    set Mesh(selnodnum) 0
    set Mesh(selnodes) {}
}

proc DeleteMeshElements {} {
    global Mesh
    set Mesh(nelems2) 0
    set Mesh(nelems3) 0
    set Mesh(nelems4) 0
    set Mesh(totnelems) 0
    set Mesh(Nodes2) {}
    set Mesh(Nodes3) {}
    set Mesh(Nodes4) {}
}


proc DeleteAllNurbs {} {
    global Nurbs
    set Nurbs(nnodes) 0
    set Nurbs(def) {}
    set Nurbs(skel) {}
    set Nurbs(ntotal) 0
    set Nurbs(nurbslist) {}
    set Nurbs(curnurbs) -1
    set Nurbs(curnode) -1
    set Nurbs(startNode) 0
    set Nurbs(endNode) 0
    set Nurbs(showbase) 1  
    set Nurbs(nurbslist) {}
}


proc DeleteNode {} {
    global Mesh

}

# -- RemoveNurbsFromNodeAdjacency --
#
# Remove Nurbs from adjacency list of its end nodes 
#
# Arguments:
#   id - nurbs id number
# Results:
#   Returns NULL.
# Calls:
#
proc RemoveNurbsFromNodeAdjacency {id} {
    global Mesh

    set ep [expr [etliNurbs$id info -ep1] - 1]
    set epnurbsAdj [lindex $Mesh(nodenurbs) $ep]
    set s [list $id 0]
    set ix [lsearch -exact $epnurbsAdj $s]
    if {$ix >= 0} {
       set epnurbsAdj [lreplace $epnurbsAdj $ix $ix]
       set Mesh(nodenurbs) [lreplace $Mesh(nodenurbs) $ep $ep $epnurbsAdj]
    } 

    set ep [expr [etliNurbs$id info -ep2] - 1]
    set epnurbsAdj [lindex $Mesh(nodenurbs) $ep]
    set s [list $id 1]
    set ix [lsearch -exact $epnurbsAdj $s]
    if {$ix >= 0} {
       set epnurbsAdj [lreplace $epnurbsAdj $ix $ix]
       set Mesh(nodenurbs) [lreplace $Mesh(nodenurbs) $ep $ep $epnurbsAdj]
    } 
    return 
} 


proc DeleteNurbs {id} {
    global Nurbs
    global drawArea
   
    $drawArea(cv) delete nurbsCurve$id  nurbsBase$id \
                  nurbsNode$id.1 nurbsNode$id.2 \
                  nurbsNode$id.3 nurbsNode$id.4 \
                  ns$id nnu$id.1 nnu$id.2 nnu$id.3 nnu$id.4 \
                  nurbsNum$id

    set ix [ lsearch -exact $Nurbs(nurbslist) $id]

    set Nurbs(nurbslist) [lreplace $Nurbs(nurbslist) $ix $ix]

    RemoveNurbsFromNodeAdjacency $id

    etliNurbs$id delete 
    return
}

proc DeleteElement {index} {
    global Mesh

    set index [expr $index - 1]
    if { $Mesh(nelems$Mesh(nvertex)) > 0 } {
      set Mesh(Nodes$Mesh(nvertex)) [ lreplace $Mesh(Nodes$Mesh(nvertex)) $index $index ]
      set Mesh(nelems$Mesh(nvertex)) [expr $Mesh(nelems$Mesh(nvertex)) - 1]
      set Mesh(totnelems) [expr $Mesh(totnelems) - 1]
    }
    RedrawCanvas
}



proc SetLimitsAction {} {
    global Entries
    global drawArea 
    set status [expr $Entries(xlow) * $Entries(ylow) * $Entries(xhigh) \
          * $Entries(yhigh)]

    if {$status != 1} {
        SetErrorMessage "Limits not specified correctly"
        return
    }
    set xlow  [expr double ([.setlimitsbox.input.xlowent get] )]
    set ylow  [expr double ([.setlimitsbox.input.ylowent get] )]
    set xhigh [expr double ([.setlimitsbox.input.xhighent get] )] 
    set yhigh [expr double ([.setlimitsbox.input.yhighent get] )]

    set drawArea(region) [list $xlow $ylow $xhigh $yhigh]
    return 
}

proc SetRegionBoxAction {} {
    global Entries
    global drawArea 
    if {$Entries(region) != 1} {
        SetErrorMessage "Zoom region fraction not specified correctly"
        return
    }
    set drawArea(zoomRegionRatio) [.setregionbox.input.regionent get]
    $drawArea(cv) delete "zoom_window"
    SetRegionSize
    DrawZoomRegionWindow 2 2
    DestroySetRegionBox
    return 
}

#EOF


