# -- CalcTransform --
#
# Calculates coordinates transfromation factor.
# 
# Arguments:
#   NONE
# Results:
#   Result: The transformation coefficient.
#
proc CalcTransform { } {
  global drawArea  
  set width [expr [lindex $drawArea(view) 2] - [lindex $drawArea(view) 0]]
  set height [expr [lindex $drawArea(view) 3] - [lindex $drawArea(view) 1]]
  set sx [expr $drawArea(width) / $width]
  set sy [expr  $drawArea(height) / $height]
  return [expr abs($sx) < abs($sy) ? $sx : $sy]
}

# -- Viewport2Canv --
#
# Calculates point canvas coordinates for given viewport coordinates.
#
# Arguments:
#     point -- coordintates list {x, y}
# Results:
#     Returns:
#     Canvas coordinates list {xc, yc}
#
proc Viewport2Canv {point} {
  global drawArea  
  set x [lindex $point 0]
  set y [lindex $point 1]
  set xc [expr $drawArea(trans)  * ($x - [lindex $drawArea(view) 0])]
  set yc [expr -$drawArea(trans) * ($y - [lindex $drawArea(view) 1]) \
               + $drawArea(height)-1]
  return [list $xc $yc]  
}

# -- Canv2Viewport --
#
# Calculates point viewport coordinates for given canvas coordinates.
#
# Arguments:
#     point -- coordintates list {x, y}
# Results:
#     Returns:
#     Viewport coordinates list {xc, yc}
#

# return point viewport coordinates for given canvas coordinates
proc Canv2Viewport {point} {
  global drawArea  
  set x [lindex $point 0]
  set y [lindex $point 1]
  set xc [expr $x / $drawArea(trans) + [lindex $drawArea(view) 0] ]
  set yc [expr -($y - $drawArea(height) +1 ) / $drawArea(trans) \
               + [lindex $drawArea(view) 1]]
  return [list $xc $yc]
}

#EOF

proc JK {} {
   global drawArea
   puts "view $drawArea(view)"
   puts "region $drawArea(region)"
   puts "limits $drawArea(limits)"
   puts "heigh width $drawArea(height) $drawArea(width)"
   puts "trans $drawArea(trans)"
   puts "inittrans $drawArea(inittrans)"
   trace variable drawArea(trans) w CallTrace
} 

proc CallTrace {a b c} {
   puts "Call Trace $a $b $c"
   for {set x [expr [info level]-1]} { $x> 0} {incr x -1} {
          puts "$x : [info level $x]"
   }
}
