proc Limits {xl yl xh yh} {
   global drawArea
   set drawArea(region) [list $xl $yl $xh $yh]
}

proc ZoomLimits {} {
    global drawArea
    set drawArea(view) $drawArea(limits) 
    set drawArea(region) $drawArea(limits) 
    set drawArea(trans) $drawArea(inittrans) 
    
    CleanCanvas
    ZoomLimitsRegionImage
    RedrawMesh
    RedrawNurbs
}


proc Zoom {xl yl xh yh} {
    global drawArea
    global Nurbs
    
    puts "$xl $yl $xh $yh $drawArea(width) $drawArea(height)"
    if { $yl > $yh} {
        set dummy $yh
        set yh $yl
        set yl $dummy
    }
    if {$xl > $xh} {
        set dummy $xh
        set xh $xl
        set xl $dummy
    }
    set nurbList {}
    set p1 [list $xl $yh]
    set p1 [Canv2Viewport $p1]
    set p2 [list $xh $yl]
    set p2 [Canv2Viewport $p2]
    set drawArea(view) [concat $p1 $p2]
    set drawArea(trans) [CalcTransform]
    set objectsList [$drawArea(cv) find overlapping $xl $yl $xh $yh]
    foreach object $objectsList {
        set tagList [$drawArea(cv) gettags $object]
        foreach tag $tagList {
            if { [string compare -length 10 $tag "nurbsCurve"] == 0} then {
               lappend nurbList [string trim $tag "nurbsCurve" ]
            }
        }
    }

    CleanCanvas
    RedrawMesh
    foreach id $nurbList {
        RedrawNurbsSkel $id      
        RedrawNurbsNodes $id
        UpdateNurbs $id
        DrawNurbsBase $id
    }
}

# -- SetRegionSize --
#
# Calculates width and height of the zoom region rectangle
# as a fraction of width and height of the canvas size.
#
# Argument:
#    NONE
# Results:
#    Returns NULL.
#    Sets the globals: drawArea(regionWidth) and drawArea(regionHeight).
#
proc SetRegionSize {} {
    global drawArea 
     
    set drawArea(regionHeight) [expr floor(double($drawArea(height)) \
              / double($drawArea(zoomRegionRatio)))]
    set drawArea(regionWidth) [expr floor(double($drawArea(width)) \
              / double($drawArea(zoomRegionRatio)))]
    return 
}

# -- MoveZoomRegionWindow --
#
# Moves the zoom window as the cursor moves over canvas
# 
# Argumens:
#   x, y  -- canvas cursor position
# Rresults:
#   Returns NULL
#   Updates the position of the zoom window, set the globals
#   drawArea(currentRegionRow) and drawArea(currentRegionCol)
#
proc MoveZoomRegionWindow {x y} {
    global drawArea
    global zoomMode
  
#    if {$x > [expr $drawArea(regionWidth) * $drawArea(currentRegionCol)]} {
#        incr drawArea(currentRegionCol)
#    }
#    if {$x < [expr $drawArea(regionWidth) *($drawArea(currentRegionCol)-1)]} {
#        incr drawArea(currentRegionCol) -1
#    }
#    if {$y > [expr $drawArea(regionHeight) * $drawArea(currentRegionRow)]} {
#        incr drawArea(currentRegionRow)
#    }
#    if {$y < [expr $drawArea(regionHeight) *($drawArea(currentRegionRow)-1)]} {
#        incr drawArea(currentRegionRow) -1
#    }
#    set xl  [expr $drawArea(regionWidth) * ($drawArea(currentRegionCol)-1)]
#    set xh  [expr $drawArea(regionWidth) *  $drawArea(currentRegionCol)] 
#    set yl  [expr $drawArea(regionHeight) * ($drawArea(currentRegionRow)-1)]
#    set yh  [expr $drawArea(regionHeight) * $drawArea(currentRegionRow)]

    
    set xl  [expr int($x - $drawArea(regionWidth) * 0.5) ]
    set xh  [expr int($x + $drawArea(regionWidth) * 0.5) ] 
    set yl  [expr int($y - $drawArea(regionHeight) * 0.5) ]
    set yh  [expr int($y + $drawArea(regionHeight) * 0.5)]

    if { $xl < 0 } { 
      set xl 0 
      set xh $drawArea(regionWidth)
    }
    if { $yl < 0 } { 
      set yl 0
      set yh $drawArea(regionHeight)
    }
    if { $xh > $drawArea(width) } { 
      set xl [expr $drawArea(width) - $drawArea(regionWidth)] 
      set xh [expr $drawArea(width)]
    }
    if { $yh > $drawArea(height) } { 
      set yl [expr  $drawArea(height) - $drawArea(regionHeight)]
      set yh [expr $drawArea(height)]
    }

    

    set p1 [list $xl $yl]
    set p2 [list $xh $yl]
    set p3 [list $xh $yh]
    set p4 [list $xl $yh]
    set zoomMode(regioncoords) [list $xl $yl $xh $yh]
    eval $drawArea(cv) coords zoom_window $p1 $p2 $p3 $p4 $p1
}

proc ExecuteCommand {} {
    global drawArea
    global Nurbs

    set command [.nvCommand.commandEntry get]
    set error [catch {uplevel $command} result]
    if {$error != 0} {
        .nvCommand.commandInfo configure \
            -text "Error: $result"
    } else {
        .nvCommand.commandInfo configure \
            -text $result
    } 
    return
}


#I/O operations
proc SaveAsFile {} {
    global drawArea 
    set types {
        {{All files} {*}}
    }
    set filename [tk_getSaveFile -initialdir $drawArea(currentpath) \
                -initialfile $drawArea(currentbasename) \
                -filetypes $types]
    if {$filename  != ""} {
        set drawArea(currentpath) [file dirname $filename]
        set drawArea(currentbasename) [file tail $filename]
#        set extension [file extension $filename]
#        set isMdf [string compare $extension ".mdf"]
#        set isGmf  [string compare $extension ".gmf"]
#        set idDom [string compare $extension ".dom"]
#        set isMpr [string compare $extension ".mpr"]
        SaveFile 
    }
   return
}

proc SaveFile {} {
     global drawArea
     SaveMdf [file join $drawArea(currentpath) $drawArea(currentbasename)]
     SaveGmf [file join $drawArea(currentpath) $drawArea(currentbasename)]
}

proc SaveGmf {basename} {
    global Mesh
    global Nurbs
    for {set nurbsId 0} {$nurbsId < $Nurbs(id)} {incr nurbsId} {
        etliNurbs$nurbsId mesh -add
    }
 
    set nodeNum 0
    set numOfNodes [llength $Mesh(nodecoords)]
    foreach nodeCoords $Mesh(nodecoords) {
       incr nodeNum
       etli_nurbs_set_endpoints $numOfNodes $nodeNum \
                [lindex $nodeCoords 0]  [lindex $nodeCoords 1]
    }

    etli_save_nurbs $basename
    return 
}

proc SaveMdf {basename} {
    global drawArea
    global Mesh
    global Nurbs 
    set chanel [open $basename.mdf w]

    puts $chanel "TITLE  \"simple mesh\""
    puts $chanel "NMESHPOINTS $Mesh(nnodes)"
    puts $chanel "NNODES $Mesh(nnodes)\n"
    if {$Mesh(nelems2) != 0} {
      puts $chanel "NELEMENTS_LINK1 $Mesh(nelems2)"
    }
    if {$Mesh(nelems3) != 0} {
      puts $chanel "NELEMENTS_TRIANG1 $Mesh(nelems3)"
    }
    if {$Mesh(nelems4) != 0} {
      puts $chanel "NELEMENTS_QUAD1 $Mesh(nelems4)"
    }
    if {[llength $Nurbs(nurbslist)] != 0} {
      puts $chanel "NNURBS_CURVES   [llength $Nurbs(nurbslist)]"
    }

    puts $chanel "\nMESHPOINT_COORDINATES"
    set i 1
    foreach point $Mesh(nodecoords) {
        set line [format "%d\t%f\t%f\t%f" $i [lindex $point 0] \
            [lindex $point 1] 0.0]
        puts $chanel $line
        incr i    
    }

    if {$Mesh(nelems2) != 0} {
      set i 1  
      puts $chanel "\nNODES_LINK1\n"
      foreach elem $Mesh(Nodes2) {
          set line [eval \
             format {"%d\t\t%d\t%d"} $i "$elem" ]
        puts $chanel $line
        incr i
      }

    }
    if {$Mesh(nelems3) != 0} {
      set i 1  
      puts $chanel "NODES_TRIANG1\n"
       foreach elem $Mesh(Nodes3) {
          set line [eval \
             format {"%d\t\t%d\t%d\t%d"} $i "$elem" ]
        puts $chanel $line
        incr i
      }
     
    }
    if {$Mesh(nelems4) != 0} {
      set i 1  
      puts $chanel "NODES_QUAD1\n"
      foreach elem $Mesh(Nodes4) {
          set line [eval \
             format {"%d\t\t%d\t%d\t%d\t%d"} $i "$elem" ]
        puts $chanel $line
        incr i
      }
    }
    if {[llength $Nurbs(nurbslist)] > 0} { 
       puts $chanel "\n\nREMESH_DATA" 
       set i 1
       foreach point $Mesh(nodecoords) {
          set line [format "%d\t\"EXTERIOR\"" $i]
          puts $chanel $line
          set nurbscurves [lindex $Mesh(nodenurbs) [expr $i -1]]
          set numOfCurves [llength $nurbscurves]
          if {$numOfCurves != 0} {
              set line [format "\tNNURBS_CURVES\t%d" $numOfCurves]
              puts $chanel $line
              set j 1
              foreach nurbs $nurbscurves {
                  set curveId [lindex $nurbs 0]
                  set param [lindex $nurbs 1]
                  set nurbsname [etliNurbs$curveId info -name]
                  set line [format "\t%d\tNURBS_CURVE\t\"%s\"" $j $nurbsname]
                  puts $chanel $line
                  set line [format "\t\tPARAMETER %f" $param]
                  puts $chanel $line
                  incr j
              } 
           }
           incr i    
      }
    }

    close $chanel
    return
}


proc ReadMdf {basename} {
    global Mesh
    set mesh [etli_mesh]
    $mesh get -mesh $basename
    set Mesh(meshobj) $mesh

    set Mesh(nnodes) [$mesh info -NNodes]
    set Mesh(nelems2) [$mesh info -NElemsLink1]
    set Mesh(nelems3) [$mesh info -NElemsTriang1]
    set Mesh(nelems4) [$mesh info -NElemsQuad1]
    set Mesh(totnelems) [$mesh info -TotalNElems] 
    set Mesh(basename) " "
    set Mesh(nvertex) 3
    set Mesh(Nodes2) {}
    for {set i 1} {$i <= $Mesh(nelems2)} {incr i} {
       lappend Mesh(Nodes2) [$mesh info -NodesLink1 $i]
    }
    set Mesh(Nodes3)
    for {set i 1} {$i <= $Mesh(nelems3)} {incr i} {
       lappend Mesh(Nodes3) [$mesh info -NodesTriang1 $i]
    }
    set Mesh(Nodes3)
    for {set i 1} {$i <= $Mesh(nelems4)} {incr i} {
       lappend Mesh(Nodes4) [$mesh info -NodesQuad1 $i]
    }
    set Mesh(nodecoords) {}
    set Mesh(nodenurbs) {}
    set Mesh(nodemeshparam) {}
    for {set i 1} {$i <= $Mesh(nnodes)} {incr i} {
       lappend Mesh(nodecoords) [lrange [$mesh info -MeshPointCoords $i] 0 1]
       lappend Mesh(nodenurbs) {}
       lappend Mesh(nodemeshparam) 1.0
    }
}

proc ReadGmf {basename} {
    global Nurbs 
    global Mesh
    set mesh $Mesh(meshobj) 
    $mesh get -geom $basename
    
    set nnurbs [$mesh info -NNurbsCurves]
    
    for {set i 1} { $i <= $nnurbs } {incr i} {
        puts $i
        for {set j 1} { $j <= 4 } {incr j} {
          set ctrlpoint [$mesh info -NurbsCurves $i -ctrlpoint $j]
          set pp [lrange $ctrlpoint 0 1]
          set pc [Viewport2Canv $pp]
          ConstructNurbs [lindex $pc 0] [lindex $pc 1]
          puts "Constructing nurbs $i $pc"
        }
    } 
}

# print a list of tags for a given object id
proc GetTags {id} {
    global drawArea
    return [$drawArea(cv) gettags $id]
}

# print coordinates of a given object 
proc GetCoords {id} {
    global drawArea
    return [$drawArea(cv) coords $id]
}
#find an element in the list with given prefix
#removes that prefix from it and returns what has left
proc GetNumberFromTag {tagList prefix} {
    foreach i $tagList {
        if { [ regexp $prefix.* $i] == 1} then {
           set s [expr  [string last $prefix $i] + [string length $prefix]]  
           return [string range $i $s [string length $i]] 
        }
    }
     
}

# Calcutates gravity center of the given point set
proc CalcGravityCenter {pointList} {
   set cx 0
   set cy 0
   set n [llength $pointList]
   foreach pp $pointList {
       set cx [expr $cx + [lindex $pp 0]]    
       set cy [expr $cy + [lindex $pp 1]]    
   }
   set cx [expr $cx/$n]
   set cy [expr $cy/$n]
   return [list $cx $cy]  
}

# --GiveMeshNode --
#
# Return coordinates and number of mesh node selected by
# clicking at canvas position x, y
#
# Arguments:
#   x, y  -- canvas position
# Results:
#   Returns list {node {x y}} where 'node' is node nuber and 'x', 'y'
#   are its viewport coordinates.
# Calls:
#   GiveOverlappingItems, GetNumberFromTag
#
proc GiveMeshNode {x y} {
    global drawArea
    global Mesh     
    set nodenum 0
    set coords {}
    set items [GiveOverlappingItems $drawArea(cv) \
                MESHNODE  \
                $x $y \
                $drawArea(searchwidth) $drawArea(searchheight)]
    if {[llength $items] > 0} then {
       set tagList [$drawArea(cv) gettags [lindex $items 0]]
       set nodenum [GetNumberFromTag $tagList meshNode]   
       set coords [lindex $Mesh(nodecoords) [expr $nodenum  -1]]
    return [list $nodenum $coords]
    } else {
      return {} 
    }
}


proc GiveNurbsNode {x y} {
    global drawArea
    set nodenum 0
    set nurbsnm -1 
    set items [GiveOverlappingItems $drawArea(cv) \
                NURBSNODE \
                $x $y \
                $drawArea(searchwidth) $drawArea(searchheight)]
    if {[llength $items] > 0} then {
       set tagList [$drawArea(cv) gettags [lindex $items 0]]
       puts $tagList
       set nurbs [GetNumberFromTag $tagList "nurbsNode" ]   
       set nurbs [split $nurbs "."]
       set nurbsnum  [lindex $nurbs 0 ]
       set nodenum [lindex $nurbs 1]
    }
    return [list $nurbsnum $nodenum]
}


proc AddNurbsToNode {nodeNum start_end  nurbsId} {
    global Mesh
    set nurbsItem [list $nurbsId $start_end]
    set nodeIndex [expr $nodeNum - 1]
    set nodeNurbsList [lindex $Mesh(nodenurbs) $nodeIndex]
#    set nodeNurbsList [concat $nodeNurbsList {$nurbsItem}]
    lappend  nodeNurbsList $nurbsItem
    set Mesh(nodenurbs) [lreplace $Mesh(nodenurbs) $nodeIndex $nodeIndex \
        $nodeNurbsList]
    return
}

proc NotifyEntry {entrywidget entryname entrytype entrystring} {
    global Entries
#    set entrystring  [$entrywidget get]
    if {[string is $entrytype $entrystring] != 1} {
        SetErrorMessage "Expecting $entrytype but got $entrystring"
        return 0
    }
    set Entries($entryname) [string is $entrytype -strict $entrystring]
    return 1
}

proc CommandNotImplemented {} {
    set messageText "Sorry, command not implemented :-("
    append messageText "\n\nA small donation will definitely\n"
    append messageText "speed up the development ;-)"
    notice_show $messageText 
}


#proc bgerror {error} {
#   email_bug_report_send romek@sect.mce.hw.ac.uk $error
#}

proc SetColor {ofObject} {
    global drawArea
    global Nurbs
    global Mesh
    if {[string compare $ofObject background] == 0} {
        set color $drawArea(bg)
        set new_color  [tk_chooseColor -initialcolor $color \
                 -title "Choose color"]
        if {[string equal $new_color ""] == 0} { 
            set drawArea(bg) $new_color 
        }
        $drawArea(cv) configure -bg $drawArea(bg)
    } else {
         eval set color $$ofObject
         set new_color [tk_chooseColor -initialcolor $color -title "Choose color"]
         if {[string equal $new_color ""] == 0} {
            set $ofObject $new_color
         }
    }
    RedrawCanvas
}

proc ExportPostScript {} {
    global drawArea 

    set file $drawArea(currentbasename).ps

    $drawArea(cv) postscript -file $file -colormode color
    return 
}
#EOF


