proc MakeDrawArea {parent} {
    global drawArea
    set drawArea(cv) $parent.canvas
    canvas $drawArea(cv)   -width $drawArea(width) \
                           -height $drawArea(height) \
                           -relief sunken \
                           -scrollregion $drawArea(canv) \
                           -bg $drawArea(bg) \
                           -borderwidth 0 \
                           -cursor "" 
    pack $drawArea(cv) -expand 1 \
            -side left \
            -fill both 
    return
}

proc UpdateWeightDisplay {} {
    global Nurbs
    set weight [etliNurbs$Nurbs(curnurbs) \
                    info -ctrlpoint $Nurbs(curnode)]
    set weight [lindex $weight 3]
    .nvWork.right.nurbs.weight set $weight
}

# Sets the coordinate text displayed
proc SetCoordsInfo {pp} {
    global drawArea
    set drawArea(coordText)  [format "x=%-*.*f y=%-*.*f" \
            $drawArea(ctwidth) $drawArea(ctprec) [lindex $pp 0] \
            $drawArea(ctwidth) $drawArea(ctprec) [lindex $pp 1]]
    return
}

proc SetInfoMessage {text} {
    .nvCommand.commandInfo configure  -foreground black  -text $text
    return
}

proc SetWarningMessage {text} {
    .nvCommand.commandInfo configure  -foreground magenta  -text $text
    return
}

proc SetErrorMessage {text} {
    .nvCommand.commandInfo configure  -foreground red -text $text
    return
}

proc SetNurbsEditInfo {nurbsNum nodeNum} {
    .nvWork.right.nurbs.curnurbs configure -text "Nurb: $nurbsNum" 
    .nvWork.right.nurbs.curnode configure -text "Nurb: $nodeNum" 
     return
}

proc SetWeightRange {min max} {
  .nvWork.right.nurbs.weight configure -from $min
  .nvWork.right.nurbs.weight configure -to $max
  return
}

proc ShowNurbsEditPanel {flag} {
  global drawArea
  if { $flag == 1} then {
     if { [string compare $drawArea(currentpanel) ""] != 0} {
        pack forget .nvWork.right.$drawArea(currentpanel)
     }
     set drawArea(currentpanel) "nurbs"
     pack configure .nvWork.right.nurbs \
       -fill both \
       -side left
  } else {
     pack forget .nvWork.right.nurbs
     set drawArea(currentpanel) {} 
  }
}

proc ShowEditMeshNodePanel {flag} {
  global drawArea
  if { $flag == 1} then {
     if { [string compare $drawArea(currentpanel) ""] != 0} {
        pack forget .nvWork.right.$drawArea(currentpanel)
     }
     set drawArea(currentpanel) "editmeshnode"
     pack configure .nvWork.right.editmeshnode \
       -fill both \
       -side left
  } else {
     pack forget .nvWork.right.editmeshnode \
     set drawArea(currentpanel) {} 
  }
}


proc ShowWindow {args} {
  global drawArea
  global Nurbs    
  global Mesh
  # Window manager configurations
  wm positionfrom . user
  wm sizefrom . ""
  wm maxsize . 1137 870
#  wm minsize . [expr 20+500+40+24+30]  433 
  wm title . {nurbview}


  # build widget .nvMain
  frame .nvMain \
    -borderwidth {2} \
    -relief {raised}

  # build widget .nvMain.file
  menubutton .nvMain.file \
    -menu {.nvMain.file.m} \
    -padx {5} \
    -pady {4} \
    -text {File}

  # build widget .nvMain.file.m
  menu .nvMain.file.m \
    -tearoff {0}
  .nvMain.file.m add command \
    -command {CommandNotImplemented} \
    -label {New}
  .nvMain.file.m add command \
    -command {OpenNewFile} \
    -label {Open}
  .nvMain.file.m add separator
  .nvMain.file.m add command \
    -command {SaveFile} \
    -label {Save}  
  .nvMain.file.m add command \
    -command {SaveAsFile} \
    -label {Save As}  
  .nvMain.file.m add separator
  .nvMain.file.m add cascade \
      -menu {.nvMain.file.m.readmodule} \
      -label {Read module}
  .nvMain.file.m add cascade \
      -menu {.nvMain.file.m.savemodule} \
      -label {Save module}
  .nvMain.file.m add separator
  .nvMain.file.m add cascade \
      -menu {.nvMain.file.m.export} \
      -label {Export}
  .nvMain.file.m add separator
  .nvMain.file.m add command \
    -command {ConfirmExit} \
    -label {Close}

  menu .nvMain.file.m.savemodule \
    -tearoff {0}
  .nvMain.file.m.savemodule add command \
    -command {SaveModuleMdf} \
    -label {Save mdf}
  .nvMain.file.m.savemodule add command \
    -command {SaveModuleGmf} \
    -label {Save gmf}
  .nvMain.file.m.savemodule add command \
    -command {SaveModuleMpr} \
    -label {Save mpr} \
    -state disabled 
  .nvMain.file.m.savemodule add command \
    -command {SaveModuleDom} \
    -label {Save dom} \
    -state disabled

  menu .nvMain.file.m.readmodule \
    -tearoff {0}
  .nvMain.file.m.readmodule add command \
    -command {ReadModuleGmf} \
    -label {Read gmf}
  .nvMain.file.m.readmodule add command \
    -command {ReadModuleMpr} \
    -label {Read mpr} \
    -state disabled
  .nvMain.file.m.readmodule add command \
    -command {ReadModuleDom} \
    -label {Read dom} \
    -state disabled 



  menu .nvMain.file.m.export \
    -tearoff {0}
  .nvMain.file.m.export add command \
    -command {ExportPostScript} \
    -label {Export PostScript}
  .nvMain.file.m.export add command \
    -command {CommandNotImplemented} \
    -label {Export DX} \
    -state disabled

  # build widget .nvMain.edit
  menubutton .nvMain.edit \
    -menu {.nvMain.edit.m} \
    -padx {5} \
    -pady {4} \
    -text {Edit}

  # build widget .nvMain.edit.m
  menu .nvMain.edit.m
  .nvMain.edit.m add command \
     -command {EnterEditNurbsMode} \
     -label {Edit NURBS}
  .nvMain.edit.m add command \
     -command {EnterEditMeshNodeMode} \
     -label {Edit nodes}
  .nvMain.edit.m add command \
     -command {CommandNotImplemented} \
     -label {Edit elements}
  .nvMain.edit.m add separator 
  .nvMain.edit.m add command \
     -command {EnterDeleteNurbsMode} \
     -label {Delete NURBS}   
  .nvMain.edit.m add separator 
  .nvMain.edit.m add command \
     -command {DeleteNode
               RedrawCanvas} \
     -label {Delete node}
  .nvMain.edit.m add command \
     -command {EnterDelElemMode 2} \
     -label {Delete Link1}
  .nvMain.edit.m add command \
     -command {EnterDelElemMode 3} \
     -label {Delete Triangle1}
  .nvMain.edit.m add command \
     -command {EnterDelElemMode 4} \
     -label {Delete Quad1}
  .nvMain.edit.m add separator 
  .nvMain.edit.m add command \
     -command {DeleteMeshElements
               RedrawCanvas} \
     -label {Delete all elements}


  # build widget .nvMain.draw
  menubutton .nvMain.draw \
    -menu {.nvMain.draw.m} \
    -padx {5} \
    -pady {4} \
    -text {Draw}

  # build widget .nvMain.draw.m
  menu .nvMain.draw.m \
    -tearoff {1}
  .nvMain.draw.m add command \
    -command {EnterDrawMeshNodeMode} \
    -label {Nodes}
  .nvMain.draw.m add separator
  .nvMain.draw.m add command \
     -command {EnterElementsMode 2} \
     -label {Link1}
  .nvMain.draw.m add command \
     -command {EnterElementsMode 3} \
     -label {Triangle1}      
  .nvMain.draw.m add command \
     -command {EnterElementsMode 4} \
     -label {Quad1}
  .nvMain.draw.m add separator
  .nvMain.draw.m add command \
    -command {EnterDrawNurbsMode} \
    -label {NURBS}

 
  # build widget .nvMain.draw.m.m
  menu .nvMain.draw.m.m \
    -tearoff {0}
  .nvMain.draw.m.m add command \
    -command {CommandNotImplemented} \
    -label {Center Radius}
  .nvMain.draw.m.m add command \
    -command {CommandNotImplemented} \
    -label {Center Diameter}
  .nvMain.draw.m.m add command \
    -command {CommandNotImplemented} \
    -label {3 Points}

  # build widget .nvMain.view
  menubutton .nvMain.view \
    -menu {.nvMain.view.m} \
    -padx {5} \
    -pady {4} \
    -text {View}

  # build widget .nvMain.view.m
  menu .nvMain.view.m
  .nvMain.view.m add checkbutton \
    -label {Show mesh nodes} \
    -variable {Mesh(shownodes)} \
    -onvalue 1 \
    -offvalue 0 \
    -command {ShowMeshNodes $Mesh(shownodes)}
  .nvMain.view.m add checkbutton \
    -label {Show mesh node numbers} \
    -variable {Mesh(shownodesnum)} \
    -onvalue 1 \
    -offvalue 0 \
    -command {ShowMeshNodesNumbers $Mesh(shownodesnum)}
  .nvMain.view.m add checkbutton \
    -label {Show mesh elements} \
    -variable {Mesh(showelems)} \
    -onvalue 1 \
    -offvalue 0 \
    -command {ShowMeshElems $Mesh(showelems)}
  .nvMain.view.m add checkbutton \
    -label {Show mesh element numbers} \
    -variable {Mesh(showelemsnum)} \
    -onvalue 1 \
    -offvalue 0 \
    -command {ShowMeshElemsNumbers $Mesh(showelemsnum)}

  .nvMain.view.m add separator
  .nvMain.view.m add checkbutton \
    -label {Show nurbs nodes} \
    -variable {Nurbs(shownodes)} \
    -onvalue 1 \
    -offvalue 0 \
    -command {ShowNurbsNodes $Nurbs(shownodes)}
  .nvMain.view.m add checkbutton \
      -label {Show nurbs skelleton} \
      -variable {Nurbs(showskel)} \
      -onvalue 1 \
      -offvalue 0 \
      -command {ShowNurbsSkel $Nurbs(showskel)}      
  .nvMain.view.m add checkbutton \
      -label {Show nurbs base} \
      -variable {Nurbs(showbase)} \
      -onvalue 1 \
      -offvalue 0 \
      -command {ShowNurbsBase $Nurbs(showbase)}
 .nvMain.view.m add checkbutton \
      -label {Show nurbs numbers} \
      -variable {Nurbs(shownnum)} \
      -onvalue 1 \
      -offvalue 0 \
      -command {ShowNurbsNumbers $Nurbs(shownnum)} 
 .nvMain.view.m add checkbutton \
      -label {Show control points numbers} \
      -variable {Nurbs(shownodesnum)} \
      -onvalue 1 \
      -offvalue 0 \
      -command {ShowNurbsNodesNumbers $Nurbs(shownodesnum)} 
  .nvMain.view.m add separator
  .nvMain.view.m add checkbutton \
      -label {Show axes} \
      -variable {drawArea(showaxes)} \
      -onvalue 1 \
      -offvalue 0 \
      -command {DrawAxes} 
  .nvMain.view.m add separator
  .nvMain.view.m add command \
    -command {RedrawCanvas} \
    -label {Redraw}
#  .nvMain.view.m add separator
#  .nvMain.view.m add command \
#    -command {CleanCanvas} \
#    -label {Clean Canvas}  

  # build widget .nvMain.zoom
  menubutton .nvMain.zoom \
    -menu {.nvMain.zoom.m} \
    -padx {5} \
    -pady {4} \
    -text {Zoom}

  menu .nvMain.zoom.m \
    -tearoff {1}
  .nvMain.zoom.m add command \
    -command {EnterZoomWindowMode} \
    -label {Zoom window}
  .nvMain.zoom.m add command \
    -command {EnterZoomRegionMode} \
    -label {Zoom region}
#  .nvMain.zoom.m add command \
#    -command {CommandNotImplemented} \
#    -label {Zoom extent}
  .nvMain.zoom.m add separator
  .nvMain.zoom.m add command \
    -command {ZoomLimits} \
    -label {Zoom limits}
  .nvMain.zoom.m add separator
  .nvMain.zoom.m add command \
    -command {ShowSetRegionBox} \
    -label {Zoom region size}



  # build widget .nvMain.special
  menubutton .nvMain.special\
    -menu {.nvMain.special.m} \
    -padx {5} \
    -pady {4} \
    -text {Special}

  menu .nvMain.special.m \
    -tearoff {0}
  .nvMain.special.m add command \
    -command {LoadImage} \
    -label {Load background image}
  .nvMain.special.m add command \
    -command {RemoveImage} \
    -label {Remove background image} \
    -state disable

  # build widget .nvMain.settings
  menubutton .nvMain.settings \
    -menu {.nvMain.settings.m} \
    -padx {5} \
    -pady {4} \
    -text {Settings}

  # build widget .nvMain.settings.m
  menu .nvMain.settings.m \
    -tearoff {0}
  .nvMain.settings.m add command \
    -command {CommandNotImplemented} \
    -label {Snap}
  .nvMain.settings.m add command \
    -command {CommandNotImplemented} \
    -label {Grid}
  .nvMain.settings.m add command \
    -command {ShowSetLimitsBox} \
    -label {Limits}	
  .nvMain.settings.m add separator 
  .nvMain.settings.m add cascade \
    -menu {.nvMain.settings.m.colors} \
    -label {Colors} \
    -state active
#  .nvMain.settings.m add separator 

  menu .nvMain.settings.m.colors \
    -tearoff {0}
  .nvMain.settings.m.colors add command \
    -command {SetColor drawArea(elcolor)} \
    -label {Trinag & Quad elements}
  .nvMain.settings.m.colors add command \
    -command {SetColor drawArea(linkcolor)} \
    -label {Link elements}
  .nvMain.settings.m.colors add command \
    -command {SetColor drawArea(elemoutline)} \
    -label {Elements outline}
  .nvMain.settings.m.colors add command \
    -command {SetColor Nurbs(lcolor)} \
    -label {NURBS line}
   .nvMain.settings.m.colors add separator
  .nvMain.settings.m.colors add command \
    -command {SetColor drawArea(ncolor)} \
    -label {Mesh node}
  .nvMain.settings.m.colors add command \
    -command {SetColor Nurbs(ncurcolor)} \
    -label {NURBS node}
   .nvMain.settings.m.colors add separator
   .nvMain.settings.m.colors add command \
    -command {SetColor Nurbs(scolor)} \
    -label {NURBS skeleton}
   .nvMain.settings.m.colors add command \
    -command {SetColor Nurbs(nbasecolor)} \
    -label {NURBS base}
   .nvMain.settings.m.colors add separator
   .nvMain.settings.m.colors add command \
    -command {SetColor drawArea(numberscolor)} \
    -label {Text}
   .nvMain.settings.m.colors add separator
   .nvMain.settings.m.colors add command \
    -command {SetColor background} \
    -label {Background}
 
   



  # build widget .nvMain.debug
  menubutton .nvMain.debug \
    -menu {.nvMain.debug.m} \
    -padx {5} \
    -pady {4} \
    -text {Debug}

  # build widget .nvMain.debug.m
  menu .nvMain.debug.m \
    -tearoff {0}
  .nvMain.debug.m add command \
    -command {EnterDebugTagsMode} \
    -label {DebugTags}


  # build widget .nvMain.help
  menubutton .nvMain.help \
    -menu {.nvMain.help.m} \
    -padx {5} \
    -pady {4} \
    -text {Help}

  # build widget .nvMain.help.m
  menu .nvMain.help.m

  # build widget .nvTools
  frame .nvTools \
    -borderwidth {2} \
    -relief {raised}

  # build widget .nvTools.coords
  label .nvTools.coords \
    -relief {sunken} \
    -padx 5 \
    -text {x=9  y=48} \
    -width 30 \
    -textvariable {drawArea(coordText)}

  frame .nvTools.modeFrame
      
  label .nvTools.modeFrame.modeText \
      -text {Mode:}
  label .nvTools.modeFrame.mode \
      -width 15 \
      -anchor w\
      -textvariable {drawArea(mode)}
  label .nvTools.modeFrame.separator \
      -text {   } 

  # build widget .nvTools.layer
  menubutton .nvTools.layer \
    -menu {.nvTools.layer.m} \
    -padx {5} \
    -pady {4} \
    -text {Layer}

  # build widget .nvTools.layer.m
  menu .nvTools.layer.m

  # build widget .nvTools.ortho
  checkbutton .nvTools.ortho \
    -command {CommandNotImplemented} \
    -text {Ortho} \
    -variable {checkbutton11}

  # build widget .nvTools.grid
  checkbutton .nvTools.grid \
    -command {CommandNotImplemented} \
    -text {Grid} \
    -variable {canvasGrid}

  # build widget .nvTools.snap
  checkbutton .nvTools.snap \
    -text {Snap} \
    -variable {checkbutton14}

  # build widget .nvWork
  frame .nvWork \
    -borderwidth {2}

  # build widget .nvWork.left
  frame .nvWork.left \
    -borderwidth {2} \
    -relief {raised} \
    -width {20}

  # build widget .nvWork.right
  frame .nvWork.right \
    -width {75} \
    -height {300}

   
  # build widget .nvWork.right.nurbs
  frame .nvWork.right.nurbs 

  label .nvWork.right.nurbs.curnurbs \
      -text {Nurbs:}
  label .nvWork.right.nurbs.curnode \
      -text {Node:}
  scale .nvWork.right.nurbs.weight \
      -orient vertical \
      -from 0.01 \
      -to 1.0 \
      -resolution 0.01 \
      -command SetNurbsNodeWeight \
      -state normal \
      -highlightbackground yellow \
      -activebackground red

  button .nvWork.right.nurbs.edit \
      -text {Edit} \
      -command {CommandNotImplemented}


  # build widget .nvWork.right.editmeshnode
  frame .nvWork.right.editmeshnode \

  button .nvWork.right.editmeshnode.edit \
    -text {Edit} \
    -command {ShowEditMeshNodeBox}

  # build widget .nvCommand
  frame .nvCommand \
    -borderwidth {2}

  # build widget .nvCommand.commandInfo
  label .nvCommand.commandInfo \
    -anchor {w} \
    -text {Enter command}

  # build widget .nvCommand.commandEntry
  entry .nvCommand.commandEntry

  # pack master .nvMain
  pack configure .nvMain.file \
    -anchor nw \
    -side left
  pack configure .nvMain.edit \
    -anchor nw \
    -side left
  pack configure .nvMain.draw \
    -anchor nw \
    -side left
  pack configure .nvMain.view \
    -anchor nw \
    -side left
  pack configure .nvMain.settings \
    -anchor nw \
    -side left
  pack configure .nvMain.zoom \
    -anchor nw \
    -side left
  pack configure .nvMain.special \
    -anchor nw \
    -side left
  pack configure .nvMain.help \
    -anchor ne \
    -side right

  # pack master .nvTools
  pack configure .nvTools.coords \
    -anchor ne \
    -side right
  pack configure .nvTools.modeFrame \
    -side right \
    -anchor w   
  pack configure .nvTools.modeFrame.mode \
    -side right \
    -anchor w
  pack configure .nvTools.modeFrame.modeText \
    -side right \
    -anchor w
  pack configure .nvTools.modeFrame.separator 
  pack configure .nvTools.layer \
    -side left
  pack configure .nvTools.layer \
    -side left
  pack configure .nvTools.ortho \
    -side left
  pack configure .nvTools.grid \
    -side left
  pack configure .nvTools.snap \
    -side left

  # pack master .nvWork
  pack configure .nvWork.left \
    -fill y \
    -side left
    
  MakeDrawArea .nvWork  


  pack propagate .nvWork.right 0 
  pack configure .nvWork.right  \
      -fill both


  pack configure .nvWork.right.nurbs.curnurbs
  pack configure .nvWork.right.nurbs.curnode
  pack configure .nvWork.right.nurbs.weight \
     -side top
  pack configure .nvWork.right.nurbs.edit

# pack and configure panel for nodes editing
  pack configure .nvWork.right.editmeshnode.edit \
     -fill x

  # pack master .nvCommand
  pack configure .nvCommand.commandInfo \
    -anchor nw \
    -fill x
  pack configure .nvCommand.commandEntry \
    -fill x

  # pack master .
  pack configure .nvMain \
    -fill x
  pack configure .nvTools \
    -fill x
  pack configure .nvWork \
    -expand 1 \
    -fill both
  pack configure .nvCommand \
    -fill x

}

proc ShowEditMeshNodeBox {{nodenum 0}} {

  .nvWork.right.editmeshnode.edit configure \
        -state disable

  toplevel .meshnodebox \
       -borderwidth 10
  
  wm positionfrom .meshnodebox "program" 
  wm title .meshnodebox {nurbsview : Edit Mesh Node}
  frame .meshnodebox.input
  frame .meshnodebox.separ \
        -relief ridge \
        -bd 1 \
        -height 2
  frame .meshnodebox.accept \

  label .meshnodebox.input.nodenumlab \
        -text {node nuber} \
        -width 15 \
        -anchor e
  label .meshnodebox.input.xcoordlab \
        -text x \
        -width 15 \
        -anchor e
  label .meshnodebox.input.ycoordlab \
        -text y \
        -width 15 \
        -anchor e
  label .meshnodebox.input.meshparamlab \
        -text {mesh parameter} \
        -width 15 \
        -anchor e

  button .meshnodebox.accept.ok \
         -command {EditMeshNodeAction} \
         -text {OK} 
  button .meshnodebox.accept.clear \
         -command {ClearEditMeshNodeBox}\
         -text {Clear}
  button .meshnodebox.accept.dismis \
         -text {Dismis} \
         -command DestroyEditMeshNodeBox

  if {$nodenum >0} {
      label .meshnodebox.input.nodenument \
          -text $nodenum \
          -anchor e
  } else {
      entry .meshnodebox.input.nodenument \
          -validate focusout \
          -validatecommand {NotifyEntry %W nodenum integer %P} 
  }

  entry .meshnodebox.input.xcoordent \
        -validate focusout \
        -validatecommand {NotifyEntry %W xcoord double %P} 

  entry .meshnodebox.input.ycoordent\
        -validate focusout \
        -validatecommand {NotifyEntry %W ycoord double %P} 

  entry .meshnodebox.input.meshparament \
        -validate focusout \
        -validatecommand {NotifyEntry %W meshparam double %P} 
 
  grid .meshnodebox.input.nodenumlab \
       .meshnodebox.input.nodenument -pady 15
  grid .meshnodebox.input.xcoordlab .meshnodebox.input.xcoordent
  grid .meshnodebox.input.ycoordlab  \
       .meshnodebox.input.ycoordent 
  grid .meshnodebox.input.meshparamlab .meshnodebox.input.meshparament
 
  pack configure .meshnodebox.accept.ok \
      -side left \
      -padx 5
  pack configure .meshnodebox.accept.clear \
      -side left \
      -padx 5
  pack configure .meshnodebox.accept.dismis \
      -side left \
      -padx 5

  pack configure .meshnodebox.input
  pack configure .meshnodebox.separ \
       -fill x \
       -pady 10
   
  pack configure .meshnodebox.accept 
  if {$nodenum != 0} {
        InitEditMeshNodeBox $nodenum
  }
}


proc InitEditMeshNodeBox {nodenum} {
   global Mesh
   global Entries
   global drawArea
   ClearEditMeshNodeBox 
   set nodeindex [expr $nodenum -1]
   set meshparam [lindex $Mesh(nodemeshparam) $nodeindex]
   set pp [lindex $Mesh(nodecoords) $nodeindex] 
   set x [lindex $pp 0]
   set y [lindex $pp 1]
   .meshnodebox.input.meshparament insert end $meshparam
   .meshnodebox.input.xcoordent insert end $x 
   .meshnodebox.input.ycoordent insert end $y 
   if {[string equal [winfo class .meshnodebox.input.nodenument] "Label"]} {
     .meshnodebox.input.nodenument configure -text $drawArea(nodeselected) 
     set Entries(nodenum) 0
   }


   set Entries(xcoord) 1
   set Entries(ycoord) 1
   set Entries(meshparam) 1
   set Entries(nodenum) 1
   return
}

proc ClearEditMeshNodeBox {} {
   global Entries
   set Entries(xcoord) 0
   set Entries(ycoord) 0
   set Entries(meshparam) 0
   .meshnodebox.input.meshparament delete 0 end 
   .meshnodebox.input.xcoordent delete 0 end  
   .meshnodebox.input.ycoordent delete 0 end  

   if {[string equal [winfo class .meshnodebox.input.nodenument] "Entry"]} {
     .meshnodebox.input.nodenument delete 0 end 
     set Entries(nodenum) 0
   }
   return
}

proc DestroyEditMeshNodeBox {} {
   catch "destroy .meshnodebox"
  .nvWork.right.editmeshnode.edit configure \
        -state normal 
   update
}



proc ShowEditNurbsBox {{nurbsnum -1} {nurbsnode 0}} {

  .nvWork.right.nurbs.edit configure \
        -state disable

  toplevel .nurbsnodebox \
       -borderwidth 10
  
  wm positionfrom .nurbsnodebox ""
  wm title .nurbsnodebox {nurbsview : Edit NURBS Node}
  frame .nurbsnodebox.input
  frame .nurbsnodebox.separ \
        -relief ridge \
        -bd 1 \
        -height 2
  frame .nurbsnodebox.accept \

  label .nurbsnodebox.input.nurbsnumlab \
        -text {nurbs number} \
        -width 15 \
        -anchor e
  label .nurbsnodebox.input.nurbsnodenumlab \
        -text {node number} \
        -width 15 \
        -anchor e
  label .nurbsnodebox.input.xcoordlab \
        -text x \
        -width 15 \
        -anchor e
  label .nurbsnodebox.input.ycoordlab \
        -text y \
        -width 15 \
        -anchor e
  label .nurbsnodebox.input.nurbsweightlab \
        -text {nurbs weight} \
        -width 15 \
        -anchor e

  button .nurbsnodebox.accept.ok \
         -command {EditNurbsAction} \
         -text {OK} 
  button .nurbsnodebox.accept.clear \
         -command {ClearEditNurbsBox}\
         -text {Clear}
  button .nurbsnodebox.accept.dismis \
         -text {Dismis} \
         -command DestroyEditNurbsBox


  entry .nurbsnodebox.input.nurbsnument \
        -validate focusout \
        -validatecommand {NotifyEntry %W nurbsnum integer %P} 

  entry .nurbsnodebox.input.nurbsnodenument \
        -validate focusout \
        -validatecommand {NotifyEntry %W nodenum integer %P} 

  entry .nurbsnodebox.input.xcoordent \
        -validate focusout \
        -validatecommand {NotifyEntry %W xcoord double %P} 

  entry .nurbsnodebox.input.ycoordent\
        -validate focusout \
        -validatecommand {NotifyEntry %W ycoord double %P} 

  entry .nurbsnodebox.input.nurbsweightent \
        -validate focusout \
        -validatecommand {NotifyEntry %W nurbsweight double %P} 
 

  grid .nurbsnodebox.input.nurbsnumlab \
       .nurbsnodebox.input.nurbsnument 
  grid .nurbsnodebox.input.nurbsnodenumlab \
       .nurbsnodebox.input.nurbsnodenument 

  grid .nurbsnodebox.input.nurbsweightlab \
              .nurbsnodebox.input.nurbsweightent -pady 15

  grid .nurbsnodebox.input.xcoordlab .nurbsnodebox.input.xcoordent
  grid .nurbsnodebox.input.ycoordlab  .nurbsnodebox.input.ycoordent 
 
  pack configure .nurbsnodebox.accept.ok \
      -side left \
      -padx 5
  pack configure .nurbsnodebox.accept.clear \
      -side left \
      -padx 5
  pack configure .nurbsnodebox.accept.dismis \
      -side left \
      -padx 5

  pack configure .nurbsnodebox.input
  pack configure .nurbsnodebox.separ \
       -fill x \
       -pady 10
   
  pack configure .nurbsnodebox.accept 
  
  if {$nurbsnode != 0} {
        InitEditNurbsBox $nurbsnum $nurbsnode
  }
}


proc InitEditNurbsBox {nurbsnum nodenum} {
   global Nursb 
   global Entries
   ClearEditNurbsBox
   set nurbsInfo [etliNurbs$nurbsnum info -ctrlpoint $nodenum]
   set x [lindex $nurbsInfo 0]
   set y [lindex $nurbsInfo 1]
   set weight [lindex $nurbsInfo 3]
   .nurbsnodebox.input.nurbsweightent insert end $weight
   .nurbsnodebox.input.xcoordent insert end $x 
   .nurbsnodebox.input.ycoordent insert end $y 
   .nurbsnodebox.input.nurbsnument insert end $nurbsnum
   .nurbsnodebox.input.nurbsnodenument insert end $nodenum
   set Entries(nurbsnum) 1
   set Entries(nodenum) 1
   set Entries(xcoord) 1
   set Entries(ycoord) 1
   set Entries(weight) 1
   return
}


proc DestroyEditNurbsBox {} {
   catch "destroy .nurbsnodebox"
  .nvWork.right.nurbs.edit configure \
        -state normal 
   update
}

proc ClearEditNurbsBox {} {
   global Entries
   set Entries(nurbsnum) 0
   set Entries(nodenum) 0
   set Entries(xcoord) 0
   set Entries(ycoord) 0
   set Entries(weight) 0
   .nurbsnodebox.input.nurbsweightent delete 0 end 
   .nurbsnodebox.input.xcoordent delete 0 end  
   .nurbsnodebox.input.ycoordent delete 0 end  
   .nurbsnodebox.input.nurbsnument delete 0 end 
   .nurbsnodebox.input.nurbsnodenument delete 0 end 
   return
}


proc ShowSetLimitsBox {} {
  global drawArea

  if { [winfo exist .setlimitsbox] == 1} {
     return 
  }

  toplevel .setlimitsbox \
       -borderwidth 10
  
  wm positionfrom .setlimitsbox ""
  wm title .setlimitsbox {nurbsview : Set limits}
  frame .setlimitsbox.input
  frame .setlimitsbox.separ \
        -relief ridge \
        -bd 1 \
        -height 2
  frame .setlimitsbox.accept 

  label .setlimitsbox.input.xlowlab \
        -text {left bottom x} \
        -width 15 \
        -anchor e
  label .setlimitsbox.input.ylowlab \
        -text {left bottom y} \
        -width 15 \
        -anchor e
  label .setlimitsbox.input.xhighlab \
        -text {right top x} \
        -width 15 \
        -anchor e
  label .setlimitsbox.input.yhighlab \
        -text {right top y} \
        -width 15 \
        -anchor e

  button .setlimitsbox.accept.ok \
         -command {SetLimitsAction} \
         -text {OK} 

  button .setlimitsbox.accept.clear \
         -command {ClearLimitsBox}\
         -text {Clear}
  button .setlimitsbox.accept.dismis \
         -text {Dismis} \
         -command DestroySetLimitsBox


  entry .setlimitsbox.input.xlowent \
        -validate all \
        -validatecommand {NotifyEntry %W xlow double %P} 

  entry .setlimitsbox.input.ylowent \
        -validate all \
        -validatecommand {NotifyEntry %W ylow double %P} 

  entry .setlimitsbox.input.xhighent \
        -validate all \
        -validatecommand {NotifyEntry %W xhigh double %P} 

  entry .setlimitsbox.input.yhighent \
        -validate all \
        -validatecommand {NotifyEntry %W yhigh double %P} 

  grid .setlimitsbox.input.xlowlab \
       .setlimitsbox.input.xlowent 
  grid .setlimitsbox.input.ylowlab \
       .setlimitsbox.input.ylowent 

  grid .setlimitsbox.input.xhighlab \
       .setlimitsbox.input.xhighent 
  grid .setlimitsbox.input.yhighlab \
       .setlimitsbox.input.yhighent 
 
  pack configure .setlimitsbox.accept.ok \
      -side left \
      -padx 5
  pack configure .setlimitsbox.accept.clear \
      -side left \
      -padx 5
  pack configure .setlimitsbox.accept.dismis \
      -side left \
      -padx 5

  pack configure .setlimitsbox.input
  pack configure .setlimitsbox.separ \
       -fill x \
       -pady 10
   
  pack configure .setlimitsbox.accept 

   set xlow [lindex $drawArea(region) 0]
   set ylow [lindex $drawArea(region) 1]
   set xhigh [lindex $drawArea(region) 2]
   set yhigh [lindex $drawArea(region) 3]

   set Entries(xlow) 1
   set Entries(ylow) 1
   set Entries(xhigh) 1
   set Entries(yhigh) 1
   .setlimitsbox.input.xlowent  insert  end  $xlow
   .setlimitsbox.input.ylowent  insert  end  $ylow
   .setlimitsbox.input.xhighent insert  end  $xhigh
   .setlimitsbox.input.yhighent insert  end  $yhigh
}

proc ClearLimitsBox {} {
   global Entries
   set Entries(xlow) 0
   set Entries(ylow) 0
   set Entries(xhigh) 0
   set Entries(yhigh) 0
   .setlimitsbox.input.xlowent delete 0 end 
   .setlimitsbox.input.ylowent delete 0 end  
   .setlimitsbox.input.xhighent delete 0 end  
   .setlimitsbox.input.yhighent delete 0 end 
   return
}

proc DestroySetLimitsBox {} {
   catch "destroy .setlimitsbox"
   update
}


proc ShowSetRegionBox {} {
  global drawArea
  toplevel .setregionbox \
       -borderwidth 10
  
  wm positionfrom .setregionbox ""
  wm title .setregionbox {nurbsview : Set zoom region size}
  frame .setregionbox.input
  frame .setregionbox.separ \
        -relief ridge \
        -bd 1 \
        -height 2
  frame .setregionbox.accept \

  label .setregionbox.input.regionlab \
        -text {Canvas fraction} \
        -width 15 \
        -anchor e

  button .setregionbox.accept.ok \
         -command {SetRegionBoxAction} \
         -text {OK} 
  button .setregionbox.accept.dismis \
         -text {Cancel} \
         -command DestroySetRegionBox


  entry .setregionbox.input.regionent \
        -validate all \
        -validatecommand {NotifyEntry %W region integer %P} 

  grid .setregionbox.input.regionlab \
       .setregionbox.input.regionent 

  pack configure .setregionbox.accept.ok \
      -side left \
      -padx 5
  pack configure .setregionbox.accept.dismis \
      -side left \
      -padx 5

  pack configure .setregionbox.input
  pack configure .setregionbox.separ \
       -fill x \
       -pady 10
   
  pack configure .setregionbox.accept 
   .setregionbox.input.regionent  insert  end  $drawArea(zoomRegionRatio)
}

proc DestroySetRegionBox {} {
   catch "destroy .setregionbox"
   update
}

proc ShowInitWindow {} {
#  tkwait visibility .nvWork.right.nurbs
#  ShowNurbsEditPanel 0
#  tkwait visibility .nvWork.right.editsetlimits
#  ShowEditMeshNodePanel 0
}


proc ConfirmExit {} {
    set answer [tk_messageBox -message "Really quit?" -type yesno -icon question]
    switch -- $answer {
       yes exit
       no {}
   }
}
#EOF


